package com.tztzf.base;

import com.tztzf.baseos.tztZFBaseFunc;
import com.tztzf.baseos.tztZFDoneFile;
import com.tztzf.baseos.tztZFTextUtils;

import java.util.ArrayList;

/**
 * 作者: yangdl 2019.06
 * 用途: 公共函数
 * 修改记录如下:
 * <p>
 * 版本号: 1.00.001 2019.06.10
 * 更新说明: 第一版正式发布
 */

public final class tztZFPublicFun {

    /**
     * 判断字符串是否为空
     *
     * @param str 字符串
     * @return 字符串是否为空（true=空 false=不为空）
     */
    public static boolean isEmpty(String str) {
        return tztZFTextUtils.isEmpty(str);
    }

    /**
     * 截取字符串
     *
     * @param str 字符串
     * @param key 分隔符
     * @return 截取后字符串
     */
    public static String getSubString(String str, String key) {
        String bkey = "<" + key + ">";
        String ekey = "</" + key + ">";
        int l = bkey.length();
        int b = str.indexOf(bkey);
        if (b < 0) {
            return "";
        }
        int e = str.indexOf(ekey, b);
        if (e < 0) {
            return "";
        }
        return str.substring(b + l, e);
    }

    /**
     * 分割字符串放入ArrayList里
     * 备注：字符串:Android安卓是向前分割 iOS是向后分割 导致不一样
     * 场景
     * 字符串:123|123| 分割符 ｜ androiod返回2个 iOS有3个
     * 字符串:|||| 分割符 ｜ androiod返回空 iOS有多个
     * 字符串|123||||| 分隔符| Android返回2个 iOS多个，会数组内长度不匹配，特别是在解析grid的时候
     *
     * @param original 字符串
     * @param regex    分隔符
     * @param AyList   ArrayList
     * @return 空
     */
    public static void split(String original, String regex, ArrayList<String> AyList) {
        int startIndex = 0;
        if (AyList == null)
            AyList = new ArrayList<>();
        if (isEmpty(original))
            return;
        int index = 0;
        startIndex = original.indexOf(regex);
        while (startIndex < original.length() && startIndex != -1) {
            String temp = original.substring(index, startIndex);
            AyList.add(temp);
            index = startIndex + regex.length();
            startIndex = original.indexOf(regex, startIndex + regex.length());
        }
        if (index < original.length())
            AyList.add(original.substring(index));
    }

    /**
     * 获取字符串包含char个数
     *
     * @param s  字符串
     * @param ch char数据
     * @return 包含个数
     */
    public static int CharCount(String s, int ch) {
        int i = s.indexOf(ch, 0), j = 0;
        while (i >= 0) {
            j++;
            i = s.indexOf(ch, i + 1);
        }
        return j;
    }

    /**
     * 分割字符串
     *
     * @param txt   字符串
     * @param split 分隔符
     * @return 分割后字符串数组
     */
    public static String[] getArray(String txt, char split) {
        int count = CharCount(txt, split);
        String[] arrTemp = new String[count];
        int begin = 0, end;
        for (int i = 0; i < count; i++) {
            end = txt.indexOf(split, begin);
            arrTemp[i] = txt.substring(begin, end);
            begin = end + 1;
        }
        return arrTemp;
    }

    /**
     * 得到从start开始，占位数为length的int值,length为此int值所占的位数
     *
     * @param data   数据流
     * @param start  起始位置
     * @param length 长度
     * @return int值
     */
    public static int getInt(byte[] data, int start, int length) {
        int temp = 0;
        for (int i = 0; i < length; i++) {
            byte a = data[i + start];
            temp += ((a >> 4 & 15) << 8 * i + 4) + ((a & 15) << 8 * i);
        }
        return temp;
    }

    /**
     * 数据流转int
     *
     * @param data 数据流
     * @return int值
     */
    public static int getInt(byte[] data) {
        return getInt(data, 0, data.length);
    }

    /**
     * int值转byte数组，低位在前
     *
     * @param value int数据
     * @return 转换后的byte数组
     */
    public static byte[] intToByteArray(int value) {
        return new byte[]{
                (byte) (value & 0xFF),
                (byte) ((value >> 8) & 0xFF),
                (byte) ((value >> 16) & 0xFF),
                (byte) ((value >> 24) & 0xFF)
        };
    }

    /**
     * 将long型数据转换为Dword的字节数组（C/C++的无符号整数）
     *
     * @param value 长整型数据
     * @return 数据流
     */
    public static byte[] longToDword(long value) {
        byte[] data = new byte[4];

        for (int i = 0; i < data.length; i++) {
            data[i] = (byte) (value >> (8 * i));
        }

        return data;
    }

    /**
     * 字符串转int
     *
     * @param nStr     字符串
     * @param defValue 默认int值
     * @return int数据
     */
    public static int parseInt(String nStr, int defValue) {
        return tztZFBaseFunc.parseInt(nStr, defValue);
    }

    /**
     * 获取配置文件到字节数组
     *
     * @param context      上下文
     * @param strAssetFile Asset目录下的文件名
     * @return 文件数据
     */
    public static byte[] getAssetsBytes(Object context, String strAssetFile) {
        return tztZFDoneFile.getAssetsBytes(context, strAssetFile);
    }

    /**
     * 写文件
     *
     * @param context  上下文
     * @param filename 文件名
     * @param fileData 文件数据
     */
    public static boolean writeFileData(Object context, String filename, byte[] fileData) {
        return tztZFDoneFile.writeFileData(context, filename, fileData);
    }

    /**
     * 读文件
     *
     * @param context  上下文
     * @param filename 文件名
     * @return 文件数据
     */
    public static byte[] readFileData(Object context, String filename) {
        return tztZFDoneFile.readFileData(context, filename);
    }

    /**
     * 获取APK包路径
     *
     * @param context 上下文
     * @return APK包路径
     */
    public static String getPackagePath(Object context) {
        return tztZFBaseFunc.getPackagePath(context);
    }

    /**
     * 获取指定文件在应用下的完整路径
     *
     * @param fileName 文件名称
     * @return 文件完整路径
     */
    public static String getFileDocumentPath(Object context, String fileName) {
        return tztZFDoneFile.getFileDocumentPath(context, fileName);
    }

}
