package com.tztzf.network.cosign;

import com.tztzf.baseos.tztZFNullable;
import com.tztzf.baseos.tztZFTextUtils;
import com.tztzf.log.tztZFLog;
import com.tztzf.log.tztZFLogLevel;

import java.io.Serializable;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

/**
 * 证书信息对象
 */
public class tztZFCertificateInfo implements Serializable {
    /** 用户ID */
    private String userID;
    /** 证书序列号 */
    private String serialNumber;
    /** 证书subject */
    private String subject;
    /** 结束日期 */
    private Date notAfter;
    /** 开始日期 */
    private Date notBefore;
    /** 签发日期 */
    private Date signDate;
    /** 状态 */
    private String status;
    /** 设备信息 */
    private String deviceInfo;
    /** 设备唯一号 */
    private String deviceID;
    /** 是否本地证书 */
    private boolean isLocal;

    //region get/set
    public String getUserID() {
        return userID;
    }

    public void setUserID(String userID) {
        this.userID = userID;
    }

    public String getSerialNumber() {
        return serialNumber;
    }

    public void setSerialNumber(String serialNumber) {
        this.serialNumber = serialNumber;
    }

    public String getSubject() {
        return subject;
    }

    public void setSubject(String subject) {
        this.subject = subject;
    }

    public Date getNotAfter() {
        return notAfter;
    }

    public void setNotAfter(Date notAfter) {
        this.notAfter = notAfter;
    }

    public Date getNotBefore() {
        return notBefore;
    }

    public void setNotBefore(Date notBefore) {
        this.notBefore = notBefore;
    }

    public Date getSignDate() {
        return signDate;
    }

    public void setSignDate(Date signDate) {
        this.signDate = signDate;
    }

    public String getStatus() {
        return status;
    }

    public void setStatus(String status) {
        this.status = status;
    }

    public String getDeviceInfo() {
        return deviceInfo;
    }

    public void setDeviceInfo(String deviceInfo) {
        this.deviceInfo = deviceInfo;
    }

    public String getDeviceID() {
        return deviceID;
    }

    public void setDeviceID(String deviceID) {
        this.deviceID = deviceID;
    }

    public boolean isLocal() {
        return isLocal;
    }

    public void setLocal(boolean local) {
        isLocal = local;
    }

    //endregion

    /**
     * 清空所有信息
     */
    public void clear() {
        userID = "";
        serialNumber = "";
        subject = "";
        notAfter = null;
        notBefore = null;
        signDate = null;
        status = "";
        deviceInfo = "";
        isLocal = false;
    }

    /**
     * 日期格式化成字符串
     *
     * @param date   日期
     * @param format 格式化形式，默认yyyy-MM-dd HH:mm:ss
     * @return 格式化后的日期
     */
    public static String dataFormatter(Date date, String format) {
        if (date == null)
            return "";
        if (format == null || format.length() < 1)
            format = "yyyy-MM-dd HH:mm:ss";

        SimpleDateFormat dateFormat = new SimpleDateFormat(format, Locale.getDefault());
        return dateFormat.format(date);
    }

    /**
     * 格式化日期 yyyy-MM-dd HH:mm:ss形式
     *
     * @param date 日期
     * @return 格式化后的日期
     */
    public static String dateFormatter(Date date) {
        return dataFormatter(date, "yyyy-MM-dd HH:mm:ss");
    }

    /**
     * 字符串转换成日期
     *
     * @param strDate 字符串
     * @param format  格式化方式，默认yyyyMMddHHmmss
     * @return 转换后的日期，可为null
     */
    @tztZFNullable
    public static Date dateFromString(String strDate, String format) {
        if (tztZFTextUtils.isEmpty(strDate))
            return null;
        boolean isEmpty = tztZFTextUtils.isEmpty(format);
        try {
            if (isEmpty || !format.contains("-")) {
                strDate = strDate.replaceAll("-", "");
            }
            if (isEmpty || !format.contains(":")) {
                strDate = strDate.replaceAll(":", "");
            }
            if (isEmpty || !format.contains(" ")) {
                strDate = strDate.replaceAll(" ", "");
            }
            if (strDate.length() < 1)
                return null;
            if (isEmpty) {
                if (strDate.length() == 17) {
                    format = "yyyyMMddHHmmssSSS";
                } else if (strDate.length() == 14) {
                    format = "yyyyMMddHHmmss";
                } else if (strDate.length() == 8) {
                    format = "yyyyMMdd";
                } else {
                    return null;
                }
            }
            SimpleDateFormat dateFormat = new SimpleDateFormat(format, Locale.getDefault());
            return dateFormat.parse(strDate);
        } catch (Exception e) {
            tztZFLog.log(tztZFLogLevel.tztZFLogLevel_Debug, "tztZFCertification-dateFormat", "%s", tztZFLog.getStackTraceString(e));
        }
        return null;
    }

    /**
     * 字符串格式化成日期，yyyyMMddHHmmss形式
     *
     * @param strDate 日期字符串
     * @return 转换后的日期，可为null
     */
    @tztZFNullable
    public static Date dateFromString(String strDate) {
        return dateFromString(strDate, "");
    }
}