package com.tztzf.network.cosign;

import com.tztzf.base.tztZFPublicFun;
import com.tztzf.baseos.tztZFNonNull;
import com.tztzf.baseos.tztZFNullable;
import com.tztzf.baseos.tztZFTextUtils;
import com.tztzf.commbase.tztZFDataMapBase;
import com.tztzf.commbase.tztZFDataMapCallBack;
import com.tztzf.commbase.tztZFThreadPool;
import com.tztzf.log.tztZFLog;
import com.tztzf.log.tztZFLogLevel;
import com.tztzf.network.callback.tztZFLinkCallBack;
import com.tztzf.network.define.tztZFErrorMessageDefine;
import com.tztzf.network.define.tztZFLinkType;
import com.tztzf.network.gm.tztZFGMManage;
import com.tztzf.network.tztZFNetWork;
import com.tztzf.network.tztZFNetWorkRespObj;
import com.tztzf.protocol.callback.tztZFBlocktztCosign;
import com.tztzf.protocol.core.tztZFCosignSSL;
import com.tztzf.protocol.core.tztZFDataEncrypt;
import com.tztzf.protocol.core.tztZFProtocolCert;
import com.tztzf.protocol.core.tztZFSSLCert;
import com.tztzf.protocol.tztZFEncryptKit;
import com.tztzf.recordprocesslog.tztZFRecordEventType;
import com.tztzf.recordprocesslog.tztZFRecordProcessLogTools;
import com.tztzf.recordprocesslog.tztZFReocordLogType;
import com.tztzf.tztZFNotificationObj;

import org.greenrobot.eventbus.EventBus;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Locale;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

import static com.tztzf.network.cosign.tztZFCosignErr.tztZFCosignErrClientSign0;
import static com.tztzf.network.cosign.tztZFCosignErr.tztZFCosignErrConnect;
import static com.tztzf.network.cosign.tztZFCosignErr.tztZFCosignErrCreateP10;
import static com.tztzf.network.cosign.tztZFCosignErr.tztZFCosignErrDevice;
import static com.tztzf.network.cosign.tztZFCosignErr.tztZFCosignErrExpire;
import static com.tztzf.network.cosign.tztZFCosignErr.tztZFCosignErrNoLoad;
import static com.tztzf.network.cosign.tztZFCosignErr.tztZFCosignErrOK;
import static com.tztzf.network.cosign.tztZFCosignErr.tztZFCosignErrOther;
import static com.tztzf.network.cosign.tztZFCosignErr.tztZFCosignErrParam;
import static com.tztzf.network.cosign.tztZFCosignErr.tztZFCosignErrPin;
import static com.tztzf.network.cosign.tztZFCosignErr.tztZFCosignErrServerCertUnfind;
import static com.tztzf.network.cosign.tztZFCosignErr.tztZFCosignErrSign;
import static com.tztzf.network.cosign.tztZFNetCosignEx.tztCosignEx_CACert;
import static com.tztzf.network.cosign.tztZFNetCosignEx.tztCosignEx_CurrentTime;
import static com.tztzf.network.cosign.tztZFNetCosignEx.tztCosignEx_Status;
import static com.tztzf.network.define.tztZFLinkType.tztZFLinkType_Cosign;
import static com.tztzf.network.define.tztZFLinkType.tztZFLinkType_JY;
import static com.tztzf.network.gm.tztZFGMManage.tztZFJYLoginGMProcessStep;
import static com.tztzf.network.gm.tztZFSocketSSLStep.tztZFSocketSSLStepDowngrade;
import static com.tztzf.protocol.define.tztZFCertType.tztZFCertPEMChain;

/**
 * 协同签名
 */
public class tztZFNetCosign {
    /** 单例使用 */
    private static tztZFNetCosign g_netCosign = null;
    /** 初始化注册的appkey */
    private String appKey;
    /** APPCosignType，三方使用 */
    private int appCosignType;
    /** 当前的用户 */
    private String loginUserID;
    /** 绑定的用户列表 */
    private tztZFDataMapBase<String, tztZFCosignUser> dictLoginUser;

    private tztZFDataMapBase<String, tztBlockCreateP10> dictBlockCreate;
    private tztZFDataMapBase<String, tztBlockImportCert> dictBlockImport;
    private tztZFDataMapBase<String, tztBlockLoadCert> dictBlockLoad;
    private tztZFDataMapBase<String, tztBlockCosign> dictBlockCosign;
    private tztZFDataMapBase<String, tztBlockDeleteCert> dictBlockDelete;

    private tztCosignRespCallBack cosignRespCallBack;
    private tztZFThreadPool respThreadPool = null;
    private tztZFThreadPool createP10ThreadPool = null;

    private Lock cosignLock;
    private Object context;

    private boolean bCallBackAsync = true;

    private static final String TAG = "tztCert";

    /** 发送请求回调，若不为nil，使用回调外层自行进行数据收发处理 */
    private tztZFNetCosignSendBlock sendRecvBlock;
    /** 构造时间 */
    private final long classCreateTime;

    public static void init(Object context) {
        getShareInstance().context = context;
    }

    public static Object getContext() {
        return getShareInstance().context;
    }

    public static synchronized tztZFNetCosign getShareInstance() {
        if (g_netCosign == null) {
            g_netCosign = new tztZFNetCosign();
        }
        return g_netCosign;
    }

    private tztZFNetCosign() {
        classCreateTime = System.currentTimeMillis();
        initdata();
    }

    /**
     * 协同签名回调
     */
    public static void setCosignRespCallBack(tztCosignRespCallBack callBack) {
        getShareInstance().cosignRespCallBack = callBack;
    }

    private void cosignCallback(final String userID, final tztZFNetCosignHead.tztZFCosignResp resp) {
        if (cosignRespCallBack == null)
            return;
        tztZFLog.log(tztZFLogLevel.tztZFLogLevel_Debug, TAG, "cosignCallback:------%s", userID);
        if (bCallBackAsync) {
            respThreadPool.useThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    if (cosignRespCallBack != null) cosignRespCallBack.onCallBack(userID, resp);
                }
            });
        } else {
            if (cosignRespCallBack != null) cosignRespCallBack.onCallBack(userID, resp);
        }
    }

    /**
     * 设置协同签名服务器地址列表
     *
     * @param addlist 地址列表
     */
    private tztZFNetCosign withAddlist(String addlist) {
        tztZFNetWork.registLink(context, tztZFLinkType.tztZFLinkType_Cosign.value(), 8);
        tztZFNetWork.setAddressList(tztZFLinkType.tztZFLinkType_Cosign.value(), addlist, -1);
        tztZFNetWork.setConnectTimeOut(tztZFLinkType.tztZFLinkType_Cosign.value(), 6);
        tztZFNetWork.setLinkBlock(tztZFLinkType.tztZFLinkType_Cosign.value(), new tztZFLinkCallBack() {
            @Override
            public void onHandler(int c_blockType, int c_sdkType, int c_linkType, @tztZFNonNull tztZFNetWorkRespObj c_respObj) {
                if (c_respObj != null) {
                    int gmStatus = c_respObj.getInt("GMStatus");
                    if (gmStatus == 2) {
                        //通道返回国密降级状态后，也需要做降级处理
                        tztZFNetCosign.didGMDowngrade();
                        tztZFNetWork.setLinkTypeGMDowngrade(tztZFLinkType_JY.value());
                        //通知上层
                        tztZFNotificationObj notificationObj = new tztZFNotificationObj();
                        notificationObj.setName(tztZFJYLoginGMProcessStep);
                        tztZFDataMapBase<String, Object> dataMapBase = new tztZFDataMapBase<>();
                        dataMapBase.setInt("step", tztZFSocketSSLStepDowngrade);
                        String userID = tztZFNetCosign.getBindUserID();
                        if (userID == null)
                            userID = "";
                        dataMapBase.setStr("userID", userID);
                        notificationObj.setData(dataMapBase);
                        EventBus.getDefault().post(notificationObj);
                    }
                    tztZFLog.log(tztZFLogLevel.tztZFLogLevel_Debug, TAG, "协同签名-网络服务:%s", c_respObj.toString());
                }
            }
        });
        return this;
    }

    /**
     * 设置协同签名服务器地址列表
     *
     * @param addlist 地址列表
     */
    public static tztZFNetCosign initWithAddlist(String addlist) {
        tztZFNetCosign obj = getShareInstance();
        return obj.withAddlist(addlist);
    }

    /**
     * 设置协同签名AppKey、CA
     *
     * @param appKey appKey
     * @param cert   CA
     */
    public tztZFNetCosign withAppKey(String appKey, tztZFDataMapBase<String, Object> cert) {
        setAppKey(appKey, cert);
        return this;
    }

    private static String blockKey(int cosignType) {
        return String.format(Locale.getDefault(), "BLOCKKEY_%d", cosignType);
    }

    /**
     * 生成P10回调
     *
     * @param cosignType  类型
     * @param blockCreate 回调
     */
    public tztZFNetCosign withCreateP10(int cosignType, tztBlockCreateP10 blockCreate) {
        this.dictBlockCreate.setObj(blockKey(cosignType), blockCreate);
        return this;
    }

    /**
     * 导入证书
     *
     * @param cosignType  类型
     * @param blockImport 回调
     */
    public tztZFNetCosign withImport(int cosignType, tztBlockImportCert blockImport) {
        this.dictBlockImport.setObj(blockKey(cosignType), blockImport);
        return this;
    }

    /**
     * 加载
     *
     * @param cosignType 类型
     * @param blockLoad  回调
     */
    public tztZFNetCosign withLoad(int cosignType, tztBlockLoadCert blockLoad) {
        this.dictBlockLoad.setObj(blockKey(cosignType), blockLoad);
        return this;
    }

    /**
     * 签名
     *
     * @param cosignType 类型
     * @param blockSign  回调
     */
    public tztZFNetCosign withCosign(int cosignType, tztBlockCosign blockSign) {
        this.dictBlockCosign.setObj(blockKey(cosignType), blockSign);
        return this;
    }

    /**
     * 删除
     *
     * @param cosignType  类型
     * @param blockDelete 回调
     */
    public tztZFNetCosign withDelete(int cosignType, tztBlockDeleteCert blockDelete) {
        this.dictBlockDelete.setObj(blockKey(cosignType), blockDelete);
        return this;
    }

    private void initdata() {
        String pinLable = tztZFErrorMessageDefine.getUserDefineError(tztZFErrorMessageDefine.tztZFNetWorkErrorKey_PinLable, tztZFErrorMessageDefine.tztZFPinLable);
        tztZFLog.log(tztZFLogLevel.tztZFLogLevel_Info, TAG, "%s管理处理初始化", pinLable);
        loginUserID = "";
        dictLoginUser = new tztZFDataMapBase<>();
        dictBlockCosign = new tztZFDataMapBase<>();
        dictBlockCreate = new tztZFDataMapBase<>();
        dictBlockImport = new tztZFDataMapBase<>();
        dictBlockLoad = new tztZFDataMapBase<>();
        dictBlockDelete = new tztZFDataMapBase<>();
        if (respThreadPool == null) {
            respThreadPool = new tztZFThreadPool(1, 1, "cosignRespThread");
            respThreadPool.setUsed(true);
        }
        if (createP10ThreadPool == null) {
            createP10ThreadPool = new tztZFThreadPool(1, 1, "cosignCreateP10Thread");
            createP10ThreadPool.setUsed(true);
        }
        cosignLock = new ReentrantLock();
        tztZFCosignSSL.withConnectType(8, new tztZFBlocktztCosign() {
            @Override
            public tztZFDataMapBase<String, Object> onHandler(int type, byte[] org) {
                return tztZFNetCosign.this.cosignOrgData(type, org, 0);
            }
        });
    }

    private tztZFDataMapBase<String, Object> cosignOrgData(int type, byte[] org, int fromType) {
        getShareInstance().cosignLock.lock();
        final tztZFDataMapBase<String, Object> signMap = new tztZFDataMapBase<>();

        final tztZFCosignSSL.tztZFResponse response = cosignOrg(type, org, fromType, new tztZFNetCosignCallBack() {
            @Override
            public void netCosignCallBack(boolean c_bOK, tztZFDataMapBase<String, Object> c_cosign) {
                synchronized (tztZFNetCosign.this) {
                    if (c_bOK && c_cosign != null) {
                        signMap.setBytes("signData", c_cosign.getBytes("signData"));
                        signMap.setBytes("signType", c_cosign.getBytes("signType"));
                    }
                    tztZFNetCosign.this.notify();
                }
            }
        });

        boolean bOK = response != null && response.getRetCode() == tztZFCosignErrOK;
        //增加签名数据获取判断，上层签名要是同步处理，无需进行等待
        if (bOK && signMap.getBytes("signData") == null) {
            synchronized (tztZFNetCosign.this) {
                try {
                    tztZFNetCosign.this.wait(30 * 1000);
                } catch (Exception e) {
                    cosignLock.unlock();
                }
            }
        }

        signMap.setObj(tztZFCosignSSL.tztZFResponse.class.getSimpleName(), response);
        cosignLock.unlock();
        return signMap;
    }

    /**
     * 设置三方签名CA
     *
     * @param cosignType 签名type
     * @param cacert     ca证书
     */
    private void _setCosignType(int cosignType, byte[] cacert) {
        this.appCosignType = cosignType;
        tztZFDataMapBase<String, Object> exInfo = tztZFNetCosignEx.cosignExInfo();
        if (exInfo != null) {
            exInfo.setBytes(tztCosignEx_CACert, cacert);
            exInfo.setInt(tztCosignEx_Status, 1);
        }
    }

    /**
     * 设置三方签名CA
     *
     * @param cosignType 签名type
     * @param cacert     ca证书
     */
    public static void setCosignType(int cosignType, byte[] cacert) {
        getShareInstance()._setCosignType(cosignType, cacert);
    }

    /**
     * 导入客户端key、CA证书
     *
     * @param appKey key
     * @param cert   ca证书
     */
    public static boolean setAppKey(String appKey, tztZFDataMapBase<String, Object> cert) {
        getShareInstance().appKey = appKey;
        return tztZFNetCosignEx.setAppKey(appKey, cert);
    }

    /**
     * 设置证书Sub
     *
     * @param certSub 证书Sub字典， cn，o，ou不支持设置，其他的ST，L等支持设置
     */
    public static boolean setCertPub(tztZFDataMapBase<String, Object> certSub) {
        return tztZFNetCosignEx.setCertPub(certSub);
    }

    /**
     * 根据userID生成Sub
     *
     * @param userID 客户号
     */
    public static String getCertSub(String userID) {
        return tztZFNetCosignEx.getCertSub(userID);
    }

    /**
     * 请求协同签名扩展
     *
     * @param reqInfo  附加信息字典，可为null
     * @param callBack 结果回调
     * @return true=请求发送成功
     */
    public static boolean reqCosignExInfo(tztZFDataMapBase<String, Object> reqInfo, tztZFNetCosignCallBack callBack) {
        return tztZFNetCosignEx.reqCosignExInfo(reqInfo, callBack);
    }

    /**
     * 协同签名服务状态
     *
     * @return true=国密
     */
    public static boolean getCosignStatus() {
        return tztZFNetCosignEx.getExIntForKey(tztZFNetCosignEx.tztCosignEx_Status) == 1;
    }

    /**
     * 获取协同相关参数信息字典
     *
     * @return 数据字典
     */
    public static tztZFDataMapBase<String, Object> getCosignExInfo() {
        return tztZFNetCosignEx.cosignExInfo();
    }

    private tztZFCosignUser _bindUserID(String userID) {
        tztZFLog.log(tztZFLogLevel.tztZFLogLevel_Debug, TAG, "绑定用户------%s", userID);
        if (tztZFRecordProcessLogTools.isRecord()) {
            ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(tztZFRecordProcessLogTools.stringFormat("userID=%s", userID));
            tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_Cosign.value()), tztZFRecordEventType.tztZFRecordEventType_BindUserID,
                    tztZFReocordLogType.tztZFReocordLogType_Debug, params, "");
        }
        this.loginUserID = userID;
        if (userID == null || userID.length() < 1) {
            return null;
        }
        tztZFCosignUser userObj = dictLoginUser.getObj(userID);
        if (userObj == null) {
            userObj = new tztZFCosignUser(context, userID);
            userObj.setAppCosigntype(appCosignType);
            dictLoginUser.setObj(userID, userObj);
        }
        if (tztZFRecordProcessLogTools.isRecord()) {
            ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(tztZFRecordProcessLogTools.stringFormat("userID=%s", userID), tztZFRecordProcessLogTools.stringFormat("cosignUser=%s", userObj.string()));
            tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_Cosign.value()), tztZFRecordEventType.tztZFRecordEventType_BindUserID,
                    tztZFReocordLogType.tztZFReocordLogType_Debug, params, "");
        }
        return userObj;
    }

    private tztZFCosignUser _bindUserID(String userID, String pin, boolean bPinEncrypt) {
        tztZFCosignUser cosignUser = bindUserID(userID);
        if (cosignUser != null) {
            if (!tztZFTextUtils.isEmpty(pin) && bPinEncrypt && tztZFNetWork.getEncryptCallBack() != null) {
                byte[] bytePin = tztZFNetWork.getEncryptCallBack().decrypt(pin.getBytes());
                if (bytePin != null && bytePin.length > 0) {
                    pin = new String(bytePin);
                }
            }
            cosignUser.loadCertWithPin(pin);
            if (tztZFRecordProcessLogTools.isRecord()) {
                String param1 = tztZFRecordProcessLogTools.stringFormat("userID=%s;pin=%s", userID, pin);
                String param2 = tztZFRecordProcessLogTools.stringFormat("cosignUser=%s", cosignUser == null ? "" : cosignUser.string());
                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1, param2);
                tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_Cosign.value()), tztZFRecordEventType.tztZFRecordEventType_BindUserID,
                        tztZFReocordLogType.tztZFReocordLogType_Debug, params, "");
            }
        }
        return cosignUser;
    }

    /**
     * 绑定用户
     *
     * @param userID 用户ID
     * @return 用户信息对象
     */
    public static tztZFCosignUser bindUserID(String userID) {
        return getShareInstance()._bindUserID(userID);
    }

    /**
     * 绑定用户
     *
     * @param userID 用户ID
     * @param pin    pin码
     * @return 用户信息对象
     */
    public static tztZFCosignUser bindUserID(String userID, String pin) {
        return getShareInstance()._bindUserID(userID, pin, false);
    }

    /**
     * 绑定用户
     *
     * @param userID      用户ID
     * @param pin         pin码
     * @param bPinEncrypt pin码是否加密
     * @return 用户信息对象
     */
    public static tztZFCosignUser bindUserID(String userID, String pin, boolean bPinEncrypt) {
        return getShareInstance()._bindUserID(userID, pin, bPinEncrypt);
    }

    /**
     * 绑定用户
     *
     * @param userID 用户ID
     * @param token  用户CAToken
     * @return 用户信息对象
     */
    public static tztZFCosignUser bindUserIDWithToken(String userID, String token) {
        tztZFCosignUser cosignUser = bindUserID(userID);
        if (cosignUser != null) {
            if (tztZFRecordProcessLogTools.isRecord()) {
                String param1 = tztZFRecordProcessLogTools.stringFormat("userID=%s;token=%s", userID, token);
                String param2 = tztZFRecordProcessLogTools.stringFormat("cosignUser=%s", cosignUser.string());
                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1, param2);
                tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_Cosign.value()), tztZFRecordEventType.tztZFRecordEventType_BindUserID,
                        tztZFReocordLogType.tztZFReocordLogType_Debug, params, "");
            }
            cosignUser.setLoginCaToken(token);
        }
        return cosignUser;
    }


    /**
     * 获取当前绑定的用户
     */
    public static String getBindUserID() {
        return getShareInstance().loginUserID;
    }

    private tztZFCosignUser _getBindCosignUser() {
        if (dictLoginUser.getCount() < 1 || loginUserID == null || loginUserID.length() < 1)
            return null;
        return dictLoginUser.getObj(loginUserID);
    }

    /**
     * 获取当前绑定的用户对象
     */
    public static tztZFCosignUser getBindCosignUser() {
        return getShareInstance()._getBindCosignUser();
    }

    /**
     * 清空当前绑定的userid
     */
    public static void clearBindUser() {
        removeCosignUserForID(getShareInstance().loginUserID);
        getShareInstance().loginUserID = "";
    }

    private tztZFCosignUser _cosignUserForID(String userID) {
        if (dictLoginUser.getCount() < 1 || userID == null || userID.length() < 1)
            return null;
        return dictLoginUser.getObj(userID);
    }

    /**
     * 根据userID获取用户信息对象
     *
     * @param userID 客户号
     */
    public static tztZFCosignUser cosignUserForID(String userID) {
        return getShareInstance()._cosignUserForID(userID);
    }

    private void _removeCosignUserForID(String userID) {
        tztZFLog.log(tztZFLogLevel.tztZFLogLevel_Debug, TAG, "解除绑定用户------%s", userID);
        if (tztZFRecordProcessLogTools.isRecord()) {
            String param = tztZFRecordProcessLogTools.stringFormat("userID=%s;loginUserID=%s;message=%s", userID, loginUserID, "removeCosignUserForID");
            ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param);
            tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_Cosign.value()), tztZFRecordEventType.tztZFRecordEventType_DeleteCert,
                    tztZFReocordLogType.tztZFReocordLogType_Debug, params, "");
        }
        if (!tztZFTextUtils.isEmpty(userID) && dictLoginUser.getCount() > 0) {
            dictLoginUser.removeByKey(userID);
        }
    }

    private void removeUserCertForID(String userID) {
        if (userID != null && userID.length() > 0 && dictLoginUser.getCount() > 0) {
            tztZFCosignUser cosignUser = cosignUserForID(userID);
            if (cosignUser != null) {
                cosignUser.removeUserCert();
            }
        }
    }

    /**
     * 移除已经绑定的用户
     *
     * @param userID 用户ID
     */
    public static void removeCosignUserForID(String userID) {
        getShareInstance()._removeCosignUserForID(userID);
    }


    private static String sessionFile(String userID, int connectType, int linkType) {
        String fileName = "";
        if (connectType == 9) {
            if (userID == null || userID.length() < 1) {
                return "";
            }
            fileName = String.format(Locale.getDefault(), "tztZFUserSessionData_%d_%d_%s.data", connectType, linkType, userID);
            byte[] dFile = tztZFDataEncrypt.encryptMD5_32(fileName.getBytes());
            String strFileName = new String(dFile);
            return tztZFPublicFun.getFileDocumentPath(getShareInstance().context, strFileName);
        }

        fileName = String.format(Locale.getDefault(), "tztZFSessionData_%d_%d.data", connectType, linkType);
        byte[] dFile = tztZFDataEncrypt.encryptMD5_32(fileName.getBytes());
        String strFileName = new String(dFile);
        return tztZFPublicFun.getFileDocumentPath(getShareInstance().context, strFileName);
    }

    public static void getDXSSLCert(tztZFSSLCert sslCert) {
        if (sslCert == null) return;
        byte[] ca = tztZFNetCosignEx.getExDataForKey(tztZFNetCosignEx.tztCosignEx_CACert);
        if (ca == null || ca.length < 1)
            return;
        sslCert.setCertCA(ca);
        sslCert.setCertCAType(tztZFCertPEMChain);
    }

    private void _getSXSSLCert(tztZFSSLCert sslCert) {
        if (this.appCosignType == 0)
            return;
        tztBlockLoadCert blockLoadCert = this.dictBlockLoad.getObj(blockKey(this.appCosignType));
        if (blockLoadCert == null)
            return;
        blockLoadCert.onCallBack(this.appCosignType, this.loginUserID, sslCert);
    }

    public static void getSXSSLCert(tztZFSSLCert sslCert) {
        if (sslCert == null) return;
        byte[] ca = tztZFNetCosignEx.getExDataForKey(tztZFNetCosignEx.tztCosignEx_CACert);
        if (ca != null && ca.length > 0) {
            sslCert.setCertCA(ca);
            sslCert.setCertCAType(tztZFCertPEMChain);
        }
        tztZFCosignUser cosignUser = getBindCosignUser();
        if (cosignUser != null && cosignUser.haveCosignCert()) {
            sslCert.setCertUser(cosignUser.userID());
            tztZFCosignCert cosignCert = cosignUser.getLoginCosignCert();
            if (cosignCert != null && cosignCert.getSignInfo() != null) {
                sslCert.setCertSignSN(cosignCert.getSignInfo().getSerialNumber());
            } else {
                sslCert.setCertSignSN("");
            }
            if (cosignCert != null) {
                sslCert.setCertSign(cosignCert.getSignCert());
                sslCert.setCertSignType(cosignCert.getSignType());
                sslCert.setCertEnc(cosignCert.getEncCert());
                sslCert.setCertEncKey(cosignCert.getEncKey());
                sslCert.setCertEncType(cosignCert.getEncType());
                sslCert.setCertType(cosignCert.getCertType());
            }
        } else {
            getShareInstance()._getSXSSLCert(sslCert);
        }
    }

    public static tztZFProtocolCert getSXProtocolCert() {
        tztZFProtocolCert protocolCert = new tztZFProtocolCert();
        protocolCert.protocolType = tztZFNetCosignEx.getExIntForKey(tztZFNetCosignEx.tztCosignEx_Protocol);
        protocolCert.handType = tztZFNetCosignEx.getExIntForKey(tztZFNetCosignEx.tztCosignEx_Handtype);
        ///加密算法第encrypttype位为1，刚好比值大1，-1后刚好是加密算法值
        protocolCert.encryptType = tztZFNetCosignEx.getExIntForKey(tztZFNetCosignEx.tztCosignEx_Encrypttype) - 1;
        protocolCert.sPubKey = tztZFNetCosignEx.getExDataForKey(tztZFNetCosignEx.tztCosignEx_ServerPubKey);
        protocolCert.sPubKeyIndex = tztZFNetCosignEx.getExIntForKey(tztZFNetCosignEx.tztCosignEx_ServerPubKeyIndex);
        protocolCert.iDA = tztZFNetCosignEx.getExDataForKey(tztZFNetCosignEx.tztCosignEx_IDA);
        protocolCert.iDB = tztZFNetCosignEx.getExDataForKey(tztZFNetCosignEx.tztCosignEx_IDB);
        protocolCert.cPubKey = tztZFNetCosignEx.getExDataForKey(tztZFNetCosignEx.tztCosignEx_ClientPubKey);
        protocolCert.cPubKeyIndex = tztZFNetCosignEx.getExIntForKey(tztZFNetCosignEx.tztCosignEx_ClientPubKeyIndex);
        protocolCert.handParameter = 0;
        return protocolCert;
    }

    /**
     * 保存sslsession
     *
     * @param session     session
     * @param userID      用户id
     * @param connectType 连接类型
     * @param linkType    通道类型
     */
    public static int saveSSLSession(byte[] session, String userID, int connectType,
                                     int linkType) {
        if (session == null || session.length < 1)
            return 0;
        String file = sessionFile(userID, connectType, linkType);
        if (file == null || file.length() < 1)
            return 0;
        if (tztZFPublicFun.writeFileData(getShareInstance().context, file, session))
            return 1;
        return 0;
    }

    /**
     * 读取session
     *
     * @param userID      用户id
     * @param connectType 连接类型
     * @param linkType    通道类型
     */
    @tztZFNullable
    public static byte[] restoreSSLSession(String userID, int connectType, int linkType) {
        String file = sessionFile(userID, connectType, linkType);
        if (file == null || file.length() < 1)
            return null;

        return tztZFPublicFun.readFileData(getShareInstance().context, file);
    }

    /**
     * 关键密钥加密
     *
     * @param orgData 待加密数据
     */
    @tztZFNullable
    public static tztZFDataMapBase<String, Object> encData(byte[] orgData) {
        if (getCosignStatus()) {
            return tztZFNetCosignEx.encData(orgData);
        }
        return null;
    }

    /**
     * SM4加密
     *
     * @param str 加密数据
     * @return 加密后的内容
     */
    public static String encryptStrSM(String str) {
        return tztZFNetCosignEx.encryptStrSM(str);
    }

    /**
     * SM4解密
     *
     * @param str 解密数据
     * @return 解密后的内容
     */
    public static String decryptStrSM(String str) {
        return tztZFNetCosignEx.decryptStrSM(str);
    }

    /**
     * 随机
     *
     * @param len   长度
     * @param block 回调
     */
    public static boolean reqCosignRand(int len, tztZFNetCosignCallBack block) {
        return tztZFNetCosignEx.reqCosignRand(len, block);
    }

    private static tztZFCosignSSL.tztZFResponse cosignOrg(int type, byte[] org, int fromType, tztZFNetCosignCallBack callBack) {
        tztZFCosignUser cosignUser = getBindCosignUser();
        if (cosignUser == null || !cosignUser.isCosigned())
            return null;
        return getShareInstance().tztCosign(cosignUser, type, org, fromType, callBack);
    }

    private tztZFCosignSSL.tztZFResponse tztCosign(final tztZFCosignUser cosignUser, int type, byte[] org, int fromType, final tztZFNetCosignCallBack callBack) {
        final tztZFCosignSSL.tztZFResponse response = tztZFCosignSSL.tztZFResponse.withCode(tztZFCosignErrOther, "未绑定账号");
        if (cosignUser == null || org == null || org.length < 1)
            return response;
        int cosignType = 0;
        if (this.appCosignType != 0) {
            cosignType = this.appCosignType;
        } else {
            if (!cosignUser.isCosigned() || cosignUser.getLoginCosignCert() == null) {
                response.setRetCode(tztZFCosignErrNoLoad);
                response.setRetMessage("未签名初始化");
                return response;
            }
            cosignType = cosignUser.getLoginCosignCert().getCosignType();
        }
        final String userID = cosignUser.userID();
        if (userID == null || userID.length() < 1) {
            response.setRetCode(tztZFCosignErrPin);
            response.setRetMessage("参数错误，无法签名");
            return response;
        }
        tztZFCosignCert cosignCert = cosignUser.getLoginCosignCert();
        if (cosignCert == null) {
            response.setRetCode(tztZFCosignErrPin);
            response.setRetMessage("参数错误，无法签名");
            return response;
        }
        String userPin = cosignCert.getPin();
        if (userPin == null || userPin.length() < 1) {
            response.setRetCode(tztZFCosignErrPin);
            response.setRetMessage("参数错误，无法签名");
            return response;
        }
        int signtype = cosignType;
        if (cosignType != 0) {
            tztBlockCosign blockCosign = dictBlockCosign.getObj(blockKey(cosignType));
            if (blockCosign == null) {
                response.setRetCode(tztZFCosignErrOther);
                response.setRetMessage("未注册签名方法");
                return response;
            }
            return blockCosign.onCallBack(cosignType, userID, userPin, org, callBack);
        }

        tztZFNetCosignHead.tztZFCosignResp resp = new tztZFNetCosignHead.tztZFCosignResp();
        resp.setCosignType(0);
        resp.setFromType(fromType);
        signtype = cosignType;
        final tztZFNetCosignHead.tztZFCosignResp finalResp = resp;
        final int finalSigntype = signtype;
        resp = tztZFNetCosignService.tztCosignData(org, userID, fromType, new tztZFNetCosignHead.tztZFNetCosignRespCosignDataCallBack() {
            @Override
            public void onCallBack(tztZFNetCosignHead.tztZFNetCosignRespCosignData c_resp) {
                tztZFDataMapBase<String, Object> retMap = new tztZFDataMapBase<>();
                boolean isSignSucc = false;
                if (c_resp == null) {
                    finalResp.setErrorNO(tztZFCosignErrConnect);
                    finalResp.setErrorMessage("签名失败");
                } else {
                    finalResp.setDictExtern(c_resp.getDictExtern());
                    finalResp.setCosignDowngrade(c_resp.isCosignDowngrade());
                    //降级处理
                    if (finalResp.isCosignDowngrade()) {
                        didGMDowngrade();
                        finalResp.setErrorNO(c_resp.errorNO);
                        finalResp.setErrorMessage(c_resp.errorMessage);
                    }
                    if (c_resp.getErrorNO() != tztZFCosignErrOK) {
                        finalResp.setErrorNO(c_resp.getErrorNO());
                        finalResp.setErrorMessage(c_resp.getErrorMessage());
                    } else {
                        byte[] retData = c_resp != null ? c_resp.getSignData() : null;
                        String signUser = c_resp != null ? c_resp.getSignUser() : null;
                        if (retData == null || retData.length < 1 || tztZFTextUtils.isEmpty(signUser) || !signUser.equals(userID)) {
                            finalResp.setErrorNO(tztZFCosignErrSign);
                            finalResp.setErrorMessage("签名失败");
                        } else {
                            finalResp.setErrorNO(tztZFCosignErrOK);
                            finalResp.setErrorMessage("签名成功");
                            retMap.setBytes("signData", retData);
                            retMap.setInt("signType", finalSigntype);
                            response.setRetData(retData);
                            response.setRetCode(retData.length);
                            isSignSucc = true;
                        }
                        if (!isSignSucc) {
                            response.setRetCode(finalResp.getErrorNO());
                            if (finalResp.getErrorNO() == tztZFCosignErrServerCertUnfind && cosignUser.getLoginCosignCert() != null) {
                                cosignUser.getLoginCosignCert().setnHave(-1);
                            }
                            //签名失败回调
                            tztZFNetWork.setGMCosignUserID(tztZFLinkType_JY.value(), userID, finalResp.getErrorNO(), finalResp.getErrorMessage());
                        }
                    }
                }

                response.getRetEx().setInt("errorno", finalResp.getErrorNO());
                response.getRetEx().setStr("errormessage", finalResp.getErrorMessage());
                response.getRetEx().setStr("userID", userID);
                response.getRetEx().setInt("cosignDowngrade", finalResp.isCosignDowngrade() ? 1 : 0);
                response.getRetEx().setInt("fromType", finalResp.getFromType());
                response.getRetEx().setInt("cosignType", finalResp.getCosignType());

                //安卓的netCosignCallBack接口上层调用，为减少修改，放在字典中返回
                retMap.setObj(tztZFCosignSSL.tztZFResponse.class.getSimpleName(), response);
                if (callBack != null) {
                    callBack.netCosignCallBack(true, retMap);
                }
                if (!isSignSucc) {
                    tztZFNetCosign.this.cosignCallback(userID, finalResp);
                }
            }
        });

        //安卓是异步的，所以要把resp的内容先返回给前面调用者，前面同步阻塞等待的
        response.setRetCode(resp.getErrorNO());
        response.setRetMessage(resp.getErrorMessage());
        response.getRetEx().setInt("errorno", resp.getErrorNO());
        response.getRetEx().setStr("errormessage", resp.getErrorMessage());
        response.getRetEx().setStr("userID", userID);
        response.getRetEx().setInt("cosignDowngrade", resp.isCosignDowngrade() ? 1 : 0);
        response.getRetEx().setInt("fromType", resp.getFromType());
        response.getRetEx().setInt("cosignType", resp.getCosignType());
        return response;
    }

    /**
     * 申请用户证书
     *
     * @param accountMap 用户相关信息字典
     * @param userID     客户号
     * @param userToken  客户token
     * @param pin        证书pin码
     * @param applyType  =1强制申请
     * @param block      回调
     */
    public static boolean applyCertWithAccount(tztZFDataMapBase<String, Object> accountMap, String userID, String userToken, String pin, int applyType, tztZFNetCosignHead.tztZFCosignResponse block) {
        return getShareInstance()._applyCertWithAccount(accountMap, userID, userToken, pin, false, applyType, true, block);
    }

    /**
     * 申请用户证书 -账号登录（上层使用这个方法）
     *
     * @param accountMap           用户相关信息字典
     * @param userID               用户id
     * @param userToken            用户token
     * @param pin                  证书安全码（系统安全码时可以不传，底层会自动生成）
     * @param applyType            int    申请类型，=1 强制重新申请， 0 可以复用原来存在的协同签名参数信息；
     * @param bUseCustomSafetyCode 否使用自定义安全码， NO使用系统安全码，YES使用自定义安全码
     * @param block                申请结果回调
     * @return
     */
    public static boolean applyCertWithAccount(tztZFDataMapBase<String, Object> accountMap, String userID, String userToken, String pin, int applyType, boolean bUseCustomSafetyCode, tztZFNetCosignHead.tztZFCosignResponse block) {
        if (!bUseCustomSafetyCode) {
            pin = tztZFGMManage.getNormalSafetyCode(userID);
        }
        return getShareInstance()._applyCertWithAccount(accountMap, userID, userToken, pin, false, applyType, bUseCustomSafetyCode, block);
    }

    /**
     * 申请用户证书
     *
     * @param accountMap  用户相关信息字典
     * @param userID      客户号
     * @param userToken   客户token
     * @param pin         证书pin码
     * @param bPinEncrypt pin码是否加密
     * @param applyType   =1强制申请
     * @param block       回调
     */
    public static boolean applyCertWithAccount(tztZFDataMapBase<String, Object> accountMap, String userID, String userToken, String pin, boolean bPinEncrypt, int applyType, tztZFNetCosignHead.tztZFCosignResponse block) {
        return getShareInstance()._applyCertWithAccount(accountMap, userID, userToken, pin, bPinEncrypt, applyType, true, block);
    }

    /**
     * 申请用户证书 -账号登录(tztZFGMMange内生成证书使用)
     *
     * @param accountMap           用户相关信息字典
     * @param userID               用户id
     * @param userToken            用户token
     * @param pin                  证书安全码（安全码必传,系统安全码不会自动生成）
     * @param applyType            int    申请类型，=1 强制重新申请， 0 可以复用原来存在的协同签名参数信息；
     * @param bUseCustomSafetyCode 否使用自定义安全码， NO使用系统安全码，YES使用自定义安全码
     * @param block                申请结果回调
     * @return
     */
    public static boolean gmManageApplyCert(tztZFDataMapBase<String, Object> accountMap, String userID, String userToken, String pin, int applyType, boolean bUseCustomSafetyCode, tztZFNetCosignHead.tztZFCosignResponse block) {
        return getShareInstance()._applyCertWithAccount(accountMap, userID, userToken, pin, false, applyType, bUseCustomSafetyCode, block);
    }

    private boolean _applyCertWithAccount(tztZFDataMapBase<String, Object> accountMap, String userID, String userToken, String
            pin, boolean bPinEncrypt, int applyType, boolean bUseCustomSafetyCode, tztZFNetCosignHead.tztZFCosignResponse block) {
        if (block == null)
            return false;
        tztZFNetCosignHead.tztZFCosignResp resp = new tztZFNetCosignHead.tztZFCosignResp();
        if ((accountMap == null || accountMap.getCount() < 1
                || tztZFTextUtils.isEmpty(accountMap.getStr("Account"))
                || tztZFTextUtils.isEmpty(accountMap.getStr("Password"))) &&
                (tztZFTextUtils.isEmpty(userID) || tztZFTextUtils.isEmpty(userToken))) {
            if (tztZFRecordProcessLogTools.isRecord()) {
                String param1 = tztZFRecordProcessLogTools.stringFormat("userID=%s;message=%s", userID, "账号信息不能为空");
                String param2 = tztZFRecordProcessLogTools.stringFormat("accountMap=%s", accountMap == null ? "" : accountMap.toString());
                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1, param2);
                tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_Cosign.value()), tztZFRecordEventType.tztZFRecordEventType_CreateCert,
                        tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
            }
            resp.setErrorMessage("账号信息不能为空");
            resp.setErrorNO(tztZFCosignErrParam);
            block.callBack("", resp);
            return true;
        }

        if (tztZFTextUtils.isEmpty(pin)) {
            String pinLable = tztZFErrorMessageDefine.getUserDefineError(tztZFErrorMessageDefine.tztZFNetWorkErrorKey_PinLable, tztZFErrorMessageDefine.tztZFPinLable);
            String emptyPin = String.format(Locale.getDefault(), "%s不能为空！", pinLable);
            if (tztZFRecordProcessLogTools.isRecord()) {
                String param1 = tztZFRecordProcessLogTools.stringFormat("userID=%s;message=%s", userID, emptyPin);
                String param2 = tztZFRecordProcessLogTools.stringFormat("accountMap=%s", accountMap == null ? "" : accountMap.toString());
                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1, param2);
                tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_Cosign.value()), tztZFRecordEventType.tztZFRecordEventType_CreateCert,
                        tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
            }
            resp.setErrorMessage(emptyPin);
            resp.setErrorNO(tztZFCosignErrPin);
            block.callBack("", resp);
            return true;
        }
        if (bPinEncrypt && tztZFNetWork.getEncryptCallBack() != null) {
            byte[] decryptPin = tztZFNetWork.getEncryptCallBack().decrypt(pin.getBytes());
            if (decryptPin != null && decryptPin.length > 0) {
                pin = new String(decryptPin);
            }
        }

        String deviceID = tztZFCosignCert.getDeviceID();
        if (tztZFTextUtils.isEmpty(deviceID)) {
            if (tztZFRecordProcessLogTools.isRecord()) {
                String param1 = tztZFRecordProcessLogTools.stringFormat("userID=%s;message=%s", userID, "获取设备号失败，请重试");
                String param2 = tztZFRecordProcessLogTools.stringFormat("accountMap=%s", accountMap == null ? "" : accountMap.toString());
                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1, param2);
                tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_Cosign.value()), tztZFRecordEventType.tztZFRecordEventType_CreateCert,
                        tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
            }
            resp.setErrorMessage("获取设备号失败，请重试！");
            resp.setErrorNO(tztZFCosignErrDevice);
            block.callBack("", resp);
            return true;
        }
        //存储账号信息：防止本地存储有客户号，国密登录存储无客户号
        //先关闭安全码进行普通交易登录并开通指纹登录，后开启安全码登录界面提示证书申请，点击申请（本地已经有userid，没有执行tztZFGMManage的存储），申请完成后使用指纹登录，此时提示账号信息不正确
        String userid = accountMap != null ? accountMap.getStr("userid") : "";
        if (!tztZFTextUtils.isEmpty(userid)) {
            tztZFGMManage.saveAccount(tztZFNetCosign.getContext(), accountMap.getStr("Account"), accountMap.getStr("AccountType"), userid);
        }
        int nActiveCosignType = tztZFNetCosignEx.getExIntForKey(tztZFNetCosignEx.tztCosignEx_ActiveCosignType);
        tztZFCosignCertApplyReq applyReq = new tztZFCosignCertApplyReq();
        applyReq.setAccountInfo(accountMap);
        applyReq.setCosignType(nActiveCosignType);
        applyReq.setbUseCustomSafetyCode(bUseCustomSafetyCode);
        applyReq.setStrPin(pin);
        if (!tztZFTextUtils.isEmpty(userID)) {
            applyReq.setCosignUser(bindUserID(userID));
        }
        /// 中焯协同签名证书初始化申请
        if (nActiveCosignType == 0 || dictBlockCreate.getCount() < 1) {
            applyCertZZ(applyReq, applyType, block);
            return true;
        }
        // 生成签名证书P10申请
        applyCertCreateP10(applyReq, block);
        return true;
    }

    private void applyCertZZ(final tztZFCosignCertApplyReq reqObj, final int applyType, final tztZFNetCosignHead.tztZFCosignResponse block) {
        if (reqObj == null)
            return;
        final int cosignType = 0;
        reqObj.setCosignType(cosignType);
        final String pin = reqObj.getStrPin();
        tztZFNetCosignHead.tztZFCosignResp ret = tztZFNetCosignService.tztCosignInit(reqObj.getAccountInfo(),
                reqObj.cosignUser != null ? reqObj.cosignUser.getLoginUserID() : null,
                reqObj.cosignUser != null ? reqObj.cosignUser.getLoginCaToken() : null,
                reqObj.getStrPin(), applyType, new tztZFNetCosignHead.tztZFNetCosignRespCosignInitCallBack() {
                    @Override
                    public void onCallBack(final tztZFNetCosignHead.tztZFNetCosignRespCosignInit c_resp) {
                        final tztZFNetCosignHead.tztZFCosignResp resp = new tztZFNetCosignHead.tztZFCosignResp();
                        if (c_resp != null) {
                            resp.getDictExtern().setDataMap(c_resp.getDictExtern());
                            resp.setCosignDowngrade(c_resp.cosignDowngrade);
                            if (resp.isCosignDowngrade()) {
                                //通知上层
                                tztZFNotificationObj notificationObj = new tztZFNotificationObj();
                                notificationObj.setName(tztZFJYLoginGMProcessStep);
                                tztZFDataMapBase<String, Object> dataMapBase = new tztZFDataMapBase<>();
                                dataMapBase.setInt("step", tztZFSocketSSLStepDowngrade);
                                String userID = tztZFNetCosign.getBindUserID();
                                if (userID == null)
                                    userID = "";
                                dataMapBase.setStr("userID", userID);
                                notificationObj.setData(dataMapBase);
                                EventBus.getDefault().post(notificationObj);
                            }
                        }
                        resp.setCosignType(cosignType);
                        if (c_resp == null || tztZFTextUtils.isEmpty(c_resp.getUserID())
                                || c_resp.getD() == null || c_resp.getD().length < 1
                                || c_resp.getP() == null || c_resp.getP().length < 1) {
                            String errMsg = "协同签名初始化失败!";
                            if (c_resp != null && !tztZFTextUtils.isEmpty(c_resp.getErrorMessage())) {
                                errMsg = String.format(Locale.getDefault(), "%s{[%d]%s}", errMsg, c_resp.getErrorNO(), c_resp.getErrorMessage());
                            }
                            if (tztZFRecordProcessLogTools.isRecord()) {
                                String param1 = tztZFRecordProcessLogTools.stringFormat("message=%s", errMsg);
                                String param2 = tztZFRecordProcessLogTools.stringFormat("c_resp=%s", c_resp == null ? "" : c_resp.toString());
                                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1, param2);
                                tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_Cosign.value()), tztZFRecordEventType.tztZFRecordEventType_CreateCert,
                                        tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
                            }
                            resp.setErrorMessage(errMsg);
                            resp.setErrorNO(tztZFCosignErrConnect);
                            block.callBack("", resp);
                            return;
                        }
                        resp.setCosignDowngrade(c_resp.isCosignDowngrade());
                        final String certUserID = c_resp.getUserID();
                        String certUserToken = "";
                        if (c_resp.getErrorNO() != tztZFCosignErrOK) {
                            resp.setErrorNO(c_resp.getErrorNO());
                            resp.setErrorMessage(c_resp.getErrorMessage());
                            block.callBack(certUserID, resp);
                            return;
                        }
                        final tztZFCosignUser cosignUser = bindUserID(certUserID);
                        if (cosignUser == null) {
                            if (tztZFRecordProcessLogTools.isRecord()) {
                                String param = tztZFRecordProcessLogTools.stringFormat("certUserID=%s;message=%s", certUserID, "绑定账号失败");
                                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param);
                                tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_Cosign.value()), tztZFRecordEventType.tztZFRecordEventType_CreateCert,
                                        tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
                            }
                            resp.setErrorNO(tztZFCosignErrOther);
                            resp.setErrorMessage("绑定账号失败！");
                            block.callBack(certUserID, resp);
                            return;
                        }
                        cosignUser.setLoginCaToken(c_resp.getUserToken());
                        certUserToken = c_resp.getUserToken();
                        reqObj.setCosignUser(cosignUser);
                        reqObj.setDMark(c_resp.getDMark());
                        //新生成
                        if (c_resp.getIsNew() == 1) {
                            if (!cosignUser.newCertWithPin(pin)) {
                                if (tztZFRecordProcessLogTools.isRecord()) {
                                    String param1 = tztZFRecordProcessLogTools.stringFormat("certUserID=%s;message=%s", certUserID, "账号协同签名初始化失败");
                                    String param2 = tztZFRecordProcessLogTools.stringFormat("cosignUser=%s", cosignUser.string());
                                    ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1, param2);
                                    tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_Cosign.value()), tztZFRecordEventType.tztZFRecordEventType_CreateCert,
                                            tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
                                }
                                resp.setErrorNO(tztZFCosignErrOther);
                                resp.setErrorMessage("账号协同签名初始化失败！");
                                block.callBack(certUserID, resp);
                                return;
                            }
                            createP10ThreadPool.useThreadPool().execute(new Runnable() {
                                @Override
                                public void run() {
                                    //设置新生成的D、P
                                    if (!cosignUser.getLoginCosignCert().cosignInitZZ(c_resp.getD(), c_resp.getP1(), c_resp.getP(), c_resp.getDMark())) {
                                        if (tztZFRecordProcessLogTools.isRecord()) {
                                            String param1 = tztZFRecordProcessLogTools.stringFormat("certUserID=%s;message=%s", certUserID, "协同签名初始化设置失败");
                                            String param2 = tztZFRecordProcessLogTools.stringFormat("cosignUser=%s", cosignUser == null ? "" : cosignUser.string());
                                            ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1, param2);
                                            tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_Cosign.value()), tztZFRecordEventType.tztZFRecordEventType_CreateCert,
                                                    tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
                                        }
                                        resp.setErrorNO(tztZFCosignErrClientSign0);
                                        resp.setErrorMessage("协同签名初始化设置失败");
                                        block.callBack(certUserID, resp);
                                        return;
                                    }
                                    //设置P，生成P10申请证书
                                    reqObj.setDataP(c_resp.getP());
                                    tztZFNetCosign.this.applyCertCreateP10(reqObj, block);
                                }
                            });
                            return;
                        }

                        if (applyType == 1) {
                            if (tztZFRecordProcessLogTools.isRecord()) {
                                String param1 = tztZFRecordProcessLogTools.stringFormat("certUserID=%s;message=%s", certUserID, "协同签名初始化失败");
                                String param2 = tztZFRecordProcessLogTools.stringFormat("cosignUser=%s", cosignUser == null ? "" : cosignUser.string());
                                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1, param2);
                                tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_Cosign.value()), tztZFRecordEventType.tztZFRecordEventType_CreateCert,
                                        tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
                            }
                            resp.setErrorNO(tztZFCosignErrClientSign0);
                            resp.setErrorMessage("协同签名初始化失败!");
                            block.callBack(certUserID, resp);
                            return;
                        }
                        //重用
                        String pinLable = tztZFErrorMessageDefine.getUserDefineError(tztZFErrorMessageDefine.tztZFNetWorkErrorKey_PinLable, tztZFErrorMessageDefine.tztZFPinLable);
                        if (!cosignUser.loadCertWithPin(pin)) {
                            String errorPin = String.format(Locale.getDefault(), "%s错误", pinLable);
                            if (tztZFRecordProcessLogTools.isRecord()) {
                                String param1 = tztZFRecordProcessLogTools.stringFormat("certUserID=%s;message=%s", certUserID, errorPin);
                                String param2 = tztZFRecordProcessLogTools.stringFormat("cosignUser=%s", cosignUser == null ? "" : cosignUser.string());
                                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1, param2);
                                tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_Cosign.value()), tztZFRecordEventType.tztZFRecordEventType_CreateCert,
                                        tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
                            }
                            resp.setErrorMessage(errorPin);
                            resp.setErrorNO(tztZFCosignErrPin);
                            block.callBack(certUserID, resp);
                            return;
                        }

                        int nStep = cosignUser.getLoginCosignCert().cosignStepZZ(c_resp.getP());
                        //校验成功，证书存在
                        if (nStep == 2) {
                            resp.setErrorMessage(String.format(Locale.getDefault(), "%s申请成功(2)", pinLable));
                            resp.setErrorNO(tztZFCosignErrOK);
                            block.callBack(certUserID, resp);
                            return;
                        }
                        //校验成功，证书不存在
                        if (nStep == 1) {
                            /// 设置P、生成P10申请证书
                            createP10ThreadPool.useThreadPool().execute(new Runnable() {
                                @Override
                                public void run() {
                                    reqObj.setDataP(c_resp.getP());
                                    tztZFNetCosign.this.applyCertCreateP10(reqObj, block);
                                }
                            });
                            return;
                        }
                        //没P或者P校验不过
                        if (nStep == 0) {
                            /// 初始化后申请新证书
                            tztZFNetCosign.this.applyCertZZ(reqObj, 1, block);
                        }
                        return;
                    }
                });
        ret.setCosignType(cosignType);
        if (ret.getErrorNO() != tztZFCosignErrOK) {
            block.callBack("", ret);
        }
    }

    private void applyCertCreateP10(tztZFCosignCertApplyReq reqObj, tztZFNetCosignHead.tztZFCosignResponse block) {
        if (tztZFRecordProcessLogTools.isRecord()) {
            String param1 = tztZFRecordProcessLogTools.stringFormat("message=%s", "创建P10");
            String param2 = tztZFRecordProcessLogTools.stringFormat("请求参数=%s", reqObj == null ? "" : reqObj.toString());
            ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1, param2);
            tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_Cosign.value()), tztZFRecordEventType.tztZFRecordEventType_CreateCert,
                    tztZFReocordLogType.tztZFReocordLogType_Debug, params, "");
        }
        tztZFNetCosignHead.tztZFCosignResp resp = new tztZFNetCosignHead.tztZFCosignResp();
        int cosignType = reqObj.getCosignType();
        resp.setCosignType(cosignType);
        String certUserID = reqObj.getCosignUser().userID();
        String pin = reqObj.getStrPin();
        //申请证书并导入
        String strP10 = "";
        String retMessage = "";
        int retCode = 0;
        if (cosignType == 0) {
            String sub = tztZFNetCosignEx.getCertSub(certUserID);
            tztZFCosignSSL.tztZFResponse response = tztZFCosignSSL.createPKCS10String(reqObj.getDataP(), sub, pin);
            retCode = response.getRetCode();
            strP10 = response.getRetValue();
            retMessage = response.getRetMessage();
        } else {
            tztBlockCreateP10 blockCreate = this.dictBlockCreate.getObj(blockKey(cosignType));
            if (blockCreate != null) {
                strP10 = blockCreate.onCallBack(cosignType, certUserID, pin);
            }
        }

        if (tztZFTextUtils.isEmpty(strP10)) {
            String pinLable = tztZFErrorMessageDefine.getUserDefineError(tztZFErrorMessageDefine.tztZFNetWorkErrorKey_PinLable, tztZFErrorMessageDefine.tztZFPinLable);
            String errorMessage = String.format(Locale.getDefault(), "[%d%d]生成%s请求文件失败，请重试!%s", cosignType, retCode, pinLable, retMessage == null ? "" : retMessage);
            if (tztZFRecordProcessLogTools.isRecord()) {
                String param1 = tztZFRecordProcessLogTools.stringFormat("message=%s", errorMessage);
                String param2 = tztZFRecordProcessLogTools.stringFormat("请求参数=%s", reqObj == null ? "" : reqObj.toString());
                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1, param2);
                tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_Cosign.value()), tztZFRecordEventType.tztZFRecordEventType_CreateCert,
                        tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
            }
            resp.setErrorMessage(errorMessage);
            resp.setErrorNO(tztZFCosignErrCreateP10);
            block.callBack(certUserID, resp);
            return;
        }
        reqObj.setStrP10(strP10);
        applyCertWithP10(reqObj, block);
    }

    private void applyCertWithP10(tztZFCosignCertApplyReq reqObj, final tztZFNetCosignHead.tztZFCosignResponse block) {
        final tztZFNetCosignHead.tztZFCosignResp resp = new tztZFNetCosignHead.tztZFCosignResp();
        final int cosignType = reqObj.getCosignType();
        resp.setCosignType(cosignType);
        final String certUserID = reqObj.getCosignUser().userID();
        String certUserToken = reqObj.getCosignUser().getLoginCaToken();
        String strP10 = reqObj.getStrP10();
        final tztZFCosignCert cosignCert = reqObj.getCosignUser().getLoginCosignCert();
        cosignCert.setBUseCustomSafetyCode(reqObj.bUseCustomSafetyCode);
        //申请证书并导入
        tztZFNetCosignService.tztCertApply(strP10, reqObj.getAccountInfo(), certUserID, certUserToken, new tztZFNetCosignHead.tztZFNetCosignRespApplyCertCallBack() {
            @Override
            public void onCallBack(tztZFNetCosignHead.tztZFNetCosignRespApplyCert c_resp) {
                String pinLable = tztZFErrorMessageDefine.getUserDefineError(tztZFErrorMessageDefine.tztZFNetWorkErrorKey_PinLable, tztZFErrorMessageDefine.tztZFPinLable);
                if (c_resp == null) {
                    String applyFail = String.format(Locale.getDefault(), "申请%s失败，请重试！", pinLable);
                    if (tztZFRecordProcessLogTools.isRecord()) {
                        ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(tztZFRecordProcessLogTools.stringFormat("message=%s", applyFail));
                        tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_Cosign.value()), tztZFRecordEventType.tztZFRecordEventType_CreateCert,
                                tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
                    }
                    resp.setErrorMessage(applyFail);
                    resp.setErrorNO(tztZFCosignErrConnect);
                    block.callBack(certUserID, resp);
                    return;
                }
                resp.getDictExtern().setDataMap(c_resp.getDictExtern());
                resp.setCosignDowngrade(c_resp.isCosignDowngrade());
                resp.setErrorNO(c_resp.getErrorNO());
                resp.setErrorMessage(c_resp.getErrorMessage());
                if (c_resp.getErrorNO() != tztZFCosignErrOK) {
                    block.callBack(certUserID, resp);
                    return;
                }

                if (cosignType != 0) {
                    tztBlockImportCert blockImport = dictBlockImport.getObj(blockKey(cosignType));
                    if (blockImport != null) {
                        int nRet = blockImport.onCallBack(cosignType, certUserID, c_resp.getCertSignBuf());
                        if (nRet < 0) {
                            String errorMessage = String.format(Locale.getDefault(), "[%d]%s导入失败[%d], 请重试！", cosignType, pinLable, nRet);
                            if (tztZFRecordProcessLogTools.isRecord()) {
                                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(tztZFRecordProcessLogTools.stringFormat("message=%s", errorMessage));
                                tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_Cosign.value()), tztZFRecordEventType.tztZFRecordEventType_CreateCert,
                                        tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
                            }
                            resp.setErrorMessage(errorMessage);
                            resp.setErrorNO(tztZFCosignErrCreateP10);
                            return;
                        }
                    }
                }
                if (cosignCert.cosignImportZZ(c_resp, cosignType, appKey)) {
                    resp.setErrorMessage(String.format(Locale.getDefault(), "%s导入成功", pinLable));
                    resp.setErrorNO(tztZFCosignErrOK);
                    block.callBack(certUserID, resp);
                    return;
                }
                resp.setErrorMessage(String.format(Locale.getDefault(), "[%d]%s导入失败，请重试！", cosignType, pinLable));
                if (tztZFRecordProcessLogTools.isRecord()) {
                    ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(tztZFRecordProcessLogTools.stringFormat("message=%s", resp.getErrorMessage()));
                    tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_Cosign.value()), tztZFRecordEventType.tztZFRecordEventType_CreateCert,
                            tztZFReocordLogType.tztZFReocordLogType_Error, params,
                            "");
                }
                resp.setErrorNO(tztZFCosignErrCreateP10);
                block.callBack(certUserID, resp);
                return;
            }
        });
    }

    /**
     * 获取证书列表
     *
     * @param userInfo      用户信息字典
     * @param responseBlock 回调
     */
    public static void queryCert(tztZFDataMapBase<String, Object> userInfo, tztZFNetCosignHead.tztZFNetCosignRespCertListCallBack responseBlock) {
        tztZFNetCosignService.tztCertQuery(userInfo, responseBlock);
    }

    /**
     * 获取当前绑定的用户证书列表
     *
     * @param responseBlock 回调
     */
    public static void queryBindUserCert(tztZFNetCosignHead.tztZFNetCosignRespCertListCallBack responseBlock) {
        tztZFCosignUser cosignUser = getBindCosignUser();
        String userID = "";
        String userToken = "";
        if (cosignUser != null) {
            userID = cosignUser.userID();
            userToken = cosignUser.getLoginCaToken();
        }
        tztZFNetCosignService.tztCertQuery(userID, userToken, responseBlock);
    }

    /**
     * 获取用户证书列表(双向)
     *
     * @param serialNumber  当前用户证书序列号
     * @param userID        用户ID
     * @param jyInfo        交易参数字典
     * @param responseBlock 回调
     */
    public static void queryBindUserCert(String serialNumber, String userID, tztZFDataMapBase<String, Object> jyInfo, tztZFNetCosignHead.tztZFNetCosignRespCertListCallBack responseBlock) {
        tztZFNetCosignService.tztCertQueryFromJY(serialNumber, userID, jyInfo, responseBlock);
    }

    /**
     * 撤销证书
     *
     * @param serialNumber  证书序列号
     * @param reason        撤销原因
     * @param userID        用户ID
     * @param userToken     用户token
     * @param responseBlock 回调
     */
    public static void revokeCert(String serialNumber, String reason, String userID, String userToken, tztZFNetCosignHead.tztZFNetCosignRespCallBack responseBlock) {
        tztZFNetCosignService.tztCertRevoke(serialNumber, reason, userID, userToken, responseBlock);
    }

    /**
     * 查询客户
     *
     * @param userInfo      用户登录账号信息
     * @param responseBlock 回调
     */
    public static void queryUserWithAccount(tztZFDataMapBase<String, Object> userInfo, tztZFNetCosignHead.tztZFNetCosignRespCallBack responseBlock) {
        tztZFNetCosignService.tztQueryUserWithAccount(userInfo, responseBlock);
    }


    /**
     * 重置证书Pin码
     *
     * @param userID        用户ID
     * @param userToken     用户Token
     * @param jyInfo        交易信息扩展字典
     * @param newPin        新的pin码
     * @param bPinEncrypt   pin码是否加密
     * @param responseBlock 结果回调
     */
    public static void resetCert(String userID, String userToken, tztZFDataMapBase<String, Object> jyInfo, String newPin, boolean bPinEncrypt, final boolean bUseCustomSafetyCode, final tztZFNetCosignHead.tztZFNetCosignRespCallBack responseBlock) {
        if (!tztZFTextUtils.isEmpty(newPin) && bPinEncrypt && tztZFNetWork.getEncryptCallBack() != null) {
            byte[] decryptPin = tztZFNetWork.getEncryptCallBack().decrypt(newPin.getBytes());
            if (decryptPin != null && decryptPin.length > 0) {
                newPin = new String(decryptPin);
            }
        }
        if (tztZFRecordProcessLogTools.isRecord()) {
            String param1 = tztZFRecordProcessLogTools.stringFormat("request51953userID:%s,Pin:%s", userID, newPin);
            ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1);
            tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_Cosign.value()), tztZFRecordEventType.tztZFRecordEventType_ResetCert,
                    tztZFReocordLogType.tztZFReocordLogType_Debug, params, "");
        }
        final String newpin = newPin;
        tztZFNetCosignService.tztResetPinWithAccount(jyInfo, userID, userToken, newpin, new tztZFNetCosignHead.tztZFNetCosignResetRespCallBack() {
            @Override
            public void onCallBack(byte[] P1, tztZFNetCosignHead.tztZFNetCosignResp c_resp) {
                if (tztZFRecordProcessLogTools.isRecord()) {
                    String param1 = tztZFRecordProcessLogTools.stringFormat("p1:%s c_resp:%s, userid:%s", P1, c_resp, c_resp == null ? "" : c_resp.userID);
                    ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1);
                    tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_Cosign.value()), tztZFRecordEventType.tztZFRecordEventType_ResetCert,
                            tztZFReocordLogType.tztZFReocordLogType_Debug, params, "");
                }
                if (c_resp == null || c_resp.getErrorNO() < 0 || tztZFTextUtils.isEmpty(c_resp.getUserID())) {
                    responseBlock.onCallBack(c_resp);
                    return;
                }
                if (P1 == null || P1.length < 1) {
                    responseBlock.onCallBack(c_resp);
                    return;
                }
                String certUserID = c_resp.getUserID();
                tztZFDataMapBase<String, Object> retDict = tztZFCosignCert.resetCertPin(certUserID, newpin, P1, bUseCustomSafetyCode);
                c_resp.setErrorNO(retDict.getErrorNo());
                String pinLable = tztZFErrorMessageDefine.getUserDefineError(tztZFErrorMessageDefine.tztZFNetWorkErrorKey_PinLable, tztZFErrorMessageDefine.tztZFPinLable);
                if (retDict.getErrorNo() != tztZFCosignErrOK) {
                    c_resp.setErrorMessage(String.format(Locale.getDefault(), "重置%s失败，请重试[%d]", pinLable, retDict.getErrorNo()));
                    c_resp.setLocalError(true);
                } else {
                    tztZFCosignUser cosignUser = bindUserID(certUserID, newpin);
                    cosignUser.setLoginCaToken(c_resp.getUserToken());
                    if (cosignUser.getLoginError() != null) {
                        c_resp.setErrorNO(tztZFCosignErrOther);
                        c_resp.setErrorMessage(String.format(Locale.getDefault(), "重置%s失败，请重试[%d]", pinLable, tztZFCosignErrOther));
                        c_resp.setLocalError(true);
                        if (tztZFRecordProcessLogTools.isRecord()) {
                            String param1 = tztZFRecordProcessLogTools.stringFormat("cosignUser.loginError:%s", cosignUser.getLoginError());
                            ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1);
                            tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_Cosign.value()), tztZFRecordEventType.tztZFRecordEventType_ResetCert,
                                    tztZFReocordLogType.tztZFReocordLogType_Debug, params, "");
                        }
                    } else if (!cosignUser.isValidity()) {
                        //判断证书是否过期
                        c_resp.setErrorNO(tztZFCosignErrExpire);
                        c_resp.setErrorMessage(String.format(Locale.getDefault(), "%s已过期，请重新申请[%d]", pinLable, tztZFCosignErrExpire));
                        c_resp.setLocalError(true);
                        if (tztZFRecordProcessLogTools.isRecord()) {
                            String param1 = tztZFRecordProcessLogTools.stringFormat("cosignUser.isValidity:%s", cosignUser.isValidity());
                            ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1);
                            tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_Cosign.value()), tztZFRecordEventType.tztZFRecordEventType_ResetCert,
                                    tztZFReocordLogType.tztZFReocordLogType_Debug, params, "");
                        }
                    } else {
                        c_resp.setErrorMessage(String.format(Locale.getDefault(), "重置%s成功,并重新绑定用户", pinLable));
                    }
                }
                responseBlock.onCallBack(c_resp);
            }
        });
    }

    /**
     * 重置证书Pin码
     *
     * @param userID        用户ID
     * @param userToken     用户Token
     * @param jyInfo        交易信息扩展字典
     * @param newPin        新的pin码
     * @param responseBlock 结果回调
     */
    public static void resetCert(String userID, String userToken, tztZFDataMapBase<String, Object> jyInfo, final String newPin, final tztZFNetCosignHead.tztZFNetCosignRespCallBack responseBlock) {
        resetCert(userID, userToken, jyInfo, newPin, false, responseBlock);
    }

    /**
     * 重置证书Pin码
     *
     * @param userID        用户ID
     * @param userToken     用户Token
     * @param jyInfo        交易信息扩展字典
     * @param newPin        新的pin码
     * @param responseBlock 结果回调
     */
    public static void resetCert(String userID, String userToken, tztZFDataMapBase<String, Object> jyInfo, String newPin, boolean bPinEncrypt, final tztZFNetCosignHead.tztZFNetCosignRespCallBack responseBlock) {
        if (!tztZFTextUtils.isEmpty(newPin) && bPinEncrypt && tztZFNetWork.getEncryptCallBack() != null) {
            byte[] decryptPin = tztZFNetWork.getEncryptCallBack().decrypt(newPin.getBytes());
            if (decryptPin != null && decryptPin.length > 0) {
                newPin = new String(decryptPin);
            }
        }
        resetCert(jyInfo, newPin, userID, true, responseBlock);
    }

    /**
     * 重置证书Pin码
     *
     * @param userInfo      用户信息
     * @param pin           新的Pin码
     * @param responseBlock 结果回调
     */
    public static void resetCert(tztZFDataMapBase<String, Object> userInfo, final String pin, final tztZFNetCosignHead.tztZFNetCosignRespCallBack responseBlock) {
        resetCert(userInfo, pin, false, true, responseBlock);
    }

    public static void resetCert(tztZFDataMapBase<String, Object> userInfo, String pin, String userID, boolean bUseCustomSafetyCode, final tztZFNetCosignHead.tztZFNetCosignRespCallBack responseBlock) {
        if (!bUseCustomSafetyCode) {
            pin = tztZFGMManage.getNormalSafetyCode(userID);
        }
        resetCert(userID, "", userInfo, pin, false, bUseCustomSafetyCode, responseBlock);
    }

    /**
     * 重置证书Pin码
     *
     * @param userInfo      用户信息
     * @param pin           新的Pin码
     * @param bPinEncrypt   Pin码是否加密
     * @param responseBlock 结果回调
     */
    public static void resetCert(tztZFDataMapBase<String, Object> userInfo, String pin, boolean bPinEncrypt, final boolean bUseCustomSafetyCode, final tztZFNetCosignHead.tztZFNetCosignRespCallBack responseBlock) {
        resetCert("", "", userInfo, pin, bPinEncrypt, bUseCustomSafetyCode, responseBlock);
    }

    /**
     * 获取本地证书列表
     */
    public static ArrayList<tztZFCertificateInfo> localCertList() {
        return tztZFCosignCert.localCertListZZ();
    }

    private void _removeLocalCert(final String userID) {
        if (tztZFRecordProcessLogTools.isRecord()) {
            ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(tztZFRecordProcessLogTools.stringFormat("userID=%s;loginUserID=%s;message=%s", userID, loginUserID, "removeLocalCert"));
            tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_Cosign.value()), tztZFRecordEventType.tztZFRecordEventType_DeleteCert,
                    tztZFReocordLogType.tztZFReocordLogType_Debug, params, "");
        }
        this.dictBlockDelete.enumerateKeysAndObjectsUsingBlock(new tztZFDataMapCallBack() {
            @Override
            public boolean enumerate(String lowKey, String key, Object value) {
                if (value != null) {
                    tztBlockDeleteCert blockDeleteCert = (tztBlockDeleteCert) value;
                    blockDeleteCert.onCallBack(userID);
                }
                return false;
            }
        });
        removeUserCertForID(userID);
    }

    /**
     * 删除证书
     *
     * @param userID 客户号
     */
    public static void removeLocalCert(String userID) {
        getShareInstance()._removeLocalCert(userID);
        //是否和当前一致
        if (!tztZFTextUtils.isEmpty(userID) && !tztZFTextUtils.isEmpty(getShareInstance().loginUserID) && getShareInstance().loginUserID.equals(userID)) {
            getShareInstance().loginUserID = "";
        }
    }

    /**
     * 证书错误
     *
     * @param userID 客户号
     * @param code   -1协同前面错误；-2证书过期；-3证书无效
     */
    public static void expiredCert(String userID, int code) {
        getShareInstance()._expiredCert(userID, code);
    }

    private void _expiredCert(String userID, int code) {
        if (tztZFTextUtils.isEmpty(userID)) return;
        tztZFCosignUser cosignUser = cosignUserForID(userID);
        if (cosignUser != null && cosignUser.getLoginCosignCert() != null) {
            cosignUser.getLoginCosignCert().setnHave(code);
        }
    }

    /**
     * 是否在证书过期days天前提示, 0=证书存在，不需要提醒； 1=即将过期提醒；-1=客户没有证书或UserID为空；-2=证书已经过期
     *
     * @param userID 客户号，若传空，取当前绑定的客户号
     * @param days   提前提示天数
     */
    public static int needAlertBeforeExpire(String userID, int days) {
        //
        if (tztZFTextUtils.isEmpty(userID)) {
            userID = tztZFNetCosign.getBindUserID();
        }
        if (tztZFTextUtils.isEmpty(userID)) {
            return -1;
        }
        //判断剩余天数
        //取证书到期日期，取服务器时间，取到期提醒天数
        tztZFCertificateInfo loginCosignCert = tztZFNetCosign.getCertInfoWithUserID(userID);
        if (loginCosignCert == null) {
            return -1;
        }
        Date certEndDate = loginCosignCert.getNotAfter();
        //证书是否过期、证书是否超过10天
        Date serverDate = new Date();
        String strServerDate = tztZFNetCosignEx.getExStrForKey(tztCosignEx_CurrentTime);
        if (!tztZFTextUtils.isEmpty(strServerDate)) {
            serverDate = tztZFCertificateInfo.dateFromString(strServerDate, "yyyyMMddHHmmss");
        }
        if (certEndDate != null) {
            if (days <= 0) {
                //未设置
                //取服务器配置的过期天数
                days = tztZFNetCosignEx.getExIntForKey(tztZFNetCosignEx.tztCosignEx_AlertBeforeExpire);
            }
            if (days <= 0) {
                days = 10;//默认10天
            }
            long time = certEndDate.getTime() - serverDate.getTime();
            TimeUnit timeunit = TimeUnit.DAYS;
            long day = timeunit.convert(time, TimeUnit.MILLISECONDS);
            if (time <= 0) {
                return -2;//过期
            } else if (day < days) {
                return 1;//证书有效，临近到期，进行提醒
            }
        }
        return 0;
    }

    /**
     * 通过客户号查询本地证书数据
     *
     * @param userID 客户号
     */
    @tztZFNullable
    public static tztZFCertificateInfo getCertInfoWithUserID(String userID) {
        if (tztZFTextUtils.isEmpty(userID)) {
            return null;
        }
        ArrayList<tztZFCertificateInfo> certList = tztZFNetCosign.localCertList();
        if (certList == null || certList.size() < 1) {
            return null;
        }
        tztZFCertificateInfo certificateInfo = null;
        for (tztZFCertificateInfo cert : certList) {
            if (cert != null && cert.getUserID() != null && cert.getUserID().equals(userID)) {
                certificateInfo = cert;
                break;
            }
        }
        return certificateInfo;
    }

    /**
     * 设置降级模式
     */
    public static void didGMDowngrade() {
        tztZFNetCosignEx.didGMDowngrade();
    }

    /**
     * 进行数据签名
     *
     * @param orgData 原始数据
     */
    public static tztZFCosignSSL.tztZFResponse cosignECDSASIG(byte[] orgData) {
        tztZFCosignSSL.tztZFResponse response = tztZFCosignSSL.tztZFResponse.withCode(-1, "");
        if (orgData == null || orgData.length < 1) {
            response.setRetMessage("签名信息不能为空");
            return response;
        }

        //先对原文进行SM3摘要处理，然后将签名数据进行服务器端协同签名
        byte[] dealData = tztZFNetCosignEx.SM3(orgData);
        tztZFDataMapBase<String, Object> retItems = getShareInstance().cosignOrgData(0, dealData, 1);
        if (retItems.getCount() > 0) {
            int signType = retItems.getInt("signtype", 0);
            byte[] signData = retItems.getBytes("signdata");
            byte[] resultData = tztZFCosignSSL.cosign_ECDSA_SIG(signData, signType);
            tztZFCosignSSL.tztZFResponse cosignResp = retItems.getObj(tztZFCosignSSL.tztZFResponse.class.getSimpleName());
            if (cosignResp != null) {
                response = cosignResp;
            }
            response.setRetData(resultData);
        }
        return response;
    }

    /**
     * 设置cosignResp是否为异步回调，默认异步回调
     *
     * @param bCallBackAsync true=异步回调；false=同步回调
     */
    public static void setCallBackAsync(boolean bCallBackAsync) {
        tztZFNetCosign.getShareInstance().bCallBackAsync = bCallBackAsync;
    }

    public tztZFNetCosignSendBlock getSendRecvBlock() {
        return sendRecvBlock;
    }

    public static void setSendRecvBlock(tztZFNetCosignSendBlock sendRecvBlock) {
        tztZFNetCosign.getShareInstance().sendRecvBlock = sendRecvBlock;
    }

    public static String getLocalCertMessage() {
        Date date = new Date(tztZFNetCosign.getShareInstance().classCreateTime);
        String format = "yyyy-MM-dd HH:mm:ss";
        SimpleDateFormat dateFormat = new SimpleDateFormat(format, Locale.getDefault());
        String createTime = "tztZFNetCosign构造时间：" + dateFormat.format(date) + "\r\n";
        String bindUserid = createTime + "绑定客户号：" + tztZFNetCosign.getBindUserID() + "\r\n\r\n";
        tztZFDataMapBase<String, tztZFCosignUser> dictLoginUser = new tztZFDataMapBase<>();
        dictLoginUser.setDataMap(tztZFNetCosign.getShareInstance().dictLoginUser);
        final ArrayList<String> arrayLocalCertMessage = new ArrayList<>();
        dictLoginUser.enumerateKeysAndObjectsUsingBlock(new tztZFDataMapCallBack() {
            @Override
            public boolean enumerate(String s, String s1, Object value) {
                if (value != null && value instanceof tztZFCosignUser) {
                    String pinLable = tztZFErrorMessageDefine.getUserDefineError(tztZFErrorMessageDefine.tztZFNetWorkErrorKey_PinLable, tztZFErrorMessageDefine.tztZFPinLable);
                    String certLable = tztZFErrorMessageDefine.getUserDefineError(tztZFErrorMessageDefine.tztZFNetWorkErrorKey_CertLable, tztZFErrorMessageDefine.tztZFCertLable);
                    tztZFCosignUser cosignUser = (tztZFCosignUser) value;
                    String localCertMessage = String.format(Locale.getDefault(), "本地%s信息\r\n", certLable);
                    localCertMessage += "客户号：" + cosignUser.getLoginUserID() + "\r\n";
                    localCertMessage += String.format(Locale.getDefault(), "%s：%s\r\n", pinLable, cosignUser.getLoginPin());
                    localCertMessage += "是否有效：" + cosignUser.isValid() + "\r\n";
                    localCertMessage += "是否过期：" + cosignUser.isValidity() + "\r\n";
                    localCertMessage += "是否可签名：" + cosignUser.isCosigned() + "\r\n\r\n";
                    arrayLocalCertMessage.add(localCertMessage);
                }
                return false;
            }
        });
        String returnValue = bindUserid;
        for (int i = 0; i < arrayLocalCertMessage.size(); i++) {
            returnValue += arrayLocalCertMessage.get(i);
        }
        return returnValue;
    }

    //MARK: - interface define

    public interface tztBlockCosign {
        tztZFCosignSSL.tztZFResponse onCallBack(int cosignType, String userID, String pin, byte[] org, tztZFNetCosignCallBack callBack);
    }

    public interface tztBlockCreateP10 {
        String onCallBack(int cosignType, String userID, String pin);
    }

    public interface tztBlockImportCert {
        int onCallBack(int cosignType, String userID, String signCert);
    }

    public interface tztBlockLoadCert {
        int onCallBack(int cosignType, String userID, tztZFSSLCert signCert);
    }

    public interface tztBlockDeleteCert {
        int onCallBack(String userID);
    }

    /**
     * 协同签名回调
     */
    public interface tztCosignRespCallBack {
        void onCallBack(String userID, tztZFNetCosignHead.tztZFCosignResp resp);
    }

    /**
     * 证书申请对象
     */
    static class tztZFCosignCertApplyReq {
        private int cosignType;
        private tztZFDataMapBase<String, Object> accountInfo;
        private tztZFCosignUser cosignUser;
        private String strPin;
        private String strP10;
        private byte[] dataP;
        private String DMark;
        /**
         * 使用自定义安全码 (YES使用自定义安全码, NO使用系统安全码)
         */
        private boolean bUseCustomSafetyCode;

        public int getCosignType() {
            return cosignType;
        }

        public void setCosignType(int cosignType) {
            this.cosignType = cosignType;
        }

        public tztZFDataMapBase<String, Object> getAccountInfo() {
            return accountInfo;
        }

        public void setAccountInfo(tztZFDataMapBase<String, Object> accountInfo) {
            this.accountInfo = accountInfo;
        }

        public tztZFCosignUser getCosignUser() {
            return cosignUser;
        }

        public void setCosignUser(tztZFCosignUser cosignUser) {
            this.cosignUser = cosignUser;
        }

        public String getStrPin() {
            return strPin;
        }

        public void setStrPin(String strPin) {
            this.strPin = strPin;
        }

        public String getStrP10() {
            return strP10;
        }

        public void setStrP10(String strP10) {
            this.strP10 = strP10;
        }

        public byte[] getDataP() {
            return dataP;
        }

        public void setDataP(byte[] dataP) {
            this.dataP = dataP;
        }

        public String getDMark() {
            return DMark;
        }

        public void setDMark(String DMark) {
            this.DMark = DMark;
        }

        public boolean isbUseCustomSafetyCode() {
            return bUseCustomSafetyCode;
        }

        public void setbUseCustomSafetyCode(boolean bUseCustomSafetyCode) {
            this.bUseCustomSafetyCode = bUseCustomSafetyCode;
        }

        @tztZFNonNull
        @Override
        public String toString() {
            return String.format("cosignType=%s;strPin=%s;strP10=%s;DMark=%s;dataP=%s;accountInfo=%s;cosignUser=%s"
                    , cosignType, strPin, strP10, DMark
                    , dataP == null ? "" : tztZFEncryptKit.hexEncrypt(dataP, dataP.length)
                    , accountInfo == null ? "" : accountInfo.toString()
                    , cosignUser == null ? "" : cosignUser.string());
        }
    }


    public static class tztZFCosignSendRecvObj {
        /** 通道类型 */
        private tztZFLinkType linkType;
        /** 发送的数据字典 */
        private tztZFDataMapBase<String, Object> sendData;
        /** 接受数据回调 */
        private tztZFNetCosignReceiveBlock recvBlock;

        public static boolean sendData(tztZFLinkType linkType, tztZFDataMapBase<String, Object> sendData, tztZFNetCosignReceiveBlock resp) {
            tztZFCosignSendRecvObj obj = new tztZFCosignSendRecvObj();
            obj.linkType = linkType;
            obj.sendData = sendData;
            obj.recvBlock = resp;
            if (tztZFNetCosign.getShareInstance().getSendRecvBlock() != null) {
                return tztZFNetCosign.getShareInstance().getSendRecvBlock().onCallBack(obj);
            }
            return false;
        }

        public tztZFLinkType getLinkType() {
            return linkType;
        }

        public void setLinkType(tztZFLinkType linkType) {
            this.linkType = linkType;
        }

        public tztZFDataMapBase<String, Object> getSendData() {
            return sendData;
        }

        public void setSendData(tztZFDataMapBase<String, Object> sendData) {
            this.sendData = sendData;
        }

        public tztZFNetCosignReceiveBlock getRecvBlock() {
            return recvBlock;
        }

        public void setRecvBlock(tztZFNetCosignReceiveBlock recvBlock) {
            this.recvBlock = recvBlock;
        }
    }

    public interface tztZFNetCosignSendBlock {
        boolean onCallBack(tztZFCosignSendRecvObj obj);
    }

    public interface tztZFNetCosignReceiveBlock {
        void onCallBack(tztZFDataMapBase<String, Object> receiveData);
    }


}
