package com.tztzf.network.cosign;

import com.tztzf.baseos.tztZFBaseDeviceInfo;
import com.tztzf.baseos.tztZFTextUtils;
import com.tztzf.commbase.tztZFDataMapBase;
import com.tztzf.keychain.tztZFKeyChain;
import com.tztzf.log.tztZFLog;
import com.tztzf.log.tztZFLogLevel;
import com.tztzf.network.callback.tztZFBlockCertCallBack;
import com.tztzf.network.callback.tztZFBlockCosignCallBack;
import com.tztzf.network.callback.tztZFBlockRandCallBack;
import com.tztzf.network.define.tztZFLinkType;
import com.tztzf.protocol.callback.tztZFBlocktztCosign;
import com.tztzf.protocol.core.tztZFCosignSSL;

import java.util.UUID;

public class tztZFNetCosignCustom {
    private static tztZFNetCosignCustom g_netCosignCustom = null;
    private Object context;

    private static final String tztZFDeviceID_Cosign = "com.tzt.deviceid.z";
    private String userID;
    private String pin;
    private int linkType;
    private int connectType;
    private tztZFBlockCertCallBack certBlock;
    private tztZFBlockCosignCallBack signBlock;
    private tztZFBlockRandCallBack randCallBack;

    public static synchronized tztZFNetCosignCustom getShareInstance() {
        if (g_netCosignCustom == null) {
            g_netCosignCustom = new tztZFNetCosignCustom();
        }
        return g_netCosignCustom;
    }


    private tztZFNetCosignCustom() {
        initdata();
    }

    public static void withSignBlock(tztZFBlockCosignCallBack signBlock, tztZFBlockCertCallBack certBlock) {
        withSignBlock(signBlock, certBlock, null);
    }

    public static void withSignBlock(tztZFBlockCosignCallBack signBlock, tztZFBlockCertCallBack certBlock, tztZFBlockRandCallBack randCallBack) {
        getShareInstance().withSignBlockIn(signBlock, certBlock, randCallBack);
    }

    private void withSignBlockIn(final tztZFBlockCosignCallBack signBlock, tztZFBlockCertCallBack certBlock, final tztZFBlockRandCallBack randCallBack) {
        this.signBlock = signBlock;
        this.certBlock = certBlock;
        this.randCallBack = randCallBack;
        tztZFCosignSSL.withConnectType(connectType, new tztZFBlocktztCosign() {
            @Override
            public tztZFDataMapBase<String, Object> onHandler(int type, byte[] org) {
                return signBlock.onHandler(type, org);
            }
        });
    }

    private void initdata() {
        userID = "";
        pin = "";
        linkType = tztZFLinkType.tztZFLinkType_Cosign.value();
        connectType = 8;
    }

    private void withUserIDIn(String userID, String pin, int linktype, int connecttype) {
        this.userID = userID;
        this.pin = pin;
        this.linkType = linktype;
        this.connectType = connecttype;
    }

    public static void withUserID(String userID, String pin, int linktype, int connecttype) {
        getShareInstance().withUserIDIn(userID, pin, linktype, connecttype);
    }

    //获取用户证书
    public static tztZFDataMapBase<String, Object> getSignCert() {
        return getShareInstance().getSignCertIn();
    }

    private tztZFDataMapBase<String, Object> getSignCertIn() {
        if (certBlock == null)
            return null;
        tztZFLog.log(tztZFLogLevel.tztZFLogLevel_Info, "HTTEST", "getSignCert");
        return certBlock.onHandler(linkType, connectType);
    }

    public static String getDeviceID() {
        tztZFDataMapBase<String, Object> dataMap = getZFDeviceInfo();
        if (dataMap == null) {
            dataMap = new tztZFDataMapBase<>();
        }
        tztZFDataMapBase<String, Object> deviceIDMap = dataMap.getDataMap(tztZFDeviceID_Cosign);
        String deviceID = "";
        if (deviceIDMap != null) {
            deviceID = deviceIDMap.getStr(tztZFDeviceID_Cosign);
        }
        if (tztZFTextUtils.isEmpty(deviceID)) {
            //FIXME: 生成唯一号
            deviceID = UUID.randomUUID().toString();
            if (tztZFTextUtils.isEmpty(deviceID)) {
                return "";
            }
            deviceID = deviceID.replaceAll("-", "");
            tztZFDataMapBase<String, Object> map = new tztZFDataMapBase<>();
            map.setStr(tztZFDeviceID_Cosign, deviceID);
            dataMap.setObj(tztZFDeviceID_Cosign, map);
            tztZFKeyChain.save(getShareInstance().context, tztZFDeviceID_Cosign, dataMap);
        }

        return deviceID;
    }

    public static String getDeviceInfo() {
        return tztZFBaseDeviceInfo.getSysDeviceInfo();
    }


    private static tztZFDataMapBase<String, Object> getZFDeviceInfo() {
        return tztZFKeyChain.load(getShareInstance().context, tztZFDeviceID_Cosign);
    }
}
