package com.tztzf.network.cosign;

import com.tztzf.base.tztZFPublicFun;
import com.tztzf.baseos.tztZFTextUtils;
import com.tztzf.commbase.tztZFDataMapBase;
import com.tztzf.log.tztZFLog;
import com.tztzf.log.tztZFLogLevel;
import com.tztzf.network.callback.tztZFNetWorkResponseCallBack;
import com.tztzf.network.define.tztZFErrorMessageDefine;
import com.tztzf.network.gm.tztZFGMManage;
import com.tztzf.network.tztZFNetWork;
import com.tztzf.network.tztZFNetWorkReqObj;
import com.tztzf.network.tztZFNetWorkRespObj;
import com.tztzf.protocol.core.tztZFCosignSSL;
import com.tztzf.protocol.core.tztZFDataEncrypt;
import com.tztzf.protocol.tztZFEncryptKit;
import com.tztzf.recordprocesslog.tztZFRecordEventType;
import com.tztzf.recordprocesslog.tztZFRecordProcessLogTools;
import com.tztzf.recordprocesslog.tztZFReocordLogType;

import java.util.ArrayList;
import java.util.Locale;

import static com.tztzf.log.tztZFLogLevel.tztZFLogLevel_Info;
import static com.tztzf.network.cosign.tztZFCosignErr.tztZFCosignErrClientSign0;
import static com.tztzf.network.cosign.tztZFCosignErr.tztZFCosignErrClientSign1;
import static com.tztzf.network.cosign.tztZFCosignErr.tztZFCosignErrCreateP10;
import static com.tztzf.network.cosign.tztZFCosignErr.tztZFCosignErrDevice;
import static com.tztzf.network.cosign.tztZFCosignErr.tztZFCosignErrOK;
import static com.tztzf.network.cosign.tztZFCosignErr.tztZFCosignErrOther;
import static com.tztzf.network.cosign.tztZFCosignErr.tztZFCosignErrParam;
import static com.tztzf.network.cosign.tztZFCosignErr.tztZFCosignErrPin;
import static com.tztzf.network.cosign.tztZFCosignErr.tztZFCosignErrSign;
import static com.tztzf.network.cosign.tztZFCosignErr.tztZFCosignErrUnfind;
import static com.tztzf.network.cosign.tztZFNetCosignEx.tztCosignEx_ActiveCertType;
import static com.tztzf.network.cosign.tztZFNetCosignEx.tztCosignEx_CurrentTime;
import static com.tztzf.network.define.tztZFLinkType.tztZFLinkType_Cosign;
import static com.tztzf.network.define.tztZFLinkType.tztZFLinkType_JY;

/**
 * 协同签名接口服务
 */
public class tztZFNetCosignService {

    /**
     * 中焯交易通道证书查询
     *
     * @param serialNumber  证书序列号
     * @param userID        用户ID
     * @param jyInfo        交易相关信息
     * @param responseBlock 查询回调
     */
    public static void tztCertQueryFromJY(String serialNumber, String userID, tztZFDataMapBase<String, Object> jyInfo, final tztZFNetCosignHead.tztZFNetCosignRespCertListCallBack responseBlock) {
        if (responseBlock == null)
            return;
        if (serialNumber == null || serialNumber.length() < 1 || userID == null || userID.length() < 1 || jyInfo == null || jyInfo.getCount() < 1) {
            tztZFNetCosignHead.tztZFNetCosignRespCertList resp = new tztZFNetCosignHead.tztZFNetCosignRespCertList();
            cosignRespWithClientErr(resp, "参数错误!");
            resp.setUserID(userID);
            responseBlock.onCallBack(resp);
            return;
        }

        tztZFNetWorkReqObj reqObj = new tztZFNetWorkReqObj();
        if (jyInfo.getCount() > 0) {
            tztZFDataMapBase_AddToReq.addToReqObj(jyInfo, reqObj);
        }
        reqObj.setStr("Action", "51973");
        reqObj.setStr("DeviceId", tztZFCosignCert.getDeviceID());
        reqObj.setStr("DeviceInfo", tztZFCosignCert.getDeviceInfo());

        reqObj.setStr("certUserId", userID);
        reqObj.setStr("certSerialNumber", serialNumber);

        if (tztZFNetCosign.getShareInstance().getSendRecvBlock() != null) {
            tztZFNetCosign.tztZFCosignSendRecvObj.sendData(tztZFLinkType_JY, reqObj.getSendValues(), new tztZFNetCosign.tztZFNetCosignReceiveBlock() {
                @Override
                public void onCallBack(tztZFDataMapBase<String, Object> receiveData) {
                    responseBlock.onCallBack(toRespCertList(receiveData));
                }
            });
        } else {
            tztZFNetWork.sendDataAction(tztZFLinkType_JY.value(), reqObj, new tztZFNetWorkResponseCallBack() {
                @Override
                public void OnError(tztZFNetWorkReqObj c_reqObj, int c_reqObjTag, int c_recvType, int c_errorNo, String c_errorMsg) {
                    tztZFNetCosignHead.tztZFNetCosignRespCertList resp = new tztZFNetCosignHead.tztZFNetCosignRespCertList();
                    cosignRespWithClientErr(resp, c_errorMsg, c_errorNo);
                    responseBlock.onCallBack(resp);
                }

                @Override
                public void OnAns(tztZFNetWorkReqObj c_reqObj, int c_reqObjTag, tztZFNetWorkRespObj c_respObj) {
                    tztZFNetCosignHead.tztZFNetCosignRespCertList resp = toRespCertList(c_respObj.getDataMap());
                    resp.setRespObj(c_respObj);
                    responseBlock.onCallBack(resp);
                }
            });
        }
    }

    /*============中焯账号查询====================*/

    /**
     * 证书申请 - 账号密码校验  51954
     *
     * @param userInfo      {Account    string    账号;
     *                      AccountType    string    账号类型;
     *                      Password    string    密码;
     *                      Type    int    柜台类型（1 普通 2 两融 3 期权）;}
     * @param responseBlock 回调
     */
    public static void tztQueryUserWithAccount(final tztZFDataMapBase<String, Object> userInfo, final tztZFNetCosignHead.tztZFNetCosignRespCallBack responseBlock) {
        if (responseBlock == null)
            return;
        tztZFNetCosignHead.tztZFNetCosignResp resp = new tztZFNetCosignHead.tztZFNetCosignResp();
        if (userInfo == null || userInfo.getCount() < 1
                || tztZFTextUtils.isEmpty(userInfo.getStr("Account"))
                || tztZFTextUtils.isEmpty(userInfo.getStr("Password"))) {
            cosignRespWithClientErr(resp, "账号信息不能为空!");
            resp.setErrorNO(tztZFCosignErrParam);
            responseBlock.onCallBack(resp);
            return;
        }

        tztZFNetWorkReqObj reqObj = new tztZFNetWorkReqObj();
        tztZFDataMapBase_AddToReq.addToReqObj(userInfo, reqObj);
        reqObj.setStr("Action", "51954");

        reqObj.setStr("DeviceId", tztZFCosignCert.getDeviceID());
        reqObj.setStr("DeviceInfo", tztZFCosignCert.getDeviceInfo());
        reqObj.setStr("Encrypt", "0");

        if (tztZFNetCosign.getShareInstance().getSendRecvBlock() != null) {
            tztZFNetCosign.tztZFCosignSendRecvObj.sendData(tztZFLinkType_Cosign, reqObj.getSendValues(), new tztZFNetCosign.tztZFNetCosignReceiveBlock() {
                @Override
                public void onCallBack(tztZFDataMapBase<String, Object> receiveData) {
                    if (responseBlock == null)
                        return;
                    tztZFNetCosignHead.tztZFNetCosignResp resp = dealWithAccountReply(receiveData, userInfo);
                    responseBlock.onCallBack(resp);
                }
            });
        } else {
            tztZFNetWork.sendDataAction(tztZFLinkType_Cosign.value(), reqObj, new tztZFNetWorkResponseCallBack() {
                @Override
                public void OnError(tztZFNetWorkReqObj c_reqObj, int c_reqObjTag, int c_recvType, int c_errorNo, String c_errorMsg) {
                    if (responseBlock == null)
                        return;
                    tztZFNetCosignHead.tztZFNetCosignResp resp = new tztZFNetCosignHead.tztZFNetCosignResp();
                    cosignRespWithClientErr(resp, c_errorMsg, c_errorNo);
                    responseBlock.onCallBack(resp);
                }

                @Override
                public void OnAns(tztZFNetWorkReqObj c_reqObj, int c_reqObjTag, tztZFNetWorkRespObj c_respObj) {
                    if (responseBlock == null)
                        return;
                    tztZFNetCosignHead.tztZFNetCosignResp resp = dealWithAccountReply(c_respObj.getDataMap(), userInfo);
                    responseBlock.onCallBack(resp);
                }
            });
        }
    }

    private static tztZFNetCosignHead.tztZFNetCosignResp dealWithAccountReply(tztZFDataMapBase<String, Object> c_respObj, tztZFDataMapBase<String, Object> userInfo) {
        tztZFNetCosignHead.tztZFNetCosignResp resp = new tztZFNetCosignHead.tztZFNetCosignResp();
        cosignRespWithNetResp(resp, c_respObj);
        if (c_respObj == null) {
            return resp;
        }
        String userId = c_respObj.getStr("certUserid");
        resp.userID = userId;
        resp.userToken = c_respObj.getStr("caToken");
        if (userId != null && userId.length() > 0 && userInfo != null) {
            tztZFGMManage.saveAccount(tztZFNetCosign.getContext(), userInfo.getStr("Account"), userInfo.getStr("AccountType"), userId);
        }
        return resp;
    }

    /*============中焯Pin码重置====================*/

    /**
     * 证书申请 - 客户端重取P1接口（用于PIN码重置）  51953
     *
     * @param userInfo      Account    string    账号;
     *                      AccountType    string    账号类型;
     *                      Password    string    密码;
     *                      Type    int    柜台类型（1 普通 2 两融 3 期权）;}
     * @param userID        客户号
     * @param userToken     客户号登录Token
     * @param newPin        新Pin码
     * @param responseBlock 回调
     */
    public static void tztResetPinWithAccount(tztZFDataMapBase<String, Object> userInfo, String userID, String userToken, String newPin, final tztZFNetCosignHead.tztZFNetCosignResetRespCallBack responseBlock) {
        if (responseBlock == null)
            return;
        final tztZFNetCosignHead.tztZFNetCosignResp resp = new tztZFNetCosignHead.tztZFNetCosignResp();
        if (tztZFTextUtils.isEmpty(newPin)) {
            String pinLable = tztZFErrorMessageDefine.getUserDefineError(tztZFErrorMessageDefine.tztZFNetWorkErrorKey_PinLable, tztZFErrorMessageDefine.tztZFPinLable);
            String errorNewPin = String.format(Locale.getDefault(), "新%s参数错误！", pinLable);
            if (tztZFRecordProcessLogTools.isRecord()) {
                String param1 = tztZFRecordProcessLogTools.stringFormat("userID=%s;message=%s", userID, errorNewPin);
                String param2 = tztZFRecordProcessLogTools.stringFormat("userInfo=%s", userInfo == null ? "" : userInfo.toString());
                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1, param2);
                tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_Cosign.value()), tztZFRecordEventType.tztZFRecordEventType_ResetCert,
                        tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
            }
            cosignRespWithClientErr(resp, errorNewPin);
            responseBlock.onCallBack(null, resp);
            return;
        }

        if ((userInfo == null || userInfo.getCount() < 1
                || tztZFTextUtils.isEmpty(userInfo.getStr("Account"))
                || tztZFTextUtils.isEmpty(userInfo.getStr("Password"))) &&
                (tztZFTextUtils.isEmpty(userID) || tztZFTextUtils.isEmpty(userToken))) {
            if (tztZFRecordProcessLogTools.isRecord()) {
                String param1 = tztZFRecordProcessLogTools.stringFormat("userID=%s;message=%s", userID, "账号信息不能为空");
                String param2 = tztZFRecordProcessLogTools.stringFormat("userInfo=%s", userInfo == null ? "" : userInfo.toString());
                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1, param2);
                tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_Cosign.value()), tztZFRecordEventType.tztZFRecordEventType_ResetCert,
                        tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
            }
            cosignRespWithClientErr(resp, "账号信息不能为空!");
            responseBlock.onCallBack(null, resp);
            return;
        }

        tztZFNetWorkReqObj reqObj = new tztZFNetWorkReqObj();
        if (userInfo != null && userInfo.getCount() > 0)
            tztZFDataMapBase_AddToReq.addToReqObj(userInfo, reqObj);
        if (userID != null && userID.length() > 0 && userToken != null && userToken.length() > 0) {
            reqObj.setStr("certUserId", userID);
            reqObj.setStr("caToken", userToken);
        }
        reqObj.setStr("Action", "51953");
        reqObj.setStr("DeviceId", tztZFCosignCert.getDeviceID());
        reqObj.setStr("DeviceInfo", tztZFCosignCert.getDeviceInfo());
        reqObj.setStr("Encrypt", "0");

        if (tztZFNetCosign.getShareInstance().getSendRecvBlock() != null) {
            tztZFNetCosign.tztZFCosignSendRecvObj.sendData(tztZFLinkType_Cosign, reqObj.getSendValues(), new tztZFNetCosign.tztZFNetCosignReceiveBlock() {
                @Override
                public void onCallBack(tztZFDataMapBase<String, Object> receiveData) {
                    if (responseBlock == null)
                        return;
                    tztZFNetCosignHead.tztZFNetCosignResp resp = dealWithResetPinReply(receiveData);
                    byte[] P1 = receiveData.getBytes("P1");
                    ///P1不能为空
                    if (P1 == null || P1.length < 1) {
                        if (tztZFRecordProcessLogTools.isRecord()) {
                            String param1 = tztZFRecordProcessLogTools.stringFormat("message=%s", "P1为空");
                            String param2 = tztZFRecordProcessLogTools.stringFormat("c_respObj=%s", receiveData == null ? "" : receiveData.toString());
                            ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1, param2);
                            tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_Cosign.value()), tztZFRecordEventType.tztZFRecordEventType_ResetCert,
                                    tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
                        }
                        if (resp.errorNO >= 0)
                            resp.errorNO = tztZFCosignErrOther;
                    }
                    responseBlock.onCallBack(P1, resp);
                }
            });
        } else {
            tztZFNetWork.sendDataAction(tztZFLinkType_Cosign.value(), reqObj, new tztZFNetWorkResponseCallBack() {
                @Override
                public void OnError(tztZFNetWorkReqObj c_reqObj, int c_reqObjTag, int c_recvType, int c_errorNo, String c_errorMsg) {
                    tztZFNetCosignHead.tztZFNetCosignResp resp = new tztZFNetCosignHead.tztZFNetCosignResp();
                    cosignRespWithClientErr(resp, c_errorMsg, c_errorNo);
                    resp.setLocalError(c_reqObj.isLocalError());
                    responseBlock.onCallBack(null, resp);
                }

                @Override
                public void OnAns(tztZFNetWorkReqObj c_reqObj, int c_reqObjTag, tztZFNetWorkRespObj c_respObj) {
                    if (responseBlock == null)
                        return;
                    resp.setLocalError(c_reqObj.isLocalError());
                    tztZFNetCosignHead.tztZFNetCosignResp resp = dealWithResetPinReply(c_respObj.getDataMap());
                    byte[] P1 = c_respObj.getBytes("P1");
                    ///P1不能为空
                    if (P1 == null || P1.length < 1) {
                        if (tztZFRecordProcessLogTools.isRecord()) {
                            String param1 = tztZFRecordProcessLogTools.stringFormat("message=%s", "P1为空");
                            String param2 = tztZFRecordProcessLogTools.stringFormat("c_respObj=%s", c_respObj.getDataMap() == null ? "" : c_respObj.getDataMap().toString());
                            ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1, param2);
                            tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_Cosign.value()), tztZFRecordEventType.tztZFRecordEventType_ResetCert,
                                    tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
                        }
                        if (resp.errorNO >= 0)
                            resp.errorNO = tztZFCosignErrOther;
                    }
                    responseBlock.onCallBack(P1, resp);
                }
            });
        }
    }

    private static tztZFNetCosignHead.tztZFNetCosignResp dealWithResetPinReply(tztZFDataMapBase<String, Object> c_respObj) {
        tztZFNetCosignHead.tztZFNetCosignResp resp = new tztZFNetCosignHead.tztZFNetCosignResp();
        cosignRespWithNetResp(resp, c_respObj);
        if (c_respObj == null) {
            return resp;
        }
        resp.userID = c_respObj.getStr("certUserid");
        resp.userToken = c_respObj.getStr("caToken");
        return resp;
    }

    /**
     * 中焯证书申请
     *
     * @param P10           证书申请串
     * @param userInfo      用户信息 {Account:账号，AccountType: 账号类型，Password: 密码， Type:类型(1-普通，2-两融，3-期权)}
     * @param userID        用户ID，可为空
     * @param userToken     用户token，可为空
     * @param responseBlock 结果回调
     */
    public static void tztCertApply(String P10, tztZFDataMapBase<String, Object> userInfo, String userID, String userToken, final tztZFNetCosignHead.tztZFNetCosignRespApplyCertCallBack responseBlock) {
        if (responseBlock == null)
            return;
        tztZFNetCosignHead.tztZFNetCosignRespApplyCert resp = new tztZFNetCosignHead.tztZFNetCosignRespApplyCert();
        if (P10 == null || P10.length() < 1) {
            if (tztZFRecordProcessLogTools.isRecord()) {
                String param1 = tztZFRecordProcessLogTools.stringFormat("userid=%s;message=%s", userID, "P10参数错误!");
                String param2 = tztZFRecordProcessLogTools.stringFormat("userInfo=%s", userInfo == null ? "" : userInfo.toString());
                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1, param2);
                tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_Cosign.value()), tztZFRecordEventType.tztZFRecordEventType_CreateCert,
                        tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
            }
            cosignRespWithClientErr(resp, "P10参数错误!", tztZFCosignErrCreateP10);
            responseBlock.onCallBack(resp);
            return;
        }

        if ((userInfo == null || userInfo.getCount() < 1
                || tztZFTextUtils.isEmpty(userInfo.getStr("Account"))
                || tztZFTextUtils.isEmpty(userInfo.getStr("Password")))
                && (tztZFTextUtils.isEmpty(userID) || tztZFTextUtils.isEmpty(userToken))) {
            if (tztZFRecordProcessLogTools.isRecord()) {
                String param1 = tztZFRecordProcessLogTools.stringFormat("userid=%s;message=%s", userID, "账号信息不能为空!");
                String param2 = tztZFRecordProcessLogTools.stringFormat("userInfo=%s", userInfo == null ? "" : userInfo.toString());
                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1, param2);
                tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_Cosign.value()), tztZFRecordEventType.tztZFRecordEventType_CreateCert,
                        tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
            }
            cosignRespWithClientErr(resp, "账号信息不能为空！");
            responseBlock.onCallBack(resp);
            return;
        }

        tztZFNetWorkReqObj reqObj = new tztZFNetWorkReqObj();
        if (userInfo != null && userInfo.getCount() > 0) {
            tztZFDataMapBase_AddToReq.addToReqObj(userInfo, reqObj);
        }
        if (!tztZFTextUtils.isEmpty(userID) && !tztZFTextUtils.isEmpty(userToken)) {
            reqObj.setStr("certUserId", userID);
            reqObj.setStr("caToken", userToken);
        }
        reqObj.setStr("Action", "51970");
        reqObj.setStr("certReqBuf", P10);
        reqObj.setStr("DeviceId", tztZFCosignCert.getDeviceID());
        reqObj.setStr("DeviceInfo", tztZFCosignCert.getDeviceInfo());
        reqObj.setStr("Encrypt", "0");

        int n_ActiveCertType = tztZFNetCosignEx.getExIntForKey(tztCosignEx_ActiveCertType);
        reqObj.setInt("certType", n_ActiveCertType);

        if (tztZFNetCosign.getShareInstance().getSendRecvBlock() != null) {
            tztZFNetCosign.tztZFCosignSendRecvObj.sendData(tztZFLinkType_Cosign, reqObj.getSendValues(), new tztZFNetCosign.tztZFNetCosignReceiveBlock() {
                @Override
                public void onCallBack(tztZFDataMapBase<String, Object> receiveData) {
                    tztZFNetCosignHead.tztZFNetCosignRespApplyCert resp = toRespApplyCert(receiveData);
                    responseBlock.onCallBack(resp);
                }
            });
        } else {
            tztZFNetWork.sendDataAction(tztZFLinkType_Cosign.value(), reqObj, new tztZFNetWorkResponseCallBack() {
                @Override
                public void OnError(tztZFNetWorkReqObj c_reqObj, int c_reqObjTag, int c_recvType, int c_errorNo, String c_errorMsg) {
                    tztZFNetCosignHead.tztZFNetCosignRespApplyCert resp = new tztZFNetCosignHead.tztZFNetCosignRespApplyCert();
                    cosignRespWithClientErr(resp, c_errorMsg, c_errorNo);
                    responseBlock.onCallBack(resp);
                }

                @Override
                public void OnAns(tztZFNetWorkReqObj c_reqObj, int c_reqObjTag, tztZFNetWorkRespObj c_respObj) {
                    tztZFNetCosignHead.tztZFNetCosignRespApplyCert resp = toRespApplyCert(c_respObj.getDataMap());
                    if (c_respObj.getReqObj() != null && c_respObj.getReqObj().getSendValues() != null) {
                        resp.setDeviceID(c_respObj.getReqObj().getSendValues().getStr("deviceID"));
                        resp.setDeviceInfo(c_respObj.getReqObj().getSendValues().getStr("deviceInfo"));
                    }
                    responseBlock.onCallBack(resp);
                }
            });
        }
    }

    /**
     * 证书注销
     *
     * @param serialNumber  证书序列号 (!表示所有)
     * @param reason        注销原因
     * @param userID        用户ID
     * @param userToken     用户token
     * @param responseBlock 结果回调
     */
    public static void tztCertRevoke(String serialNumber, String reason, String userID, String userToken, final tztZFNetCosignHead.tztZFNetCosignRespCallBack responseBlock) {
        if (tztZFTextUtils.isEmpty(userID) || tztZFTextUtils.isEmpty(userToken) || tztZFTextUtils.isEmpty(serialNumber)) {
            if (responseBlock == null)
                return;
            tztZFNetCosignHead.tztZFNetCosignResp resp = new tztZFNetCosignHead.tztZFNetCosignResp();
            cosignRespWithClientErr(resp, "参数错误!");
            resp.setUserID(userID);
            resp.setUserToken(userToken);
            responseBlock.onCallBack(resp);
            return;
        }

        tztZFNetWorkReqObj reqObj = new tztZFNetWorkReqObj();
        reqObj.setStr("Action", "51972");
        reqObj.setStr("certSerialNumber", serialNumber);
        reqObj.setStr("certRevokeReason", reason);
        reqObj.setStr("certUserId", userID);
        reqObj.setStr("caToken", userToken);
        reqObj.setStr("DeviceId", tztZFCosignCert.getDeviceID());
        reqObj.setStr("DeviceInfo", tztZFCosignCert.getDeviceInfo());

        if (tztZFNetCosign.getShareInstance().getSendRecvBlock() != null) {
            tztZFNetCosign.tztZFCosignSendRecvObj.sendData(tztZFLinkType_Cosign, reqObj.getSendValues(), new tztZFNetCosign.tztZFNetCosignReceiveBlock() {
                @Override
                public void onCallBack(tztZFDataMapBase<String, Object> receiveData) {
                    if (responseBlock == null)
                        return;
                    tztZFNetCosignHead.tztZFNetCosignResp resp = new tztZFNetCosignHead.tztZFNetCosignResp();
                    cosignRespWithNetResp(resp, receiveData);
                    responseBlock.onCallBack(resp);
                }
            });
        } else {
            tztZFNetWork.sendDataAction(tztZFLinkType_Cosign.value(), reqObj, new tztZFNetWorkResponseCallBack() {
                @Override
                public void OnError(tztZFNetWorkReqObj c_reqObj, int c_reqObjTag, int c_recvType, int c_errorNo, String c_errorMsg) {
                    if (responseBlock != null) {
                        tztZFNetCosignHead.tztZFNetCosignResp resp = new tztZFNetCosignHead.tztZFNetCosignResp();
                        cosignRespWithClientErr(resp, c_errorMsg, c_errorNo);
                        responseBlock.onCallBack(resp);
                    }
                }

                @Override
                public void OnAns(tztZFNetWorkReqObj c_reqObj, int c_reqObjTag, tztZFNetWorkRespObj c_respObj) {
                    if (responseBlock == null)
                        return;
                    tztZFNetCosignHead.tztZFNetCosignResp resp = new tztZFNetCosignHead.tztZFNetCosignResp();
                    cosignRespWithNetResp(resp, c_respObj.getDataMap());
                    responseBlock.onCallBack(resp);
                }
            });
        }
    }

    /**
     * 证书查询
     *
     * @param userInfo      用户信息字典 {Account: 账号 AccountType: 账号类型 Password: 密码 Type:类型(1-普通，2-两融，3-期权)}
     * @param responseBlock 结果回调
     */
    public static void tztCertQuery(tztZFDataMapBase<String, Object> userInfo, final tztZFNetCosignHead.tztZFNetCosignRespCertListCallBack responseBlock) {
        if (responseBlock == null)
            return;
        if (userInfo == null || userInfo.getCount() < 1) {
            tztZFNetCosignHead.tztZFNetCosignRespCertList resp = new tztZFNetCosignHead.tztZFNetCosignRespCertList();
            cosignRespWithClientErr(resp, "账号不允许为空或账号校验失败");
            responseBlock.onCallBack(resp);
            return;
        }

        tztZFNetWorkReqObj reqObj = new tztZFNetWorkReqObj();
        tztZFDataMapBase_AddToReq.addToReqObj(userInfo, reqObj);
        reqObj.setStr("Action", "51974");
        reqObj.setStr("DeviceId", tztZFCosignCert.getDeviceID());
        reqObj.setStr("DeviceInfo", tztZFCosignCert.getDeviceInfo());
        reqObj.setStr("Encrypt", "0");

        if (tztZFNetCosign.getShareInstance().getSendRecvBlock() != null) {
            tztZFNetCosign.tztZFCosignSendRecvObj.sendData(tztZFLinkType_Cosign, reqObj.getSendValues(), new tztZFNetCosign.tztZFNetCosignReceiveBlock() {
                @Override
                public void onCallBack(tztZFDataMapBase<String, Object> receiveData) {
                    tztZFNetCosignHead.tztZFNetCosignRespCertList resp = toRespCertList(receiveData);
                    responseBlock.onCallBack(resp);
                }
            });
        } else {
            tztZFNetWork.sendDataAction(tztZFLinkType_Cosign.value(), reqObj, new tztZFNetWorkResponseCallBack() {
                @Override
                public void OnError(tztZFNetWorkReqObj c_reqObj, int c_reqObjTag, int c_recvType, int c_errorNo, String c_errorMsg) {
                    tztZFNetCosignHead.tztZFNetCosignRespCertList resp = new tztZFNetCosignHead.tztZFNetCosignRespCertList();
                    cosignRespWithClientErr(resp, c_errorMsg, c_errorNo);
                    responseBlock.onCallBack(resp);
                }

                @Override
                public void OnAns(tztZFNetWorkReqObj c_reqObj, int c_reqObjTag, tztZFNetWorkRespObj c_respObj) {
                    tztZFNetCosignHead.tztZFNetCosignRespCertList resp = toRespCertList(c_respObj.getDataMap());
                    resp.setRespObj(c_respObj);
                    responseBlock.onCallBack(resp);
                }
            });
        }
    }

    /**
     * 证书查询
     *
     * @param userID        用户ID
     * @param userToken     用户Token
     * @param responseBlock 回调
     */
    public static void tztCertQuery(String userID, String userToken, final tztZFNetCosignHead.tztZFNetCosignRespCertListCallBack responseBlock) {
        if (responseBlock == null)
            return;
        if (tztZFTextUtils.isEmpty(userID) || tztZFTextUtils.isEmpty(userToken)) {
            tztZFNetCosignHead.tztZFNetCosignRespCertList resp = new tztZFNetCosignHead.tztZFNetCosignRespCertList();
            cosignRespWithClientErr(resp, "参数错误");
            resp.setUserID(userID);
            resp.setUserToken(userToken);
            responseBlock.onCallBack(resp);
            return;
        }

        tztZFNetWorkReqObj reqObj = new tztZFNetWorkReqObj();
        reqObj.setStr("Action", "51974");
        reqObj.setStr("certUserId", userID);
        reqObj.setStr("caToken", userToken);

        reqObj.setStr("DeviceId", tztZFCosignCert.getDeviceID());
        reqObj.setStr("DeviceInfo", tztZFCosignCert.getDeviceInfo());
        reqObj.setStr("Encrypt", "0");

        if (tztZFNetCosign.getShareInstance().getSendRecvBlock() != null) {
            tztZFNetCosign.tztZFCosignSendRecvObj.sendData(tztZFLinkType_Cosign, reqObj.getSendValues(), new tztZFNetCosign.tztZFNetCosignReceiveBlock() {
                @Override
                public void onCallBack(tztZFDataMapBase<String, Object> receiveData) {
                    tztZFNetCosignHead.tztZFNetCosignRespCertList resp = toRespCertList(receiveData);
                    responseBlock.onCallBack(resp);
                }
            });
        } else {
            tztZFNetWork.sendDataAction(tztZFLinkType_Cosign.value(), reqObj, new tztZFNetWorkResponseCallBack() {
                @Override
                public void OnError(tztZFNetWorkReqObj c_reqObj, int c_reqObjTag, int c_recvType, int c_errorNo, String c_errorMsg) {
                    tztZFNetCosignHead.tztZFNetCosignRespCertList resp = new tztZFNetCosignHead.tztZFNetCosignRespCertList();
                    cosignRespWithClientErr(resp, c_errorMsg, c_errorNo);
                    responseBlock.onCallBack(resp);
                }

                @Override
                public void OnAns(tztZFNetWorkReqObj c_reqObj, int c_reqObjTag, tztZFNetWorkRespObj c_respObj) {
                    tztZFNetCosignHead.tztZFNetCosignRespCertList resp = toRespCertList(c_respObj.getDataMap());
                    resp.setRespObj(c_respObj);
                    responseBlock.onCallBack(resp);
                }
            });
        }
    }

    /**
     * 协同签名P10 51950 init
     *
     * @param userInfo      用户信息字典 {Account: 账号 AccountType: 账号类型 Password: 密码 Type:类型(1-普通，2-两融，3-期权)}
     * @param userID        客户号
     * @param userToken     客户号登录Token
     * @param pin           PIN码
     * @param applyType     申请类型 1-强制重新申请 0-可以复用原来存在的协同签名数据信息
     * @param responseBlock 回调
     * @return 调用结果
     */
    public static tztZFNetCosignHead.tztZFCosignResp tztCosignInit(tztZFDataMapBase<String, Object> userInfo, String userID, String userToken,
                                                                   String pin, int applyType, final tztZFNetCosignHead.tztZFNetCosignRespCosignInitCallBack responseBlock) {
        tztZFNetCosignHead.tztZFCosignResp ret = new tztZFNetCosignHead.tztZFCosignResp();
        ret.setCosignType(0);
        if (userInfo == null || userInfo.getCount() < 1 || responseBlock == null) {
            if (tztZFRecordProcessLogTools.isRecord()) {
                String param1 = tztZFRecordProcessLogTools.stringFormat("userID=%s;message=%s", userID, "参数错误");
                String param2 = tztZFRecordProcessLogTools.stringFormat("userInfo=%d;responseBlock=%b", userInfo == null ? 0 : userInfo.getCount(), responseBlock != null);
                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1, param2);
                tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_Cosign.value()), tztZFRecordEventType.tztZFRecordEventType_CreateCert,
                        tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
            }
            ret.setErrorNO(tztZFCosignErrParam);
            ret.setErrorMessage("[C]参数错误");
            return ret;
        }

        String strAccount = userInfo.getStr("Account");
        String strPass = userInfo.getStr("Password");
        if ((tztZFTextUtils.isEmpty(strAccount) || tztZFTextUtils.isEmpty(strPass))/* && (tztZFTextUtils.isEmpty(userID) || tztZFTextUtils.isEmpty(userToken))*/) {
            if (tztZFRecordProcessLogTools.isRecord()) {
                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(tztZFRecordProcessLogTools.stringFormat("userID=%s;message=%s", userID, "入参错误，账号信息不全"));
                tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_Cosign.value()), tztZFRecordEventType.tztZFRecordEventType_CreateCert,
                        tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
            }
            ret.setErrorNO(tztZFCosignErrParam);
            ret.setErrorMessage("[C]入参错误，账号信息不全");
            return ret;
        }
        if (tztZFTextUtils.isEmpty(pin)) {
            String pinLable = tztZFErrorMessageDefine.getUserDefineError(tztZFErrorMessageDefine.tztZFNetWorkErrorKey_PinLable, tztZFErrorMessageDefine.tztZFPinLable);
            String errorPin = String.format(Locale.getDefault(), "[C]入参%s错误", pinLable);
            if (tztZFRecordProcessLogTools.isRecord()) {
                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(tztZFRecordProcessLogTools.stringFormat("userID=%s;message=%s", userID, errorPin));
                tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_Cosign.value()), tztZFRecordEventType.tztZFRecordEventType_CreateCert,
                        tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
            }
            ret.setErrorNO(tztZFCosignErrPin);
            ret.setErrorMessage(errorPin);
            return ret;
        }

        String deviceId = tztZFCosignCert.getDeviceID();
        if (tztZFTextUtils.isEmpty(deviceId)) {
            if (tztZFRecordProcessLogTools.isRecord()) {
                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(tztZFRecordProcessLogTools.stringFormat("userID=%s;message=%s", userID, "获取设备信息错误，请重试"));
                tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_Cosign.value()), tztZFRecordEventType.tztZFRecordEventType_CreateCert,
                        tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
            }
            ret.setErrorNO(tztZFCosignErrDevice);
            ret.setErrorMessage("[C]获取设备信息错误，请重试");
            return ret;
        }
        byte[] D1 = null;
        byte[] P1 = null;
        ArrayList<byte[]> ayInfo = tztZFCosignSSL.cosignInit();
        if (ayInfo != null && ayInfo.size() == 2) {
            D1 = ayInfo.get(0);
            P1 = ayInfo.get(1);
        }
        if (D1 == null || P1 == null) {
            if (tztZFRecordProcessLogTools.isRecord()) {
                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(tztZFRecordProcessLogTools.stringFormat("userID=%s;message=%s", userID, "协同签名初始化错误"));
                tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_Cosign.value()), tztZFRecordEventType.tztZFRecordEventType_CreateCert,
                        tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
            }
            ret.setErrorMessage("[C]协同签名初始化错误");
            ret.setErrorNO(tztZFCosignErrClientSign0);
            return ret;
        }

        tztZFNetWorkReqObj reqObj = new tztZFNetWorkReqObj();


        if (userInfo.getCount() > 0)
            tztZFDataMapBase_AddToReq.addToReqObj(userInfo, reqObj);
        //modify 20220822 51950界面输入账号密码，去掉catoken入参，避免底层优先校验catoken，但上层输入密码错误时，仍旧可以执行通过的问题
//        if(userID != null && userID.length() > 0 && userToken != null && userToken.length() > 0){
//            reqObj.setStr("certUserId", userID);
//            reqObj.setStr("caToken", userToken);
//        }
        reqObj.setStr("Action", "51950");
        reqObj.setStr("DeviceId", deviceId);
        reqObj.setInt("ApplyType", applyType);
        reqObj.setBytes("P1", P1);
        reqObj.setTimeOut(1000 * 30);
        reqObj.setNativeObj("D1", D1);
        reqObj.setNativeObj("P1", P1);
        reqObj.setStr("Encrypt", "0");

        final byte[] finalD = D1;
        final byte[] finalP = P1;
        if (tztZFNetCosign.getShareInstance().getSendRecvBlock() != null) {
            tztZFNetCosign.tztZFCosignSendRecvObj.sendData(tztZFLinkType_Cosign, reqObj.getSendValues(), new tztZFNetCosign.tztZFNetCosignReceiveBlock() {
                @Override
                public void onCallBack(tztZFDataMapBase<String, Object> receiveData) {
                    tztZFNetCosignHead.tztZFNetCosignRespCosignInit resp = dealCosignRespInitReply(receiveData, finalD, finalP);
                    responseBlock.onCallBack(resp);
                }
            });
        } else {
            tztZFNetWork.sendDataAction(tztZFLinkType_Cosign.value(), reqObj, new tztZFNetWorkResponseCallBack() {
                @Override
                public void OnError(tztZFNetWorkReqObj c_reqObj, int c_reqObjTag, int c_recvType, int c_errorNo, String c_errorMsg) {
                    tztZFNetCosignHead.tztZFNetCosignRespCosignInit resp = new tztZFNetCosignHead.tztZFNetCosignRespCosignInit();
                    cosignRespWithClientErr(resp, c_errorMsg, c_errorNo);
                    responseBlock.onCallBack(resp);
                }

                @Override
                public void OnAns(tztZFNetWorkReqObj c_reqObj, int c_reqObjTag, tztZFNetWorkRespObj c_respObj) {
                    tztZFNetCosignHead.tztZFNetCosignRespCosignInit resp = dealCosignRespInitReply(c_respObj.getDataMap(), finalD, finalP);
                    responseBlock.onCallBack(resp);
                }
            });
        }
        ret.setErrorNO(tztZFCosignErrOK);
        ret.setErrorMessage("");
        return ret;
    }

    private static tztZFNetCosignHead.tztZFNetCosignRespCosignInit dealCosignRespInitReply(tztZFDataMapBase<String, Object> c_respObj, byte[] D1, byte[] P1) {
        tztZFNetCosignHead.tztZFNetCosignRespCosignInit resp = new tztZFNetCosignHead.tztZFNetCosignRespCosignInit();
        cosignRespWithNetResp(resp, c_respObj);
        if (c_respObj == null) {
            return resp;
        }
        if (tztZFRecordProcessLogTools.isRecord()) {
            String param1 = tztZFRecordProcessLogTools.stringFormat("message=%s", "51950应答成功");
            String param2 = tztZFRecordProcessLogTools.stringFormat("51950应答数据=%s", c_respObj.toString());
            ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1, param2);
            tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_Cosign.value()), tztZFRecordEventType.tztZFRecordEventType_CreateCert,
                    tztZFReocordLogType.tztZFReocordLogType_Debug, params, "");
        }
        if (resp.errorNO >= 0) {
            String strUserID = c_respObj.getStr("certUserId");
            if (tztZFTextUtils.isEmpty(strUserID)) {
                strUserID = c_respObj.getStr("UserID");
            }
            resp.setUserID(strUserID);
            resp.setUserToken(c_respObj.getStr("caToken"));
            resp.setErrorNO(tztZFCosignErrOK);
            resp.setP(c_respObj.getBytes("P"));
            resp.setIsNew(c_respObj.getInt("isNew"));
            if (D1 != null && D1.length > 0) {
                resp.setD(D1);
            }
            if (P1 != null && P1.length > 0) {
                resp.setP1(P1);
            }
            //密钥分量标志，后台返回，客户端51951协同签名时需要带上
            resp.setDMark(c_respObj.getStr("DMark"));
        }
        return resp;
    }

    /**
     * 协同签名
     *
     * @param org           内容
     * @param userID        用户ID
     * @param responseBlock 回调
     * @return 调用结果
     */
    public static tztZFNetCosignHead.tztZFCosignResp tztCosignData(byte[] org, final String userID, int fromType, final tztZFNetCosignHead.tztZFNetCosignRespCosignDataCallBack responseBlock) {
    /*
     Action:51951 中焯协同签名
     Req:
     UserId    string    客户号
     DeviceId    string    设备Id
     e    string    摘要值
     Q1    string    客户端签名临时结果Q1

     r    string    协同签名服务端签名结果r
     s2    string    协同签名服务端签名结果s2
     s3    string    协同签名服务端签名结果s3
     */
        tztZFNetCosignHead.tztZFCosignResp ret = new tztZFNetCosignHead.tztZFCosignResp();
        ret.setCosignType(0);
        ret.setFromType(fromType);
        if (org == null || org.length != 32 || responseBlock == null) {
            ret.setErrorNO(tztZFCosignErrParam);
            ret.setErrorMessage("[C]入参错误");
            return ret;
        }
        if (tztZFTextUtils.isEmpty(userID)) {
            ret.setErrorNO(tztZFCosignErrParam);
            ret.setErrorMessage("[C]入参错误，客户号为空");
            return ret;
        }

        String deviceId = tztZFCosignCert.getDeviceID();
        if (tztZFTextUtils.isEmpty(deviceId)) {
            ret.setErrorNO(tztZFCosignErrDevice);
            ret.setErrorMessage("[C]获取设备信息错误,请重试");
            return ret;
        }

        final tztZFCosignUser cosignUser = tztZFNetCosign.cosignUserForID(userID);
        if (cosignUser == null || !cosignUser.isCosigned()) {
            ret.setErrorNO(tztZFCosignErrUnfind);
            String pinLable = tztZFErrorMessageDefine.getUserDefineError(tztZFErrorMessageDefine.tztZFNetWorkErrorKey_PinLable, tztZFErrorMessageDefine.tztZFPinLable);
            ret.setErrorMessage(String.format(Locale.getDefault(), "[C]获取用户签名%s失败", pinLable));
            return ret;
        }
        byte[] K1 = null;
        byte[] Q1 = null;
        ArrayList<byte[]> ayInfo = tztZFCosignSSL.cosignStep1();
        if (ayInfo != null) {
            K1 = ayInfo.get(0);
            Q1 = ayInfo.get(1);
        }

        if (K1 == null || K1.length < 1 || Q1 == null || Q1.length < 1) {
            ret.setErrorNO(tztZFCosignErrClientSign1);
            ret.setErrorMessage("[C]协同签名初始化失败");
            return ret;
        }

        tztZFNetWorkReqObj req = new tztZFNetWorkReqObj();
        req.setStr("Action", "51951");
        req.setStr("UserId", userID);
        req.setStr("certUserId", userID);
        req.setStr("DeviceId", deviceId);
        req.setNativeObj("CosignUser", cosignUser);
        //20220908 51951协同签名时，带上51950返回的密钥分量标志
        //该标志内容51950返回，客户端本地存储；服务器端先校验该标志，不匹配直接返回找不到密钥分量
        if (cosignUser.getLoginCosignCert() != null) {
            req.setStr("DMark", cosignUser.getLoginCosignCert().getDMark());
        }
        req.setTimeOut(20 * 1000);

        byte[] dataE = tztZFDataEncrypt.hexEncrypt(org);
        String strE = "";
        if (dataE != null && dataE.length > 0) {
            strE = new String(dataE);
        }
        req.setStr("e", strE);
        req.setBytes("Q1", Q1);

        final byte[] finalK = K1;
        if (tztZFNetCosign.getShareInstance().getSendRecvBlock() != null) {
            tztZFNetCosign.tztZFCosignSendRecvObj.sendData(tztZFLinkType_Cosign, req.getSendValues(), new tztZFNetCosign.tztZFNetCosignReceiveBlock() {
                @Override
                public void onCallBack(tztZFDataMapBase<String, Object> receiveData) {
                    tztZFNetCosignHead.tztZFNetCosignRespCosignData resp = dealCosignRespDataReply(receiveData, finalK, cosignUser, userID);
                    responseBlock.onCallBack(resp);
                }
            });
        } else {
            tztZFNetWork.sendDataAction(tztZFLinkType_Cosign.value(), req, new tztZFNetWorkResponseCallBack() {
                @Override
                public void OnError(tztZFNetWorkReqObj c_reqObj, int c_reqObjTag, int c_recvType, int c_errorNo, String c_errorMsg) {
                    tztZFNetCosignHead.tztZFNetCosignRespCosignData resp = new tztZFNetCosignHead.tztZFNetCosignRespCosignData();
                    cosignRespWithClientErr(resp, c_errorMsg, c_errorNo);
                    responseBlock.onCallBack(resp);
                }

                @Override
                public void OnAns(tztZFNetWorkReqObj c_reqObj, int c_reqObjTag, tztZFNetWorkRespObj c_respObj) {
                    tztZFCosignUser reqCosignUser = (tztZFCosignUser) c_respObj.getReqObj().getNativeObj("CosignUser");
                    tztZFNetCosignHead.tztZFNetCosignRespCosignData resp = dealCosignRespDataReply(c_respObj.getDataMap(), finalK, reqCosignUser, userID);
                    responseBlock.onCallBack(resp);
                }
            });
        }

        ret.setErrorNO(tztZFCosignErrOK);
        ret.setErrorMessage("");
        return ret;
    }

    private static tztZFNetCosignHead.tztZFNetCosignRespCosignData dealCosignRespDataReply(tztZFDataMapBase<String, Object> c_respObj, byte[] K1, tztZFCosignUser reqCosignUser, String userID) {
        tztZFNetCosignHead.tztZFNetCosignRespCosignData resp = new tztZFNetCosignHead.tztZFNetCosignRespCosignData();
        cosignRespWithNetResp(resp, c_respObj);
        if (c_respObj == null) {
            resp.setErrorMessage("[S]协同签名服务器应答错误");
            return resp;
        }
        if (resp.errorNO < 0) {
            return resp;
        }
        if (reqCosignUser == null || !reqCosignUser.isCosigned()) {
            resp.setErrorNO(tztZFCosignErrUnfind);
            resp.setErrorMessage(String.format(Locale.getDefault(), "[%s]协同签名未初始化", userID));
            return resp;
        }

        byte[] P = c_respObj.getBytes("P");
        //51951返回的P与本地存储的P进行比较，如果不一样，删掉本地的证书
        if (reqCosignUser.getLoginCosignCert() != null && !reqCosignUser.getLoginCosignCert().compareP(P)) {
            tztZFNetCosign.removeLocalCert(userID);
        }

        byte[] r = c_respObj.getBytes("R");
        byte[] s2 = c_respObj.getBytes("S2");
        byte[] s3 = c_respObj.getBytes("S3");
        if (r == null || r.length < 1 || s2 == null || s2.length < 1 || s3 == null || s3.length < 1) {
            resp.setErrorNO(tztZFCosignErrSign);
            resp.setErrorMessage(String.format(Locale.getDefault(), "[%s]协同签名失败2", userID));
            return resp;
        }

        byte[] signData = reqCosignUser.getLoginCosignCert().cosignSignZZ(K1, r, s2, s3);
        if (signData == null || signData.length < 1) {
            resp.setErrorNO(tztZFCosignErrSign);
            resp.setErrorMessage(String.format(Locale.getDefault(), "[%s]协同签名失败3", userID));
            return resp;
        }
        if (tztZFLog.getLevel() <= tztZFLogLevel_Info.value()) {
            byte[] signDataEncrypt = tztZFEncryptKit.hexEncrypt(signData, signData.length);
            byte[] rEncrypt = tztZFEncryptKit.hexEncrypt(r, r.length);
            byte[] s2Encrypt = tztZFEncryptKit.hexEncrypt(s2, s2.length);
            byte[] s3Encrypt = tztZFEncryptKit.hexEncrypt(s3, s3.length);
            String signStr = new String(signDataEncrypt);
            String rStr = new String(rEncrypt);
            String s2Str = new String(s2Encrypt);
            String s3Str = new String(s3Encrypt);
            if (signDataEncrypt != null && signDataEncrypt.length > 0) {
                tztZFLog.log(tztZFLogLevel.tztZFLogLevel_error, "signData", "r=%s;s2=%s;s3=%s;signData=%s", rStr, s2Str, s3Str, signStr);
            }
        }
        resp.setSignData(signData);
        resp.setSignUser(reqCosignUser.userID());
        resp.setErrorNO(tztZFCosignErrOK);
        resp.setErrorMessage(String.format(Locale.getDefault(), "[%s]签名成功", userID));
        return resp;
    }

    /**
     * 证书申请数据解析
     *
     * @param c_respObj 应答数据
     * @return 解析后的应答数据
     */
    private static tztZFNetCosignHead.tztZFNetCosignRespApplyCert toRespApplyCert(tztZFDataMapBase<String, Object> c_respObj) {
        tztZFNetCosignHead.tztZFNetCosignRespApplyCert resp = new tztZFNetCosignHead.tztZFNetCosignRespApplyCert();
        cosignRespWithNetResp(resp, c_respObj);
        if (c_respObj == null)
            return resp;

        resp.setUserID(c_respObj.getStr("certUserId"));
        resp.setUserToken(c_respObj.getStr("caToken"));

        resp.setUserName(c_respObj.getStr("userName"));

        String deviceInfo = c_respObj.getStr("deviceInfo");
        if (tztZFTextUtils.isEmpty(deviceInfo))
            deviceInfo = tztZFCosignCert.getDeviceInfo();
        resp.setDeviceInfo(deviceInfo);
        String deviceID = c_respObj.getStr("deviceID");
        if (tztZFTextUtils.isEmpty(deviceID))
            deviceID = tztZFCosignCert.getDeviceID();
        resp.setDeviceID(deviceID);

        resp.setCertSerialNumber(c_respObj.getStr("certSerialNumber"));

        String strCert = c_respObj.getStr("certSignBuf");
        if (!tztZFTextUtils.isEmpty(strCert) && !strCert.startsWith("-----BEGIN CERTIFICATE-----")) {
            strCert = String.format(Locale.CHINA, "%s%s%s",
                    "-----BEGIN CERTIFICATE-----\n",
                    strCert,
                    "\n-----END CERTIFICATE-----");
        }
        resp.setCertSignBuf(strCert);
        resp.setCertNotBefore(c_respObj.getStr("certNotBefore"));
        resp.setCertNotAfter(c_respObj.getStr("certNotAfter"));
        resp.setCertStatus(c_respObj.getStr("certStatus"));
        resp.setCertSignDate(c_respObj.getStr("certSignDate"));
        resp.setCertSubjectDN(c_respObj.getStr("certSubjectDN"));
        resp.setCertSerialNumberKmc(c_respObj.getStr("certSerialnumberKmc"));
        resp.setCertSignBufKmc(c_respObj.getBytes("certSignBufKmc"));
        resp.setEncPrivateKeyUser(c_respObj.getBytes("encPrivateKeyUser"));
        resp.setCertType(c_respObj.getInt("certType"));
        return resp;
    }

    /**
     * 证书列表数据解析
     *
     * @param c_resp 应答数据
     * @return 解析后的数据对象
     */
    private static tztZFNetCosignHead.tztZFNetCosignRespCertList toRespCertList(tztZFDataMapBase<String, Object> c_resp) {
        tztZFNetCosignHead.tztZFNetCosignRespCertList resp = new tztZFNetCosignHead.tztZFNetCosignRespCertList();
        cosignRespWithNetResp(resp, c_resp);
        if (c_resp == null)
            return null;
        resp.setUserID(c_resp.getStr("certUserId"));
        resp.setUserToken(c_resp.getStr("caToken"));
        String currentTime = c_resp.getStr(tztCosignEx_CurrentTime);
        if (!tztZFPublicFun.isEmpty(currentTime) && tztZFNetCosignEx.cosignExInfo() != null) {
            tztZFNetCosignEx.cosignExInfo().setStr(tztCosignEx_CurrentTime, currentTime);
        }

        ArrayList<tztZFCertificateInfo> array = new ArrayList<>();
        ArrayList<ArrayList<String>> ayGrid = tztZFDataMapBase_AddToReq.getArray(c_resp, "Grid");
        int UserId_idx = c_resp.getInt("UserId_idx", -1);
        int certSerialNumber_idx = c_resp.getInt("certSerialNumber_idx", -1);
        int certSubjectDN_idx = c_resp.getInt("certSubjectDN_idx", -1);
        int certNotAfter_idx = c_resp.getInt("certNotAfter_idx", -1);
        int certNotBefore_idx = c_resp.getInt("certNotBefore_idx", -1);
        int certSignDate_idx = c_resp.getInt("certSignDate_idx", -1);
        int certStatus_idx = c_resp.getInt("certStatus_idx", -1);
        int DeviceId_idx = c_resp.getInt("DeviceId_idx", -1);
        int DeviceInfo_idx = c_resp.getInt("DeviceInfo_idx", -1);

//        String userLocSN = "";
//        tztZFCosignUser cosignUser = tztZFNetCosign.getBindCosignUser();
//        if(cosignUser != null && cosignUser.getLoginUserID() != null && cosignUser.getLoginUserID().equals(resp.getUserID()) && cosignUser.getLoginCosignCert() != null && cosignUser.getLoginCosignCert().getSignInfo() != null)
//        userLocSN = cosignUser.getLoginCosignCert().getSignInfo().getSerialNumber();

        if (ayGrid != null) {
            for (ArrayList<String> ayData : ayGrid) {
                tztZFCertificateInfo info = new tztZFCertificateInfo();
                info.setLocal(false);
                if (UserId_idx >= 0 && UserId_idx < ayData.size()) {
                    info.setUserID(ayData.get(UserId_idx));
                }
                if (certSerialNumber_idx >= 0 && certSerialNumber_idx < ayData.size()) {
                    info.setSerialNumber(ayData.get(certSerialNumber_idx));
//                    info.setLocal(!tztZFTextUtils.isEmpty(userLocSN) && userLocSN.equals(info.getSerialNumber()));
                }
                if (certSubjectDN_idx >= 0 && certSubjectDN_idx < ayData.size()) {
                    info.setSubject(ayData.get(certSubjectDN_idx));
                }
                if (certNotAfter_idx >= 0 && certNotAfter_idx < ayData.size()) {
                    String strAfter = ayData.get(certNotAfter_idx);
                    info.setNotAfter(tztZFCertificateInfo.dateFromString(strAfter));
                }
                if (certNotBefore_idx >= 0 && certNotBefore_idx < ayData.size()) {
                    String strBefore = ayData.get(certNotBefore_idx);
                    info.setNotBefore(tztZFCertificateInfo.dateFromString(strBefore));
                }
                if (certSignDate_idx >= 0 && certSignDate_idx < ayData.size()) {
                    String strSign = ayData.get(certSignDate_idx);
                    info.setSignDate(tztZFCertificateInfo.dateFromString(strSign));
                }
                if (certStatus_idx >= 0 && certStatus_idx < ayData.size()) {
                    info.setStatus(ayData.get(certStatus_idx));
                }
                if (DeviceInfo_idx >= 0 && DeviceInfo_idx < ayData.size()) {
                    info.setDeviceInfo(ayData.get(DeviceInfo_idx));
                }
                if (DeviceId_idx >= 0 && DeviceId_idx < ayData.size()) {
                    info.setDeviceID(ayData.get(DeviceId_idx));
                }

                array.add(info);
            }
        }
        resp.setCertList(array);
        return resp;
    }

    private static void cosignRespWithClientErr(tztZFNetCosignHead.tztZFNetCosignResp resp, String errorMessage, int errorNo) {
        if (resp == null)
            return;
        resp.setErrorNO(errorNo);
        resp.setErrorMessage(errorMessage);
    }

    private static void cosignRespWithClientErr(tztZFNetCosignHead.tztZFNetCosignResp resp, String errorMessage) {
        cosignRespWithClientErr(resp, errorMessage, tztZFCosignErrParam);
    }

    private static void cosignRespWithNetResp(tztZFNetCosignHead.tztZFNetCosignResp resp, tztZFDataMapBase<String, Object> respObj) {
        if (resp == null)
            return;
        if (respObj == null) {
            resp.setErrorNO(tztZFCosignErrOther);
            return;
        }
        int errorNo = respObj.getInt("ErrorNo");
        resp.setErrorMessage(respObj.getStr("ErrorMessage"));
        if (errorNo >= 0) {
            resp.setErrorNO(tztZFCosignErrOK);
        } else {
            resp.setErrorNO(errorNo);
        }
        resp.setCosignDowngrade(respObj.getInt("GMStatus") == 2);
        ///数据返回上层，便于扩展使用
        resp.getDictExtern().setDataMap(respObj);
    }
}
