package com.tztzf.network.gm;


import com.tztzf.baseos.tztZFTextUtils;
import com.tztzf.commbase.tztZFDataMapBase;
import com.tztzf.commbase.tztZFThreadPool;
import com.tztzf.network.callback.tztZFGMSSLStepCallBack;
import com.tztzf.network.callback.tztZFLinkCallBack;
import com.tztzf.network.callback.tztZFNetWorkResponseCallBack;
import com.tztzf.network.cosign.tztZFCertificateInfo;
import com.tztzf.network.cosign.tztZFCosignCert;
import com.tztzf.network.cosign.tztZFCosignUser;
import com.tztzf.network.cosign.tztZFNetCosign;
import com.tztzf.network.cosign.tztZFNetCosignCallBack;
import com.tztzf.network.cosign.tztZFNetCosignEx;
import com.tztzf.network.cosign.tztZFNetCosignHead;
import com.tztzf.network.define.tztZFErrorMessageDefine;
import com.tztzf.network.define.tztZFLinkType;
import com.tztzf.network.define.tztZFRecvType;
import com.tztzf.network.tztZFNetWork;
import com.tztzf.network.tztZFNetWorkReqObj;
import com.tztzf.network.tztZFNetWorkRespObj;
import com.tztzf.protocol.core.tztZFCosignSSL;
import com.tztzf.protocol.core.tztZFDataEncrypt;
import com.tztzf.protocol.tztZFEncryptKit;
import com.tztzf.recordprocesslog.tztZFRecordEventType;
import com.tztzf.recordprocesslog.tztZFRecordProcessLogTools;
import com.tztzf.recordprocesslog.tztZFReocordLogType;
import com.tztzf.tztZFNotificationObj;

import org.greenrobot.eventbus.EventBus;

import java.util.ArrayList;
import java.util.Locale;
import java.util.Timer;
import java.util.TimerTask;

import static com.tztzf.network.cosign.tztZFCosignErr.tztZFCosignErrExpire;
import static com.tztzf.network.cosign.tztZFCosignErr.tztZFCosignErrOK;
import static com.tztzf.network.cosign.tztZFCosignErr.tztZFCosignErrPin;
import static com.tztzf.network.cosign.tztZFCosignErr.tztZFCosignErrServerCertUnfind;
import static com.tztzf.network.define.tztZFErrorMessageDefine.tztZFNetWorkErrorKey_NotGMStatus;
import static com.tztzf.network.define.tztZFErrorMessageDefine.tztZFNetWorkErrorKey_WriteError;
import static com.tztzf.network.define.tztZFErrorMessageDefine.tztZFWriteError;
import static com.tztzf.network.define.tztZFLinkType.tztZFLinkType_Cosign;
import static com.tztzf.network.define.tztZFLinkType.tztZFLinkType_JY;
import static com.tztzf.network.gm.tztZFGMStep.tztZFGMStepApplyCert;
import static com.tztzf.network.gm.tztZFGMStep.tztZFGMStepBindUserID;
import static com.tztzf.network.gm.tztZFGMStep.tztZFGMStepGMDowngrade;
import static com.tztzf.network.gm.tztZFGMStep.tztZFGMStepGMStatus;
import static com.tztzf.network.gm.tztZFGMStep.tztZFGMStepLogin;
import static com.tztzf.network.gm.tztZFGMStep.tztZFGMStepLoginSign;
import static com.tztzf.network.gm.tztZFGMStep.tztZFGMStepLoginTimeout;
import static com.tztzf.network.gm.tztZFGMStep.tztZFGMStepQueryCert;
import static com.tztzf.network.gm.tztZFGMStep.tztZFGMStepQueryUserID;
import static com.tztzf.network.gm.tztZFGMStep.tztZFGMStepResetPin;
import static com.tztzf.network.gm.tztZFGMStep.tztZFGMStepRevokeCert;
import static com.tztzf.network.gm.tztZFGMStep.tztZFGMStepSSLStep;
import static com.tztzf.network.gm.tztZFSocketSSLStep.tztZFSocketSSLStepClose;
import static com.tztzf.network.gm.tztZFSocketSSLStep.tztZFSocketSSLStepDowngrade;
import static com.tztzf.network.gm.tztZFSocketSSLStep.tztZFSocketSSLStepSucess;
import static com.tztzf.network.gm.tztZFSocketSSLStep.tztZFSocketSSLStepUnClientCert;
import static com.tztzf.network.gm.tztZFSocketSSLStep.tztZFSocketSSLStepUnServerCert;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_Send;

/**
 * 国密初始化对象
 */
public class tztZFGMManage {

    //region class tztZFGMLoginReqObj
    public static class tztZFGMLoginReqObj {
        private tztZFNetWorkReqObj reqObj;
        private String userID;
        private String userToken;
        private String userPin;
        private boolean bGMLogin;
        private boolean bAutoResetPin;
        private boolean checkKHBH;
        private boolean bUseCustomSafetyCode;
        private boolean isSend;
        private boolean isResp;

        //region get/set

        public tztZFNetWorkReqObj getReqObj() {
            return reqObj;
        }

        public void setReqObj(tztZFNetWorkReqObj reqObj) {
            this.reqObj = reqObj;
        }

        public String getUserID() {
            return userID;
        }

        public void setUserID(String userID) {
            this.userID = userID;
        }

        public String getUserToken() {
            return userToken;
        }

        public void setUserToken(String userToken) {
            this.userToken = userToken;
        }

        public String getUserPin() {
            return userPin;
        }

        public void setUserPin(String userPin) {
            this.userPin = userPin;
        }

        public boolean isbGMLogin() {
            return bGMLogin;
        }

        public void setbGMLogin(boolean bGMLogin) {
            this.bGMLogin = bGMLogin;
        }

        public boolean isbAutoResetPin() {
            return bAutoResetPin;
        }

        public void setbAutoResetPin(boolean bAutoResetPin) {
            this.bAutoResetPin = bAutoResetPin;
        }

        public boolean isCheckKHBH() {
            return checkKHBH;
        }

        public void setCheckKHBH(boolean checkKHBH) {
            this.checkKHBH = checkKHBH;
        }

        public boolean isbUseCustomSafetyCode() {
            return bUseCustomSafetyCode;
        }

        public void setbUseCustomSafetyCode(boolean bUseCustomSafetyCode) {
            this.bUseCustomSafetyCode = bUseCustomSafetyCode;
        }

        public boolean isSend() {
            return isSend;
        }

        public void setSend(boolean send) {
            isSend = send;
        }

        public boolean isResp() {
            return isResp;
        }

        public void setResp(boolean resp) {
            isResp = resp;
        }

        //endregion

        public tztZFGMLoginReqObj() {
            reqObj = null;
            userPin = null;
            userID = null;
            userToken = null;
            bGMLogin = true;
            bAutoResetPin = true;
            checkKHBH = true;
            bUseCustomSafetyCode = true;
            isResp = false;
            isSend = false;
        }

        private boolean isInvalid() {
            if (reqObj == null)
                return true;
            if (bGMLogin && (bUseCustomSafetyCode && tztZFTextUtils.isEmpty(userPin)))
                return true;
            tztZFDataMapBase<String, Object> reqVals = reqObj.getSendValues();
            if (reqVals == null || reqVals.getCount() < 1)
                return true;
            tztZFCosignAccount account = new tztZFCosignAccount(tztZFNetCosign.getContext());
            account.setAccount(reqVals.getStr("Account"));
            account.setAccountType(reqVals.getStr("Accounttype"));
            if (account.isInvalid())
                return true;
            if (!tztZFTextUtils.isEmpty(userID))
                return false;

            userID = tztZFCosignAccount.loadUserID(account, checkKHBH);
            return false;
        }

        private void saveAccount() {
            if (reqObj == null)
                return;
            tztZFDataMapBase<String, Object> reqVals = reqObj.getSendValues();
            if (reqVals == null || reqVals.getCount() < 1)
                return;
            tztZFCosignAccount account = new tztZFCosignAccount(tztZFNetCosign.getContext());
            account.setAccount(reqVals.getStr("Account"));
            account.setAccountType(reqVals.getStr("Accounttype"));
            tztZFCosignAccount.saveAccount(account, userID);
        }

        private String getSafetyCode() {
            if (!bUseCustomSafetyCode) {
                String safetyCode = tztZFCosignCert.loadSystemSafetyCode(userID);
                if (!tztZFTextUtils.isEmpty(safetyCode)) {
                    this.userPin = safetyCode;
                } else {
                    this.userPin = tztZFGMManage.getNormalSafetyCode(this.userID);
                }

            }
            return userPin;
        }
    }
    //endregion

    //region class tztZFGMLogin

    /**
     * 国密登录对象
     */
    public static class tztZFGMLogin {
        private tztZFGMLoginReqObj loginReq;
        private tztZFGMManageCallBack gmBlock;
        private tztZFGMLoginResponseCallBack respBlock;
        private int timeOut;
        private Timer connectTimer;
        private Object context;
        private tztZFThreadPool m_pThreadPool = null;

        private tztZFGMLogin() {
        }

        public tztZFGMLogin(Object context) {
            this.context = context;
            m_pThreadPool = new tztZFThreadPool(3, 3, "jylogin");
            m_pThreadPool.setUsed(true);
        }

        private void startLogin() {
            m_pThreadPool.useThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    if (loginReq == null || loginReq.isInvalid()) {
                        responseGMStep(tztZFGMStepQueryUserID, -1, "参数错误", null);
                        return;
                    }

                    if (!loginReq.bGMLogin) {
                        //普通登录
                        tztZFCosignUser cosignUser = tztZFNetCosign.getBindCosignUser();
                        if (cosignUser != null) {
                            cosignUser.setNeedGMLogin(loginReq.bGMLogin);
                        }
                        if (tztZFRecordProcessLogTools.isRecord()) {
                            String param1 = tztZFRecordProcessLogTools.stringFormat("mesage=%s", "普通登录交易");
                            String param2 = tztZFRecordProcessLogTools.stringFormat("cosignUser=%s", cosignUser == null ? "" : cosignUser.string());
                            ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1, param2);
                            tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_JY.value()), tztZFRecordEventType.tztZFRecordEventType_TradeLogin,
                                    tztZFReocordLogType.tztZFReocordLogType_Debug, params, "");
                        }
                        doLogin();
                        return;
                    }
                    timeOut = loginReq.reqObj.getTimeOut();
                    if (timeOut < 1)
                        timeOut = 30 * 1000;
                    startConnectTimeout();
                    if (!tztZFNetCosign.getCosignStatus()) {
                        if (stop(tztZFGMStepGMStatus, -1, tztZFErrorMessageDefine.getUserDefineError(tztZFNetWorkErrorKey_NotGMStatus, "当前非国密状态"), null))
                            return;
                        if (tztZFRecordProcessLogTools.isRecord()) {
                            ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(tztZFRecordProcessLogTools.stringFormat("mesage=%s", "当前非国密状态:登录交易"));
                            tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_JY.value()), tztZFRecordEventType.tztZFRecordEventType_TradeLogin,
                                    tztZFReocordLogType.tztZFReocordLogType_Debug, params, "");
                        }
                        doLogin();//直接登录
                        return;
                    }
                    if (tztZFTextUtils.isEmpty(loginReq.userID)) {
                        queryUserID();
                    } else {
                        bindUserID();
                    }
                }
            });
        }

        public synchronized void endConnectTimeOut() {
            cancelLogin();
        }

        public void cancelLogin() {
            endConnectTimeout(true);
        }

        private void startConnectTimeout() {
            if (timeOut > 0 && connectTimer == null && !(loginReq == null || loginReq.isSend || loginReq.isResp)) {
                connectTimer = new Timer();
                connectTimer.schedule(new TimerTask() {
                    @Override
                    public void run() {
                        cancelLogin();
                    }
                }, timeOut, timeOut);
            }
        }

        private synchronized void endConnectTimeout(boolean bResp) {
            if (connectTimer != null) {
                connectTimer.cancel();
                connectTimer = null;
                if (!bResp)
                    return;
                @tztZFGMStep int step = tztZFGMStepLoginTimeout;
                String msg = tztZFErrorMessageDefine.getUserDefineError(tztZFNetWorkErrorKey_WriteError, tztZFWriteError);
                if (doStopStep(step, loginReq.userID, msg))
                    return;
                if (tztZFRecordProcessLogTools.isRecord()) {
                    ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(tztZFRecordProcessLogTools.stringFormat("endConnectTimeout=%s", msg));
                    tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_JY.value()), tztZFRecordEventType.tztZFRecordEventType_TradeLogin,
                            tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
                }
                responseGMStep(step, -1 * step, msg, null);
            }
        }

        private void responseGMStep(@tztZFGMStep int step, int errorNo, String errorMessage, tztZFDataMapBase<String, Object> dataMapBase) {
            if (respBlock == null || loginReq == null || loginReq.isResp)
                return;
            //应答时，结束超时
            endConnectTimeout(false);
            tztZFDataMapBase<String, Object> dataMap = new tztZFDataMapBase<>();
            if (dataMapBase != null) {
                dataMap.setObj(tztZFDataMapBase.class.getSimpleName(), dataMapBase);
            }
            dataMap.setInt("ErrorNo", errorNo);
            dataMap.setStr("ErrorMessage", errorMessage);
            dataMap.setInt("Action", loginReq.reqObj.getSendValues().getAction());
            dataMap.setStr("ReqNo", loginReq.reqObj.getSendValues().getStr("ReqNo"));
            tztZFNetWorkRespObj response = loginReq.reqObj.convertToResponse(dataMap);
            response.setRecvType(tztZFRecvType.tztZFSend_SendError);
            response.setReqObjTag(loginReq.reqObj.getReqObjTag());
            response.setReqObj(loginReq.reqObj);
            responseBlock(step, errorNo, response);
        }

        private synchronized void responseBlock(@tztZFGMStep int step, int errorNo, tztZFNetWorkRespObj response) {
            if (respBlock == null || loginReq == null || loginReq.isResp)
                return;
            synchronized (loginReq) {
                if (loginReq.isResp)
                    return;
                loginReq.isResp = true;
            }
            respBlock.onCallBack(step, errorNo >= 0, loginReq, response);
        }

        private boolean stop(@tztZFGMStep int step, int error, String msg, tztZFDataMapBase<String, Object> dataMapBase) {
            boolean bStop = true;
            if (gmBlock != null) {
                tztZFDataMapBase<String, Object> data = new tztZFDataMapBase<>();
                data.setErrorNo(error);
                data.setErrorMessage(msg);
                bStop = gmBlock.onCallBack(step, false, bStop, data);
            }
            if (bStop) {
                responseGMStep(step, error, msg, dataMapBase);
            }
            return bStop;
        }

        private tztZFDataMapBase<String, Object> getSendDataMap() {
            tztZFDataMapBase<String, Object> sendValue = new tztZFDataMapBase<>();
            if (loginReq != null && loginReq.reqObj != null && loginReq.reqObj.getSendValues() != null) {
                sendValue.setDataMap(loginReq.reqObj.getSendValues());
                if (sendValue.getAction() == 100) {
                    sendValue.setStr("Type", "1");
                } else if (sendValue.getAction() == 104) {
                    sendValue.setStr("Type", "2");
                } else if (sendValue.getAction() == 105) {
                    sendValue.setStr("Type", "3");
                } else if (sendValue.getAction() == 101) {
                    sendValue.setStr("Type", "4");
                } else if (sendValue.getAction() == 106) {
                    sendValue.setStr("Type", "5");
                }
                String type = loginReq.reqObj.getNativeStr("type");
                if (!tztZFTextUtils.isEmpty(type)) {
                    sendValue.setStr("Type", type);
                }
            }
            return sendValue;
        }

        /**
         * 查询客户号
         *
         * @return true=发送查询
         */
        private boolean queryUserID() {
            //是否停止执行
            if (doStopStep(tztZFGMStepQueryUserID, loginReq.userID))
                return false;
            tztZFNetCosign.queryUserWithAccount(getSendDataMap(), new tztZFNetCosignHead.tztZFNetCosignRespCallBack() {
                @Override
                public void onCallBack(tztZFNetCosignHead.tztZFNetCosignResp c_resp) {
                    if (c_resp != null) {
                        boolean bSaveUserID = c_resp.getErrorNO() > 0 && !tztZFTextUtils.isEmpty(c_resp.getUserID()) && !tztZFTextUtils.isEmpty(c_resp.getUserToken());
                        if (bSaveUserID) {
                            loginReq.userID = c_resp.getUserID();
                            loginReq.userToken = c_resp.getUserToken();
                            loginReq.saveAccount();
                        }
                        if (c_resp.isCosignDowngrade()) {
                            if (tztZFRecordProcessLogTools.isRecord()) {
                                String param1 = tztZFRecordProcessLogTools.stringFormat("message=%s", "51954应答降级");
                                String param2 = tztZFRecordProcessLogTools.stringFormat("51954应答结果=%s", c_resp.toString());
                                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1, param2);
                                tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_JY.value()), tztZFRecordEventType.tztZFRecordEventType_TradeLogin,
                                        tztZFReocordLogType.tztZFReocordLogType_Debug, params, "");
                            }
                            tztZFGMLogin.this.didGMDowngrade();
                            return;
                        }
                        if (bSaveUserID) {
                            m_pThreadPool.useThreadPool().execute(new Runnable() {
                                @Override
                                public void run() {
                                    tztZFGMLogin.this.bindUserID();
                                }
                            });
                            return;
                        }
                        tztZFGMLogin.this.stop(tztZFGMStepQueryUserID, c_resp.getErrorNO(), c_resp.getErrorMessage(), c_resp.getDictExtern());
                    } else {
                        if (tztZFRecordProcessLogTools.isRecord()) {
                            ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(tztZFRecordProcessLogTools.stringFormat("mesage=%s", "查询账号客户号失败"));
                            tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_JY.value()), tztZFRecordEventType.tztZFRecordEventType_TradeLogin,
                                    tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
                        }
                        tztZFGMLogin.this.stop(tztZFGMStepQueryUserID, -1, "查询账号客户号失败", null);
                    }
                }
            });
            return true;
        }

        /**
         * 绑定客户号
         */
        private boolean bindUserID() {
            if (doStopStep(tztZFGMStepBindUserID, loginReq.userID))
                return false;
            if (!tztZFTextUtils.isEmpty(loginReq.userToken)) {
                tztZFNetCosign.bindUserIDWithToken(loginReq.userID, loginReq.userToken);
            }
            tztZFNetCosign.bindUserID(loginReq.userID, loginReq.getSafetyCode());
            tztZFCosignUser cosignUser = tztZFNetCosign.getBindCosignUser();
            if (cosignUser != null) {
                cosignUser.setNeedGMLogin(loginReq.bGMLogin);
            }
            if (tztZFRecordProcessLogTools.isRecord()) {
                String param1 = tztZFRecordProcessLogTools.stringFormat("userID=%s;mesage=%s", loginReq.userID, "绑定客户号");
                String param2 = tztZFRecordProcessLogTools.stringFormat("cosignUser=%s", cosignUser == null ? "" : cosignUser.string());
                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1, param2);
                tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_JY.value()), tztZFRecordEventType.tztZFRecordEventType_TradeLogin,
                        tztZFReocordLogType.tztZFReocordLogType_Debug, params, "");
            }
            if (!loginReq.bGMLogin) {
                return didGMLogin();
            }
            return queryUserCert();
        }

        public tztZFGMLogin(tztZFGMLoginReqObj loginReq) {
            this.loginReq = loginReq;
        }

        /**
         * 查询证书
         */
        private boolean queryUserCert() {
            if (doStopStep(tztZFGMStepQueryCert, loginReq.userID))
                return false;

            tztZFCosignUser cosignUser = tztZFNetCosign.getBindCosignUser();
            tztZFCertificateInfo certificateInfo = tztZFNetCosign.getCertInfoWithUserID(loginReq.userID);
            if (cosignUser == null || !cosignUser.isValid()) {
                if (tztZFRecordProcessLogTools.isRecord()) {
                    String param1 = tztZFRecordProcessLogTools.stringFormat("message=%s", "查询客户号错误");
                    String param2 = tztZFRecordProcessLogTools.stringFormat("cosignUser=%s", cosignUser == null ? "" : cosignUser.string());
                    ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1, param2);
                    tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_JY.value()), tztZFRecordEventType.tztZFRecordEventType_TradeLogin,
                            tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
                }
                responseGMStep(tztZFGMStepQueryCert, tztZFCosignErrPin, "查询客户号错误", null);
                return false;
            }
            String pinLable = tztZFErrorMessageDefine.getUserDefineError(tztZFErrorMessageDefine.tztZFNetWorkErrorKey_PinLable, tztZFErrorMessageDefine.tztZFPinLable);
            if (!cosignUser.haveCosignCert() && certificateInfo == null) {
                if (tztZFRecordProcessLogTools.isRecord()) {
                    String param1 = tztZFRecordProcessLogTools.stringFormat("message=%s", "无证书申请证书");
                    String param2 = tztZFRecordProcessLogTools.stringFormat("cosignUser=%s", cosignUser == null ? "" : cosignUser.string());/**/
                    ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1, param2);
                    tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_JY.value()), tztZFRecordEventType.tztZFRecordEventType_TradeLogin,
                            tztZFReocordLogType.tztZFReocordLogType_Debug, params, "");
                }
                int applyType = cosignUser.isCosigned() ? 0 : 1;
                return applyCert(applyType);
            } else if (cosignUser.getLoginCosignCert() == null) {
                String errorPin = String.format(Locale.getDefault(), "%s错误", pinLable);
                if (tztZFRecordProcessLogTools.isRecord()) {
                    String param1 = tztZFRecordProcessLogTools.stringFormat("message=%s", errorPin);
                    String param2 = tztZFRecordProcessLogTools.stringFormat("cosignUser=%s", cosignUser == null ? "" : cosignUser.string());
                    ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1, param2);
                    tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_JY.value()), tztZFRecordEventType.tztZFRecordEventType_TradeLogin,
                            tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
                }
                if (stop(tztZFGMStepQueryCert, tztZFCosignErrPin, errorPin, null)) {
                    return false;
                }
                responseGMStep(tztZFGMStepQueryCert, tztZFCosignErrPin, String.format(Locale.getDefault(), "%s错误,请确认%s", pinLable, pinLable), null);
                return false;
            } else if (!cosignUser.isValidity()) {
                int errorNo = tztZFCosignErrExpire;
                String errorMessage = String.format(Locale.getDefault(), "%s无效(过期)", pinLable);
                String strSN = "";
                if (cosignUser.getLoginCosignCert() != null) {
                    strSN = cosignUser.cosignCertSN();
                    if (cosignUser.getLoginCosignCert().getnHave() < 0) {
                        errorNo = tztZFCosignErrServerCertUnfind;
                        errorMessage = String.format(Locale.getDefault(), "%s无效(服务器端)", pinLable);
                    }
                }
                if (tztZFRecordProcessLogTools.isRecord()) {
                    String param1 = tztZFRecordProcessLogTools.stringFormat("message=%s", errorMessage);
                    String param2 = tztZFRecordProcessLogTools.stringFormat("cosignUser=%s", cosignUser == null ? "" : cosignUser.string());/**/
                    ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1, param2);
                    tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_JY.value()), tztZFRecordEventType.tztZFRecordEventType_TradeLogin,
                            tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
                }
                if (stop(tztZFGMStepQueryCert, errorNo, errorMessage, null))
                    return false;
                responseGMStep(tztZFGMStepQueryCert, errorNo, errorMessage, null);
                return false;
            }
            return didGMLogin();
        }

        /**
         * 重置pin码
         */
        private boolean resetPinCert() {
            if (doStopStep(tztZFGMStepResetPin, loginReq.userID))
                return false;

            tztZFNetCosign.resetCert(loginReq.userID, loginReq.userToken, getSendDataMap(), loginReq.getUserPin(), new tztZFNetCosignHead.tztZFNetCosignRespCallBack() {
                @Override
                public void onCallBack(tztZFNetCosignHead.tztZFNetCosignResp c_resp) {
                    if (c_resp != null && c_resp.isCosignDowngrade()) {
                        tztZFGMLogin.this.didGMDowngrade();
                        return;
                    }
                    if (c_resp != null && c_resp.getErrorNO() > 0 && !tztZFTextUtils.isEmpty(c_resp.getUserID()) && !tztZFTextUtils.isEmpty(c_resp.getUserToken())) {
                        tztZFGMLogin.this.queryUserCert();
                    } else {
                        tztZFGMLogin.this.stop(tztZFGMStepResetPin, c_resp != null ? c_resp.getErrorNO() : -1 * tztZFGMStepResetPin, c_resp != null ? c_resp.getErrorMessage() : "", c_resp == null ?
                                null : c_resp.getDictExtern());
                    }
                }
            });
            return true;
        }

        /**
         * 注销本地证书
         *
         * @param strSN  证书序列号
         * @param reason 注销原因
         */
        private boolean revokeCert(String strSN, String reason) {
            if (doStopStep(tztZFGMStepRevokeCert, loginReq.userID))
                return false;
            if (tztZFTextUtils.isEmpty(strSN))
                return false;
            if (tztZFTextUtils.isEmpty(loginReq.userToken))
                return queryUserID();

            tztZFNetCosign.revokeCert(strSN, reason, loginReq.userID, loginReq.userToken, new tztZFNetCosignHead.tztZFNetCosignRespCallBack() {
                @Override
                public void onCallBack(tztZFNetCosignHead.tztZFNetCosignResp c_resp) {
                    if (c_resp != null && c_resp.isCosignDowngrade()) {
                        tztZFGMLogin.this.didGMDowngrade();
                        return;
                    }

                    tztZFGMLogin.this.stop(tztZFGMStepRevokeCert,
                            c_resp != null ? c_resp.getErrorNO() : -1 * tztZFGMStepRevokeCert,
                            c_resp != null ? c_resp.getErrorMessage() : "",
                            c_resp == null ? null : c_resp.getDictExtern());
                }
            });

            //清除用户本地证书
            tztZFNetCosign.removeLocalCert(loginReq.userID);
            //重新查询证书
            return queryUserCert();
        }

        /**
         * 申请新证书
         *
         * @param applyType =1强制申请
         */
        private boolean applyCert(int applyType) {
            if (doStopStep(tztZFGMStepApplyCert, loginReq.userID))
                return false;
            tztZFNetCosign.gmManageApplyCert(getSendDataMap(), loginReq.userID, loginReq.userToken, loginReq.getUserPin(), applyType, loginReq.bUseCustomSafetyCode, new tztZFNetCosignHead.tztZFCosignResponse() {
                @Override
                public void callBack(String userID, tztZFNetCosignHead.tztZFCosignResp resp) {
                    if (resp != null && resp.isCosignDowngrade()) {
                        tztZFGMLogin.this.didGMDowngrade();
                        return;
                    }
                    if (resp != null && resp.getErrorNO() == tztZFCosignErrOK) {
                        m_pThreadPool.useThreadPool().execute(new Runnable() {
                            @Override
                            public void run() {
                                tztZFGMLogin.this.didGMLogin();
                            }
                        });
                    } else {
                        tztZFGMLogin.this.stop(tztZFGMStepApplyCert,
                                resp != null ? resp.getErrorNO() : -1 * tztZFGMStepApplyCert,
                                resp != null ? resp.getErrorMessage() : "",
                                resp == null ? null : resp.getDictExtern());
                    }
                }
            });
            return true;
        }

        /** 信息签名 */
        private boolean doLoginSign() {
            //非国密登录，不去做验签了
            if (loginReq != null && !loginReq.bGMLogin) {
                return true;
            }
            boolean bNeedSign = tztZFNetCosignEx.getExIntForKey(tztZFNetCosignEx.tztCosignEx_NoSignFlag) == 0;
            if (!bNeedSign || !tztZFNetCosign.getCosignStatus())
                return true;
            if (doStopStep(tztZFGMStepLoginSign, loginReq == null ? "" : loginReq.getUserID()))
                return false;
            // 验证用户身份处理，新增字段处理
            // 调用到这里说明已经证书处理完成了，使用用户证书进行签名处理
            // 并根据接口增加相关字段
            // certUserID = 客户号； DeviceId=设备id；tztGMSign=签名后的数据
            // tztGMSign=HEX(协同签名(sm3(account+password+accounttype+deviceId)))
            //登录回调
            //验证用户身份处理，新增字段处理
            //调用到这里说明已经证书处理完成了，使用用户证书进行签名处理
            //并根据接口增加相关字段
            //certUserID = 客户号； DeviceId=设备id；tztGMSign=签名后的数据
            //tztGMSign=HEX(协同签名(sm3(account+password+accounttype+deviceId)))
            if (loginReq == null || loginReq.reqObj == null) {
                responseGMStep(tztZFGMStepLoginSign, -1 * tztZFGMStepLoginSign, "参数错误，信息签名失败", null);
                return false;
            }
            tztZFDataMapBase<String, Object> sendMap = loginReq.reqObj.getSendValues();
            if (sendMap == null) {
                responseGMStep(tztZFGMStepLoginSign, -1 * tztZFGMStepLoginSign, "参数错误，信息签名失败", null);
                return false;
            }
            //已经设置过了。不在处理
            String account = sendMap.getStr("account");
            if (tztZFTextUtils.isEmpty(account))
                account = "";
            String password = sendMap.getStr("password");
            if (tztZFTextUtils.isEmpty(password))
                password = "";
            String accountType = sendMap.getStr("accountType");
            if (tztZFTextUtils.isEmpty(accountType))
                accountType = "";
            String deviceId = sendMap.getStr("deviceId");
            if (tztZFTextUtils.isEmpty(deviceId))
                deviceId = tztZFCosignUser.getDeviceID();
            if (tztZFTextUtils.isEmpty(deviceId))
                deviceId = "";

            String orgData = String.format(Locale.getDefault(), "%s%s%s%s", account, password, accountType, deviceId);
            //签名验证
            tztZFCosignSSL.tztZFResponse response = tztZFNetCosign.cosignECDSASIG(orgData.getBytes());
            String strSign = "";
            if (response != null && response.getRetData() != null && response.getRetData().length > 0) {
                byte[] signData = tztZFEncryptKit.hexEncrypt(response.getRetData(), response.getRetData().length);
                if (signData != null && signData.length > 0) {
                    strSign = new String(signData);
                    sendMap.setStr("tztGMSign", strSign);
                    sendMap.setStr("DeviceId", deviceId);
                    sendMap.setStr("certUserID", loginReq.getUserID());
                    loginReq.reqObj.setNativeInt("haveSetGMSign", 1);
                    return true;
                }
            }
            responseGMStep(tztZFGMStepLoginSign, -1 * tztZFGMStepLoginSign, response.getRetMessage(), response.getRetEx());
            return false;
        }

        /**
         * 登录
         */
        private boolean doLogin() {
            if (!doLoginSign()) {
                releaseGmLogin();
                return false;
            }
            if (doStopStep(tztZFGMStepLogin, loginReq.userID, "")) {
                releaseGmLogin();
                return false;
            }
            endConnectTimeout(false);
            if (!loginReq.isSend && !loginReq.isResp) {
                loginReq.isSend = tztZFNetWork.sendDataAction(tztZFLinkType_JY.value(), loginReq.reqObj, new tztZFNetWorkResponseCallBack() {
                    @Override
                    public void OnError(tztZFNetWorkReqObj c_reqObj, int c_reqObjTag, int c_recvType, int c_errorNo, String c_errorMsg) {
                        releaseGmLogin();
                        responseGMStep(tztZFGMStepLogin, c_errorNo, c_errorMsg, null);
                    }

                    @Override
                    public void OnAns(tztZFNetWorkReqObj c_reqObj, int c_reqObjTag, tztZFNetWorkRespObj c_respObj) {
                        releaseGmLogin();
                        if (respBlock != null) {
                            responseBlock(tztZFGMStepLogin, c_respObj.getStatusCode(), c_respObj);
                        }
                    }
                });
            }

            return loginReq.isSend;
        }

        /**
         * 是否停止执行
         *
         * @param step 步骤
         * @return true=停止执行
         */
        private boolean doStopStep(@tztZFGMStep int step, String user) {
            return doStopStep(step, user, "");
        }

        /**
         * 是否停止执行
         *
         * @param step    步骤
         * @param message 信息
         * @return true=停止执行
         */
        private boolean doStopStep(@tztZFGMStep int step, String user, String message) {
            if (gmBlock != null) {
                boolean bStop = false;
                tztZFDataMapBase<String, Object> data = new tztZFDataMapBase<>();
                data.setErrorNo(step);
                data.setErrorMessage(message);
                data.setStr("UserID", user);
                bStop = gmBlock.onCallBack(step, true, bStop, data);
                if (bStop) {
                    responseGMStep(step, -1 * step, message, null);
                    return true;
                }
            }
            return false;
        }

        private void didGMDowngrade() {
            tztZFNetCosign.didGMDowngrade();
            tztZFNetWork.setLinkTypeGMDowngrade(tztZFLinkType.tztZFLinkType_JY.value());
            if (doStopStep(tztZFGMStepGMDowngrade, loginReq.userID))
                return;
            doLogin();
        }

        private void doGMSSLStep(@tztZFSocketSSLStep int step, String certUser) {
            doStopStep(tztZFGMStepSSLStep, certUser, step + "");
            if (loginReq != null && loginReq.isResp)
                return;

            if (step == tztZFSocketSSLStepSucess) {
                doLogin();
                return;
            }
            if (step == tztZFSocketSSLStepDowngrade) {
                if (tztZFRecordProcessLogTools.isRecord()) {
                    ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(tztZFRecordProcessLogTools.stringFormat("certUser=%s;mesage=%s", certUser,
                            tztZFSocketSSLStep.tztZFSocketSSLStepTips.tips(tztZFGMStepGMDowngrade)));
                    tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_JY.value()), tztZFRecordEventType.tztZFRecordEventType_TradeLogin,
                            tztZFReocordLogType.tztZFReocordLogType_Debug, params, "");
                }
                doStopStep(tztZFGMStepGMDowngrade, certUser, tztZFSocketSSLStep.tztZFSocketSSLStepTips.tips(tztZFGMStepGMDowngrade));
                return;
            }
            if (certUser == null || certUser.length() < 1)
                return;
            if (!certUser.equals(loginReq.userID))
                return;
            if (step == tztZFSocketSSLStepUnClientCert) {
                bindUserID();
                return;
            }
            if (step == tztZFSocketSSLStepUnServerCert) {
                queryUserCert();
                return;
            }
            if (step == tztZFSocketSSLStepClose) {
                if (timeOut <= 0) { //没有超时，返回错误
                    if (tztZFRecordProcessLogTools.isRecord()) {
                        ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(tztZFRecordProcessLogTools.stringFormat("mesage=%s", "tztZFSocketSSLStepClose返回错误"));
                        tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_JY.value()), tztZFRecordEventType.tztZFRecordEventType_TradeLogin,
                                tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
                    }
                    responseGMStep(tztZFGMStepLoginTimeout, tztZFNetWorkError_Send, tztZFErrorMessageDefine.getUserDefineError(tztZFWriteError, tztZFWriteError), null);
                } else { //有超时，重新执行
                    if (tztZFRecordProcessLogTools.isRecord()) {
                        ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(tztZFRecordProcessLogTools.stringFormat("mesage=%s", "tztZFSocketSSLStepClose重发交易登录"));
                        tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_JY.value()), tztZFRecordEventType.tztZFRecordEventType_TradeLogin,
                                tztZFReocordLogType.tztZFReocordLogType_Debug, params, "");
                    }
                    startLogin();
                }
            }
        }

        private boolean didGMLogin() {
            if (loginReq == null || loginReq.isSend || loginReq.isResp)
                return true;
            tztZFNetWork.setGMBindUserID(tztZFLinkType_JY.value(), loginReq.userID, new tztZFGMSSLStepCallBack() {
                @Override
                public void onCallBack(int step, int code, String certUser) {
                    //通知上层
                    tztZFNotificationObj notificationObj = new tztZFNotificationObj();
                    notificationObj.setName(tztZFJYLoginGMProcessStep);
                    tztZFDataMapBase<String, Object> dataMapBase = new tztZFDataMapBase<>();
                    dataMapBase.setInt("step", step);
                    if (certUser == null)
                        certUser = "";
                    dataMapBase.setStr("userID", certUser);
                    notificationObj.setData(dataMapBase);
                    EventBus.getDefault().post(notificationObj);

                    if (step == tztZFSocketSSLStepDowngrade) {
                        tztZFNetCosign.didGMDowngrade();
                    }
                    doGMSSLStep(step, certUser);
                }
            });
            return doLogin();
        }
    }
    //endregion

    /** 单向地址列表 */
    private String dxAddress;
    /** 单向默认地址索引 */
    private int dxIndex = -1;
    /** 连接超时 */
    private int dxSignleConnTimeout;
    /** 双向(国密)地址列表 */
    private String sxAddress;
    /** 双向(国密)默认地址索引 */
    private int sxIndex = -1;
    /** 国密连接超时 */
    private int sxConnMaxTimeout = 12;
    /** 普通交易地址列表 */
    private String jyAddress;
    /** 普通交易默认地址索引 */
    private int jyIndex = -1;

    /** 单个连接超时 */
    private int jySignleConnTimeout;
    /** appkey */
    private String appKey;
    /** ca证书信息字典 */
    private tztZFDataMapBase<String, Object> caData;
    /** 证书ST */
    private String certPubST;
    /** 证书L */
    private String certPubL;
    /** Debug信息 */
    private tztZFDataMapBase<String, Object> debugEx;
    /** 状态步骤回调 */
    private tztZFGMManageCallBack callBack;
    /** 通讯连接回调 */
    private tztZFLinkCallBack linkCallBack;

    /** 国密登录连接状态通知 object:@{"step":(step)} step = tztZFSocketSSLStep */
    public static final String tztZFJYLoginGMProcessStep = "tztZFJYLogin_GMProcess_Step";
    /** 登录界面注册link是否完成(默认true) */
    private static boolean bFinishLoginRegistLink = true;
    /** 是否注册国密类型Link（connectype=9） */
    private static boolean bRegistGMLink = false;

    ///

    //region get/set

    public String getDxAddress() {
        return dxAddress;
    }

    public void setDxAddress(String dxAddress) {
        this.dxAddress = dxAddress;
    }

    public int getDxIndex() {
        return dxIndex;
    }

    public void setDxIndex(int dxIndex) {
        this.dxIndex = dxIndex;
    }

    public String getSxAddress() {
        return sxAddress;
    }

    public void setSxAddress(String sxAddress) {
        this.sxAddress = sxAddress;
    }

    public int getSxIndex() {
        return sxIndex;
    }

    public void setSxIndex(int sxIndex) {
        this.sxIndex = sxIndex;
    }

    public String getJyAddress() {
        return jyAddress;
    }

    public void setJyAddress(String jyAddress) {
        this.jyAddress = jyAddress;
    }

    public int getJyIndex() {
        return jyIndex;
    }

    public void setJyIndex(int jyIndex) {
        this.jyIndex = jyIndex;
    }

    public String getAppKey() {
        return appKey;
    }

    public void setAppKey(String appKey) {
        this.appKey = appKey;
    }

    public tztZFDataMapBase<String, Object> getCaData() {
        return caData;
    }

    public void setCaData(tztZFDataMapBase<String, Object> caData) {
        this.caData = caData;
    }

    public String getCertPubST() {
        return certPubST;
    }

    public void setCertPubST(String certPubST) {
        this.certPubST = certPubST;
    }

    public String getCertPubL() {
        return certPubL;
    }

    public void setCertPubL(String certPubL) {
        this.certPubL = certPubL;
    }

    public tztZFDataMapBase<String, Object> getDebugEx() {
        return debugEx;
    }

    public void setDebugEx(tztZFDataMapBase<String, Object> debugEx) {
        this.debugEx = debugEx;
    }

    public tztZFGMManageCallBack getCallBack() {
        return callBack;
    }

    public void setCallBack(tztZFGMManageCallBack callBack) {
        this.callBack = callBack;
    }

    public tztZFLinkCallBack getLinkCallBack() {
        return linkCallBack;
    }

    public void setLinkCallBack(tztZFLinkCallBack linkCallBack) {
        this.linkCallBack = linkCallBack;
    }

    public int getDxSignleConnTimeout() {
        return dxSignleConnTimeout;
    }

    public void setDxSignleConnTimeout(int dxSignleConnTimeout) {
        this.dxSignleConnTimeout = dxSignleConnTimeout;
    }

    public int getSxConnMaxTimeout() {
        return sxConnMaxTimeout;
    }

    public void setSxConnMaxTimeout(int sxConnMaxTimeout) {
        this.sxConnMaxTimeout = sxConnMaxTimeout;
    }

    public int getJySignleConnTimeout() {
        return jySignleConnTimeout;
    }

    public void setJySignleConnTimeout(int jySignleConnTimeout) {
        this.jySignleConnTimeout = jySignleConnTimeout;
    }
    //endregion

    private boolean isInValid() {
        if (tztZFTextUtils.isEmpty(dxAddress) || tztZFTextUtils.isEmpty(sxAddress) || tztZFTextUtils.isEmpty(jyAddress))
            return true;
        return tztZFTextUtils.isEmpty(appKey) || caData == null || caData.getCount() < 1;
    }

    /**
     * 设置登录界面是否完成注册link
     *
     * @param bFinishLoginRegistLink 登录界面是否完成注册link
     */
    public static void setFinishLoginRegistLink(boolean bFinishLoginRegistLink) {
        tztZFGMManage.bFinishLoginRegistLink = bFinishLoginRegistLink;
    }

    /**
     * 是否注册国密类型通道
     *
     * @return 是否注册国密类型通道
     */
    public static boolean isRegistGMLink() {
        return bRegistGMLink;
    }

    /**
     * 设置是否注册国密类型通道
     *
     * @param bRegistGMLink 是否注册国密类型通道
     */
    public static void setIsRegistGMLink(boolean bRegistGMLink) {
        tztZFGMManage.bRegistGMLink = bRegistGMLink;
    }

    /**
     * 初始化国密
     *
     * @param context         context
     * @param gmObj           国密对象
     * @param reqCosignStatus 是否请求国密状态
     */
    public static void GMInit(final Object context, final tztZFGMManage gmObj, boolean reqCosignStatus) {
        if (gmObj == null)
            return;
        if (gmObj.isInValid()) {
            if (gmObj.callBack != null) {
                tztZFDataMapBase<String, Object> data = new tztZFDataMapBase<>();
                data.setErrorNo(-1);
                data.setErrorMessage("参数错误");
                boolean stop = true;
                gmObj.callBack.onCallBack(tztZFGMStepGMStatus, false, stop, data);
            }
            return;
        }
        tztZFNetCosign.init(context);
        tztZFNetCosign.initWithAddlist(gmObj.dxAddress)
                .withAppKey(gmObj.appKey, gmObj.caData);
        tztZFNetCosign.setCosignRespCallBack(new tztZFNetCosign.tztCosignRespCallBack() {
            @Override
            public void onCallBack(String userID, tztZFNetCosignHead.tztZFCosignResp resp) {
                if (gmObj.callBack != null) {
                    /// 签名回调处理
                    int nError = 0;
                    String strErr = "";
                    if (resp != null) {
                        nError = resp.getErrorNO();
                        strErr = resp.getErrorMessage();
                    }
                    tztZFDataMapBase<String, Object> data = new tztZFDataMapBase<>();
                    if (resp != null) {
                        data.setDataMap("ExternDict", resp.getDictExtern());
                    }
                    data.setErrorNo(nError);
                    data.setErrorMessage(strErr);
                    data.setStr("UserID", userID);
                    boolean isDowngrade = false;
                    if (resp != null && resp.isCosignDowngrade()) {
                        data.setStr("GMStatus", "交易降级，已断开连接");
                        isDowngrade = true;
                    }
                    boolean stop = true;
                    //降级重新创建交易通道
                    if (isDowngrade) {
                        tztZFNetWork.setLinkTypeGMDowngrade(tztZFLinkType_JY.value());
                    }
                    gmObj.callBack.onCallBack(isDowngrade ? tztZFGMStepGMDowngrade : tztZFGMStepLogin, !isDowngrade && nError >= 0, stop, data);
                }
            }
        });

        tztZFNetWork.registLink(context, tztZFLinkType.tztZFLinkType_JY.value(), 0);
        tztZFNetWork.setGMAddressList(tztZFLinkType.tztZFLinkType_JY.value(), gmObj.sxAddress, "", gmObj.sxIndex);
        tztZFNetWork.setAddressList(tztZFLinkType.tztZFLinkType_JY.value(), gmObj.jyAddress, gmObj.jyIndex);
        tztZFNetWork.setLinkBlock(tztZFLinkType.tztZFLinkType_JY.value(), gmObj.linkCallBack);
        tztZFNetWork.setConnectTimeOut(tztZFLinkType_JY.value(), gmObj.jySignleConnTimeout);
        tztZFNetWork.setGMConnectTimeOut(tztZFLinkType_JY.value(), gmObj.sxConnMaxTimeout);
        tztZFNetWork.setConnectTimeOut(tztZFLinkType_Cosign.value(), gmObj.dxSignleConnTimeout);

        if (reqCosignStatus) {
            tztZFNetCosign.reqCosignExInfo(gmObj.debugEx, new tztZFNetCosignCallBack() {
                @Override
                public void netCosignCallBack(boolean c_bOK, tztZFDataMapBase<String, Object> c_cosign) {
                    tztZFGMManage.setIsRegistGMLink(c_bOK);
                    if (gmObj.callBack != null) {
                        boolean stop = true;
                        gmObj.callBack.onCallBack(tztZFGMStepGMStatus, c_bOK, stop, c_cosign);
                    }
                }
            });
        }
    }

    /**
     * 初始化国密
     *
     * @param context context
     * @param gmObj   国密对象
     */
    public static void GMInit(final Object context, final tztZFGMManage gmObj) {
        GMInit(context, gmObj, true);
    }

    /**
     * 设置地址
     *
     * @param dxAddress 单向地址列表
     * @param dxIndex   单向默认地址索引
     * @param sxAddress 国密地址列表
     * @param sxIndex   国密默认地址索引
     * @param jyAddress 普通交易地址列表
     * @param jyIndex   普通交易默认地址索引
     */
    public static void setAddressList(String dxAddress, int dxIndex, String sxAddress, int sxIndex, String jyAddress, int jyIndex) {
        if (!tztZFTextUtils.isEmpty(dxAddress)) {
            tztZFNetWork.setAddressList(tztZFLinkType_Cosign.value(), dxAddress, dxIndex);
        }
        if (!tztZFTextUtils.isEmpty(jyAddress)) {
            tztZFNetWork.setAddressList(tztZFLinkType.tztZFLinkType_JY.value(), jyAddress, jyIndex);
        }
        if (!tztZFTextUtils.isEmpty(sxAddress)) {
            tztZFNetWork.setGMAddressList(tztZFLinkType.tztZFLinkType_JY.value(), sxAddress, "", sxIndex);
        }
    }

    public static tztZFGMLogin GMLogin(Object context, tztZFGMLoginReqObj loginReq, tztZFGMManageCallBack gmBlock, tztZFGMLoginResponseCallBack responseCallBack) {
        tztZFGMLogin login = new tztZFGMLogin(context);
        login.loginReq = loginReq;
        login.gmBlock = gmBlock;
        login.respBlock = responseCallBack;
        login.startLogin();
        return login;
    }

    /**
     * 国密登录
     *
     * @param context          context
     * @param reqObj           登录请求对象
     * @param pin              pin码
     * @param gmBlock          国密状态回调
     * @param responseCallBack 登录请求应答回调
     * @return 登录对象
     */
    public static tztZFGMLogin gmLogin(Object context,
                                       tztZFNetWorkReqObj reqObj,
                                       String pin,
                                       tztZFGMManageCallBack gmBlock,
                                       tztZFGMLoginResponseCallBack responseCallBack) {
        return gmLogin(context, reqObj, pin, gmBlock, responseCallBack, true);
    }

    /**
     * 国密登录
     *
     * @param context          context
     * @param reqObj           登录请求对象
     * @param pin              pin码
     * @param gmBlock          国密状态回调
     * @param responseCallBack 登录请求应答回调
     * @param checkKHBH        是否检测KHBH
     * @return 登录对象
     */

    public static tztZFGMLogin gmLogin(Object context,
                                       tztZFNetWorkReqObj reqObj,
                                       String pin,
                                       tztZFGMManageCallBack gmBlock,
                                       tztZFGMLoginResponseCallBack responseCallBack, boolean checkKHBH) {
        return gmLogin(context, reqObj, pin, checkKHBH, true, gmBlock, responseCallBack);
    }

    /**
     * 国密登录
     *
     * @param context          context
     * @param reqObj           登录请求对象
     * @param pin              pin码
     * @param checkKHBH        是否检测KHBH
     * @param needGMLogin      是否需要国密登录
     * @param gmBlock          国密状态回调
     * @param responseCallBack 登录请求应答回调
     * @return 登录对象
     */
    public static tztZFGMLogin gmLogin(Object context,
                                       tztZFNetWorkReqObj reqObj,
                                       String pin,
                                       boolean checkKHBH,
                                       boolean needGMLogin,
                                       tztZFGMManageCallBack gmBlock,
                                       tztZFGMLoginResponseCallBack responseCallBack) {
        return gmLogin(context, reqObj, pin, false, checkKHBH, needGMLogin, gmBlock, responseCallBack);
    }

    /**
     * 国密登录
     *
     * @param context          context
     * @param reqObj           登录请求对象
     * @param pin              pin码
     * @param checkKHBH        是否检测KHBH
     * @param needGMLogin      是否需要国密登录
     * @param gmBlock          国密状态回调
     * @param responseCallBack 登录请求应答回调
     * @return 登录对象
     */
    public static tztZFGMLogin gmLogin(Object context,
                                       tztZFNetWorkReqObj reqObj,
                                       String pin,
                                       boolean bPinEnCrypt,
                                       boolean checkKHBH,
                                       boolean needGMLogin,
                                       tztZFGMManageCallBack gmBlock,
                                       tztZFGMLoginResponseCallBack responseCallBack) {
        return gmLogin(context, reqObj, pin, bPinEnCrypt, checkKHBH, needGMLogin, true, gmBlock, responseCallBack);
    }

    /**
     * 国密登录
     *
     * @param context            context
     * @param reqObj             登录请求对象
     * @param pin                pin码
     * @param checkKHBH          是否检测KHBH
     * @param needGMLogin        是否需要国密登录
     * @param bUseCustomSafeCode 是否使用自定义安全码
     * @param gmBlock            国密状态回调
     * @param responseCallBack   登录请求应答回调
     * @return 登录对象
     */
    public static tztZFGMLogin gmLogin(Object context,
                                       tztZFNetWorkReqObj reqObj,
                                       String pin,
                                       boolean bPinEnCrypt,
                                       boolean checkKHBH,
                                       boolean needGMLogin,
                                       boolean bUseCustomSafeCode,
                                       tztZFGMManageCallBack gmBlock,
                                       tztZFGMLoginResponseCallBack responseCallBack) {
        if (!tztZFTextUtils.isEmpty(pin) && bPinEnCrypt && tztZFNetWork.getEncryptCallBack() != null) {
            byte[] decryptPin = tztZFNetWork.getEncryptCallBack().decrypt(pin.getBytes());
            if (decryptPin != null && decryptPin.length > 0) {
                pin = new String(decryptPin);
            }
        }

        tztZFGMLoginReqObj loginReq = new tztZFGMLoginReqObj();
        loginReq.reqObj = reqObj;
        loginReq.userPin = pin;
        loginReq.bGMLogin = needGMLogin;
        loginReq.checkKHBH = checkKHBH;
        loginReq.bUseCustomSafetyCode = bUseCustomSafeCode;

        return GMLogin(context, loginReq, gmBlock, responseCallBack);
    }

    private static void releaseGmLogin() {
        //应答成功后，清空当前回调
        tztZFNetWork.clearGMSSLStepCallback(tztZFLinkType_JY.value());
    }

    /**
     * 保存账号信息
     *
     * @param context     Context上下文
     * @param account     账号
     * @param accountType 账号类型
     * @param userID      客户号
     */
    public static void saveAccount(Object context, String account, String accountType, String userID) {
        if (context == null || tztZFTextUtils.isEmpty(account) || tztZFTextUtils.isEmpty(accountType) || tztZFTextUtils.isEmpty(userID)) {
            return;
        }
        tztZFCosignAccount cosignAccount = new tztZFCosignAccount(context);
        cosignAccount.setAccount(account);
        cosignAccount.setAccountType(accountType);
        saveAccount(cosignAccount, userID);
    }

    /**
     * 保存账号信息
     *
     * @param account 账号信息
     * @param userID  客户号
     */
    public static void saveAccount(tztZFCosignAccount account, String userID) {
        tztZFCosignAccount.saveAccount(account, userID);
    }

    /**
     * 根据账号信息获取客户号
     *
     * @param context     Context上下文
     * @param account     账号
     * @param accountType 账号类型
     * @param checkKHBH   是否检测客户号
     * @return 客户号
     */
    public static String getUserID(Object context, String account, String accountType, boolean checkKHBH) {
        if (context == null || tztZFTextUtils.isEmpty(account) || tztZFTextUtils.isEmpty(accountType)) {
            return "";
        }
        tztZFCosignAccount cosignAccount = new tztZFCosignAccount(context);
        cosignAccount.setAccount(account);
        cosignAccount.setAccountType(accountType);
        return tztZFCosignAccount.loadUserID(cosignAccount, checkKHBH);
    }

    public static String getNormalSafetyCode(String userID) {
        if (userID == null || userID.length() < 1) {
            return null;
        }
        //生成随机数
        byte[] randomData = tztZFDataEncrypt.makeRandom(6);
        String random = new String(randomData);
        String safatyCode = userID + tztZFCosignCert.getDeviceID() + random;
        byte[] data = safatyCode.getBytes();
        byte[] userIDData = userID.getBytes();
        byte[] key = tztZFNetCosignEx.SM3(userIDData);
        if (key.length < 32) {
            return null;
        }
        //先SM4
        byte[] sm4 = tztZFEncryptKit.sm4Encrypt(data, data.length, key, key.length);
        //再做一次SM3
        byte[] encryptData = tztZFNetCosignEx.SM3(sm4);
        if (encryptData != null) {
            return new String(tztZFEncryptKit.hexEncrypt(encryptData, encryptData.length));
        }
        return "";
    }

    /**
     * 国密流程执行回调
     */
    public interface tztZFGMManageCallBack {
        /**
         * 国密流程执行回调
         *
         * @param c_step 步骤
         * @param c_bOK  成功or失败
         * @param stop   是否终止（true=终止，false=继续）
         * @param exInfo 扩展信息
         * @return true=中止，false=继续执行后续步骤
         */
        boolean onCallBack(@tztZFGMStep int c_step, boolean c_bOK, boolean stop, tztZFDataMapBase<String, Object> exInfo);
    }

    /**
     * 国密登录结果应答回调
     */
    public interface tztZFGMLoginResponseCallBack {
        /**
         * 国密登录应答
         *
         * @param c_step 步骤
         * @param c_bOK  成功or失败
         * @param c_req  国密登录对象
         * @param c_resp 应答数据对象
         */
        void onCallBack(@tztZFGMStep int c_step, boolean c_bOK, tztZFGMLoginReqObj c_req, tztZFNetWorkRespObj c_resp);
    }
}
