package com.tztzf.network.push;

import com.tztzf.baseos.tztZFTextUtils;
import com.tztzf.commbase.tztZFDataMapBase;
import com.tztzf.network.callback.tztZFNetWorkResponseCallBack;
import com.tztzf.network.define.tztZFLinkType;
import com.tztzf.network.tztZFNetWork;
import com.tztzf.network.tztZFNetWorkReqObj;
import com.tztzf.network.tztZFSDKLink;
import com.tztzf.tztZFPrivate;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Locale;

/**
 * 行情主推类
 */
public class tztZFHQPush {
    private int group = 0;
    private tztZFLinkType linkType = tztZFLinkType.tztZFLinkType_HQ;

    @tztZFPrivate
    private ArrayList<String> ayCodeInfo;
    @tztZFPrivate
    private tztZFNetWorkReqObj reqObj;

    // <editor-fold defaultstate="collapsed" desc="get/set">

    /**
     * 获取订阅组ID
     */
    public int getGroup() {
        return group;
    }

    /**
     * 设置订阅组ID
     *
     * @param group 组ID
     */
    public void setGroup(int group) {
        this.group = group;
    }

    /**
     * 获取连接类型
     *
     * @return 连接类型
     */
    public tztZFLinkType getLinkType() {
        return linkType;
    }

    /**
     * 设置连接类型
     *
     * @param linkType 连接类型
     */
    public void setLinkType(tztZFLinkType linkType) {
        this.linkType = linkType;
    }

    // </editor-fold>

    /**
     * 构造函数
     */
    public tztZFHQPush() {
        reqObj = new tztZFNetWorkReqObj();
        ayCodeInfo = new ArrayList<>();
        group = 0;
        linkType = tztZFLinkType.tztZFLinkType_HQ;
    }

    /**
     * 设置主推请求代码 （重新设置，会清除原来的）
     *
     * @param code 代码
     * @param type 市场
     */
    public void setStockCode(String code, String type) {
        if (tztZFTextUtils.isEmpty(code))
            return;
        if (tztZFTextUtils.isEmpty(type))
            type = "0";
        ayCodeInfo.clear();
        addCodeInfo(code, type);
    }

    /**
     * 设置代码列表
     *
     * @param list 代码列表，格式：code|type,code|type,...
     */
    public void setStockList(List<String> list) {
        if (list == null || list.size() < 1)
            return;
        ayCodeInfo.clear();
        for (String codeInfo : list) {
            if (!tztZFTextUtils.isEmpty(codeInfo)) {
                ayCodeInfo.add(codeInfo);
            }
        }
    }

    /**
     * 添加订阅代码
     * @param code 代码
     * @param codeType 代码市场
     */
    public void addCodeInfo(String code, String codeType) {
        if (tztZFTextUtils.isEmpty(code) || tztZFTextUtils.isEmpty(codeType))
            return;
        String codeInfo = String.format(Locale.CHINA, "%s|%s", code, codeType);
        if (!ayCodeInfo.contains(codeInfo)) {
            ayCodeInfo.add(codeInfo);
        }
    }

    /**
     * 移除订阅代码
     * @param code 代码
     * @param codeType 代码市场
     */
    public void removeCodeInfo(String code, String codeType) {
        if (tztZFTextUtils.isEmpty(code) || tztZFTextUtils.isEmpty(codeType))
            return;
        String codeInfo = String.format(Locale.CHINA, "%s|%s", code, codeType);
        ayCodeInfo.remove(codeInfo);
    }

    /**
     * 统一处理key
     * @param grid 分组 [1-5]
     */
    private static String getGridKey(int grid) {
        if (grid < 1 || grid > 5)
            return null;
        return String.format(Locale.CHINA, "Grid%d", grid);
    }

    /**
     * 获取指定分组的内容字典，若不存在，进行创建
     * @param grid 分组[1-5]
     */
    private tztZFDataMapBase<String, Object> getGridMap(int grid) {
        String strGrid = getGridKey(grid);
        if (tztZFTextUtils.isEmpty(strGrid))
            return null;
        tztZFDataMapBase<String, Object> sendMap = reqObj.getSendValues();
        if (sendMap == null)
            return null;
        tztZFDataMapBase<String, Object> gridMap = sendMap.getDataMap(strGrid);
        if (gridMap == null) {
            gridMap = new tztZFDataMapBase<>();
            sendMap.setDataMap(strGrid, gridMap);
        }
        return gridMap;
    }

    /**
     * 设置GridData 取值范围[1-5], 共5个分类
     * @param reqData 订阅信息
     * @return 成功=true
     */
    public boolean setGridData(tztZFDataMapBase<String, Object> reqData) {
        boolean bSet = false;
        for (int i = 1; i <= 5; i++) {
            if (setGrid(i, reqData)) {
                bSet = true;
            }
        }
        return bSet;
    }

    private boolean setGrid(int grid, tztZFDataMapBase<String, Object> reqData) {
        tztZFDataMapBase<String, Object> sendMap = reqObj.getSendValues();
        if (sendMap == null)
            return false;

        String strGrid = getGridKey(grid);
        if (tztZFTextUtils.isEmpty(strGrid))
            return false;
        //移除原来的主推请求
        sendMap.removeByKey(strGrid);
        if (reqData == null || reqData.getCount() < 1)
            return false;
        tztZFDataMapBase<String, Object> gridMap = reqData.getDataMap(strGrid);
        if (gridMap == null || gridMap.getCount() < 1
                || gridMap.getAction() < 1)
            return false;
        sendMap.setObj(strGrid, gridMap);
        return true;
    }

    /**
     * 添加订阅请求的字段
     * @param grid 对应分组
     * @param key 请求的key
     * @param value 请求的key对应内容
     * @return 成功=true
     */
    public boolean addGrid(int grid, String key, String value) {
        tztZFDataMapBase<String, Object> gridMap = getGridMap(grid);
        if (gridMap != null && !tztZFTextUtils.isEmpty(key)) {
            gridMap.setStr(key, value);
            return true;
        }
        return false;
    }

    /**
     * 移除订阅请求字段
     * @param grid 分组
     * @param key 要移除的key
     */
    public void removeGrid(int grid, String key) {
        tztZFDataMapBase<String, Object> gridMap = getGridMap(grid);
        if (gridMap != null && !tztZFTextUtils.isEmpty(key)) {
            gridMap.removeByKey(key);
        }
    }

    /**
     * 移除整个对应的请求grid
     * @param grid 分组
     */
    public void removeGrid(int grid) {
        String strGrid = getGridKey(grid);
        if (!tztZFTextUtils.isEmpty(strGrid)) {
            tztZFDataMapBase<String, Object> sendMap = reqObj.getSendValues();
            if (sendMap == null)
                return;
            sendMap.removeByKey(strGrid);
        }
    }

    /**
     * 设置主推请求参数
     * @param key 字段
     * @param value 内容
     */
    public void setStr(String key, String value) {
        reqObj.setStr(key, value);
    }

    /**
     * 订阅主推
     * @param callBack 应答回调，主推过来的数据也在改回调处理
     * @return true=订阅成功
     */
    public boolean hqPushSub(tztZFNetWorkResponseCallBack callBack) {
        return hqPushSub(callBack, null);
    }

    public boolean hqPushSub(tztZFNetWorkResponseCallBack callBack, tztZFSDKLink sdkLink) {
        if (group < 0 || group > 4)
            return false;
        if (ayCodeInfo == null || ayCodeInfo.size() < 1)
            return false;
        if (reqObj == null)
            reqObj = new tztZFNetWorkReqObj();

        reqObj.setStr("Action", "20253");
        reqObj.setInt("Group", group);

        String codeList = "";
        for (int i = 0; i < ayCodeInfo.size(); i++) {
            codeList += ayCodeInfo.get(i);
            if (i != ayCodeInfo.size() - 1)
                codeList += ",";
        }
        reqObj.setStr("Grid", codeList);
        if (sdkLink != null) {
            sdkLink.requestLinkType(linkType.value(), reqObj, callBack);
        } else {
            tztZFNetWork.sendDataAction(linkType.value(), reqObj, callBack);
        }
        return true;
    }

    /**
     * 取消订阅
     *
     * @param callBack 应答回调
     * @return true=成功
     */
    public boolean hqPushCancel(tztZFNetWorkResponseCallBack callBack) {
        return hqPushCancel(group, linkType, callBack, null);
    }

    public boolean hqPushCancel(tztZFNetWorkResponseCallBack callBack, tztZFSDKLink sdkLink) {
        return hqPushCancel(group, linkType, callBack, sdkLink);
    }

    private static tztZFHQPush initWithDataMap(tztZFDataMapBase<String, Object> dataMap) {
        int group = 0;
        int sessionType = tztZFLinkType.tztZFLinkType_HQ.value();
        if (dataMap != null) {
            group = dataMap.getInt("Group", group);
            sessionType = dataMap.getInt("SessionType", sessionType);
        }

        tztZFHQPush hqPush = new tztZFHQPush();
        hqPush.setGroup(group);
        hqPush.setLinkType(tztZFLinkType.valueOf(sessionType));
        return hqPush;
    }

    /**
     * 主推订阅
     * @param dataMap 订阅的主推请求数据
     * @param callBack 回调
     * @return 是否订阅
     */
    public static boolean hqPushSub(tztZFDataMapBase<String, Object> dataMap, tztZFNetWorkResponseCallBack callBack) {
        return hqPushSub(dataMap, callBack, null);
    }

    public static boolean hqPushSub(tztZFDataMapBase<String, Object> dataMap, tztZFNetWorkResponseCallBack callBack, tztZFSDKLink sdkLink) {
        if (dataMap == null)
            return false;

        tztZFHQPush hqPush = initWithDataMap(dataMap);

        tztZFDataMapBase<String, Object> pushReq = dataMap.getDataMap("pushReq");
        boolean bSet = hqPush.setGridData(pushReq);
        if (!bSet) {
            bSet = hqPush.setGridData(dataMap);
        }

        String strStockList = dataMap.getStr("StockList");
        if (!tztZFTextUtils.isEmpty(strStockList)) {
            String[] ayList = strStockList.split(",");
            int nList = ayList.length;
            hqPush.setStockList(Arrays.asList(ayList));
            if (!bSet) {
//                hqPush.addGrid(1, "Action", "60");
//                hqPush.addGrid(1, "grid", strStockList);
//                hqPush.addGrid(1, "NewMarketNo", "0");
//                hqPush.addGrid(1, "MaxCount", nList+"");
//                hqPush.addGrid(1, "StartPos", "0");
//                hqPush.addGrid(1, "Lead", "1");
//                hqPush.addGrid(1, "StockIndex", "1");
//                hqPush.addGrid(1, "AccountIndex", "9");
            }
        } else {
            String strCode = dataMap.getStr("StockCode");
            if (tztZFTextUtils.isEmpty(strCode))
                return false;
            String strType = dataMap.getStr("marketSN");
            if (tztZFTextUtils.isEmpty(strType))
                strType = "0";
            hqPush.setStockCode(strCode, strType);
            if (!bSet) {
                // hqPush.addGrid(1, "Action", "20109");
//                hqPush.addGrid(1, "StockCode", strCode);
//                hqPush.addGrid(1, "NewMarketNo", strType);
            }
        }
        if (sdkLink != null) {
            return hqPush.hqPushSub(callBack, sdkLink);
        } else {
            return hqPush.hqPushSub(callBack);
        }
    }

    /**
     * 取消主推订阅
     * @param group 分组编号，【0-4】
     * @param linkType 连接类型
     * @param callBack 回调
     * @return 是否取消
     */
    public static boolean hqPushCancel(int group, tztZFLinkType linkType, tztZFNetWorkResponseCallBack callBack) {
        return hqPushCancel(group, linkType, callBack, null);
    }

    public static boolean hqPushCancel(int group, tztZFLinkType linkType, tztZFNetWorkResponseCallBack callBack, tztZFSDKLink sdkLink) {
        if (group < 0 || group > 4)
            return false;

        tztZFNetWorkReqObj reqModel = new tztZFNetWorkReqObj();
        reqModel.setStr("Action", "20253");
        reqModel.setInt("Group", group);
        if (sdkLink != null) {
            sdkLink.requestLinkType(linkType.value(), reqModel, callBack);
        } else {
            tztZFNetWork.sendDataAction(linkType.value(), reqModel, callBack);
        }
        return true;
    }

    /**
     * 取消所有的主推订阅
     * @param linkType 连接类型
     * @param callBack 回调
     */
    public static void hqPushClear(tztZFLinkType linkType, tztZFNetWorkResponseCallBack callBack) {
        hqPushClear(linkType, callBack, null);
    }

    public static void hqPushClear(tztZFLinkType linkType, tztZFNetWorkResponseCallBack callBack, tztZFSDKLink sdkLink) {
        tztZFNetWorkReqObj reqModel = new tztZFNetWorkReqObj();
        reqModel.setStr("Action", "20253");
        tztZFNetWork.sendDataAction(linkType.value(), reqModel, callBack);
    }
}
