package com.tztzf.network.push;

import com.tztzf.baseos.tztZFTextUtils;
import com.tztzf.commbase.tztZFDataMapBase;
import com.tztzf.network.callback.tztZFNetWorkResponseCallBack;
import com.tztzf.network.define.tztZFLinkType;
import com.tztzf.network.tztZFNetWork;
import com.tztzf.network.tztZFNetWorkReqObj;

/**
 * 主推
 */
public class tztZFServicePush {

    private String reqKey = "";
    private tztZFLinkType linktype = tztZFLinkType.tztZFLinkType_ZX;

    private tztZFDataMapBase<String, Object> serviceMap;
    private String serviceCode = "";

    // <editor-fold defaultstate="collapsed" desc="get/set">

    /**
     * 获取请求key
     * @return 请求key
     */
    public String getReqKey() {
        return reqKey;
    }

    /**
     * 设置请求key
     * @param reqKey 请求key
     */
    public void setReqKey(String reqKey) {
        this.reqKey = reqKey;
    }

    /**
     * 获取通道类型
     * @return 通道类型
     */
    public tztZFLinkType getSessionType() {
        return linktype;
    }

    /**
     * 设置通道类型
     * @param linktype 通道类型
     */
    public void setSessionType(tztZFLinkType linktype) {
        this.linktype = linktype;
    }

    /**
     * 获取主推订阅信息
     * @return 主推订阅信息
     */
    public tztZFDataMapBase<String, Object> getServiceMap() {
        return serviceMap;
    }

    /**
     * 设置主推订阅信息
     * @param serviceMap 主推订阅信息
     */
    public void setServiceMap(tztZFDataMapBase<String, Object> serviceMap) {
        this.serviceMap = serviceMap;
    }

    /**
     * 获取主推代码
     * @return 主推代码
     */
    public String getServiceCode() {
        return serviceCode;
    }

    /**
     * 设置主推代码
     * @param serviceCode 主推代码
     */
    public void setServiceCode(String serviceCode) {
        this.serviceCode = serviceCode;
    }

    // </editor-fold>

    /**
     * 构造函数
     */
    public tztZFServicePush() {
        super();
        serviceMap = new tztZFDataMapBase<>();
    }

    /**
     * 构造函数
     * @param serviceCode 主推代码
     */
    public tztZFServicePush(String serviceCode) {
        super();
        serviceMap = new tztZFDataMapBase<>();
        this.serviceCode = serviceCode;
    }

    /**
     * 初始化主推服务
     * @param serviceCode 主推代码
     * @return 主推服务
     */
    public static tztZFServicePush initWithServiceCode(String serviceCode) {
        tztZFServicePush servicePush = new tztZFServicePush();
        servicePush.setServiceMap(new tztZFDataMapBase<String, Object>());
        servicePush.setReqKey("");
        servicePush.setSessionType(tztZFLinkType.tztZFLinkType_ZX);
        servicePush.setServiceCode(serviceCode);
        return servicePush;
    }

    static tztZFServicePush initWithDataMap(tztZFDataMapBase<String, Object> dataMap) {
        if (dataMap == null)
            return null;
        String serviceCode = dataMap.getStr("ServiceCode");
        if (tztZFTextUtils.isEmpty(serviceCode))
            return null;

        int sessionType = dataMap.getInt("SessionType");
        tztZFServicePush codePush = tztZFServicePush.initWithServiceCode(serviceCode);
        codePush.setSessionType(tztZFLinkType.valueOf(sessionType));

        String strReqKey = dataMap.getStr("ReqKey");
        if (!tztZFTextUtils.isEmpty(strReqKey))
            codePush.setReqKey(strReqKey);
        return codePush;
    }


    // <editor-fold defaultstate="collapsed" desc="公开接口">

    /**
     * 设置推送代码
     * @param serviceCode 推送代码
     * @return 是否设置成功
     */
    public boolean onSetServiceCode(String serviceCode) {
        if (serviceCode != null && serviceCode.length() > 0) {
            this.serviceCode = serviceCode;
            return true;
        }
        return false;
    }

    void setObj(String key, Object value) {
        if (tztZFTextUtils.isEmpty(key))
            return;
        if (value == null) {
            serviceMap.removeByKey(key);
        } else {
            serviceMap.setObj(key, value);
        }
    }

    boolean setGridData(tztZFDataMapBase<String, Object> mapBase) {
        boolean bSet = false;
        for (int i = 1; i <= 5; i++) {
            if (setGrid(i, mapBase))
                bSet = true;
        }
        return bSet;
    }

    /**
     * 添加订阅请求字段
     */
    boolean setGrid(int grid, tztZFDataMapBase<String, Object> mapBase) {
        if (serviceMap == null)
            return false;

        String strGrid = getGridKey(grid);
        if (strGrid == null || strGrid.length() < 1)
            return false;

        //移除原来的主推请求
        serviceMap.removeByKey(strGrid);
        if (mapBase == null || mapBase.getCount() < 1)
            return false;

        tztZFDataMapBase<String, Object> gridMap = mapBase.getDataMap(strGrid);
        if (gridMap == null || gridMap.getCount() < 1 || gridMap.getInt("Action") < 1)
            return false;

        serviceMap.setDataMap(strGrid, gridMap);
        return true;
    }

    //grid  取值范围[1-5]，共5个分类
    //添加grid订阅请求字段
    boolean addGrid(int grid, String value, String key) {
        if (grid < 1 || grid > 5
                || key == null || key.length() < 1
                || value == null || value.length() < 1)
            return false;

        String strGrid = getGridKey(grid);
        tztZFDataMapBase<String, Object> mapBase = serviceMap.getDataMap(strGrid);
        if (mapBase == null) {
            mapBase = new tztZFDataMapBase<>();
            serviceMap.setDataMap(strGrid, mapBase);
        }

        mapBase.setStr(key, value);
        return true;
    }

    //移除grid订阅请求字段
    void removeGrid(int grid, String key) {
        if (grid < 1 || grid > 5
                || key == null || key.length() < 1)
            return;

        if (serviceMap == null)
            return;

        String strGrid = getGridKey(grid);
        tztZFDataMapBase<String, Object> mapBase = serviceMap.getDataMap(strGrid);
        if (mapBase == null)
            return;

        mapBase.removeByKey(key);
    }

    //移除grid订阅
    void removeGrid(int grid) {
        if (grid < 1 || grid == 5)
            return;
        if (serviceMap == null)
            return;
        String strGrid = getGridKey(grid);
        serviceMap.removeByKey(strGrid);
    }

    boolean codePushSub(int optype, tztZFNetWorkResponseCallBack callBack) {
        String strCode = getCodePushCode();
        if (strCode == null || strCode.length() < 1)
            return false;
        String strReqKey = getCodePushReqKey();
        tztZFNetWorkReqObj reqModal = new tztZFNetWorkReqObj();
        reqModal.setSendMap(serviceMap);
        reqModal.setStr("Action", "40481");
        reqModal.setStr("ServiceCode", serviceCode);
        reqModal.setInt("OptType", optype);
        if (strReqKey != null && strReqKey.length() > 0)
            reqModal.setStr("ReqKey", strReqKey);

        tztZFNetWork.sendDataAction(linktype.value(), reqModal, callBack);
        return true;
    }

    /**
     * 订阅主推
     * @param callBack 回调
     * @return 是否订阅成功
     */
    public boolean codePushAddSub(tztZFNetWorkResponseCallBack callBack) {

        return codePushSub(1, callBack);
    }

    /**
     * 全新订阅-会先清除原有所有订阅
     * @param callBack 回调
     * @return 是否清除成功
     */
    public boolean codePushClearAndSub(tztZFNetWorkResponseCallBack callBack) {
        return codePushSub(2, callBack);
    }

    /**
     * 取消订阅
     * @param callBack 回调
     * @return 是否取消成功
     */
    public boolean codePushCancel(tztZFNetWorkResponseCallBack callBack) {
        String strCode = getCodePushCode();
        if (strCode == null || strCode.length() < 1)
            return false;

        String strReqKey = getCodePushReqKey();
        tztZFNetWorkReqObj reqModal = new tztZFNetWorkReqObj();
        reqModal.setStr("Action", "40482");
        reqModal.setStr("ServiceCode", strCode);
        if (strReqKey != null && strReqKey.length() > 0)
            reqModal.setStr("ReqKey", strReqKey);

        tztZFNetWork.sendDataAction(linktype.value(), reqModal, callBack);
        return true;
    }
    // </editor-fold>

    String getGridKey(int grid) {
        return "Grid" + grid;
    }

    String getCodePushCode() {
        if (serviceCode == null || serviceCode.length() < 1)
            serviceCode = serviceMap.getStr("ServiceCode");
        return serviceCode;
    }

    String getCodePushReqKey() {
        if (reqKey == null || reqKey.length() < 1)
            reqKey = serviceMap.getStr("ReqKey");

        return reqKey;
    }

    /**
     * 订阅主推
     * @param dataMap 订阅信息
     * @param callBack 回调
     * @return 是否订阅成功
     */
    public static boolean codePushAddSub(tztZFDataMapBase<String, Object> dataMap, tztZFNetWorkResponseCallBack callBack) {
        tztZFServicePush codePush = tztZFServicePush.initWithDataMap(dataMap);
        if (codePush == null)
            return false;

        tztZFDataMapBase<String, Object> pushReq = dataMap.getDataMap("pushReq");
        boolean bSet = codePush.setGridData(pushReq);
        if (!bSet) {
            bSet = codePush.setGridData(dataMap);
        }

        return codePush.codePushAddSub(callBack);
    }

    /**
     * 清除订阅
     * @param dataMap 订阅信息
     * @param callBack 回调
     * @return 是否成功
     */
    public static boolean codePushClearAndSub(tztZFDataMapBase<String, Object> dataMap, tztZFNetWorkResponseCallBack callBack) {
        tztZFServicePush codePush = tztZFServicePush.initWithDataMap(dataMap);
        tztZFDataMapBase<String, Object> pushReq = dataMap.getDataMap("pushReq");
        boolean bSet = codePush.setGridData(pushReq);
        if (!bSet) {
            bSet = codePush.setGridData(dataMap);
        }

        return codePush.codePushClearAndSub(callBack);
    }

    /**
     * 取消订阅
     * @param dataMap 订阅信息
     * @param callBack 回调
     * @return 是否成功
     */
    public static boolean codePushCancel(tztZFDataMapBase<String, Object> dataMap, tztZFNetWorkResponseCallBack callBack) {
        tztZFServicePush codePush = tztZFServicePush.initWithDataMap(dataMap);
        if (codePush == null)
            return false;

        return codePush.codePushCancel(callBack);
    }

    /**
     * 清除订阅
     * @param linktype 通道
     * @param callBack 回调
     * @return 是否清除成功
     */
    public static boolean codePushClear(tztZFLinkType linktype, tztZFNetWorkResponseCallBack callBack) {
        tztZFNetWorkReqObj reqObj = new tztZFNetWorkReqObj();
        reqObj.setStr("Action", "40482");
        reqObj.setStr("ServiceCode", "");
        tztZFNetWork.sendDataAction(linktype.value(), reqObj, callBack);
        return true;
    }
}
