package com.tztzf.network.socket.Netty;

import com.tztzf.base.tztZFPublicFun;

import java.util.ArrayList;

/**
 * ****************************************************************
 * 文件名称 : tztZFHostportIpv4Ipv6
 * 作   者 : Created by yangdl
 * 创建时间 : 20221206
 * 文件描述 : ipv4-ipv6地址
 * 注意事项 :
 * 版权声明 : Copyright (C) 2022-2027 杭州中焯信息技术股份有限公司
 * 修改历史 : 20221206 1.00 初始版本
 * ****************************************************************
 */
public class tztZFHostportIpv4Ipv6 {
    ///一地址
    private tztZFHostport hostport1 = null;
    ///二地址
    private tztZFHostport hostport2 = null;
    ///类别
    private int connecttype = 0;
    ///当前使用
    private int used = 0;

    public tztZFHostportIpv4Ipv6(String address) {
        initWithAddress(address, "", 0);
    }

    public tztZFHostportIpv4Ipv6(String address, String addrName, int connecttype) {
        initWithAddress(address, addrName, connecttype);
    }

    ///转列表 host1:port1,host2:port2&
    public static ArrayList<tztZFHostportIpv4Ipv6> withAddresslist(String addresslist) {
        return tztZFHostportIpv4Ipv6.withAddresslistNamelist(addresslist, "", 0);
    }

    public static ArrayList<tztZFHostportIpv4Ipv6> withAddresslistNamelist(String addresslist, String namelist, int connecttype) {
        if (tztZFPublicFun.isEmpty(addresslist))
            return null;
        ArrayList<String> ayName = new ArrayList<>();
        tztZFPublicFun.split(namelist, "&", ayName);
        ArrayList<String> ayList = new ArrayList<>();
        tztZFPublicFun.split(addresslist, "&", ayList);
        ArrayList<tztZFHostportIpv4Ipv6> ayResult = new ArrayList<>();
        for (int i = 0; i < ayList.size(); i++) {
            String strHostport = ayList.get(i);
            if (tztZFPublicFun.isEmpty(strHostport))
                continue;
            String strName = "";
            if (i < ayName.size())
                strName = ayName.get(i);
            tztZFHostportIpv4Ipv6 host = new tztZFHostportIpv4Ipv6(strHostport, strName, connecttype);
            if (host != null && host.isValid())
                ayResult.add(host);
        }
        return ayResult;
    }

    //host1:port1,host2:port2
    private void initWithAddress(String address, String addrName, int connecttype) {
        if (tztZFPublicFun.isEmpty(address))
            return;
        ArrayList<String> ayHostport = new ArrayList<>();
        tztZFPublicFun.split(address, ",", ayHostport);
        if (ayHostport.size() < 1)
            return;
        String strHostport1 = ayHostport.get(0);
        this.connecttype = connecttype;
        this.hostport1 = new tztZFHostport(strHostport1, addrName, connecttype);
        if (ayHostport.size() < 2)
            return;
        String strHostport2 = ayHostport.get(1);
        this.hostport2 = new tztZFHostport(strHostport2, addrName, connecttype);
    }

    public tztZFHostportIpv4Ipv6 withUse(int used) {
        tztZFHostportIpv4Ipv6 hostportIpv4Ipv6 = new tztZFHostportIpv4Ipv6("");
        hostportIpv4Ipv6.hostport1 = hostport1;
        hostportIpv4Ipv6.hostport2 = hostport2;
        hostportIpv4Ipv6.connecttype = connecttype;
        hostportIpv4Ipv6.used = used;
        return hostportIpv4Ipv6;
    }

    //当前使用地址
    public tztZFHostport usedHostport() {
        if (used == 2)
            return hostport2;
        return hostport1;
    }

    //当前使用地址
    public String usedAddress() {
        tztZFHostport hostport = usedHostport();
        if (hostport == null)
            return "";
        return hostport.address();
    }

    //相同
    public boolean isEqualHost(tztZFHostportIpv4Ipv6 host) {
        if (host == null)
            return false;
        //都无效
        if (isInvalid() && host.isInvalid())
            return true;
        //一个无效、一个有效
        if (isInvalid() || host.isInvalid())
            return false;
        //hostport1判断
        if (hostport1 != null) {
            if (!hostport1.isEqualToHostport(host.hostport1) && !hostport1.isEqualToHostport(host.hostport2)) {
                return false;
            }
        }
        //hostport2判断
        if (hostport2 != null) {
            if (!hostport2.isEqualToHostport(host.hostport1) && !hostport2.isEqualToHostport(host.hostport2)) {
                return false;
            }
        }
        return true;
    }

    //无效
    public boolean isInvalid() {
        return (hostport1 == null || hostport1.isInvalid())
                && (hostport2 == null || hostport2.isInvalid());
    }

    //有效
    public boolean isValid() {
        return !isInvalid();
    }

    //包含hostport
    protected boolean isHave(tztZFHostport hostport) {
        if (hostport == null || hostport.isInvalid() || isInvalid())
            return false;
        return hostport.isEqualToHostport(hostport1)
                || hostport.isEqualToHostport(hostport2);
    }

    //host1:port1,host2:port2
    public String address() {
        String strHostport = "";
        if (hostport1 != null)
            strHostport = hostport1.address();
        if (hostport2 == null)
            return strHostport;
        if (strHostport.length() > 0)
            return strHostport + "," + hostport2.address();
        return hostport2.address();
    }

    @Override
    public String toString() {
        return "[connecttype:" + connecttype + ",used:" + used + "]" + address();
    }

    public tztZFHostport getHostport1() {
        return hostport1;
    }

    public tztZFHostport getHostport2() {
        return hostport2;
    }

    public int getConnecttype() {
        return connecttype;
    }

    public int getUsed() {
        return used;
    }
}
