package com.tztzf.network.socket.Netty;

import com.tztzf.base.tztZFPublicFun;
import com.tztzf.commbase.tztZFDataMapBase;
import com.tztzf.commbase.tztZFDataMapCallBack;
import com.tztzf.network.define.tztZFRecvType;
import com.tztzf.network.socket.tztZFSocketSendData;

import java.util.ArrayList;
import java.util.concurrent.CopyOnWriteArrayList;

import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_Send;

public final class tztZFSocketPushCode {
    private static final String DEFServiceCode = "ServiceCode";
    private Object userData = null;
    private tztZFPushRespDelegate delegate;
    private CopyOnWriteArrayList<tztZFSocketSendData> reqList = new CopyOnWriteArrayList<>();
    private tztZFDataMapBase<String, tztZFSocketSendData> recList = new tztZFDataMapBase<>();

    public tztZFSocketPushCode(tztZFPushRespDelegate delegate) {
        initData();
        this.delegate = delegate;
    }

    public static boolean isPushAction(int nAction) {
        return isPushActionTake(nAction) || isPushActionRemove(nAction);
    }

    //40481 服务订阅 ServiceCode=live001;
    private static boolean isPushActionTake(int nAction) {
        return nAction == 40481;
    }

    //40482 服务取消 ServiceCode=live001;
    private static boolean isPushActionRemove(int nAction) {
        return nAction == 40482;
    }

    //40484 服务推送 ServiceCode=live001;
    private static boolean isPushActionPush(int nAction) {
        return nAction == 40484;
    }

    private void initData() {

    }

    public Object getUserData() {
        return userData;
    }

    public void setUserData(Object userData) {
        this.userData = userData;
    }

    public boolean have() {
        return reqList.size() > 0;
    }

    public void clearReq() {
        reqList.clear();
    }

    public void clearRec() {
        recList.removeAllObjects();
    }

    public tztZFPushRespDelegate getDelegate() {
        return delegate;
    }

    public void setDelegate(tztZFPushRespDelegate delegate) {
        this.delegate = delegate;
    }

    public void clearRecWithType(final tztZFRecvType type) {
        recList.enumerateKeysAndObjectsUsingBlock(new tztZFDataMapCallBack() {
            @Override
            public boolean enumerate(String lowKey, String key, Object value) {
                tztZFDataMapBase resp = new tztZFDataMapBase();
                resp.setInt("ErrorNo", tztZFNetWorkError_Send);
                String strMsg = String.format("[%s]订阅已被取消", key);
                resp.setStr("ErrorMessage", strMsg);
                responseTypePush(resp, type, (tztZFSocketSendData) value);
                return false;
            }
        });
        recList.removeAllObjects();
    }

    public void reSendReq(CopyOnWriteArrayList<tztZFSocketSendData> ayReSend) {
        if (ayReSend == null || ayReSend.size() < 1)
            return;
        for (tztZFSocketSendData push : reqList) {
            if (push != null) {
                push.setAddTime();
                ayReSend.add(push);
            }
        }
    }

    private void clear() {
        reqList.clear();
        recList.removeAllObjects();
    }

    private String getServiceCode(tztZFSocketSendData push) {
        String serviceCode = push.getStr(DEFServiceCode);
        if (tztZFPublicFun.isEmpty(serviceCode))
            return serviceCode;
        return serviceCode.toLowerCase();
    }

    private boolean isServiceCode(String serviceCode, tztZFSocketSendData push) {
        if (tztZFPublicFun.isEmpty(serviceCode))
            return false;
        String code = getServiceCode(push);
        return serviceCode.equalsIgnoreCase(code);
    }

    //主推请求加入队列
    public void addReqPush(tztZFSocketSendData push) {
        if (push == null)
            return;
        int nAction = push.getAction();
        if (isPushActionRemove(nAction)) {
            String serviceCode = getServiceCode(push);
            removeServiceCode(serviceCode);
        }
    }

    //清除serviceCode服务主推
    private void removeServiceCode(String serviceCode) {
        removeServiceCodeRec(serviceCode);
        removeServiceCodeReq(serviceCode);
    }

    //清除服务订阅主推
    private void removeServiceCodeRec(String serviceCode) {
        if (recList.getCount() < 1)
            return;
        //serviceCode为空，取消所有服务主推订阅
        boolean bClear = tztZFPublicFun.isEmpty(serviceCode);
        if (bClear) {
            recList.enumerateKeysAndObjectsUsingBlock(new tztZFDataMapCallBack() {
                @Override
                public boolean enumerate(String lowKey, String key, Object value) {
                    respServiceCodeRemovePush(key, (tztZFSocketSendData) value);
                    return false;
                }
            });
            recList.removeAllObjects();
            return;
        }
        //取消对应的serviceCode订阅
        tztZFSocketSendData push = recList.getObj(serviceCode);
        respServiceCodeRemovePush(serviceCode, push);
        recList.removeByKey(serviceCode);
    }

    //清除serviceCode服务订阅请求
    private void removeServiceCodeReq(String serviceCode) {
        if (reqList.size() < 1)
            return;
        boolean bClear = tztZFPublicFun.isEmpty(serviceCode);
        if (bClear) {
            reqList.clear();
            return;
        }
        for (tztZFSocketSendData push : reqList) {
            if (isServiceCode(serviceCode, push)) {
                reqList.remove(push);
            }
        }
    }

    private boolean findedServiceCodeForAy(ArrayList<String> ayCode, String findCode) {
        if (tztZFPublicFun.isEmpty(findCode))
            return false;
        if (ayCode == null || ayCode.size() < 1)
            return false;
        return ayCode.contains(findCode);
    }

    //设置serviceCode服务订阅应答
    private void recPushServiceCodeCodeList(tztZFSocketSendData push, String serviceCode, final ArrayList<String> ayCode) {
        if (!tztZFPublicFun.isEmpty(serviceCode)) {
            recList.removeByKey(serviceCode);
        }
        final ArrayList<String> ayRemove = new ArrayList<>();
        recList.enumerateKeysAndObjectsUsingBlock(new tztZFDataMapCallBack() {
            @Override
            public boolean enumerate(String lowKey, String key, Object value) {
                if (findedServiceCodeForAy(ayCode, lowKey)) {
                    respServiceCodeRemovePush(key, (tztZFSocketSendData) value);
                    ayRemove.add(lowKey);
                }
                return false;
            }
        });
        for (String key : ayRemove) {
            recList.removeByKey(key);
        }
        ///加入列表
        if (push != null && !tztZFPublicFun.isEmpty(serviceCode)) {
            recList.setObj(serviceCode, push);
        }
    }

    //设置serviceCode服务订阅请求
    private void reqPushServiceCodeCodeList(tztZFSocketSendData push, String serviceCode, ArrayList<String> ayCode) {
        for (tztZFSocketSendData req : reqList) {
            if (!tztZFPublicFun.isEmpty(serviceCode) && isServiceCode(serviceCode, req)) {
                reqList.remove(req);
                continue;
            }
            String reqCode = getServiceCode(req);
            if (!findedServiceCodeForAy(ayCode, reqCode))
                reqList.remove(req);
        }
        if (push != null && !tztZFPublicFun.isEmpty(serviceCode)) {
            push.setPushKey(serviceCode);
            reqList.add(push);
        }
    }

    //#pragma mark -根据订阅列表设置serviceCode服务主推
    private void setPushServiceCodeList(tztZFSocketSendData push, String serviceCode, ArrayList<String> ayCode) {
        recPushServiceCodeCodeList(push, serviceCode, ayCode);
        reqPushServiceCodeCodeList(push, serviceCode, ayCode);
    }

    //根据tag查找主推请求
    public tztZFSocketSendData findPushWithtag(final String recvTag) {
        if (tztZFPublicFun.isEmpty(recvTag))
            return null;
        final tztZFSocketSendData[] push = {null};
        recList.enumerateKeysAndObjectsUsingBlock(new tztZFDataMapCallBack() {
            @Override
            public boolean enumerate(String lowKey, String key, Object value) {
                tztZFSocketSendData sendData = (tztZFSocketSendData) value;
                if (sendData != null && sendData.getSendTag() == tztZFPublicFun.parseInt(recvTag, -1)) {
                    push[0] = sendData;
                    return true;
                }
                return false;
            }
        });
        return push[0];
    }

    //根据serviceCode查找主推请求
    public tztZFSocketSendData findPushWithCode(String serviceCode) {
        if (tztZFPublicFun.isEmpty(serviceCode))
            return null;
        return recList.getObj(serviceCode);
    }

    //    #pragma mark -根据应答查找主推请求
    private tztZFSocketSendData findPushWithRecv(tztZFDataMapBase recvMap, String recvTag) {
        if (recvMap == null || recvMap.getCount() < 1)
            return null;
        int nAction = recvMap.getAction();
        if (!isPushAction(nAction))
            return null;
        String serviceCode = recvMap.getStr("ServiceCode");
        return recList.getObj(serviceCode);
    }

    //serviceCode服务主推订阅/退订应答处理
    public void dealRecvPush(tztZFDataMapBase recvMap, tztZFSocketSendData push) {
        if (recvMap == null || recvMap.getInt("ErrorNo", -1) < 0)
            return;
        int nAtcion = recvMap.getAction();
        if (!isPushAction(nAtcion))
            return;
        String serviceCode = recvMap.getStr("Grid");
        if (tztZFPublicFun.isEmpty(serviceCode)) {
            removeServiceCode("");
            return;
        }
        serviceCode = serviceCode.toLowerCase();
        ArrayList<String> ayCode = new ArrayList<>();
        tztZFPublicFun.split(serviceCode, ",", ayCode);
        if (ayCode.size() < 1) {
            removeServiceCode("");
            return;
        }
        String pushCode = "";
        if (push != null && isPushAction(push.getAction())) {
            pushCode = getServiceCode(push);
            if (!tztZFPublicFun.isEmpty(pushCode)) {
                push.setPushKey(pushCode);
            } else {
                pushCode = "";
            }
        }
        setPushServiceCodeList(push, pushCode, ayCode);
    }

    private void respServiceCodeRemovePush(String serviceCode, tztZFSocketSendData push) {
        if (push == null || tztZFPublicFun.isEmpty(serviceCode))
            return;
        tztZFDataMapBase resp = new tztZFDataMapBase();
        resp.setInt("ErrorNo", -1);
        String strMsg = String.format("[%s]订阅已被取消", serviceCode);
        resp.setStr("ErrorMessage", strMsg);
        responseTypePush(resp, tztZFRecvType.tztZFRecv_DataOK, push);
    }

    private void responseTypePush(tztZFDataMapBase resp, tztZFRecvType type, tztZFSocketSendData push) {
        if (delegate != null) {
            delegate.resp(push, type, resp);
        }
    }
}
