package com.tztzf.network.socket.Netty;

import com.tztzf.base.tztZFPublicFun;
import com.tztzf.log.tztZFLog;
import com.tztzf.log.tztZFLogLevel;
import com.tztzf.log.tztZFLogPrintType;
import com.tztzf.network.cosign.tztZFNetCosign;
import com.tztzf.network.define.tztZFLinkType;
import com.tztzf.network.socket.tztZFThreadRespRunnable;
import com.tztzf.protocol.callback.tztZFBlocktztSSL;
import com.tztzf.protocol.core.tztZFSSLCert;
import com.tztzf.protocol.core.tztZFVPNSSL;

import java.io.File;
import java.util.Locale;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import static com.tztzf.network.gm.tztZFSocketSSLStep.tztZFSocketSSLStepClose;
import static com.tztzf.network.gm.tztZFSocketSSLStep.tztZFSocketSSLStepHand;
import static com.tztzf.network.gm.tztZFSocketSSLStep.tztZFSocketSSLStepUnServerCert;
import static com.tztzf.protocol.define.tztZFCertType.tztZFCertPEMChain;
import static com.tztzf.protocol.define.tztZFCertType.tztZFCertPEMData;
import static com.tztzf.protocol.define.tztZFSSLCallbackType.tztZFSSLCallback_Hand;
import static com.tztzf.protocol.define.tztZFSSLCallbackType.tztZFSSLCallback_Read;
import static com.tztzf.protocol.define.tztZFSSLCallbackType.tztZFSSLCallback_Write;
import static com.tztzf.protocol.define.tztZFSSLCallbackType.tztZFSSLCallback_close;
import static com.tztzf.protocol.define.tztZFSSLCallbackType.tztZFSSLCallback_d2iSession;
import static com.tztzf.protocol.define.tztZFSSLCallbackType.tztZFSSLCallback_err;
import static com.tztzf.protocol.define.tztZFSSLCallbackType.tztZFSSLCallback_i2dSession;

/**
 * ****************************************************************
 * 文件名称 : tztZFSocketSSL
 * 作   者 : Created by yangdl
 * 创建时间 : 20221206
 * 文件描述 : ssl
 * 注意事项 :
 * 版权声明 : Copyright (C) 2022-2027 杭州中焯信息技术股份有限公司
 * 修改历史 : 20221206 1.00 初始版本
 * ****************************************************************
 */

public final class tztZFSocketSSL {
    ///证书已注销
    private static final int SSL3_AD_CERTIFICATE_REVOKED = 44;
    ///证书已过期
    private static final int SSL3_AD_CERTIFICATE_EXPIRED = 45;
    private static final String tztZFSocketSSLClientFileDir = "tztssl_cert";
    private static final String tztZFSocketSSLClientCAFile = "tztssl_client_ca.crt";
    private static final String tztZFSocketSSLClientSignCertFile = "tztssl_client_sig.crt";
    private static final String tztZFSocketSSLClientSignKeyFile = "tztssl_client_sig.key";
    private static final String tztZFSocketSSLClientEncCertFile = "tztssl_client_enc.crt";
    private static final String tztZFSocketSSLClientEncKeyFile = "tztssl_client_enc.key";
    ///vpnssl
    private tztZFVPNSSL vpnssl = null;
    ///vpnssl回调
    private tztZFBlocktztSSL vpnsslBlock = null;
    ///socket
    private Object sslSocketFD = null;
    ///是SSL
    private boolean isSSL;
    ///连接类别
    private int linktype = tztZFLinkType.tztZFLinkType_Non.value();
    ///链接类别
    private int connecttype = 0;
    ///ajax服务
    private Boolean isAjaxServer = false;
    ///ssl回调
    private tztZFSocketSSLDelegate socketSSLDelegate = null;
    ///证书
    private tztZFSSLCert sslCert = null;
    ///证书客户号
    private String certUser = "";
    ///SSL握手
    private boolean isSSLHand = false;
    ///Context
    private Object context = null;
    ///回调线程
    private ThreadPoolExecutor pThread = null;

    public tztZFSocketSSL(Object context) {
        this.context = context;
        initData();
    }

    private void initData() {
        ///初始化vpnssl回调
        initVPNSSLBlock();
        ///初始化回调线程
        initThread();
    }

    ///是ajaxServer
    public tztZFSocketSSL withSSLAjaxServer() {
        isAjaxServer = true;
        return tztZFSocketSSL.this;
    }

    public tztZFSocketSSL withChildSSLAjaxServer(Object childSSL) {
        return tztZFSocketSSL.this;
    }

    public tztZFSocketSSL withSSLLinktype(int linktype, int connecttype) {
        this.linktype = linktype;
        this.connecttype = connecttype;
        return tztZFSocketSSL.this;
    }

    public tztZFSocketSSL withSSLCert(tztZFSSLCert sslcert) {
        this.sslCert = sslcert;
        if (sslCert != null)
            certUser = sslCert.getCertUser();
        return tztZFSocketSSL.this;
    }

    private void loadCommCert() {
        if (isAjaxServer) return;
        if (sslCert == null)
            sslCert = new tztZFSSLCert();
        if (context == null) return;
        sslCert.setCertCA(tztZFPublicFun.getAssetsBytes(context, tztZFSocketSSLClientFileDir + File.separator + tztZFSocketSSLClientCAFile));
        sslCert.setCertCAType(tztZFCertPEMChain);
        sslCert.setCertSign(tztZFPublicFun.getAssetsBytes(context, tztZFSocketSSLClientFileDir + File.separator + tztZFSocketSSLClientSignCertFile));
        sslCert.setCertSignType(tztZFCertPEMData);
        sslCert.setCertSignKey(tztZFPublicFun.getAssetsBytes(context, tztZFSocketSSLClientFileDir + File.separator + tztZFSocketSSLClientSignKeyFile));
        sslCert.setCertEnc(tztZFPublicFun.getAssetsBytes(context, tztZFSocketSSLClientFileDir + File.separator + tztZFSocketSSLClientEncCertFile));
        sslCert.setCertEncType(tztZFCertPEMData);
        sslCert.setCertEncKey(tztZFPublicFun.getAssetsBytes(context, tztZFSocketSSLClientFileDir + File.separator + tztZFSocketSSLClientEncKeyFile));
    }

    public tztZFSocketSSL loadClientSSLCert() {
        loadCommCert();
        return tztZFSocketSSL.this;
    }

    public void setDelegate(tztZFSocketSSLDelegate delegate) {
        this.socketSSLDelegate = delegate;
    }

    public boolean isUseSSL() {
        return (connecttype > 0 && isSSL);
    }

    public boolean isSSLHanded() {
        return (connecttype > 0 && isSSL && isSSLHand);
    }

    public String getCertUser() {
        return certUser;
    }


    ///初始化
    public int sslinit() {
        if (isAjaxServer)
            return -1;
        return sslvpninit();
    }

    ///连接
    public int sslconnect(Object sock) {
        int nRet = -1;
        sslSocketFD = null;
        isSSLHand = false;
        if (vpnssl != null) {
            sslSocketFD = sock;
            nRet = vpnssl.ssl_connect(sslSocketFD);
            if (nRet < 1) {

            } else {
                nRet = 0;
            }
        }
        return nRet;
    }

    ///read
    public int sslread(byte[] data, int len) {
        int nRet = -1;
        if (isAjaxServer) {
            return nRet;
        }
        if (vpnssl == null)
            return -1;
        return vpnssl.ssl_read(data, len);
    }

    ///write
    public int sslwrite(byte[] data) {
        int nRet = -1;
        if (isAjaxServer) {
            return nRet;
        }
        if (vpnssl == null)
            return -1;
        return vpnssl.ssl_write(data);
    }

    ///close
    public int sslclose() {
        if (vpnssl == null)
            return 0;
        return vpnssl.ssl_close();
    }

    ///free
    public int sslfree() {
        if (vpnssl == null)
            return 0;
        return vpnssl.ssl_free();
    }

    ///sslvpn初始化
    private int sslvpninit() {
        if (vpnssl != null) return 1;
        vpnssl = new tztZFVPNSSL();
        vpnssl.initWithConnectType(connecttype, vpnsslBlock);
        vpnssl.ssl_setSSLCert(sslCert);
        int nRet = vpnssl.ssl_init();
        isSSL = (nRet > 0);
        return nRet;
    }

    ///sslvpn回调初始化
    private void initVPNSSLBlock() {
        vpnsslBlock = new tztZFBlocktztSSL() {
            @Override
            public int onHandler(Object socket, int type, int code, byte[] data, int len) {
                tztlog("onHandler,type:%d,code:%d,len:%d", type, code, len);
                return sslBlockHandler(socket, type, code, data, len);
            }
        };
    }

    ///回调线程初始化
    private void initThread() {
        pThread = new ThreadPoolExecutor(
                1,      //核心线程数设置成1
                1,  //线程池最大线程数为1
                10, TimeUnit.SECONDS, //超过核心线程数量的线程idle 10秒之后会退出
                new ArrayBlockingQueue<Runnable>(50), //队列长度为50
                new ThreadPoolExecutor.DiscardOldestPolicy() //任务被拒绝以后的处理类
        );
    }

    ///sslblock回调处理
    private int sslBlockHandler(Object socket, int type, int code, byte[] data, int len) {
        if (sslSocketFD == null || socket != sslSocketFD) return -1;
        tztZFGCDAsyncSocket sock = (tztZFGCDAsyncSocket) socket;
        int nRet = 0;
        String logType = "";
        String retMsg = "";
        tztZFSocketSSLInfo sslInfo = new tztZFSocketSSLInfo();
        sslInfo.setCode(code);
        sslInfo.setCertUser(certUser);
        sslInfo.setSock(sock);
        switch (type) {
            ///hand
            case tztZFSSLCallback_Hand: {
                logType = "SSLHand";
                sslInfo.setMsg(logType);
                isSSLHand = true;
                asyncSSLBlockStep(tztZFSocketSSLStepHand, sslInfo);
            }
            break;
            ///write
            case tztZFSSLCallback_Write: {
                logType = "SocketWrite";
                if (sock != null) {
                    nRet = sock.socketWrite(data, data);
                } else {
                    nRet = -1;
                }
            }
            break;
            ///read
            case tztZFSSLCallback_Read: {
                logType = "SSLRead";
                if (sock != null) {
                    nRet = sock.socketRead(data, len);
                } else {
                    nRet = -1;
                }
            }
            break;
            ///存储session
            case tztZFSSLCallback_i2dSession: {
                logType = "SSLSaveSession";
                if (data != null && data.length > 0) {
                    sslSaveSession(data);
                }
            }
            break;
            ///读取session
            case tztZFSSLCallback_d2iSession: {
                logType = "SSLLoadSession";
                byte[] session = sslRestoreSession();
                if (session != null) {
                    int sessionlen = session.length;
                    if (sessionlen > 0 && sessionlen <= len) {
                        System.arraycopy(session, 0, data, 0, sessionlen);
                        nRet = sessionlen;
                    }
                }
            }
            break;
            ///错误
            case tztZFSSLCallback_err: {
                logType = "SSLErr";
                if (data != null && data.length > 0)
                    retMsg = new String(data);
                int errcode = (code & 0xFF);
                if (errcode == SSL3_AD_CERTIFICATE_REVOKED ||
                        errcode == SSL3_AD_CERTIFICATE_EXPIRED) {
                    if (!tztZFPublicFun.isEmpty(certUser)) {
                        int nError = -1;
                        if (errcode == SSL3_AD_CERTIFICATE_REVOKED) {
                            nError = -3;
                        } else {
                            nError = -2;
                        }
                        tztZFNetCosign.expiredCert(certUser, nError);
                    }
                    sslInfo.setMsg(retMsg);
                    asyncSSLBlockStep(tztZFSocketSSLStepUnServerCert, sslInfo);
                }
                nRet = -1;
            }
            break;
            ///close
            case tztZFSSLCallback_close: {
                logType = "SSLClose";
                isSSLHand = false;
                if (data != null && data.length > 0)
                    retMsg = new String(data);
                sslInfo.setMsg(retMsg);
                asyncSSLBlockStep(tztZFSocketSSLStepClose, sslInfo);
            }
            break;
            ///其他
            default:
                logType = "invaild_" + type;
                break;
        }
        tztlogI("[SSL-%d][tztZFSSL][AsyncSocket]%d %s[code:%d][%d]%s", connecttype, socket.hashCode(), logType, code, nRet, retMsg);
        return nRet;
    }

    ///存储session
    private int sslSaveSession(byte[] data) {
        if (data == null || data.length < 1)
            return 0;
        return tztZFNetCosign.saveSSLSession(data, certUser, connecttype, linktype);
    }

    ///读取session
    private byte[] sslRestoreSession() {
        return tztZFNetCosign.restoreSSLSession(certUser, connecttype, linktype);
    }

    ///异步回调
    private void asyncSSLBlockStep(final int step, final tztZFSocketSSLInfo info) {
        final tztZFSocketSSL socketSSL = tztZFSocketSSL.this;
        pThread.execute(new tztZFThreadRespRunnable(linktype, 0, null) {
            @Override
            public void run() {
                final tztZFGCDAsyncSocket sock = (tztZFGCDAsyncSocket) sslSocketFD;
                if (socketSSLDelegate != null) {
                    socketSSLDelegate.didSSLStepInfo(socketSSL, sock, step, info);
                }
            }
        });
    }

    /**
     * 打印日志
     */
    private void tztlog(String format, Object... args) {
        tztlog(tztZFLogLevel.tztZFLogLevel_Debug, format, args);
    }

    private void tztlogI(String format, Object... args) {
        tztlog(tztZFLogLevel.tztZFLogLevel_Info, format, args);
    }

    private void tztlog(tztZFLogLevel level, String format, Object... args) {
        if (tztZFLog.getLevel() <= level.value()) {
            String cTag = String.format(Locale.getDefault(), "[%d]tztZFSocketSSL[%d]",
                    linktype, hashCode());
            tztZFLog.log(level, tztZFLogPrintType.tztZFLogPrintType_tztSocket.value(), cTag, format, args);
        }
    }
}
