package com.tztzf.network.socket.Netty;

import java.util.TimerTask;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

/**
 * ****************************************************************
 * 文件名称 : tztZFTimer
 * 作   者 : Created by yangdl
 * 创建时间 : 20221206
 * 文件描述 : Timer回调
 * 注意事项 :
 * 版权声明 : Copyright (C) 2022-2027 杭州中焯信息技术股份有限公司
 * 修改历史 : 20221206 1.00 初始版本
 * ****************************************************************
 */
public final class tztZFTimer {
    private final long start;
    private final long interval;
    private final boolean repeats;
    private boolean valid = false;
    //private Timer timer;
    /** 使用ScheduledExecutorService 替换 Timer 避免多线程异常 */
    private ScheduledExecutorService executorService;
    private tztZFTimerDelegate delegate;

    public tztZFTimer(long start, long interval, boolean repeats, tztZFTimerDelegate delegate) {
        if (start < 0)
            start = 0;
        this.start = start;
        this.interval = interval;
        this.repeats = repeats;
        this.delegate = delegate;
        startTimer();
    }

    private void startTimer() {
        //如果当前定时小于1ms，则不做处理
        if (interval < 1) {
            valid = true;
            fireTimer();
            return;
        }
        if (executorService == null) {
            executorService = Executors.newScheduledThreadPool(1);
            valid = true;
            executorService.scheduleAtFixedRate(new TimerTask() {
                @Override
                public void run() {
                    fireTimer();
                }
            }, start, interval, TimeUnit.MILLISECONDS);
        }
    }

    private synchronized void fireTimer() {
        if (!valid)
            return;
        if (delegate != null)
            delegate.fire(tztZFTimer.this);
        if (!repeats) {
            invalidate();
        }
    }

    public synchronized void invalidate() {
        valid = false;
        if (executorService != null && !executorService.isShutdown()) {
            //关闭后，需要重新创建才可以，直接调用scheduleAtFixedRate无效
            executorService.shutdownNow();
            executorService = null;
        }
    }

}
