package com.tztzf.network.socket;

import com.tztzf.base.tztZFPublicFun;
import com.tztzf.baseos.net.tztZFNetAvailable;
import com.tztzf.baseos.net.tztZFNetChange;
import com.tztzf.baseos.tztZFTextUtils;
import com.tztzf.commbase.tztZFDataMapBase;
import com.tztzf.commbase.tztZFThreadPool;
import com.tztzf.log.tztZFLog;
import com.tztzf.log.tztZFLogLevel;
import com.tztzf.log.tztZFLogPrintType;
import com.tztzf.network.callback.tztZFGMSSLStepCallBack;
import com.tztzf.network.callback.tztZFLinkCallBack;
import com.tztzf.network.callback.tztZFNetWorkResponseCallBack;
import com.tztzf.network.cosign.tztZFCosignUser;
import com.tztzf.network.cosign.tztZFNetCosign;
import com.tztzf.network.cosign.tztZFNetCosignEx;
import com.tztzf.network.define.tztZFErrorMessageDefine;
import com.tztzf.network.define.tztZFLinkCallBackType;
import com.tztzf.network.define.tztZFLinkType;
import com.tztzf.network.define.tztZFRecvType;
import com.tztzf.network.define.tztZFSocketFlags;
import com.tztzf.network.gm.tztZFGMManage;
import com.tztzf.network.gm.tztZFSocketSSLStep;
import com.tztzf.network.tztZFNetWork;
import com.tztzf.network.tztZFNetWorkReqObj;
import com.tztzf.network.tztZFNetWorkRespObj;
import com.tztzf.network.tztZFQuitTradeEvent;
import com.tztzf.protocol.callback.tztZFDataProtocolInterface;
import com.tztzf.protocol.core.tztZFDataEncrypt;
import com.tztzf.protocol.core.tztZFDataProtocolKit;
import com.tztzf.protocol.core.tztZFProtocolCert;
import com.tztzf.protocol.core.tztZFRecvPackage;
import com.tztzf.protocol.core.tztZFRegister;
import com.tztzf.protocol.define.tztZFHandshakeStatus;
import com.tztzf.protocol.define.tztZFPackageType;
import com.tztzf.tztZFNotificationObj;

import org.greenrobot.eventbus.EventBus;

import java.io.ByteArrayOutputStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.RejectedExecutionHandler;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import static com.tztzf.log.tztZFLogLevel.tztZFLogLevel_Info;
import static com.tztzf.network.cosign.tztZFCosignErr.tztZFCosignErrCertDeviceDel;
import static com.tztzf.network.cosign.tztZFCosignErr.tztZFCosignErrServerCertUnfind;
import static com.tztzf.network.cosign.tztZFNetCosignEx.tztCosignEx_CACert;
import static com.tztzf.network.cosign.tztZFNetCosignEx.tztCosignEx_ClientPubKey;
import static com.tztzf.network.cosign.tztZFNetCosignEx.tztCosignEx_ClientPubKeyIndex;
import static com.tztzf.network.cosign.tztZFNetCosignEx.tztCosignEx_Encrypttype;
import static com.tztzf.network.cosign.tztZFNetCosignEx.tztCosignEx_Handtype;
import static com.tztzf.network.cosign.tztZFNetCosignEx.tztCosignEx_IDA;
import static com.tztzf.network.cosign.tztZFNetCosignEx.tztCosignEx_IDB;
import static com.tztzf.network.cosign.tztZFNetCosignEx.tztCosignEx_Protocol;
import static com.tztzf.network.cosign.tztZFNetCosignEx.tztCosignEx_ServerPubKey;
import static com.tztzf.network.cosign.tztZFNetCosignEx.tztCosignEx_ServerPubKeyIndex;
import static com.tztzf.network.define.tztZFErrorMessageDefine.tztZFConnectError;
import static com.tztzf.network.define.tztZFErrorMessageDefine.tztZFNetWorkErrorKey_ConnectError;
import static com.tztzf.network.define.tztZFErrorMessageDefine.tztZFNetWorkErrorKey_ReadError;
import static com.tztzf.network.define.tztZFErrorMessageDefine.tztZFNetWorkErrorKey_TimeOutError;
import static com.tztzf.network.define.tztZFErrorMessageDefine.tztZFNetWorkErrorKey_WriteError;
import static com.tztzf.network.define.tztZFErrorMessageDefine.tztZFOutTimeError;
import static com.tztzf.network.define.tztZFErrorMessageDefine.tztZFReadError;
import static com.tztzf.network.define.tztZFErrorMessageDefine.tztZFWriteError;
import static com.tztzf.network.define.tztZFLinkType.tztZFLinkType_JY;
import static com.tztzf.network.define.tztZFRecvType.tztZFConn_Change;
import static com.tztzf.network.define.tztZFRecvType.tztZFConn_Connected;
import static com.tztzf.network.define.tztZFRecvType.tztZFConn_DisConnect;
import static com.tztzf.network.define.tztZFRecvType.tztZFConn_Non;
import static com.tztzf.network.define.tztZFRecvType.tztZFRecv_DataOK;
import static com.tztzf.network.define.tztZFRecvType.tztZFRecv_Error;
import static com.tztzf.network.define.tztZFRecvType.tztZFRecv_TimeOut;
import static com.tztzf.network.define.tztZFRecvType.tztZFSend_MaxError;
import static com.tztzf.network.define.tztZFRecvType.tztZFSend_SendCancel;
import static com.tztzf.network.define.tztZFRecvType.tztZFSend_SendError;
import static com.tztzf.network.define.tztZFRecvType.tztZFSend_TimeOut;
import static com.tztzf.network.define.tztZFSocketFlags.tztZFSocketFlagTip;
import static com.tztzf.network.define.tztZFSocketFlags.tztZFSocketFlags_ConnFail;
import static com.tztzf.network.define.tztZFSocketFlags.tztZFSocketFlags_ConnSucess;
import static com.tztzf.network.define.tztZFSocketFlags.tztZFSocketFlags_Connect;
import static com.tztzf.network.define.tztZFSocketFlags.tztZFSocketFlags_Conning;
import static com.tztzf.network.define.tztZFSocketFlags.tztZFSocketFlags_DisConning;
import static com.tztzf.network.define.tztZFSocketFlags.tztZFSocketFlags_End;
import static com.tztzf.network.define.tztZFSocketFlags.tztZFSocketFlags_Init;
import static com.tztzf.network.gm.tztZFGMManage.tztZFJYLoginGMProcessStep;
import static com.tztzf.network.gm.tztZFSocketSSLStep.tztZFSocketSSLStepClose;
import static com.tztzf.network.gm.tztZFSocketSSLStep.tztZFSocketSSLStepDowngrade;
import static com.tztzf.network.gm.tztZFSocketSSLStep.tztZFSocketSSLStepSocket;
import static com.tztzf.network.gm.tztZFSocketSSLStep.tztZFSocketSSLStepSucess;
import static com.tztzf.network.gm.tztZFSocketSSLStep.tztZFSocketSSLStepUnServerCert;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_Disconnect;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_Hand;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_NoConnect;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_Recv;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_Send;
import static com.tztzf.protocol.define.tztZFHandshakeStatus.tztZFHandshake_Error;
import static com.tztzf.protocol.define.tztZFHandshakeStatus.tztZFHandshake_Failed;
import static com.tztzf.protocol.define.tztZFHandshakeStatus.tztZFHandshake_Handing;
import static com.tztzf.protocol.define.tztZFHandshakeStatus.tztZFHandshake_Init;
import static com.tztzf.protocol.define.tztZFHandshakeStatus.tztZFHandshake_Rehand;
import static com.tztzf.protocol.define.tztZFHandshakeStatus.tztZFHandshake_Success;
import static com.tztzf.protocol.define.tztZFPackageType.tztZFPackage_AutoPush;
import static com.tztzf.protocol.define.tztZFPackageType.tztZFPackage_Data;
import static com.tztzf.protocol.define.tztZFPackageType.tztZFPackage_Handshake;
import static com.tztzf.protocol.define.tztZFPackageType.tztZFPackage_Socket;

/**
 * 作者: yangdl 2021.02
 * 用途: 基础的请求应答类，封装了网络通讯，调用者不需要关心网络通讯与业务层协议
 * 修改记录如下:
 * 版本号: 3.00.001 2021.02.26
 * 更新说明: 第一版正式发布
 */
public final class tztZFThreadPoolSocket {
    // implements HSInterface
    private static final int m_nDefPort = 7778;//默认端口

    //=====一组连接地址接口hostport，一般一个为ipv4，一个为ipv6=======================================================
    static class tztHostport {
        String sHost1 = "";//服务器地址
        int nPort1 = 0;//服务器端口
        String sHost2 = "";//服务器地址2
        int nPort2 = 0;//服务器端口2
        int nNum = 0;
        int connectType;

        public String toPrint() {
            return String.format(Locale.getDefault(), "[%d]host1:%s:%d,host2:%s:%d", connectType, sHost1 != null ? sHost1 : "", nPort1, sHost2 != null ? sHost2 : "", nPort2);
        }

        @Override
        public String toString() {
            if (!tztZFTextUtils.isEmpty(sHost1) && !tztZFTextUtils.isEmpty(sHost2)) {
                return sHost1 + ":" + nPort1 + "," + sHost2 + ":" + nPort2;
            } else if (!tztZFTextUtils.isEmpty(sHost1)) {
                return sHost1 + ":" + nPort1;
            } else if (!tztZFTextUtils.isEmpty(sHost2)) {
                return sHost2 + ":" + nPort2;
            }
            return "";
        }

        boolean isSame(tztHostport hostport) {
            if (hostport == null)
                return false;
            boolean result = hostport.toString().equals(this.toString());
            return result && (this.connectType == hostport.connectType);
        }

        tztHostport(String strAdd) {
            this(strAdd, 0);
        }

        tztHostport(String strAdd, int connectType) {
            this.connectType = connectType;
            //防止分隔错误
            if (!strAdd.contains(",")) {
                strAdd += ",";
            }
            if (!strAdd.endsWith(","))
                strAdd = strAdd + ",";
            String[] ayHostPort = tztZFPublicFun.getArray(strAdd, ',');
            nNum = 0;
            //第一台服务器地址
            if (ayHostPort.length > 0) {
                String strHost1 = ayHostPort[0];
                int nPos = strHost1.lastIndexOf(":");
                if (nPos > 0) {
                    sHost1 = strHost1.substring(0, nPos);
                    nPort1 = tztZFPublicFun.parseInt(strHost1.substring(nPos + 1), m_nDefPort);
                } else {
                    sHost1 = strHost1;
                    nPort1 = m_nDefPort;
                }
                nNum++;
            }
            //第二台服务器
            if (ayHostPort.length > 1) {
                String strHost2 = ayHostPort[1];
                int nPos = strHost2.lastIndexOf(":");
                if (nPos > 0) {
                    sHost2 = strHost2.substring(0, nPos);
                    nPort2 = tztZFPublicFun.parseInt(strHost2.substring(nPos + 1), m_nDefPort);
                } else {
                    sHost2 = strHost2;
                    nPort2 = m_nDefPort;
                }
                nNum++;
            }
        }
    }

    //业务处理线程池
    class tztSocketDiscardOldestPolicy implements RejectedExecutionHandler {
        @Override
        public void rejectedExecution(Runnable r, ThreadPoolExecutor executor) {
            // 处理任务被拒绝的情况，记录日志
            //弹出最先加入的任务，空出一个位置，然后再次执行execute方法把任务加入队列。
            if (!executor.isShutdown()) {
                Runnable p = executor.getQueue().poll();
                if (p != null) {
                    tztlog(tztZFLogLevel.tztZFLogLevel_error, "ThreadResp", "%s", p.toString());
                }
                executor.execute(r);
            }
        }
    }

    /** 通讯库版本号 */
    private static final String Version = "V9.01.005.20230303001";
    private static final String tztZFKeyStatistic = "zzstatistic";
    private String m_strAddTag = "";
    //网络事件监听
    private Object context = null;
    //主线程和读线程
    private tztZFThreadPool m_pThreadPool = null;
    //连接线程
    private tztZFThreadPool m_pThreadConnPool = null;
    //应答处理线程
    private ThreadPoolExecutor m_pThreadResp = null;
    private final Object m_pSyncObject = new Object();
    private final Object m_pReadSyncObject = new Object();
    //耗时统计
    private String m_sStatisticVal = "";
    //  private Lock sendLock = new ReentrantLock();// 锁对象
//  private Lock recvLock = new ReentrantLock(true);
    //线程是否运行
    private volatile boolean m_bRun = false;
    private int doWaitTime = 1 * 1000;
    private volatile boolean m_bHandSucess = false;
    private boolean m_bDoConnect = false;
    @tztZFSocketFlags
    private int m_socketStep = tztZFSocketFlags_Init;//连接状态
    //错误号、错误信息
    private int m_nLastError = 0;
    private String m_sLastErrMsg = "";
    //断开连接错误号、错误信息
    private int disConnectErrorNo = 0;
    private String disConnectErrorMsg = "";
    //通讯相关参数
    private String m_sSdkID = "0";
    //券商ID
    private int m_nUserID = 0;
    //类别
    private int m_nSdkType = 0;
    private String m_sLinkkey = "";
    //通道
    private int m_nLinkType = tztZFLinkType.tztZFLinkType_HQ.value();
    //通讯通道标示
    private String m_sSocketKey = "";
    //协商数据包
    private tztZFSocketSendData m_pHandSendData = null;
    private tztZFSocketSendData m_pCheckSendData = null;
    //包序号
    private int m_nPackageindex = 1;
    //短连接
    private boolean m_bShortConn = false;

    // 请求应答队列
    private int m_nMaxReqCount = 20; //待发送队列最大值
    private ArrayList<tztZFSocketSendData> m_pReqList = new ArrayList<tztZFSocketSendData>(); // 待发送队列
    private ConcurrentHashMap<Integer, tztZFSocketSendData> m_pSendedReqList = new ConcurrentHashMap<>();
//  private HashMap<Integer, tztZFSocketSendData> m_pSendedReqList = new HashMap<>();
//  private ArrayList<tztZFSocketSendData> m_pSendedReqList = new ArrayList<tztZFSocketSendData>(); // 已经发送的队列

    private ArrayList<tztZFSocketSendData> m_pPushReqList = new ArrayList<tztZFSocketSendData>(); //主推请求队列
    private ArrayList<tztZFSocketSendData> m_pPushRecList = new ArrayList<tztZFSocketSendData>(); //主推应答队列

    // 地址相关
    private tztHostport m_pConnectAdd = null; //当前连接地址
    private int m_nMaxConn = 10;
    private int m_nConnNum = 0;//本次连接尝试次数

    //最近连接成功地址、端口
    private String m_sLastHostPort = "";
    //切换地址
    private boolean m_bNextAdd = false;

    // 地址相关 - 普通
    private ArrayList<tztHostport> m_ayAddress;
    int m_AddressIndex = 0;
    // 国密地址
    private ArrayList<tztHostport> m_ayGMAddress;
    int m_GMAddressIndex = 0;
    boolean bSetAddress;

    //参数
    private int m_nConnOutTime = 12 * 1000; //连接总超时
    private int m_nOneConnOutTime = 6 * 1000; //Socket单个连接超时时间
    private int m_nSendOutTime = 5 * 1000;  //发送超时
    private int m_nRecvOutTime = 30 * 1000; //应答超时
    private int m_nOnlineTime = 60 * 1000; //心跳间隔时间

    private int m_nMinOutTime = 3 * 1000; //最小超时时间，设置时判断用
    private int m_nCheckOutTime = 6 * 1000; //通讯验证包超时时间
    private int m_nCheckTime = 30 * 1000; //通讯验证包间隔时间
    //网络类型
    private boolean m_isSSL = false;
    private int m_connectType = 0;
    //当前双向连接类别
    private int m_GMConnectType = 0;


    private ByteArrayOutputStream recv_stream = new ByteArrayOutputStream(255);
    private byte[] m_readData = new byte[4096];
    //tztZFGcdSocket,封装的socket
    //当前使用GcdSocket
    private tztZFThreadPoolGCDAsyncSocket m_pSocket = null;
    //GCDSocket回调接口
    private tztZFThreadPoolGCDAsyncSocketEventsInterface m_pGCDSocketEvents = null;

    //连接组
    private tztZFThreadPoolGCDAsyncSocket m_pSocket1DF = null;
    private tztZFThreadPoolGCDAsyncSocket m_pSocket2DF = null;

    //协议数据对象
    private tztZFDataProtocolKit m_pDataProtocolKit = null;

    //连接状态
    private boolean m_bConnected = false;

    //最近请求、应答时间
    private long m_nLastRecvTime = 0;
    private long m_nLastSendTime = 0;
    private long m_nConnectedTime = 0;
    //最近一次应答后的请求时间
    private long m_nRecvAfterFirstSendDataTime = 0;

    //连接回调接口
    private tztZFLinkCallBack m_pLinkBlock = null;

    //网络事件监听
    private tztZFNetAvailable m_pNetAvailable = null;
    private boolean m_isProxy = false;
    private String m_sProxyAdd = "";
    private int m_nProxyPort = 0;
    //网络类型
    private String m_sNetWorkType = "";
    private tztZFCosignUser cosignUser;

    /** 连接国密绑定的账号ID */
    private String gmUserID = "";
    /** 国密回调 */
    private tztZFGMSSLStepCallBack sslStep;
    private tztZFGMSSLStepCallBack GMSSLStep;
    private boolean gmHaveCosignCert = false;
    private boolean bBindGMUser;
    //=====tztZFSocketThreadPool 构造=======================================================

    /**
     * 初始化构造 tztZFSocketThreadPool
     */
    public tztZFThreadPoolSocket() {
        m_sSocketKey = "";
    }

    //=====tztZFSocketThreadPool 设置初始属性=======================================================

    /**
     * 设置属性
     *
     * @param tag 上下文
     * @return tztZFSocketThreadPool
     */
    public tztZFThreadPoolSocket InitWithTag(String tag) {
        this.m_strAddTag = tag;
        return tztZFThreadPoolSocket.this;
    }

    /**
     * 设置属性
     *
     * @param context 上下文
     * @return tztZFSocketThreadPool
     */
    public tztZFThreadPoolSocket InitWithContext(Object context) {
        if (m_bRun)
            return tztZFThreadPoolSocket.this;

        this.context = context;
        return tztZFThreadPoolSocket.this;
    }

    /**
     * 设置属性
     *
     * @param threadPool 线程池
     * @return tztZFSocketThreadPool
     */
    public tztZFThreadPoolSocket InitWithThreadPool(tztZFThreadPool threadPool) {
        if (m_bRun)
            return tztZFThreadPoolSocket.this;

        this.m_pThreadPool = threadPool;
        return tztZFThreadPoolSocket.this;
    }

    /**
     * 设置属性
     *
     * @param userID userID
     * @return tztZFSocketThreadPool
     */
    public tztZFThreadPoolSocket InitWithUserID(int userID) {
        if (m_bRun)
            return tztZFThreadPoolSocket.this;

        this.m_nUserID = userID;
        return tztZFThreadPoolSocket.this;
    }

    /**
     * 设置属性
     *
     * @param sdktype sdktype
     * @return tztZFSocketThreadPool
     */
    public tztZFThreadPoolSocket InitWithSdktype(int sdktype) {
        if (m_bRun)
            return tztZFThreadPoolSocket.this;

        this.m_nSdkType = sdktype;
        return tztZFThreadPoolSocket.this;
    }

    /**
     * 设置属性
     *
     * @param linktype linktype
     * @return tztZFSocketThreadPool
     */
    public tztZFThreadPoolSocket InitWithLinktype(int linktype) {
        if (m_bRun)
            return tztZFThreadPoolSocket.this;

        if (linktype == tztZFLinkType.tztZFLinkType_Non.value()) {
            linktype = tztZFLinkType.tztZFLinkType_HQ.value();
        }
        this.m_nLinkType = linktype;
        return tztZFThreadPoolSocket.this;
    }

    /**
     * 设置属性
     *
     * @param linkKey linkKey
     * @return tztZFSocketThreadPool
     */
    public tztZFThreadPoolSocket InitWithLinkkey(String linkKey) {
        if (m_bRun)
            return tztZFThreadPoolSocket.this;
        this.m_sLinkkey = linkKey;
        return tztZFThreadPoolSocket.this;
    }

    /**
     * 设置属性
     *
     * @param connectType connectType
     * @return tztZFSocketThreadPool
     */
    public tztZFThreadPoolSocket InitWithConnectType(int connectType) {
        if (m_bRun)
            return tztZFThreadPoolSocket.this;

        this.m_connectType = connectType;
        this.m_isSSL = (connectType > 0);
        return tztZFThreadPoolSocket.this;
    }

    public int GetUserID() {
        return m_nUserID;
    }

    public int GetSDKType() {
        return m_nSdkType;
    }

    public int GetLinkType() {
        return m_nLinkType;
    }

    public String GetLinkKey() {
        return m_sLinkkey;
    }

    @tztZFSocketFlags
    public int GetStockStep() {
        return m_socketStep;
    }
    //=====tztZFSocketThreadPool 事件接口=======================================================

    /**
     * 通讯开启
     *
     * @param context     上下文
     * @param userID      userID
     * @param sdkType     sdkType
     * @param linkType    链路类型
     * @param connectType 连接类型（是否使用SSL，1=使用）
     * @return true：开启 false：失败
     */
    public boolean SocketConnect(Object context, int userID, int sdkType, tztZFLinkType linkType, int connectType) {
        return InitWithContext(context)
                .InitWithUserID(userID)
                .InitWithSdktype(sdkType)
                .InitWithLinktype(linkType.value())
                .InitWithConnectType(connectType)
                .SocketConnect();
    }

    /**
     * 通讯开启
     *
     * @return true：开启 false：失败
     */
    public boolean SocketConnect() {
        tztlog("SocketConnect", "%s", "SocketConnect");
        if (m_bRun)
            return true;

        if (context == null || m_pThreadPool == null)
            return false;

        if (this.m_nLinkType <= tztZFLinkType.tztZFLinkType_Non.value())
            return false;

        if (m_socketStep != tztZFSocketFlags_Init)
            return false;

        m_bDoConnect = true;

        m_bShortConn = (this.m_nLinkType >= tztZFLinkType.tztZFLinkType_Short.value() ||
                this.m_nLinkType == tztZFLinkType.tztZFLinkType_JH.value());

        //协议对象
        if (m_pDataProtocolKit == null) {
            m_pDataProtocolKit = new tztZFDataProtocolKit(context, m_nLinkType, new tztZFDataProtocolInterface() {
                @Override
                public void didHandshake(@tztZFHandshakeStatus int status) {
                    if (status != tztZFHandshake_Handing
                            && status != tztZFHandshake_Init) {
                        if (m_bRun) {
                            tztlog("Hand", "status:%s", status + "");
                            doNotify("didHandShake");
                        }
                    }
                }
            });

            m_pThreadResp = new ThreadPoolExecutor(10,      //核心线程数设置成1
                    10,  //线程池最大线程数为1
                    10, TimeUnit.SECONDS, //超过核心线程数量的线程idle 10秒之后会退出
                    new ArrayBlockingQueue<Runnable>(50), //队列长度为50
                    new tztSocketDiscardOldestPolicy() //任务被拒绝以后的处理类
            );

            boolean bLoad = m_pDataProtocolKit.loadProtocolCert(m_nUserID, m_nSdkType);
            if (!bLoad) {
                m_nLastError = m_pDataProtocolKit.getErrorNo();
                String pinLable = tztZFErrorMessageDefine.getUserDefineError(tztZFErrorMessageDefine.tztZFNetWorkErrorKey_PinLable, tztZFErrorMessageDefine.tztZFPinLable);
                m_sLastErrMsg = String.format(Locale.getDefault(), "加载通讯协商%s错误", pinLable);
                m_pDataProtocolKit.free();
                m_pDataProtocolKit = null;
                return false;
            }
            m_sSdkID = m_pDataProtocolKit.getHandAppID();
            m_sSocketKey = String.format(Locale.CHINA, "%d.%d.%d", m_nSdkType, m_nLinkType, this.hashCode());

            if (m_pCheckSendData == null) {
                m_pCheckSendData = new tztZFSocketSendData(new tztZFNetWorkReqObj(), tztZFPackage_Socket, m_sSocketKey);
                m_pCheckSendData.setTimeOut(m_nCheckOutTime);
            }

            if (m_pGCDSocketEvents == null) {

                if (this.m_nLinkType < tztZFLinkType.tztZFLinkType_Short.value()) {
                    m_pNetAvailable = new tztZFNetAvailable(context, new tztZFNetChange() {
                        @Override
                        public void netChange(boolean isAvailable, String netType, boolean isPorxy, String proxyAdd, int proxyPort) {
                            if (isAvailable) {
                                tztZFThreadPoolSocket.this.doNotify("Net Available");
                            }
                            if (!isAvailable || (m_pSocket != null && !m_sNetWorkType.equals(netType))) {
                                tztZFThreadPoolSocket.this.netChanged(netType, isPorxy, proxyAdd, proxyPort);
                            }
                        }
                    });
                    //获取上一次网络类型
                    m_sNetWorkType = m_pNetAvailable.getLastNetWorkType();
                }

                m_pGCDSocketEvents = new tztZFThreadPoolGCDAsyncSocketEventsInterface() {
                    @Override
                    public void didConnectToHost(tztZFThreadPoolGCDAsyncSocket socket, String host, int port) {
                        gcdSocketEvents_doSocketConnectToHost(socket, host, port);
                    }

                    @Override
                    public void socketDidDisconnect(tztZFThreadPoolGCDAsyncSocket socket, int errorNo, String errorMsg) {
                        gcdSocketEvents_doSocketDisconnect(socket, errorNo, errorMsg);
                    }

                    @Override
                    public int socketReadData(tztZFThreadPoolGCDAsyncSocket socket, byte[] data, int len) {
                        if (socket == m_pSocket) {
                            return recvData(data, len);
                        }
                        return -1;
                    }
                };

                m_bRun = true;
                m_pConnectAdd = getConnectHostPort(false);
                if (m_nLinkType == tztZFLinkType_JY.value()) {
                    this.sslStep = new tztZFGMSSLStepCallBack() {
                        @Override
                        public void onCallBack(@tztZFSocketSSLStep int step, int code, String certUser) {
                            if (GMSSLStep != null) {
                                GMSSLStep.onCallBack(step, code, certUser);
                            } else {
                                tztZFNotificationObj notificationObj = new tztZFNotificationObj();
                                notificationObj.setName(tztZFJYLoginGMProcessStep);
                                tztZFDataMapBase<String, Object> dataMapBase = new tztZFDataMapBase<>();
                                dataMapBase.setInt("step", tztZFSocketSSLStepClose);
                                if (certUser == null)
                                    certUser = "";
                                dataMapBase.setStr("userID", certUser);
                                notificationObj.setData(dataMapBase);
                                EventBus.getDefault().post(notificationObj);
                            }
                        }
                    };
                }
                try {
                    m_pThreadPool.useThreadPool().execute(new Runnable() {
                        @Override
                        public void run() {
                            while (m_bRun) {
                                try {
                                    tztZFThreadPoolSocket.this.readSyncRun();
                                } catch (Exception e) {
                                    tztZFThreadPoolSocket.this.CloseConnect(tztZFNetWorkError_Disconnect, tztZFErrorMessageDefine.getUserDefineError(tztZFNetWorkErrorKey_ReadError,
                                            tztZFReadError));
                                }
                            }
                        }
                    });
                    m_pThreadPool.useThreadPool().execute(new Runnable() {
                        @Override
                        public void run() {
                            while (m_socketStep != tztZFSocketFlags_End) {
                                tztZFThreadPoolSocket.this.syncRun();
                            }
                        }
                    });
                } catch (Exception e) {
                    Quit();
                    freeSocket(true);
                    return false;
                }

                tztlog("init", "初始化连接! Ver:%s", Version);
                return true;
            }
        }
        return false;
    }

    //=====tztZFSocketThreadPool GCDSocket连接回调接口=======================================================
    //连接成功回调处理
    private synchronized void gcdSocketEvents_doSocketConnectToHost(tztZFThreadPoolGCDAsyncSocket Owner, String host, int port) {
        //已设置连接，且非当前连接，直接断开
        if (m_pSocket != null && m_pSocket != Owner) {
            closeGCDSocket(Owner, "已有连接，连接断开");
            return;
        }
        //未设置连接，设置连接，断开其它连接
        if (m_pSocket == null) {
            m_pSocket = Owner;
            disConnectErrorNo = 0;
            disConnectErrorMsg = "";
            //重置连接尝试次数
            m_nConnNum = 0;
            //最近应答时间
            m_nLastSendTime = System.currentTimeMillis();
            m_nLastRecvTime = System.currentTimeMillis();
            //最近连接成功地址
            m_sLastHostPort = host + ":" + port;
            Object userdata = m_pSocket.userData;
            if (m_connectType == 9 && userdata != null && userdata instanceof tztHostport) {
                m_GMConnectType = ((tztHostport) userdata).connectType;
                //交易通道判断降级，通知上层
                if (m_GMConnectType != 9 && m_nLinkType == tztZFLinkType_JY.value()) {
                    if (didGMDowngrade(false))
                        return;
                }
                loadSSLHandShakeCert(m_GMConnectType);
            } else {
                loadSSLHandShakeCert(m_connectType);
            }

            if (m_connectType == 8 || (m_connectType == 9 && tztZFNetCosign.getCosignStatus())) {
                m_pSocket.setGMSSLStepCallBack(this.sslStep);
            }
            //重置协议数据
            if (m_pDataProtocolKit != null) {
                m_pDataProtocolKit.onProtocolInit();
            }
            //重置协议包序号
            m_nPackageindex = 1;
            tztlog("Connect", "OK:%s", m_sLastHostPort);
            m_socketStep = tztZFSocketFlags_Connect;
            if (m_bShortConn) {
                m_bHandSucess = true;
                m_socketStep = tztZFSocketFlags_ConnSucess;
                m_nConnectedTime = System.currentTimeMillis();
                //连接成功，不切换地址
                m_bNextAdd = false;
                dealConnType(tztZFConn_Connected);
            } else {
                if (!doProtocolHandshakeReq()) {
                    CloseConnect(tztZFNetWorkError_Hand, tztZFErrorMessageDefine.getUserDefineError(tztZFNetWorkErrorKey_WriteError, tztZFWriteError));
                }
            }
            closeOtherGCDSocket(Owner);
        }
        doNotify("Socket didConnect");
    }

    //连接失败回调处理
    private synchronized void gcdSocketEvents_doSocketDisconnect(tztZFThreadPoolGCDAsyncSocket Owner, int errorNo, String errorMsg) {
        Owner.setEvents(null);
        if (Owner == m_pSocket1DF) {
            m_pSocket1DF = null;
        } else if (Owner == m_pSocket2DF) {
            m_pSocket2DF = null;
        }
        //已设置连接,是当前设置的连接断开
        if (m_pSocket == Owner) {
            m_bNextAdd = true;
            closeSocket(errorNo, errorMsg);
        }

        if (m_pSocket == null && m_pSocket1DF == null && m_pSocket2DF == null) {
            if (m_nConnNum >= m_nMaxConn) {
                m_nConnNum = 0;
                //所有服务器连接均失败，短连接或网络状态不正常，待发请求返回失败
                if (m_bShortConn || (m_pNetAvailable != null && !m_pNetAvailable.checkNetworkAvailable())) {
                    doClearReqList(tztZFRecvType.tztZFConn_Non);
                    clearPushRespList(tztZFRecvType.tztZFConn_Non);
                }
                //返回网络错误消息
                dealConnType(tztZFRecvType.tztZFConn_Non, true);
            }
            checkNetWorkAvailableProxy();
            doNotify("Socket disConn");
        }
    }

    private synchronized void closeSocket(int errorNo, String errorMessage) {
        if (m_pSocket != null) {
            tztlog("Close", "[%d]%s", errorNo, errorMessage);
            recv_stream.reset();
            m_pSocket.setEvents(null);
            m_pSocket.Close(errorMessage);
            m_pSocket = null;

            m_bHandSucess = false;
            m_nConnectedTime = 0;
            m_nLastError = errorNo;
            m_sLastErrMsg = errorMessage;
            disConnectErrorNo = errorNo;
            disConnectErrorMsg = errorMessage;
            if (m_socketStep != tztZFSocketFlags_End) { //已经断开
                m_socketStep = tztZFSocketFlags_ConnFail;
            }

            closeGCDSocket(m_pSocket1DF, errorMessage);
            closeGCDSocket(m_pSocket2DF, errorMessage);

            if (m_pThreadConnPool != null) {
                m_pThreadConnPool.shutdown();
                m_pThreadConnPool = null;
            }
            //断开连接，待应答请求返回失败
            doClearSendedReqList(tztZFConn_DisConnect);
            //返回网络断开消息
            dealConnType(tztZFConn_DisConnect);
        }
    }

    private void closeGCDSocket(tztZFThreadPoolGCDAsyncSocket GCDSocket, String msg) {
        if (m_pSocket1DF != null && m_pSocket1DF == GCDSocket) {
            m_pSocket1DF.setEvents(null);
            m_pSocket1DF.Close(msg);
            m_pSocket1DF = null;
        } else if (m_pSocket2DF != null && m_pSocket2DF == GCDSocket) {
            m_pSocket2DF.setEvents(null);
            m_pSocket2DF.Close(msg);
            m_pSocket2DF = null;
        }
    }

    private void closeOtherGCDSocket(tztZFThreadPoolGCDAsyncSocket GCDSocket) {
        if (m_pSocket1DF == GCDSocket) {
            if (m_pSocket2DF != null) {
                m_pSocket2DF.setEvents(null);
                m_pSocket2DF.Close("关闭其他连接");
                m_pSocket2DF = null;
            }
        } else if (m_pSocket2DF == GCDSocket) {
            if (m_pSocket1DF != null) {
                m_pSocket1DF.setEvents(null);
                m_pSocket1DF.Close("关闭其他连接");
                m_pSocket1DF = null;
            }
        }
    }
    //=====tztZFSocketThreadPool 设置、获取属性=======================================================

    /**
     * 设置服务器地址
     *
     * @param addList     服务器地址
     * @param selectIndex 地址索引
     */
    public synchronized void SetAddressList(String addList, int selectIndex) {
        SetAddressList(addList, null, selectIndex);
    }

    public void SetAddressList(String addList, String nameList, int selectIndex) {
        threadR_setAddressList(addList, nameList, selectIndex);
    }

    public void setGMAddressList(String addList, String nameList, int selectIndex) {
        thread_didGMAddressList(addList, nameList, selectIndex);
    }

    private void didAddress(String addList, String nameList, int selectIndex) {
        if (tztZFTextUtils.isEmpty(addList))
            return;
        if (m_ayAddress == null)
            m_ayAddress = new ArrayList<>();
        m_ayAddress.clear();
        if (!addList.endsWith("&"))
            addList = addList + "&";
        String[] array = tztZFPublicFun.getArray(addList, '&');
        if (array.length > 0) {
            synchronized (this) {
                for (int i = 0; i < array.length; i++) {
                    String strHostport = array[i];
                    if (tztZFTextUtils.isEmpty(strHostport)) {
                        continue;
                    }
                    m_ayAddress.add(new tztHostport(strHostport));
                }
            }
        }
        m_AddressIndex = selectIndex;
        m_bNextAdd = false;
        bSetAddress = true;
        m_nConnNum = 0;
        if (selectIndex >= 0 || m_pConnectAdd == null) {
            m_pConnectAdd = getConnectHostPort(false);
        }
        if (selectIndex >= 0) {
            changedAddAndDidDisconnect();
        }
        if (m_bDoConnect) {
            SocketConnect();
        }
    }

    private boolean changedAddAndDidDisconnect() {
        boolean isDidDisconn = false;
        if (m_ayAddress == null && m_ayGMAddress == null) {
            return false;
        }
        int adressCount = m_ayAddress == null ? 0 : m_ayAddress.size();
        adressCount += m_ayGMAddress == null ? 0 : m_ayGMAddress.size();
        if (adressCount < 1 || m_pSocket == null || !m_pSocket.IsConnected())
            return false;
        m_pConnectAdd = getConnectHostPort(false);
        if (m_pConnectAdd != null && m_pSocket.userData != null && m_pSocket.userData instanceof tztHostport) {
            tztHostport usedhostPort = (tztHostport) m_pSocket.userData;
            if (!usedhostPort.isSame(m_pConnectAdd)) {
                isDidDisconn = true;
                closeSocket(tztZFNetWorkError_Disconnect, "切换服务器，断开连接");
            }
        }
        m_bNextAdd = false;
        bSetAddress = false;
        return isDidDisconn;
    }

    private tztHostport getConnectHostPort(boolean bNext) {
        if (m_connectType == 9 && tztZFNetCosign.getCosignStatus() && m_ayGMAddress != null) {
            m_nMaxConn = m_ayGMAddress.size();
            if (m_ayAddress != null) {
                m_nMaxConn += m_ayAddress.size();
            }
            return getNextGMHostPort(bNext);
        } else if (m_ayAddress != null) {
            m_nMaxConn = m_ayAddress.size();
            return getNextHostPort(bNext);
        }
        return null;
    }

    private void threadR_setAddressList(String addList, String nameList, int selectIndex) {
        tztlog("setAddress", "设置服务器地址列表[%d]:%s", selectIndex, addList);
        didAddress(addList, nameList, selectIndex);
        if (m_bDoConnect) {
            SocketConnect();
        }
    }

    /**
     * 设置连接回调接口
     *
     * @param linkCallBack 连接回调
     */
    public final void SetLinkBlock(tztZFLinkCallBack linkCallBack) {
        this.m_pLinkBlock = linkCallBack;
    }

    /**
     * 获取设置的连接回调接口
     *
     * @return tztZFLinkBlock
     */
    public tztZFLinkCallBack GetLinkBlock() {
        return m_pLinkBlock;
    }

    /**
     * 设置连接超时时间,单位毫秒
     *
     * @param timeout 超时时间
     */
    public void SetConnectOutTime(int timeout) {
        if (timeout < m_nMinOutTime)
            return;
        m_nOneConnOutTime = timeout;
        //根据地址列表设置计算超时时间
        m_nConnOutTime = Math.max(m_nMinOutTime, timeout) * 2 - 1;
    }

    /**
     * 设置应答超时时间,单位毫秒
     *
     * @param timeout 超时时间
     */
    public void SetRecvOutTime(int timeout) {
        if (timeout < m_nMinOutTime)
            return;
        m_nRecvOutTime = timeout;
    }

    /**
     * 获取应答超时时间,单位毫秒
     *
     * @return 设置的应答超时时间
     */
    public int GetRecvOutTime() {
        return m_nRecvOutTime;
    }

    /**
     * 设置待发送队列最多有多大，超过了就清空队列
     *
     * @param maxCount 待发送队列最大个数
     */
    public void SetMaxReqCount(int maxCount) {
        if (maxCount > 3)
            m_nMaxReqCount = maxCount;
    }

    /**
     * 获取stocket的hashcode值
     *
     * @return stocket的hashcode值
     */
    public int GetSocketID() {
        if (m_pSocket == null)
            return 0;
        else {
            return m_pSocket.GetSocketID();
        }
    }

    /**
     * 当前网络是否在连接状态
     */
    public boolean IsConnected() {
        if (!m_bRun)
            return false;
        if (m_socketStep != tztZFSocketFlags_Connect
                && m_socketStep != tztZFSocketFlags_ConnSucess)
            return false;
        if (m_pSocket == null)
            return false;
        return m_pSocket.IsConnected();
    }

    /**
     * 获取当前连接使用的地址信息(地址:端口)
     *
     * @return 当前使用的地址信息
     */
    public String GetSocketSuccessedHostPort() {
        if (tztZFTextUtils.isEmpty(m_sLastHostPort)) {
            if (m_pSocket != null && m_pSocket.IsConnected()) {
                return m_pSocket.GetAddressPort();
            } else {
                return "";
            }
        }
        return m_sLastHostPort;
    }

    /**
     * 获取当前设置的连接地址信息(地址:端口)
     *
     * @return 当前设置的连接地址信息
     */
    public String GetSocketHostPort() {
        if (m_pConnectAdd != null)
            return m_pConnectAdd.toString();
        return m_sLastHostPort;
    }

    /**
     * 获取当前设置的连接地址信息(地址:端口)
     *
     * @return 当前设置的连接地址信息
     */
    public String GetSocketHostPortGM() {
        if (m_ayGMAddress != null && m_GMAddressIndex >= 0 && m_GMAddressIndex < m_ayGMAddress.size()) {
            return m_ayGMAddress.get(m_GMAddressIndex).toString();
        } else if (m_pConnectAdd != null) {
            return m_pConnectAdd.toString();
        }
        return m_sLastHostPort;
    }

    /**
     * 获取当前设置的连接地址信息(地址:端口)
     *
     * @return 当前设置的连接地址信息
     */
    public String GetSocketHostPortPT() {
        if (m_ayAddress != null && m_AddressIndex >= 0 && m_AddressIndex < m_ayAddress.size()) {
            return m_ayAddress.get(m_AddressIndex).toString();
        } else if (m_pConnectAdd != null) {
            return m_pConnectAdd.toString();
        }
        return m_sLastHostPort;
    }


    public int getConnectType() {
        return m_connectType;
    }

    /**
     * 是否使用SSL
     *
     * @return 是否使用SSL
     */
    public boolean IsSSL() {
        return m_isSSL;
    }
    //=====tztZFSocketThreadPool 线程=======================================================

    /**
     * 启动过通讯线线程
     *
     * @throws NullPointerException 空指针异常
     */
    private void syncRun() throws NullPointerException {
        try {
            while (m_bRun && haveReqAction() || havePushReqAction()) {
                if (doGCDSocketConnect() && doProtocolHandshake()) {
                    if (!sendRequestData())
                        break;
                } else if (!doCheckReqListTimeOut()) { //判断请求超时
                    break;
                }
            }

            doCheckSendedReqListTimeOut();// 处理应答超时
            doSendHeartBeat(); //发送心跳
            QueueGMCosignDisconnect();
            if (m_bRun && m_pSyncObject != null) {
                synchronized (m_pSyncObject) {
                    if (haveReqAction() || getSendedReqCount() > 0) {
                        doWait(Thread.currentThread(), 1 * 1000);
                    } else if (IsConnected() || checkNetWorkAvailableProxy()) {
                        doWait(Thread.currentThread(), 3 * 1000);
                    } else {
                        if (doWaitTime >= 59 * 1000)
                            doWait(Thread.currentThread(), 59 * 1000);
                        else {
                            doWait(Thread.currentThread(), doWaitTime);
                        }
                    }
                }
            }

            if (!m_bRun) {
                freeSocket(false);
            }
        } catch (Exception e) {
            tztlog("Run", "%s", tztZFLog.getStackTraceString(e));
        }
    }

    private void QueueGMCosignDisconnect() {
        if (m_connectType != 8)
            return;
        if (!IsConnected())
            return;
        if (!haveReqAction() && (System.currentTimeMillis() - m_nLastRecvTime) >= 5 * 60 * 1000) {
            String gmLable = tztZFErrorMessageDefine.getUserDefineError(tztZFErrorMessageDefine.tztZFNetWorkErrorKey_GMLable, tztZFErrorMessageDefine.tztZFGMLable);
            m_sLastErrMsg = String.format(Locale.getDefault(), "%s单向主动断开连接", gmLable);
            disConnectErrorMsg = m_sLastErrMsg;
            CloseConnect(tztZFConn_DisConnect.value(), m_sLastErrMsg);
        }
    }

    /**
     * 终止线程
     */
    private void stopThread() {
        m_bRun = false;
        doNotify("stopThread");
        readDoNotify();
    }

    private void doNotify(String tag) {
        if (tag == null)
            tag = "";
        tztlog("run", "%s-%s", "doNotify", tag);
        if (m_pSyncObject == null)
            return;
        doWaitTime = 0;
        synchronized (m_pSyncObject) {
            m_pSyncObject.notify();
        }
    }

    private void doWait(Thread thread, int Howlong) {
        try {
            tztlog("run", "doWait:%d", Howlong);
            if (m_pSyncObject == null)
                return;
            if (doWaitTime > 0 && Howlong > 0) {
                m_pSyncObject.wait(Howlong);
            }
            if (doWaitTime < 59 * 1000) {
                doWaitTime += 1000;
            }
        } catch (Exception e) {
        }
    }
    //=====tztZFSocketThreadPool 连接=======================================================

    /**
     * 获取本次连接地址
     *
     * @return tztHostport
     */
    private tztHostport getNextGMHostPort(boolean bNext) {
        if (m_ayGMAddress == null || m_ayGMAddress.size() < 1)
            return getNextHostPort(bNext);
        boolean changeNext = bNext && (!bBindGMUser);
        bBindGMUser = false;
        if (changeNext) {
            m_GMAddressIndex++;
        }
        if (m_GMAddressIndex < 0)
            m_GMAddressIndex = 0;
        if (m_GMAddressIndex >= 0 && m_GMAddressIndex < m_ayGMAddress.size()) {
            m_bNextAdd = false;
            return m_ayGMAddress.get(m_GMAddressIndex);
        }
        m_GMAddressIndex = 0;
        m_bNextAdd = false;
        return getNextHostPort(bNext);
    }

    private tztHostport getNextHostPort(boolean bNext) {
        if (m_ayAddress == null || m_ayAddress.size() < 1)
            return null;
        if (bNext) {
            m_AddressIndex++;
        }
        if (m_AddressIndex < 0)
            m_AddressIndex = 0;
        if (m_AddressIndex >= 0 && m_AddressIndex < m_ayAddress.size()) {
            return m_ayAddress.get(m_AddressIndex);
        }
        m_AddressIndex = 0;
        m_bNextAdd = false;
        return m_ayAddress.get(0);
    }

    private boolean isGCDSocketTimeOut() {
        if (m_pSocket1DF != null && m_pSocket1DF.IsTimeOut()) {
            m_pSocket1DF.Close("超时关闭");
            m_pSocket1DF = null;
        }

        if (m_pSocket2DF != null && m_pSocket2DF.IsTimeOut()) {
            m_pSocket2DF.Close("超时关闭");
            m_pSocket2DF = null;
        }
        boolean bTimeOut = (m_pSocket1DF == null && m_pSocket2DF == null && m_pSocket == null);
        if (bTimeOut && m_pThreadConnPool != null) {
            m_pThreadConnPool.shutdown();
            m_pThreadConnPool = null;
        }
        return bTimeOut;
    }

    /**
     * GCDSocket是否都已断开
     *
     * @return true：都已断开
     */
    private boolean isGCDSocketClosing() {
        if (m_pSocket1DF != null && m_pSocket1DF.IsClosing()) {
            m_pSocket1DF = null;
        }
        if (m_pSocket2DF != null && m_pSocket2DF.IsClosing()) {
            m_pSocket2DF = null;
        }
        boolean bClosed = (m_pSocket1DF == null && m_pSocket2DF == null && m_pSocket == null);
        if (bClosed && m_pThreadConnPool != null) {
            m_pThreadConnPool.shutdown();
            m_pThreadConnPool = null;
        }
        return bClosed;
    }

    private boolean isGCDSocketClosed() {
        if (m_pSocket1DF != null && m_pSocket1DF.IsClosed()) {
            m_pSocket1DF = null;
        }
        if (m_pSocket2DF != null && m_pSocket2DF.IsClosed()) {
            m_pSocket2DF = null;
        }
        boolean bClosed = (m_pSocket1DF == null && m_pSocket2DF == null && m_pSocket == null);
        if (bClosed && m_pThreadConnPool != null) {
            m_pThreadConnPool.shutdown();
            m_pThreadConnPool = null;
        }
        return bClosed;
    }

    /**
     * 连接：进行Socket连接
     *
     * @return true：已连接成功 false：未连接成功
     */
    private synchronized boolean doGCDSocketConnect() {
        //已连接成功
        if (IsConnected())
            return true;
        //都已断开，重新连接
        if (m_bRun && (isGCDSocketClosing() || isGCDSocketTimeOut())) {
            m_pConnectAdd = getConnectHostPort(m_bNextAdd);
            if (m_pConnectAdd == null) {
                m_nLastError = tztZFNetWorkError_NoConnect;
                m_sLastErrMsg = "通讯错误！请设置连接地址列表";
                tztlog("Connect", "%s", m_sLastErrMsg);
                //返回网络错误消息
                dealConnType(tztZFConn_Non);
                return false;
            }
            m_bNextAdd = true;
            if (this.sslStep != null) {
                this.sslStep.onCallBack(tztZFSocketSSLStepSocket, 0, m_pSocket != null ? m_pSocket.getCertUser() : "");
            }
            if (m_pThreadConnPool != null) {
                m_pThreadConnPool.shutdownNow();
                m_pThreadConnPool = null;
            }
            m_pThreadConnPool = new tztZFThreadPool(m_pConnectAdd.nNum, m_pConnectAdd.nNum, m_sSocketKey);
            m_pThreadConnPool.setUsed(true);
            tztlog("Connect", "[%d]Host:%s,connecttype:%d", m_pConnectAdd.connectType, m_pConnectAdd.toString(), m_connectType);
            m_socketStep = tztZFSocketFlags_Conning;
            String proxyAdd = "";
            int proxyPort = 0;
            if (m_pNetAvailable != null) {
                proxyAdd = m_pNetAvailable.getProxyAddress();
                proxyPort = m_pNetAvailable.getProxyPort();
            }

            m_nConnNum++;
            //创建连接1
            if (!tztZFTextUtils.isEmpty(m_pConnectAdd.sHost1) && m_pConnectAdd.nPort1 > 0) {
                m_pSocket1DF = new tztZFThreadPoolGCDAsyncSocket();
                try {
                    m_pSocket1DF.InitWithContext(this.context)
                            .InitWithSocketKey(m_sSocketKey)
                            .InitWithIndex("1")
                            .InitWithThreadPool(m_pThreadConnPool.useThreadPool())
                            .InitWithUserID(m_nUserID)
                            .InitWithSdktype(m_nSdkType)
                            .InitWithLinktype(m_nLinkType)
                            .InitWithAddress(m_pConnectAdd.sHost1)
                            .InitWithPort(m_pConnectAdd.nPort1)
                            .InitWithIsSSL(m_isSSL)
                            .InitWithEvents(m_pGCDSocketEvents)
                            .InitWithIsProxy(m_isProxy)
                            .InitWithProxyAdd(proxyAdd, proxyPort)
                            .InitWithConnectOutTime(m_nOneConnOutTime);

                    loadSSLCert(m_pSocket1DF);
                    if (m_connectType != 9) {
                        m_pSocket1DF.tztZF_SetConnecttype(m_connectType);
                    } else {
                        m_pSocket1DF.tztZF_SetConnecttype(m_pConnectAdd.connectType);
                    }
                    m_pSocket1DF.userData = m_pConnectAdd;
                    m_pSocket1DF.Connect();
                } catch (Exception e) {
                    if (m_pSocket1DF != null) {
                        m_pSocket1DF.Close(e.getMessage());
                    }
                    m_pSocket1DF = null;
                }
            }
            //创建连接2
            if (!tztZFTextUtils.isEmpty(m_pConnectAdd.sHost2) && m_pConnectAdd.nPort2 > 0) {
                m_pSocket2DF = new tztZFThreadPoolGCDAsyncSocket();
                try {
                    m_pSocket2DF.InitWithContext(this.context)
                            .InitWithSocketKey(m_sSocketKey)
                            .InitWithIndex("2")
                            .InitWithThreadPool(m_pThreadConnPool.useThreadPool())
                            .InitWithUserID(m_nUserID)
                            .InitWithSdktype(m_nSdkType)
                            .InitWithLinktype(m_nLinkType)
                            .InitWithAddress(m_pConnectAdd.sHost2)
                            .InitWithPort(m_pConnectAdd.nPort2)
                            .InitWithIsSSL(m_isSSL)
                            .InitWithEvents(m_pGCDSocketEvents)
                            .InitWithIsProxy(m_isProxy)
                            .InitWithProxyAdd(proxyAdd, proxyPort)
                            .InitWithConnectOutTime(m_nOneConnOutTime);
                    loadSSLCert(m_pSocket2DF);
                    if (m_connectType != 9) {
                        m_pSocket2DF.tztZF_SetConnecttype(m_connectType);
                    } else {
                        m_pSocket2DF.tztZF_SetConnecttype(m_pConnectAdd.connectType);
                    }
                    m_pSocket2DF.userData = m_pConnectAdd;
                    m_pSocket2DF.Connect();
                } catch (Exception e) {
                    if (m_pSocket2DF != null) {
                        m_pSocket2DF.Close(e.getMessage());
                    }
                    m_pSocket2DF = null;
                }
            }
        }
        return false;
    }

    /**
     * 连接回调
     */
    private void dealConnType(tztZFRecvType type) {
        dealConnType(type, false);
    }

    /**
     * 连接回调
     *
     * @param type   类型
     * @param isLast 地址列表中最后一个地址=true
     */
    private void dealConnType(tztZFRecvType type, boolean isLast) {
        if (m_pLinkBlock != null && m_pThreadResp != null) {
            tztZFNetWorkRespObj resp = new tztZFNetWorkRespObj();
            if (type != tztZFRecvType.tztZFConn_Connected) {
                resp.setStatusCode(m_nLastError);
                resp.setMessage(m_sLastErrMsg);
            } else {
                resp.setStatusCode(0);
            }
            resp.setRecvType(type);
            tztZFDataMapBase<String, Object> data = new tztZFDataMapBase<>();
            data.setStr("SDKType", "" + m_nSdkType);
            data.setStr("LinkType", "" + m_nLinkType);
            if (m_pConnectAdd != null) {
                data.setStr("Address", m_pConnectAdd.toString());
            }
            data.setStr("isLastAddress", isLast ? "1" : "0");
            data.setInt("RecvType", type.value());
            resp.setResponseData(data);
            final tztZFNetWorkRespObj connResp = resp;
            m_pThreadResp.execute(new tztZFThreadRespRunnable(m_nLinkType, 0, data) {
                @Override
                public void run() {
                    m_pLinkBlock.onHandler(tztZFLinkCallBackType.tztZFLinkCallBackType_NetState.value(), m_nSdkType, m_nLinkType, connResp);
                }
            });
        }
    }

    /**
     * 数据回调
     */
    private void dealPush(tztZFDataMapBase<String, Object> data) {
        if (m_pLinkBlock != null && m_pThreadResp != null) {
            int errorNo = 0;
            tztZFNetWorkRespObj resp = new tztZFNetWorkRespObj();
            if (data != null) {
                resp.setResponseData(data);
            } else {
                resp.setStatusCode(errorNo);
            }
            resp.setRecvType(tztZFRecv_DataOK);
            final tztZFNetWorkRespObj pushResp = resp;
            m_pThreadResp.execute(new tztZFThreadRespRunnable(m_nLinkType, 0, data) {
                @Override
                public void run() {
                    m_pLinkBlock.onHandler(tztZFLinkCallBackType.tztZFLinkCallBackType_PostData.value(), m_nSdkType, m_nLinkType, pushResp);
                }
            });
        }
    }
    //=====tztZFSocketThreadPool 请求队列=======================================================

    /**
     * 加入请求到m_pReqList
     *
     * @param j     请求
     * @param bPush 是否为推送
     */
    private boolean addAction(tztZFSocketSendData j, boolean bPush) {
        if (j.getAction() == 0) {
            responseError(j, tztZFSend_SendError, tztZFNetWorkError_Send, tztZFErrorMessageDefine.getUserDefineError(tztZFNetWorkErrorKey_WriteError, tztZFWriteError));
            return false;
        }
        if (!m_bRun) {
            responseError(j, tztZFConn_Non, tztZFNetWorkError_NoConnect, tztZFErrorMessageDefine.getUserDefineError(tztZFNetWorkErrorKey_WriteError, tztZFWriteError));
            return false;
        }

        if (getReqListCount() > m_nMaxReqCount) { // 请求累积，强行断线
            doClearReqList(tztZFSend_MaxError);
            CloseConnect(tztZFNetWorkError_Send, "请求累积");
        }

        if (m_nLinkType == tztZFLinkType_JY.value() && tztZFNetCosign.getCosignStatus() && m_connectType == 9) {
            j.setStr("tztGMSSL", "1");
            if (!gmHaveCosignCert) {
                String pinLable = tztZFErrorMessageDefine.getUserDefineError(tztZFErrorMessageDefine.tztZFNetWorkErrorKey_PinLable, tztZFErrorMessageDefine.tztZFPinLable);
                if (cosignUser == null) {
                    cosignUser = tztZFNetCosign.getBindCosignUser();
                }
                if (cosignUser == null) {
                    responseError(j, tztZFSend_SendError, tztZFNetWorkError_NoConnect, tztZFErrorMessageDefine.getUserDefineError(tztZFErrorMessageDefine.tztZFNetWorkErrorKey_NoUserCert,
                            String.format(Locale.getDefault(), "未设置用户有效加密%s", pinLable)));
                    return false;
                } else if (cosignUser.isNeedGMLogin() && !cosignUser.isValidity()) {
                    String err = tztZFErrorMessageDefine.getUserDefineError(tztZFErrorMessageDefine.tztZFNetWorkErrorKey_CertInvalid, "请确认申请国密安全码且在有效期内");
                    if (tztZFLog.getLevel() <= tztZFLogLevel_Info.value()) {
                        int nStatus = cosignUser.getLoginCosignCert() == null ? -1 : cosignUser.getLoginCosignCert().getnHave();
                        err = String.format(Locale.getDefault(), "%s:[Status:%d][UserID:]", err, nStatus, cosignUser.userID() != null ? cosignUser.userID() : "");
                    }
                    responseError(j, tztZFSend_SendError, tztZFNetWorkError_NoConnect, err);
                    return false;
                }
                gmHaveCosignCert = true;
            }
        }

        synchronized (m_pReqList) {
            j.setAddTime();
            m_pReqList.add(j);
            if (bPush) {
                addActionPush(j);
            }
            tztlog("addAction", "[Action:%d]加入请求队列", j.getAction());
        }
        doNotify("AddAction");
        return true;
    }

    /**
     * 发送请求
     * 加入请求到m_pReqList
     *
     * @param reqObj   请求
     * @param callBack 回调
     */
    public boolean OnSendReqModel(tztZFNetWorkReqObj reqObj, tztZFNetWorkResponseCallBack callBack) {
        if (reqObj != null) {
            tztlog("OnSendReqModel", "[Action:%s]", reqObj.getString("Action"));
        }
        return tztZFSocketSend(reqObj, callBack);
    }

    private boolean tztZFSocketSend(tztZFNetWorkReqObj reqObj, tztZFNetWorkResponseCallBack callBack) {
        if (callBack == null)
            return false;
        @tztZFPackageType int type = tztZFPackage_Data;
        tztZFSocketSendData sendData = new tztZFSocketSendData(reqObj, type, m_sSocketKey);
        sendData.setResponseCallBack(callBack);
        if (!m_bRun) {
            return responseError(sendData, tztZFConn_Non, tztZFNetWorkError_NoConnect, tztZFErrorMessageDefine.getUserDefineError(tztZFNetWorkErrorKey_WriteError, tztZFWriteError));
        }

        if (reqObj == null || reqObj.getSendValues() == null) {
            return responseError(sendData, tztZFSend_SendError, tztZFNetWorkError_Send, tztZFErrorMessageDefine.getUserDefineError(tztZFNetWorkErrorKey_WriteError, tztZFWriteError));
        }

        int nAction = reqObj.getInt("Action");
        if (nAction <= 0) {
            return responseError(sendData, tztZFSend_SendError, tztZFNetWorkError_Send, tztZFErrorMessageDefine.getUserDefineError(tztZFNetWorkErrorKey_WriteError, tztZFWriteError));
        }
        int nTimeout = reqObj.getTimeOut();
        if (nTimeout == 0) {
            if (m_bShortConn) {
                nTimeout = 6 * 1000;
            } else {
                nTimeout = m_nRecvOutTime;
            }
        }
        sendData.setTimeOut(nTimeout);
        if (reqObj.isRetainBlock() || isPushAction(nAction)) {
            type = tztZFPackage_AutoPush;
            reqObj.setIsRetainBlock(true);
            sendData.setType(type);
        }
        return addAction(sendData, reqObj.isRetainBlock());
    }

    /**
     * 获取待请求数
     *
     * @return 获取待请求列表数量
     */
    private int getReqListCount() {
        //pjl 如果密钥协商不成功则不能发送具体的请求
        if (m_pReqList == null || m_pReqList.size() == 0)
            return 0;
        synchronized (m_pReqList) {
            return m_pReqList.size();
        }
    }

    /**
     * 从列表移除请求对象
     */
    private void removeReqList(tztZFSocketSendData req) {
        if (m_pReqList == null || req == null)
            return;
        synchronized (m_pReqList) {
            m_pReqList.remove(req);
        }
    }

    private void doClearReqList(tztZFRecvType type) {
        try {
            if (m_pReqList == null)
                return;
            synchronized (m_pReqList) {
                for (int len = m_pReqList.size(), i = len - 1; i >= 0; i--) {
                    tztZFSocketSendData req = m_pReqList.get(i);
                    m_pReqList.remove(i); //清一个删除一个，防止被递归调用，导致程序蹦溃
                    if (req.isReSend && type == tztZFRecvType.tztZFConn_Non) {
                        //连接失败才允许处理重发
                        req.setAddTime();
                        req.setIsReSend(false);
                        addAction(req, req.isRetainBlock());
                    } else {
                        responseError(req, type, tztZFNetWorkError_Send, tztZFErrorMessageDefine.getUserDefineError(tztZFNetWorkErrorKey_TimeOutError, tztZFOutTimeError));
                        tztlog("clearReqList", "[Action:%s]通讯错误[%d]", req.getAction(), type.value());
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 获取某个待请求数据
     *
     * @return 请求
     */
    private tztZFSocketSendData getAction(int id) {
        if (m_pReqList == null || id < 0)
            return null;
        synchronized (m_pReqList) {
            if (id < m_pReqList.size()) {
                return m_pReqList.get(id);// 得到请求数据
            } else {
                return null;
            }
        }
    }

    /**
     * 有待发请求
     *
     * @return true：有 false：无
     */
    private boolean haveReqAction() {
        if (!m_bRun)
            return false;
        if (m_pReqList != null && m_pReqList.size() > 0)
            return true;

        return false;
    }

    private boolean havePushReqAction() {
        if (!m_bRun)
            return false;
        if (m_pPushReqList != null && m_pPushReqList.size() > 0 && m_pNetAvailable != null)
            return m_pNetAvailable.checkNetworkAvailable();
        return false;
    }

    /**
     * 请求队列超时处理
     *
     * @return true：有超时请求
     */
    private boolean doCheckReqListTimeOut() {
        //判断发送超时
        boolean bSendTimeOut = false;
        synchronized (m_pReqList) {
            int nCount = m_pReqList.size();
            int nErrorNo = tztZFNetWorkError_Send;
            String strErrMsg = tztZFErrorMessageDefine.getUserDefineError(tztZFNetWorkErrorKey_WriteError, tztZFWriteError);
            for (int i = nCount - 1; i >= 0; i--) {
                tztZFSocketSendData req = m_pReqList.get(i);
                if (req != null) {
                    //发送超时
                    if (req.isSendTimeOut(this.m_nSendOutTime + this.m_nConnOutTime)) {
                        if (!bSendTimeOut) {
                            if (m_pNetAvailable != null && !m_pNetAvailable.checkNetworkAvailable()) {
                                nErrorNo = tztZFNetWorkError_Disconnect;
                                strErrMsg = tztZFErrorMessageDefine.getUserDefineError(tztZFNetWorkErrorKey_ConnectError, tztZFConnectError);
                            }
                        }
                        m_pReqList.remove(i);
                        //判断是否需要重发
                        if (req.isReSend) {
                            req.setAddTime();
                            req.setIsReSend(false);
                            addAction(req, req.isRetainBlock());
                        } else {
                            responseError(req, tztZFSend_TimeOut, nErrorNo, strErrMsg);
                            bSendTimeOut = true;
                            tztlog("checkSendOutTime", "[Action:%s][%d]通讯错误:%s", req.getAction(), nErrorNo, strErrMsg);
                        }
                    }
                }
            }
        }
        return bSendTimeOut;
    }

    //=====tztZFSocketThreadPool 待应答队列=======================================================

    /**
     * 获取待应答队列请求数
     *
     * @return 获取待应答队列数量
     */
    private int getSendedReqCount() {
        if (m_pSendedReqList == null)
            return 0;
        return m_pSendedReqList.size();
    }

    /**
     * 根据请求号，从待应答队列中移除，返回移除的请求对象
     *
     * @return 请求对象
     */
    private tztZFSocketSendData findRemoveSendedReq(Integer recvTag) {
        if (recvTag < 0)
            return null;
        if (m_pSendedReqList == null)
            return null;
        return m_pSendedReqList.remove(recvTag);
    }

    /**
     * 加入到已发送列表
     *
     * @param req 已发送数据对象
     */
    private void addSendedReq(tztZFSocketSendData req) {
        if (m_pSendedReqList == null)
            return;
        Integer sendTag = req.getSendTag();
        m_pSendedReqList.put(sendTag, req);
    }

    /**
     * 清空待应答队列
     *
     * @return true
     */
    private boolean doClearSendedReqList(tztZFRecvType type) {
        if (m_pSendedReqList == null)
            return true;
        String strErrMsg = tztZFErrorMessageDefine.getUserDefineError(tztZFNetWorkErrorKey_ReadError, tztZFReadError);
        int nErrorNo = tztZFNetWorkError_Disconnect;
        if (type == tztZFRecv_TimeOut || type == tztZFRecv_Error)
            nErrorNo = tztZFNetWorkError_Recv;

        if (m_pSendedReqList != null) {
            for (Iterator<Map.Entry<Integer, tztZFSocketSendData>> it = m_pSendedReqList.entrySet().iterator(); it.hasNext(); ) {
                Map.Entry<Integer, tztZFSocketSendData> item = it.next();
                if (item != null) {
                    tztZFSocketSendData req = item.getValue();
                    it.remove();
                    if (req != null) {
                        responseError(req, type, nErrorNo, strErrMsg);
                        tztlog("ClearSendedReqList", "[Action:%d][Reqno:%d][%s][%d]通讯错误:%s",
                                req.getAction(), req.getSendTag(), req.getStr("ReqTag"), nErrorNo, strErrMsg);
                    }
                }
            }
        }

        return true;
    }

    /**
     * 待应答队列超时处理
     *
     * @return true
     */
    private boolean doCheckSendedReqListTimeOut() {
        if (m_pSendedReqList == null)
            return true;
        boolean bConnected = IsConnected();//连接状态
        int nErrClose = 0;
        if (m_pSendedReqList != null) {
            for (Iterator<Map.Entry<Integer, tztZFSocketSendData>> it = m_pSendedReqList.entrySet().iterator(); it.hasNext(); ) {
                Map.Entry<Integer, tztZFSocketSendData> item = it.next();
                if (item == null) {
                    it.remove();
                    continue;
                }
                tztZFSocketSendData req = item.getValue();
                if (req != null && ((!bConnected) || req.isRecvTimeOut())) {
                    tztlog("CheckSended", "[Action:%d][Reqno:%d][%s]应答超时",
                            req.getAction(), req.getSendTag(), req.getStr("ReqTag"));
                    it.remove();
                    //判断是否需要重发
                    if (req.isReSend) {
                        req.setAddTime();
                        req.setIsReSend(false);
                        addAction(req, req.isRetainBlock());
                    } else {
                        //协商超时、验证超时
                        if (req.getType() == tztZFPackage_Handshake
                                || req.getType() == tztZFPackage_Socket) {
                            nErrClose = tztZFNetWorkError_Hand;
                            if (req.getType() == tztZFPackage_Socket)
                                nErrClose = tztZFNetWorkError_Disconnect;
                        } else if (m_nLinkType == tztZFLinkType.tztZFLinkType_HQ.value() && req.isBeforeSendTimeOut(m_nConnectedTime)) {
                            //行情连接应答超时,行情请求时间>连接成功时间。
                            nErrClose = tztZFNetWorkError_Disconnect;
                            m_nConnectedTime = 0;
                        }
                        responseError(req, bConnected ? tztZFRecv_TimeOut : tztZFConn_DisConnect, tztZFNetWorkError_Recv, tztZFErrorMessageDefine.getUserDefineError(tztZFNetWorkErrorKey_TimeOutError, tztZFOutTimeError));
                    }
                }
            }
        }
        //需断开连接
        if (nErrClose != 0) {
            m_bNextAdd = true;
            CloseConnect(nErrClose, tztZFErrorMessageDefine.getUserDefineError(tztZFNetWorkErrorKey_TimeOutError, tztZFOutTimeError));
        }
        return true;
    }

    //=====tztZFSocketThreadPool 断开连接=======================================================

    /**
     * 断开当前连接：断开类型，断开信息
     */
    public final void CloseConnect(int type, String msg) {
        CloseConnect(type, msg, -1);
    }

    public final synchronized void CloseConnect(int type, String msg, int connectIndex) {
        m_bHandSucess = false;
        m_nConnectedTime = 0;
        if (m_pSocket != null) { //正在断开
            m_socketStep = tztZFSocketFlags_DisConning;
            closeSocket(type, msg);
        }
        if (connectIndex >= 0) {
            if (m_connectType == 9 && tztZFNetCosign.getCosignStatus())
                m_GMAddressIndex = connectIndex;
            else {
                m_AddressIndex = connectIndex;
            }
            m_bNextAdd = false;
        }

        if (m_socketStep != tztZFSocketFlags_End) { //已经断开
            m_socketStep = tztZFSocketFlags_ConnFail;
        }
        doNotify("Socket close");
    }

    /**
     * 关闭连接
     */
    public synchronized void Quit() {
        if (m_socketStep == tztZFSocketFlags_End)
            return;
        stopThread();//结束线程
        CloseConnect(tztZFNetWorkError_Disconnect, "结束连接");
        tztlog("Quit", "%s", "结束连接!");
    }

    private synchronized void freeSocket(boolean bForce) {
        clearReqPush(); //清主推发队列
        clearPushRespList(tztZFSend_SendCancel);//清主推队列
        doClearReqList(tztZFSend_SendCancel); // 清待发队列
        doClearSendedReqList(tztZFSend_SendCancel);// 清已发队列
        freeSocketQueue(bForce);
    }

    private void freeSocketQueue(boolean bForce) {
        try {
            if (isGCDSocketClosed() || bForce) {
                m_socketStep = tztZFSocketFlags_End;
                if (m_pNetAvailable != null) {
                    m_pNetAvailable.onQuit();
                    m_pNetAvailable = null;
                }
                //释放通讯协议对象
                if (m_pDataProtocolKit != null) {
                    m_pDataProtocolKit.free();
                    m_pDataProtocolKit = null;
                }

                if (m_pReqList != null) {
                    m_pReqList = null;
                }

                if (m_pSendedReqList != null) {
                    m_pSendedReqList = null;
                }

                if (m_pThreadPool != null) {
                    m_pThreadPool.setUsed(false);
                }

                if (m_pThreadResp != null) {
                    m_pThreadResp.shutdown();
                    m_pThreadResp = null;
                }

            } else {
                closeGCDSocket(m_pSocket1DF, "");
                closeGCDSocket(m_pSocket2DF, "");
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    //=====tztZFSocketThreadPool 请求=======================================================

    /**
     * 发送请求
     *
     * @return 是否从列表里删除
     */
    private boolean sendRequestData() {
        tztZFSocketSendData req = getAction(0);
        if (req == null) {
            return false;
        }
        return sendTcpData(req, true);
    }

    private boolean sendTcpData(tztZFSocketSendData req, boolean bRemoveReqList) {
        if (req == null) {
            return false;
        }
        if (!m_bRun || m_pSocket == null)
            return false;
        try {
            //zzstatistic=功能号,耗时(ms),网络类型
            if (!tztZFTextUtils.isEmpty(m_sStatisticVal)) {
                req.setStr(tztZFKeyStatistic, m_sStatisticVal);
                m_sStatisticVal = "";
            }
            if (m_nLinkType == tztZFLinkType_JY.value() && (!tztZFNetCosign.getCosignStatus() || m_connectType != 9)) {
                //国密状态不正常，则移除tztZFGMSSL参数
                req.removeByKey("tztGMSSL");
            }
            byte[] outData = getRequestSendData(req);
            if (outData != null && outData.length > 0) {
                req.setSendTime();
                doWriteInfo(outData.length);//写入口
                if (m_pSocket.Write(outData)) {
                    this.m_nPackageindex++;
                    m_nLastSendTime = System.currentTimeMillis();
                    //应答后的第一次请求时间
                    if (m_nLastRecvTime == 0 || m_nRecvAfterFirstSendDataTime == 0 || m_nRecvAfterFirstSendDataTime < m_nLastRecvTime) {
                        m_nRecvAfterFirstSendDataTime = m_nLastSendTime;
                    }
                    //加入待接收应答队列
                    addSendedReq(req);
                    if (bRemoveReqList) {
                        //从请求队列移除
                        removeReqList(req);
                    }
                    readDoNotify();
                    String hostPort = m_sLastHostPort;
                    if (m_pSocket != null && m_pSocket.userData != null && m_pSocket.userData instanceof tztHostport) {
                        hostPort = ((tztHostport) m_pSocket.userData).toPrint();
                    }
                    tztlogI("Send", "[Action:%d][Reqno:%d]Host:%s;json=%s",
                            req.getAction(), req.getSendTag(), hostPort, req.getReqMapdata().toString());
                    return true;
                }
            }
            return false;
        } catch (Exception e) {
            CloseConnect(tztZFNetWorkError_Send, "打包错误");
            return false;
        }
    }

    /**
     * 发送心跳请求
     */
    private void doSendHeartBeat() {
        if (m_bShortConn || !m_bRun || m_isProxy)
            return;
        if (m_socketStep != tztZFSocketFlags_ConnSucess)
            return;
        if (m_pSocket == null || m_pDataProtocolKit == null)
            return;
        if (m_nLastSendTime <= 0)
            return;

        //距离上一次发送成功已有30S没收到应答，发送验证包验证连接状态，防止假连接
        if (m_nLastRecvTime > 0
                && m_nRecvAfterFirstSendDataTime > 0
                && m_nRecvAfterFirstSendDataTime - m_nLastRecvTime > 0
                && System.currentTimeMillis() - m_nRecvAfterFirstSendDataTime > m_nCheckTime) {
            m_pCheckSendData.setAddTime();
            m_nRecvAfterFirstSendDataTime = System.currentTimeMillis();
            sendTcpData(m_pCheckSendData, false);
            return;
        }

        if (System.currentTimeMillis() - m_nLastSendTime < m_nOnlineTime) {
            return;
        }
        byte[] abyte1 = m_pDataProtocolKit.transferOnlineData();
        try {
            if (abyte1 != null && abyte1.length > 0) {

                if (m_pSocket.Write(abyte1)) {
                    this.m_nPackageindex++;
                    m_nLastSendTime = System.currentTimeMillis();
                    readDoNotify();
                    tztlog("packageindex", "[%d]:SendHeartBeat:OK!", m_nPackageindex - 1);
                }
            }
        } catch (Exception e) {
            m_nLastError = tztZFNetWorkError_Send;
            m_sLastErrMsg = "连接已断开:" + e.getMessage();
            CloseConnect(m_nLastError, m_sLastErrMsg);
        }
    }

    //由于包序号的关系，组包改成在write时组包

    /**
     * 发送数据包协议打包
     *
     * @return 协议数据包
     */
    private byte[] getRequestSendData(tztZFSocketSendData req) {
        if (m_pDataProtocolKit == null)
            return null;
        req.setSendTime();
        req.setPackageIndex(m_nPackageindex);
        return m_pDataProtocolKit.transferSendData(req.getPackage());
    }
    //=====tztZFSocketThreadPool 应答=======================================================
    //=====tztZFSocketThreadPool 应答线程=======================================================

    /**
     * 唤起应答处理
     */
    private void readDoNotify() {
        synchronized (m_pReadSyncObject) {
            m_pReadSyncObject.notify();
        }
    }

    /**
     * 应答线程处理
     */
    private void readSyncRun() throws Exception {
        tztlog("readRun", "%s", "readSyncRun");
        if (m_bRun && (!IsConnected())) {
            synchronized (m_pReadSyncObject) {
                m_pReadSyncObject.wait();
            }
        } else {
            doRecvData();
        }
    }
    //=====tztZFSocketThreadPool 应答=======================================================

    /**
     * Http代理应答，HTTP头
     */
    private void doReadHttpHeader() throws Exception {
//        int reads = 0;
//        while (true) {
//            if (m_pSocket == null || (reads = m_pSocket.Read()) < 0) {
//                throw new Exception();
//            }
//            if (reads == 13 && m_pSocket.Read() == 10 && m_pSocket.Read() == 13 && m_pSocket.Read() == 10) {
//                break;
//            }
//        }
    }

    public int recvData(byte[] data, int len) {
        if (m_pDataProtocolKit == null) {
            return -1;
        }
        recv_stream.write(data, 0, len);
        boolean bNotify = false;
        while (recv_stream.size() >= 6) {
            int headSize = 6;
            byte[] headData = new byte[headSize];
            byte[] recv_data = recv_stream.toByteArray();
            System.arraycopy(recv_data, 0, headData, 0, headSize);
            tztlog(tztZFLogLevel.tztZFLogLevel_Debug, "PoolSocket", "%s", "checkPackageHeader！");
            int datalen = m_pDataProtocolKit.checkPackageHeader(headData);
            if (datalen >= 0) {
                if (datalen + headSize <= recv_data.length) {
                    if (datalen > 0) {
                        byte[] abyte0 = new byte[datalen];// 具体额数据内容
                        System.arraycopy(recv_data, headSize, abyte0, 0, datalen);
                        m_nLastRecvTime = System.currentTimeMillis();
                        dealReceiveData(abyte0);
                    }
                    recv_stream.reset();
                    if (recv_data.length > headSize + datalen) {
                        recv_stream.write(recv_data, headSize + datalen, recv_data.length - headSize - datalen);
                    }
                    bNotify = true;
                } else {
                    //没接收完，需要继续读取
                    tztlog(tztZFLogLevel.tztZFLogLevel_Debug, "PoolSocket", "datalen:%d, recvLen:%d", datalen, recv_data.length);
                    break;
                }
            } else {
                tztlog(tztZFLogLevel.tztZFLogLevel_Debug, "PoolSocket", "%s", "数据包头校验错误！");
                break;
            }
        }
        if (bNotify) {
            doNotify("doRecvData");//收到一个完整包，唤醒发送线程
        }
        return len;
    }

    /**
     * 读取应答数据
     */
    private void doRecvData() throws Exception {
//        if (m_isProxy) {
//            doReadHttpHeader();
//        }
        // ========读数据========
        int size = m_readData.length;// 要读取的长度，包长
        int reads = 0;
        try {
            if (m_pDataProtocolKit == null) {
                throw new Exception("通讯错误！");
            }
            if (m_pSocket == null || (reads = m_pSocket.ReadData(m_readData, size)) < 0) {
                tztlog(tztZFLogLevel.tztZFLogLevel_Debug, "PoolSocket", "%s", "socket Read Error");
                throw new Exception("socket Read Error");
            } else if (reads == 0) {
                tztlog(tztZFLogLevel.tztZFLogLevel_Debug, "PoolSocket", "%s", "reads = 0");
                return;
            } else if (reads > 0) {
                tztlog(tztZFLogLevel.tztZFLogLevel_Debug, "PoolSocket", "reads :%d;recv_stream:%d", reads, recv_stream.size());
                recvData(m_readData, reads);
            }
        } catch (Exception e) {
            tztlog(tztZFLogLevel.tztZFLogLevel_Debug, "PoolSocket", "%s", tztZFLog.getStackTraceString(e));
            throw new Exception(e.getMessage());
        }
    }

    //给第三方做计算耗时用，不要删除
    public final void doWriteInfo(int len) {

    }

    //给第三方做计算耗时用，不要删除
    public final void doReadInfo(int len) {

    }

    /**
     * 应答数据协议处理
     */
    private void dealReceiveData(byte[] RecData) {
        if (m_pDataProtocolKit == null)
            return;
        tztZFDataMapBase<String, Object> mapData = new tztZFDataMapBase<>();
        tztZFRecvPackage recvPackage = new tztZFRecvPackage(mapData);
        if (!m_pDataProtocolKit.transferPackageData(recvPackage, RecData)) {
            recvPackage = null;
        }
        // ====检查数据长度完整性====
        if (recvPackage == null) {
            m_nLastError = tztZFNetWorkError_Recv;
            m_sLastErrMsg = "数据解析错误！";
            CloseConnect(m_nLastError, m_sLastErrMsg);
            return;
        }
        dealRecvPackageData(recvPackage);
    }

    /**
     * 应答协议数据包处理
     */
    private void dealRecvPackageData(tztZFRecvPackage recvPackage) {
        int nRecvTag = recvPackage.getTag();
        String hostPort = m_sLastHostPort;
        if (m_pSocket != null && m_pSocket.userData != null && m_pSocket.userData instanceof tztHostport) {
            hostPort = ((tztHostport) m_pSocket.userData).toPrint();
        }
        tztZFDataMapBase<String, Object> pMapData = recvPackage.getRecvData();
        tztZFSocketSendData sendData = findRemoveSendedReq(nRecvTag);
        if (sendData != null) { // 找到: 执行应答处理过程
            int nAction = sendData.getAction();
            long nUsedtime = sendData.getUsedTime();
            String strReqno = sendData.getReqno();
            if (tztZFTextUtils.isEmpty(strReqno)) {
                pMapData.removeByKey("Reqno");
            } else {
                pMapData.setStr("Reqno", strReqno);
            }
            //是主推请求的应答
            if (sendData.getType() == tztZFPackage_AutoPush) {
                takeOrClearPush(pMapData, sendData);
            } else {
                if (nAction == 2 && pMapData.getInt("errorno") > 0) {
                    int percent = pMapData.getInt("ZZPercent", 100);
                    tztZFSocketLink.setStatisticPercent(percent);
                }
                //处理耗时
                if (tztZFSocketLink.getStatisticPercent()) {
                    setStatistic(nAction, nUsedtime);
                }
            }
            //判断是否含主动挑战
            checkInitiativeChallenge(pMapData, sendData);
            //在有主动挑战的情况下，判断是是否挑战成功
            checkAnsInitiativeChallenge(pMapData);
            responseData(sendData, true, tztZFRecv_DataOK, pMapData);
            tztlogI("Recv", "[Action:%d][Reqno:%d][%s]JSON:%s",
                    nAction, nRecvTag, sendData.getStr("ReqTag"), pMapData.toString());
        } else {
            tztlog("findSendDataTakePush", "%d[Action:%s][Reqno:%d]",
                    m_pSendedReqList.size(), pMapData.getStr("Action"), nRecvTag);
            sendData = findSendDataTakePush(pMapData, nRecvTag);
            if (sendData != null) {
                tztlogI("Recv", "[TakePushData][Action:%s][Reqno:%d][%s]Host:%s;JSON:%s",
                        pMapData.getStr("Action"), nRecvTag, sendData.getStr("ReqTag"), hostPort, pMapData.toString());
                responseData(sendData, false, tztZFRecv_DataOK, pMapData);
            } else {  // 找不到：主推的应答或对应请求已经被超时、断线等过程清除，都当主推处理（因为没法区分）
                tztlogI("Recv", "[PushData][Action:%s][Reqno:%d]Host:%s;JSON:%s",
                        pMapData.getStr("Action"), nRecvTag, hostPort, pMapData.toString());
                dealPush(pMapData);
            }
        }
    }

    //=====tztZFSocketThreadPool 协商 BEGIN =======================================================
    //根据握手状态进行处理
    protected synchronized boolean doProtocolHandshake() {
        if (m_pDataProtocolKit == null)
            return false;
        @tztZFHandshakeStatus int handStatus = m_pDataProtocolKit.getCurrentStatus();
        if (handStatus == tztZFHandshake_Success) {
            //已经握手成功
            m_socketStep = tztZFSocketFlags_ConnSucess;
            if (!m_bHandSucess) {
                m_bHandSucess = true;
                if (this.sslStep != null) {
                    this.sslStep.onCallBack(tztZFSocketSSLStepSucess, 0, m_pSocket != null ? m_pSocket.getCertUser() : "");
                }
                m_nConnectedTime = System.currentTimeMillis();
                //连接成功，不切换地址
                m_bNextAdd = false;
                dealConnType(tztZFConn_Connected);
//              //连接成功，重新发起主推
                reSendAutoPush();
            }
            return true;
        }

        if (handStatus == tztZFHandshake_Handing) {
            //握手中
            tztlog("Hand", "%s", "Handing协商中");
            return false;
        }

        if (handStatus == tztZFHandshake_Rehand
                || handStatus == tztZFHandshake_Error
                || handStatus == tztZFHandshake_Failed
        ) {
            //重新握手状态
            m_socketStep = tztZFSocketFlags_ConnFail;
            m_bNextAdd = (handStatus != tztZFHandshake_Rehand);
            m_nLastError = m_pDataProtocolKit.getErrorNo();
            if (handStatus == tztZFHandshake_Failed)
                tztZFRegister.onHandler(m_nSdkType, m_sSdkID, m_nLastError);
            m_sLastErrMsg = "[" + m_nLastError + "][Status:" + handStatus + "]协商失败！";
            tztlog("Hand", m_sLastErrMsg);
            CloseConnect(tztZFNetWorkError_Hand, m_sLastErrMsg);
        }
        return false;
    }

    protected boolean doProtocolHandshakeReq() {
        if (m_pDataProtocolKit == null)
            return false;

        if (m_pHandSendData == null) {
            tztZFNetWorkReqObj reqHand = new tztZFNetWorkReqObj();
            m_pHandSendData = new tztZFSocketSendData(reqHand, tztZFPackage_Handshake, m_sSocketKey);
            m_pHandSendData.setTimeOut(m_nCheckOutTime);
            m_pHandSendData.setResponseCallBack(new tztZFNetWorkResponseCallBack() {
                @Override
                public void OnError(tztZFNetWorkReqObj c_reqObj, int c_reqObjTag, int c_recvtype, int c_errorNo, String c_errorMsg) {
                    tztlog("Hand", "握手应答Error:%s", c_errorMsg);
                    m_socketStep = tztZFSocketFlags_ConnFail;
                    if (m_pDataProtocolKit != null) {
                        m_pDataProtocolKit.onHandshakeErr(-1);
                        m_nLastError = m_pDataProtocolKit.getErrorNo();
                    }
                    m_sLastErrMsg = c_errorMsg + "[" + m_nLastError + "]";
                    CloseConnect(tztZFNetWorkError_Hand, m_sLastErrMsg);
                }

                @Override
                public void OnAns(tztZFNetWorkReqObj c_reqObj, int c_reqObjTag, tztZFNetWorkRespObj c_respObj) {
                    tztlog("Hand", "握手应答:%s", c_respObj.getStr("Action"));
                    doProtocolHandshake();
                }
            });
        }
        return sendTcpData(m_pHandSendData, false);
    }

    private boolean responseError(tztZFSocketSendData req, tztZFRecvType type, int errorNo, String errorMessage) {
        if (req == null)
            return false;
        errorMessage = convertResponseErrorMsg(errorMessage, req.getAction());
        return req.responseError(m_pThreadResp, m_nLinkType, type, errorNo, errorMessage,true);
    }

    private boolean responseData(tztZFSocketSendData req, boolean bUsedTime, tztZFRecvType type, tztZFDataMapBase<String, Object> dataMap) {
        if (req == null)
            return false;
        if (dataMap != null) {
            String errorMsg = dataMap.getStr("ErrorMessage");
            errorMsg = convertResponseErrorMsg(errorMsg, req.getAction());
            dataMap.setStr("ErrorMessage", errorMsg);
        }
        return req.responseData(m_pThreadResp, m_nLinkType, type, bUsedTime, dataMap);
    }

    private String convertResponseErrorMsg(String errorMsg, int action) {
        if (tztZFLog.getLevel() <= tztZFLogLevel_Info.value()) {
            String errorHost = "";
            if (m_pSocket != null && m_pSocket.userData != null && m_pSocket.userData instanceof tztHostport) {
                errorHost = ((tztHostport) m_pSocket.userData).toPrint();
            } else if (!tztZFTextUtils.isEmpty(m_sLastHostPort)) {
                errorHost = String.format("last:%s", m_sLastHostPort);
            } else if (m_pConnectAdd != null) {
                errorHost = String.format("conn:%s", m_pConnectAdd.toPrint());
            }
            String strExInfo = String.format("LinkType:%d,Connecttype:%d,Action:%s,Host:%s,step:%s,disconnectErrorNo:%d,disconnectErrorMsg:%s",
                    m_nLinkType, m_connectType, action, errorHost, tztZFSocketFlagTip.tips(m_socketStep),
                    disConnectErrorNo, disConnectErrorMsg);
            errorMsg = String.format("%s,exInfo:%s", errorMsg != null ? errorMsg : "", strExInfo);
        }
        return errorMsg;
    }

    //=====tztZFSocketThreadPool 主推 BEGIN =======================================================
    private boolean isPushAction(int nAction) {
        return isHQPushAction(nAction) ||
                isServicePushAction(nAction);
    }

    //20253 行情主推 订阅和取消
    private boolean isHQPushAction(int nAction) {
        return nAction == 20253;
    }

    private boolean isHQPushTakeData(tztZFSocketSendData sendData) {
        if (sendData == null)
            return false;
        if (!isHQPushAction(sendData.getAction()))
            return false;
        String strGrid = sendData.getStr("Grid");
        if (strGrid == null || strGrid.length() < 1)
            return false;
        return true;
    }

    private boolean isHQPushClearData(tztZFSocketSendData sendData) {
        if (sendData == null)
            return false;
        if (!isHQPushAction(sendData.getAction()))
            return false;
        String strGrid = sendData.getStr("Grid");
        if (strGrid == null || strGrid.length() < 1)
            return true;
        return false;
    }

    private boolean isServicePushAction(int nAction) {
        return isServicePushActionPush(nAction) ||
                isServicePushActionClear(nAction) ||
                isServicePushActionTake(nAction);
    }

    //40484 服务推送 ServiceCode=live001;
    private boolean isServicePushActionPush(int nAction) {
        return (nAction == 40484);
    }

    //40482 服务取消 ServiceCode=live001;
    private boolean isServicePushActionClear(int nAction) {
        return (nAction == 40482);
    }

    //40481 服务订阅 ServiceCode=live001;
    private boolean isServicePushActionTake(int nAction) {
        return (nAction == 40481);
    }

    private int getPushType(int nAction) {
        if (isHQPushAction(nAction)) {
            return 20253;
        } else if (isServicePushAction(nAction)) {
            return 40481;
        }
        return 0;
    }

    //将主推请求加入发送队列，重新发起主推请求
    private void reSendAutoPush() {

        synchronized (m_pPushRecList) {
            m_pPushRecList.clear();
        }
        if (m_pPushReqList == null)
            return;
        synchronized (m_pPushReqList) {
            for (int len = m_pPushReqList.size(), i = len - 1; i >= 0; i--) {
                tztZFSocketSendData req = m_pPushReqList.get(i);
                m_pPushReqList.remove(i);
                addAction(req, false);
            }
        }
    }

    //添加清除主推请求
    private void addActionPush(tztZFSocketSendData reqData) {
        if (reqData == null)
            return;
        if (isHQPushClearData(reqData)) {
            int nGroup = reqData.getInt("Group", -1);
            clearGroupHQPush(nGroup);
        } else if (isHQPushTakeData(reqData)) {
            int nGroup = reqData.getInt("Group", -1);
            if (nGroup == -1) {
                reqData.setStr("Group", "0");
            }
        } else {
            int nAction = reqData.getAction();
            if (isServicePushActionClear(nAction)) {
                String serviceCode = reqData.getStr("ServiceCode");
                clearServiceCodePush(serviceCode);
            }
        }
    }

    //清除Group行情主推应答
    private void clearRecListHQPush(int nGroup, tztZFSocketSendData takeData, boolean bTake) {
        if (m_pPushRecList == null)
            return;
        synchronized (m_pPushRecList) {
            tztZFDataMapBase<String, Object> ans = new tztZFDataMapBase<>();
            ans.setInt("errorno", -1);
            for (int nSize = m_pPushRecList.size(), i = nSize - 1; i >= 0; i--) {
                tztZFSocketSendData recData = m_pPushRecList.get(i);
                if (recData == null) {
                    m_pPushRecList.remove(i);
                    continue;
                }
                //是行情主推
                if (isHQPushAction(recData.getAction())) {
                    String pushKey = recData.getPushKey();
                    try {
                        int recGroup = tztZFPublicFun.parseInt(pushKey, -1);
                        if (recGroup == -1 || nGroup == -1 || recGroup == nGroup) {
                            m_pPushRecList.remove(i);
                            if (!recData.isEqualReq(takeData)) {
                                //跟新增的是同一个请求，不通知取消
                                ans.setStr("errormessage", "Group[" + recGroup + "]订阅已被取消");
                                responseData(recData, false, tztZFRecv_DataOK, ans);
                            }
                        }
                    } catch (Exception ex) {
                        continue;
                    }
                }
            }
            //添加新的订阅
            if (bTake && takeData != null) {
                m_pPushRecList.add(takeData);
            }
        }
    }

    //清除Group行情主推请求
    private void clearReqListHQPush(int nGroup, tztZFSocketSendData takeData, boolean bTake) {
        if (m_pPushReqList == null)
            return;
        synchronized (m_pPushReqList) {
            for (int nSize = m_pPushReqList.size(), i = nSize - 1; i >= 0; i--) {
                tztZFSocketSendData reqData = m_pPushReqList.get(i);
                if (reqData == null) {
                    m_pPushReqList.remove(i);
                    continue;
                }
                if (isHQPushAction(reqData.getAction())) {
                    int reqGroup = reqData.getInt("Group", -1);
                    if (reqGroup == -1 || nGroup == -1 || reqGroup == nGroup) {
                        m_pPushReqList.remove(i);
                    }
                }
            }
            //添加新的订阅
            if (bTake && takeData != null) {
                m_pPushReqList.add(takeData);
            }
        }
    }

    //清除行情Group主推
    private void clearGroupHQPush(int nGroup) {
        clearGroupHQPush(nGroup, null, false);
    }

    private void clearGroupHQPush(int nGroup, tztZFSocketSendData takeData, boolean bTake) {
        clearRecListHQPush(nGroup, takeData, bTake);
        clearReqListHQPush(nGroup, takeData, bTake);
    }

    //清除serviceCode服务订阅应答
    private void clearRecListServiceCodePush(String serviceCode) {
        if (m_pPushRecList == null)
            return;
        boolean bClearAll = (serviceCode == null || serviceCode.length() < 1);
        synchronized (m_pPushRecList) {
            tztZFDataMapBase<String, Object> ans = new tztZFDataMapBase<>();
            ans.setInt("errorno", -1);
            for (int nSize = m_pPushRecList.size(), i = nSize - 1; i >= 0; i--) {
                tztZFSocketSendData recData = m_pPushRecList.get(i);
                if (recData == null) {
                    m_pPushRecList.remove(i);
                    continue;
                }
                int nAction = recData.getAction();
                if (isServicePushActionTake(nAction)) {
                    String recCode = recData.getPushKey();
                    if (bClearAll || recCode == null || (recCode.compareToIgnoreCase(serviceCode) == 0)) {
                        m_pPushRecList.remove(i);
                        ans.setStr("errormessage", "[" + serviceCode + "]订阅已被取消");
                        responseData(recData, false, tztZFRecv_DataOK, ans);
                    }
                }
            }
        }
    }

    //清除serviceCode服务订阅请求
    private void clearReqListServiceCodePush(String serviceCode) {
        if (m_pPushReqList == null)
            return;
        boolean bClearAll = (serviceCode == null || serviceCode.length() < 1);
        synchronized (m_pPushReqList) {
            for (int nSize = m_pPushReqList.size(), i = nSize - 1; i >= 0; i--) {
                tztZFSocketSendData reqData = m_pPushReqList.get(i);
                if (reqData == null) {
                    m_pPushReqList.remove(i);
                    continue;
                }
                if (isServicePushActionTake(reqData.getAction())) {
                    String reqCode = reqData.getStr("ServiceCode");
                    if (bClearAll || reqCode == null || (reqCode.compareToIgnoreCase(serviceCode) == 0)) {
                        m_pPushReqList.remove(i);
                    }
                }
            }
        }
    }

    //清除serviceCode服务主推
    private void clearServiceCodePush(String serviceCode) {
        clearRecListServiceCodePush(serviceCode);
        clearReqListServiceCodePush(serviceCode);
    }

    private boolean findedServiceCode(String[] serviceCode, String findCode) {
        for (int i = 0; i < serviceCode.length; i++) {
            String strCode = serviceCode[i];
            if (findCode.compareToIgnoreCase(strCode) == 0)
                return true;
        }
        return false;
    }

    //设置serviceCode服务订阅应答
    private void setRecListServiceCodePush(String[] serviceCode, String takeCode, tztZFSocketSendData takeData) {
        if (m_pPushRecList == null)
            return;
        synchronized (m_pPushRecList) {
            tztZFDataMapBase<String, Object> ans = new tztZFDataMapBase<>();
            ans.setInt("errorno", -1);
            for (int nSize = m_pPushRecList.size(), i = nSize - 1; i >= 0; i--) {
                tztZFSocketSendData sendData = m_pPushRecList.get(i);
                if (sendData == null) {
                    m_pPushRecList.remove(i);
                    continue;
                }
                int nAction = sendData.getAction();
                if (isServicePushActionTake(nAction)) {
                    String recCode = sendData.getPushKey();
                    if (recCode != null && recCode.length() > 0) {
                        if (recCode.compareToIgnoreCase(takeCode) != 0) {
                            //不是takecode
                            if (findedServiceCode(serviceCode, recCode)) {
                                //在订阅列表中，不移除
                                continue;
                            }
                        }
                    }
                    m_pPushRecList.remove(i);
                    if (!sendData.isEqualReq(takeData)) {
                        //不与take同一个请求
                        ans.setStr("errormessage", "[" + recCode + "]订阅已被取消");
                        responseData(sendData, false, tztZFRecv_DataOK, ans);
                    }
                }
            }
            if (takeData != null) {
                m_pPushRecList.add(takeData);
            }
        }
    }

    //设置serviceCode服务订阅请求
    private void setReqListServiceCodePush(String[] serviceCode, String takeCode, tztZFSocketSendData takeData) {
        if (m_pPushReqList == null)
            return;
        synchronized (m_pPushReqList) {
            for (int nSize = m_pPushReqList.size(), i = nSize - 1; i >= 0; i--) {
                tztZFSocketSendData reqData = m_pPushReqList.get(i);
                if (reqData == null) {
                    m_pPushReqList.remove(i);
                    continue;
                }
                if (isServicePushActionTake(reqData.getAction())) {
                    String reqCode = reqData.getStr("ServiceCode");
                    if (reqCode == null || reqCode.length() < 1 ||              //值为空
                            (reqCode.compareToIgnoreCase(takeCode) == 0) ||     //是takeCode
                            (!findedServiceCode(serviceCode, reqCode))) {       //不在订阅列表中
                        m_pPushReqList.remove(i);
                    }
                }
            }
            if (takeData != null) {
                m_pPushReqList.add(takeData);
            }
        }
    }

    //设置serviceCode服务主推
    private void setServiceCodePush(String[] serviceCode, String takeCode, tztZFSocketSendData takeData) {
        setRecListServiceCodePush(serviceCode, takeCode, takeData);
        setReqListServiceCodePush(serviceCode, takeCode, takeData);
    }

    //根据tag查找主推请求
    private tztZFSocketSendData findSendDataTakeHqPush(int recTag) {
        if (m_pPushRecList == null || m_pPushRecList.size() < 1)
            return null;
        synchronized (m_pPushRecList) {
            for (int nSize = m_pPushRecList.size(), i = nSize - 1; i >= 0; i--) {
                tztZFSocketSendData recData = m_pPushRecList.get(i);
                if (recData != null && isHQPushAction(recData.getAction())) {
                    if (recData.getSendTag() == recTag) {
                        return recData;
                    }
                }
            }
        }
        return null;
    }

    //根据serviceCode查找主推请求
    private tztZFSocketSendData findSendDataTakeServiceCodePush(String serviceCode) {
        if (serviceCode == null || serviceCode.length() < 1 || m_pPushRecList == null || m_pPushRecList.size() < 1)
            return null;
        synchronized (m_pPushRecList) {
            for (int nSize = m_pPushRecList.size(), i = nSize - 1; i >= 0; i--) {
                tztZFSocketSendData recData = m_pPushRecList.get(i);
                if (recData != null && isServicePushActionTake(recData.getAction())) {
                    String recCode = recData.getPushKey();
                    if (recCode == null || recCode.length() < 1)
                        continue;
                    if (serviceCode.compareToIgnoreCase(recCode) == 0) {
                        return recData;
                    }
                }
            }
        }
        return null;
    }

    //根据应答查找主推请求
    private tztZFSocketSendData findSendDataTakePush(tztZFDataMapBase<String, Object> recvMap, int recTag) {
        if (recvMap == null || recvMap.getCount() < 1)
            return null;
        int nAction = recvMap.getInt("Action");
        if (isHQPushAction(nAction)) {
            return findSendDataTakeHqPush(recTag);
        }
        if (isServicePushActionPush(nAction)) {
            String serviceCode = recvMap.getStr("ServiceCode");
            if (serviceCode == null || serviceCode.length() < 1)
                return null;
            return findSendDataTakeServiceCodePush(serviceCode);
        }
        return null;
    }

    //行情主推订阅/退订应答处理
    private void takeOrClearGroupHQPush(tztZFDataMapBase<String, Object> recvMap, tztZFSocketSendData reqData) {
        if (reqData == null)
            return;
        if (recvMap == null || recvMap.getInt("ErrorNo", -1) < 0)
            return;
        int nGroup = reqData.getInt("Group", -1);
        if (isHQPushClearData(reqData)) { //是清除请求
            clearGroupHQPush(nGroup);
        } else if (isHQPushTakeData(reqData)) {
            if (nGroup < 0)
                return;
            reqData.setPushKey(String.valueOf(nGroup));
            //清除老的订阅,添加新的订阅
            clearGroupHQPush(nGroup, reqData, true);
        }
    }

    //serviceCode服务主推订阅/退订应答处理
    private void takeOrClearServiceCodePush(tztZFDataMapBase<String, Object> recvMap, tztZFSocketSendData reqData) {
        if (recvMap == null || recvMap.getInt("ErrorNo", -1) < 0)
            return;
        //当前订阅列表
        String serviceCode = recvMap.getStr("Grid");
        if (serviceCode == null)
            serviceCode = "";
        //清空全部订阅列表
        if (serviceCode.length() < 1) {
            clearServiceCodePush(serviceCode);
            return;
        }
        serviceCode = serviceCode.toLowerCase();
        String[] ayCode = serviceCode.split(",");
        if (ayCode == null || ayCode.length < 1) {
            clearServiceCodePush("");
            return;
        }
        boolean bTake = false;
        String takeCode = "";
        if (reqData != null) {
            int nAction = recvMap.getInt("Action", -1);
            bTake = isServicePushActionTake(nAction);
            if (bTake) {
                takeCode = reqData.getStr("ServiceCode");
                if (takeCode == null || takeCode.length() < 1) {
                    bTake = false;
                } else {
                    takeCode = takeCode.toLowerCase();
                    reqData.setPushKey(takeCode);
                }
            }
        }
        setServiceCodePush(ayCode, takeCode, (bTake ? reqData : null));
    }

    //处理主推订阅/退订应答
    private void takeOrClearPush(tztZFDataMapBase<String, Object> recvMap, tztZFSocketSendData reqData) {
        if (recvMap == null || recvMap.getInt("ErrorNo", -1) < 0)
            return;
        int nAction = recvMap.getInt("Action");
        if (isHQPushAction(nAction)) {
            takeOrClearGroupHQPush(recvMap, reqData);
        } else if (isServicePushActionTake(nAction) ||
                isServicePushActionClear(nAction)) {
            takeOrClearServiceCodePush(recvMap, reqData);
        } else if (isServicePushActionPush(nAction)) {
            return;
        }
    }

    private void clearReqPush() {
        if (m_pPushReqList == null)
            return;
        m_pPushReqList.clear(); //主推队列清空
    }

    private void clearPushRespList(tztZFRecvType type) {
        if (m_pPushRecList == null)
            return;
        synchronized (m_pPushRecList) {
            tztZFDataMapBase<String, Object> ans = new tztZFDataMapBase<>();
            ans.setInt("errorno", -1);
            for (int nSize = m_pPushRecList.size(), i = nSize - 1; i >= 0; i--) {
                tztZFSocketSendData recData = m_pPushRecList.get(i);
                if (recData == null) {
                    m_pPushRecList.remove(i);
                    continue;
                }
                int nAction = recData.getAction();
                String pushKey = recData.getPushKey();
                if (isServicePushActionTake(nAction)) {
                    m_pPushRecList.remove(i);
                    ans.setStr("errormessage", "[" + pushKey + "]订阅已被取消");
                    responseData(recData, false, type, ans);
                } else {
                    m_pPushRecList.remove(i);
                    ans.setStr("errormessage", "Group[" + pushKey + "]订阅已被取消");
                    responseData(recData, false, type, ans);
                }
            }
        }
    }
    //=====tztZFSocketThreadPool 命中事件处理 BEGIN =======================================================

    /**
     * 命中统计
     *
     * @param nAction 功能号
     * @param time    耗时
     */
    private void setStatistic(int nAction, long time) {
        //组织上报的数据格式: 功能号,耗时,网络类型
        if (time > 0) {
            if (m_pNetAvailable != null)
                m_sNetWorkType = m_pNetAvailable.getNetWorkType();
            m_sStatisticVal = String.format(Locale.CHINA, "%d,%d,%s", nAction, time, m_sNetWorkType);
        }
    }

    //=====tztZFSocketThreadPool 主动挑战 BEGIN =======================================================

    /**
     * 判断是否有主动性挑战失败的情况，如果失败，则提出所有账号信息
     *
     * @param ans 数据字典
     */
    private void checkAnsInitiativeChallenge(tztZFDataMapBase<String, Object> ans) {
        int nErrorNo = ans.getInt("ErrorNo");
        if (nErrorNo == -204021 || nErrorNo == -204022) {
            String errorMessage = ans.getStr("ErrorMessage");
            //挑战失败，交易强制退出
            if (m_pNetAvailable != null)
                m_pNetAvailable.eventBus(new tztZFQuitTradeEvent(errorMessage));
        }
    }

    /**
     * 主动性挑战
     * 底层接收到数据后，发现应答数据中有challenge_question和challenge_formulaid字段，
     * 根据challenge_formulaid对应的值(暂定 1-MD5计算 2-Base64计算 )，
     * 将challenge_question用对应的算法计算后，发送6700给服务器进行校验，
     * 在收到-204021或者-204022的情况下，说明挑战出错失败了，
     * 需要退出交易，其他情况下，继续正常使用
     */
    private void checkInitiativeChallenge(tztZFDataMapBase<String, Object> ans, tztZFSocketSendData sendData) {

        if (ans == null || sendData == null || sendData.getReqMapdata() == null || tztZFTextUtils.isEmpty(sendData.getReqMapdata().getStr("mobileCode"))) {
            return;
        }
        String challengeQuestion = ans.getStr("CHALLENGE_QUESTION");
        String challengeFormulaid = ans.getStr("CHALLENGE_FORMULAID");
        String token = ans.getStr("TOKEN");
        if (!tztZFTextUtils.isEmpty(challengeQuestion)
                && !tztZFTextUtils.isEmpty(challengeFormulaid)
                && !tztZFTextUtils.isEmpty(token)) {
            String challengeAns = "";
            switch (Integer.valueOf(challengeFormulaid)) {
                case 1:
                    challengeAns = tztZFDataEncrypt.encryptMD532Str(challengeQuestion.getBytes());
                    break;
                case 2:
                    challengeAns = tztZFDataEncrypt.encryptBase64Str(challengeQuestion.getBytes());
                    break;
            }
            if (!tztZFTextUtils.isEmpty(challengeAns)) {
                //构造挑战请求
                tztZFNetWorkReqObj reqObj = new tztZFNetWorkReqObj();
                reqObj.setInt("action", 6700);
                reqObj.setStr("mobileCode", sendData.getReqMapdata().getStr("mobileCode"));
                reqObj.setStr("token", token);
                reqObj.setStr("answer", challengeAns);
                tztZFSocketSendData challengesendData = new tztZFSocketSendData(reqObj, tztZFPackage_Data, m_sSocketKey);
                challengesendData.setResponseCallBack(sendData.getResponseCallBack());
                addAction(challengesendData, false);
            }
        }
    }

    //=====tztZFSocketThreadPool 网络状态=======================================================
    private boolean checkNetWorkAvailableProxy() {
        // 检测网络状态也是不通的
        if (m_pNetAvailable != null && !m_pNetAvailable.checkNetworkAvailable()) {
            return false;
        }
        boolean Proxy = m_pNetAvailable != null ? m_pNetAvailable.checkProxy() : false;
        if (Proxy) {
            if (m_nConnNum >= m_nMaxConn && m_nLastRecvTime == 0) {
                m_isProxy = true;
            }
        } else {
            m_isProxy = false;
        }
        return true;
    }

    private void netChanged(String strNetWorkType, boolean isProxy, String proxyAdd, int proxyPort) {
        tztlog("Net", "系统网络切换:%s==>%s", m_sNetWorkType, strNetWorkType);
        CloseConnect(tztZFNetWorkError_Disconnect, "系统网络切换:" + m_sNetWorkType + "==>" + strNetWorkType); // 切换网络时先关闭连接
        dealConnType(tztZFConn_Change);
        m_sNetWorkType = strNetWorkType;
        m_isProxy = isProxy;
        m_sProxyAdd = proxyAdd;
        m_nProxyPort = proxyPort;
    }
    //=====tztZFSocket log=======================================================

    /**
     * 打印日志
     */
    private void tztlog(String tag, String format, Object... args) {
        tztlog(tztZFLogLevel.tztZFLogLevel_Debug, tag, format, args);
    }

    private void tztlogI(String tag, String format, Object... args) {
        tztlog(tztZFLogLevel_Info, tag, format, args);
    }

    private void tztlog(tztZFLogLevel level, String tag, String format, Object... args) {
        if (tztZFLog.getLevel() <= level.value()) {
            String cTag = String.format(Locale.getDefault(), "[%d]ZFSocket[%d][%d-%s][%s]",
                    m_nLinkType, this.hashCode(), m_nSdkType, tztZFLinkType.nameOf(m_nLinkType), tag);
            String logTag;
            if (tztZFTextUtils.isEmpty(m_strAddTag)) {
                logTag = cTag;
                cTag = "";
            } else {
                logTag = m_strAddTag;
            }
            tztZFLog.log(level, tztZFLogPrintType.tztZFLogPrintType_tztSocket.value(), logTag, cTag + format, args);
        }
    }

    ///证书加载
    private boolean loadSSLHandShakeCert(int connecttype) {
        if (m_nLinkType != tztZFLinkType_JY.value())
            return true;
        if (!tztZFNetCosign.getCosignStatus() || connecttype != 9) {
            return m_pDataProtocolKit.switchToLoadCert();
        }
        tztZFProtocolCert protocolCert = new tztZFProtocolCert();
        protocolCert.protocolType = tztZFNetCosignEx.getExIntForKey(tztCosignEx_Protocol);
        protocolCert.handType = tztZFNetCosignEx.getExIntForKey(tztCosignEx_Handtype);
        protocolCert.encryptType = tztZFNetCosignEx.getExIntForKey(tztCosignEx_Encrypttype);
        protocolCert.sPubKey = tztZFNetCosignEx.getExDataForKey(tztCosignEx_ServerPubKey);
        protocolCert.sPubKeyIndex = tztZFNetCosignEx.getExIntForKey(tztCosignEx_ServerPubKeyIndex);
        protocolCert.iDA = tztZFNetCosignEx.getExDataForKey(tztCosignEx_IDA);
        protocolCert.iDB = tztZFNetCosignEx.getExDataForKey(tztCosignEx_IDB);
        protocolCert.cPubKey = tztZFNetCosignEx.getExDataForKey(tztCosignEx_ClientPubKey);
        protocolCert.cPubKeyIndex = tztZFNetCosignEx.getExIntForKey(tztCosignEx_ClientPubKeyIndex);
        if (protocolCert.isInvalid())
            return true;
        boolean bLoad = m_pDataProtocolKit.setProtocolCert(m_nUserID, m_nSdkType, protocolCert);
        if (!bLoad) {
            m_nLastError = m_pDataProtocolKit.getErrorNo();
            m_sLastErrMsg = "加载通讯协商数据错误";
        }
        return bLoad;
    }

    // 功能:将一个整型放入数组数组缓
    // 参数:
    // nSrc 将要转换的整型数
    // szDest 要放入的目标数组
    // nStart 数组的起始偏移位
    // nLength 在数组中占用缓存空间( 1 ~ 4)
    private void ConvertIntToArray(int nSrc, byte[] szDest, int nStart, int nLength) {
        if (nLength > 4) {
            return;
        }
        for (int n = 0; n < nLength; n++) {
            szDest[nStart + n] = (byte) (nSrc >>> (n * 8));
        }
    }

    private void loadSSLCert(tztZFThreadPoolGCDAsyncSocket socket) {
        if (m_connectType != 9 && m_connectType != 8)
            return;

        byte[] ca = tztZFNetCosignEx.getExDataForKey(tztCosignEx_CACert);
        if (ca == null || ca.length < 1)
            return;

        byte[] sign = null;
        byte[] signKey = null;
        byte[] enc = null;
        byte[] encKey = null;

        int signType = 1;
        int encType = 1;
        int certType = 1;
        if (m_connectType == 9) {
            this.cosignUser = tztZFNetCosign.getBindCosignUser();
            if (cosignUser != null && cosignUser.haveCosignCert() && cosignUser.isNeedGMLogin()) {
                gmHaveCosignCert = true;
                socket.withZFSSL_LinkCosignUserID(m_nLinkType, cosignUser.userID());
                sign = cosignUser.getLoginCosignCert().getSignCert();
                if (sign != null) {
                    tztlog("VPNSign", "SignCert:%s", new String(sign));
                }
                signType = cosignUser.getLoginCosignCert().getSignType();

                enc = cosignUser.getLoginCosignCert().getEncCert();
                encKey = cosignUser.getLoginCosignCert().getEncKey();
                encType = cosignUser.getLoginCosignCert().getEncType();
                certType = cosignUser.getLoginCosignCert().getCertType();
            } else {
                gmHaveCosignCert = false;
            }
            signKey = null;
        }

        socket.withZFSSL_UserID(cosignUser != null ? cosignUser.userID() : null)
                .withZFSSL_CA(ca, 1)
                .withZFSSL_Enc(enc, encKey, encType, certType)
                .withZFSSL_Sign(sign, signKey, signType);
        socket.setCosignUserInfo(cosignUser);
    }

    ///国密降级不可恢复
    public void didGMDowngrade() {
        didGMDowngrade(true);
    }

    private boolean didGMDowngrade(boolean close) {
        boolean bDisConn = false;
        if (m_connectType != 9)
            return bDisConn;
        tztZFGMManage.setIsRegistGMLink(false);
        m_connectType = 0;
        bSetAddress = true;
        if (this.sslStep != null) {
            this.sslStep.onCallBack(tztZFSocketSSLStepDowngrade, 0, m_pSocket != null ? m_pSocket.getCertUser() : gmUserID);
        }
        bDisConn = changedAddAndDidDisconnect();
        if (close && !bDisConn) {
            bDisConn = true;
            String gmLable = tztZFErrorMessageDefine.getUserDefineError(tztZFErrorMessageDefine.tztZFNetWorkErrorKey_GMLable, tztZFErrorMessageDefine.tztZFGMLable);
            closeSocket(tztZFNetWorkError_Disconnect, String.format(Locale.getDefault(), "%s降级，断开连接", gmLable));
        }
        //通知兼容层国密降级
        if (tztZFNetWork.getGmLinkDelegate() != null) {
            tztZFNetWork.getGmLinkDelegate().didGMDowngrade();
        }
        return bDisConn;
    }

    public void didGMBindUserID(String userID, tztZFGMSSLStepCallBack SSLStep) {
        didGMBindUserID(userID, true, -1, SSLStep);
    }

    public void GMBindUserID(String userID, boolean isSSL, int addressIndex, tztZFGMSSLStepCallBack SSLStep) {
        bBindGMUser = true;
        boolean bChangeUser = false;
        boolean bChangeSSL = false;
        synchronized (gmUserID) {
            if (SSLStep != null) GMSSLStep = SSLStep;
            if (isSSL && tztZFTextUtils.isEmpty(userID)) userID = tztZFNetCosign.getBindUserID();

            if (!tztZFTextUtils.isEmpty(userID)) {
                tztZFCosignUser cosignUser = tztZFNetCosign.cosignUserForID(userID);
                isSSL = isSSL && cosignUser != null && cosignUser.isNeedGMLogin();
            }
            tztlog("ZFSocket", "绑定新用户:%s,SSL:%s,当前用户:%s", userID, String.valueOf(isSSL), gmUserID);
            int connecttype = 0;
            if (tztZFTextUtils.isEmpty(userID) || !isSSL || !tztZFNetCosign.getCosignStatus()) {
                gmUserID = "";
                cosignUser = null;
                gmHaveCosignCert = false;
            } else {
                connecttype = 9;
            }
            bChangeSSL = (this.m_connectType != connecttype);
            this.m_connectType = connecttype;
            ///连接状态
            if (m_pSocket != null) {
                ///非SSL连接
                if (GetStockStep() == tztZFSocketFlags_ConnSucess && !m_pSocket.tztZF_GetUseSSL() && this.m_connectType == 0) {
                    if (this.sslStep != null) {
                        this.sslStep.onCallBack(tztZFSocketSSLStepSucess, 0, "");
                    }
                    doNotify("setGMUserID");
                    return;
                }
            }
            if (this.m_connectType == 9) {
                if (!this.gmUserID.equals(userID)) {
                    this.gmUserID = userID;
                    bChangeUser = true;
                }
                cosignUser = tztZFNetCosign.cosignUserForID(this.gmUserID);
                gmHaveCosignCert = cosignUser != null && cosignUser.isNeedGMLogin() && cosignUser.isValidity();
            }
        }
        if (bChangeUser) {
            doClearReqList(tztZFSend_SendCancel);
        }
        boolean bDisConn = false;
        if (bChangeSSL) {
            bSetAddress = true;
            bDisConn = changedAddAndDidDisconnect();
        }
        if (!bDisConn && (bChangeSSL || bChangeUser)) {
            if (m_socketStep != tztZFSocketFlags_End) {
                m_socketStep = tztZFSocketFlags_ConnFail;
            }
            closeSocket(tztZFNetWorkError_Disconnect, "账号切换");
        } else {
            doNotify(String.format(Locale.getDefault(), "%s:user:%s,connecttype:%d", "账号切换", gmUserID, m_connectType));
        }
    }

    public void didGMBindUserID(String userID, boolean isSSL, int addressIndex, tztZFGMSSLStepCallBack SSLStep) {
        if (this.m_nLinkType != tztZFLinkType_JY.value()) return;
        GMBindUserID(userID, isSSL, addressIndex, SSLStep);
    }

    public void didGMCosignUserID(String userID, int errorNo, String errorMessage) {
        if (tztZFTextUtils.isEmpty(gmUserID) || tztZFTextUtils.isEmpty(userID) || m_connectType != 9)
            return;
        synchronized (gmUserID) {
            if (!gmUserID.equals(userID))
                return;
            if (errorNo == tztZFCosignErrServerCertUnfind || errorNo == tztZFCosignErrCertDeviceDel) {
                gmUserID = "";//无网络证书，清除userid，防止重新申请证书后交易通道重新链接导致降级
                m_nLastError = errorNo;
                m_sLastErrMsg = errorMessage;
                if (this.sslStep != null) {
                    this.sslStep.onCallBack(tztZFSocketSSLStepUnServerCert, 0, gmUserID);
                }
                doClearReqList(tztZFSend_SendError);
            }
        }
    }

    private boolean didGMUserID() {
        if (tztZFTextUtils.isEmpty(gmUserID))
            return false;
        synchronized (gmUserID) {
            tztZFCosignUser cosignUser = tztZFNetCosign.cosignUserForID(gmUserID);
            return cosignUser != null && cosignUser.isValidity();
        }
    }

    private void didGMAddressList(String addList, String nameList, int selectIndex) {
        if (m_nLinkType != tztZFLinkType_JY.value())
            return;
        if (m_ayGMAddress == null)
            m_ayGMAddress = new ArrayList<>();
        if (tztZFTextUtils.isEmpty(addList)) {
            return;
        }
        if (!addList.endsWith("&"))
            addList = addList + "&";
        String[] array = tztZFPublicFun.getArray(addList, '&');
        if (array.length > 0) {
            synchronized (this) {
                m_ayGMAddress.clear();
                for (int i = 0; i < array.length; i++) {
                    String strHostport = array[i];
                    if (tztZFTextUtils.isEmpty(strHostport)) {
                        continue;
                    }
                    m_ayGMAddress.add(new tztHostport(strHostport, 9));
                }
            }
        }
        bSetAddress = true;
        m_bNextAdd = false;
        m_GMAddressIndex = selectIndex;
        m_nConnNum = 0;
        if (selectIndex >= 0 || m_pConnectAdd == null) {
            m_pConnectAdd = getConnectHostPort(false);
        }
        if (selectIndex >= 0) {
            changedAddAndDidDisconnect();
        }
        if (m_bDoConnect) {
            SocketConnect();
        }
    }

    private void thread_didGMAddressList(String addList, String nameList, int selectIndex) {
        tztlog("setAddress", "设置国密服务器地址列表[%d]:%s", selectIndex, addList);
        didGMAddressList(addList, nameList, selectIndex);
    }
}