package com.tztzf.network;

import com.tztzf.log.tztZFLog;
import com.tztzf.log.tztZFLogLevel;
import com.tztzf.network.callback.tztZFNetPoolsCallBack;
import com.tztzf.network.define.tztZFLinkType;

public class tztZFNetBalance {
    static tztZFNetBalance _obj;

    private Object context;
    private String m_sAddlist;
    private int m_nLinkType;
    private int m_nNum;
    private int m_nTimeOut;
    private int m_nConnTimeOut = 3 * 1000;

    private tztZFPoolsShortNet _poolNet = null;

    private static synchronized tztZFNetBalance getShareInstance() {
        if (_obj == null) {
            _obj = new tztZFNetBalance();
        }
        return _obj;
    }

    private tztZFNetBalance() {
        initData();
    }

    private void initData() {
        context = null;
        m_sAddlist = "";
        _poolNet = null;
        m_nLinkType = tztZFLinkType.tztZFLinkType_JH.value();
        m_nNum = 3;
        m_nTimeOut = 0;
    }

    private tztZFNetBalance initWithContext(Object context) {
        this.context = context;
        return tztZFNetBalance.this;
    }

    /**
     * 构造tztZFNetBalance对象
     * @param context 上下文
     * @return tztZFNetBalance对象
     */
    public static tztZFNetBalance WithContext(Object context) {
        return getShareInstance().initWithContext(context);
    }

    private tztZFNetBalance initWithAddlist(String strAddList) {
        this.m_sAddlist = strAddList;
        return tztZFNetBalance.this;
    }

    /**
     * 构造tztZFNetBalance对象
     * @param strAddList 服务器地址
     * @return tztZFNetBalance对象
     */
    public static tztZFNetBalance WithAddlist(String strAddList) {
        return getShareInstance().initWithAddlist(strAddList);
    }

    private tztZFNetBalance initWithNum(int num) {
        this.m_nNum = num;
        return tztZFNetBalance.this;
    }

    /**
     * 构造tztZFNetBalance对象
     * @param num 线程池最大线程数
     * @return tztZFNetBalance对象
     */
    public static tztZFNetBalance WithNum(int num) {
        return getShareInstance().initWithNum(num);
    }

    private tztZFNetBalance initWithTimeOut(int timeOut) {
        this.m_nTimeOut = timeOut;
        return tztZFNetBalance.this;
    }

    /**
     * 构造tztZFNetBalance对象
     * @param timeOut 超时时间
     * @return tztZFNetBalance对象
     */
    public static tztZFNetBalance WithTimeOut(int timeOut) {
        return getShareInstance().initWithTimeOut(timeOut);
    }

    private tztZFNetBalance initWithConnTimeOut(int timeOut) {
        this.m_nTimeOut = timeOut;
        return tztZFNetBalance.this;
    }

    /**
     * 构造tztZFNetBalance对象
     * @param timeOut 链接超时时间
     * @return tztZFNetBalance对象
     */
    public static tztZFNetBalance WithConnTimeOut(int timeOut) {
        return getShareInstance().initWithConnTimeOut(timeOut);
    }

    private synchronized boolean dobalance(tztZFNetWorkReqObj reqObj, final tztZFNetPoolsCallBack callBack) {
        if (_poolNet == null) {
            _poolNet = new tztZFPoolsShortNet("tztZFBalace");
            return _poolNet.InitWithContext(context)
                    .InitWithNum(m_nNum)
                    .InitWithTimeOut(m_nTimeOut)
                    .InitWithConnTimeOut(m_nConnTimeOut)
                    .InitWithAddlist(m_sAddlist)
                    .InitWithLinktype(m_nLinkType)
                    .DoPoolsAction(reqObj, new tztZFNetPoolsCallBack() {
                        @Override
                        public boolean onPools(int c_netIndex, String c_netAddress, tztZFNetWorkReqObj c_reqObj, tztZFNetWorkRespObj c_respObj) {
                            if (callBack != null) {
                                return callBack.onPools(c_netIndex, c_netAddress, c_reqObj, c_respObj);
                            }
                            return true;
                        }

                        @Override
                        public void onFinish(int c_netPools, String c_netAddList, tztZFNetWorkReqObj c_reqObj, tztZFNetWorkRespObj c_respObj) {
                            if (callBack != null) {
                                callBack.onFinish(c_netPools, c_netAddList, c_reqObj, c_respObj);
                                tztZFLog.log(tztZFLogLevel.tztZFLogLevel_Debug, "onFinished", "%s", "onFinish end");
                            }
                            onQuit();
                        }
                    });
        }
        return false;
    }

    /**
     * 执行均衡
     * @param reqObj 均衡请求参数数据
     * @param callBack 均衡应答回调
     * @return 是否执行均衡
     */
    public static boolean Balance(tztZFNetWorkReqObj reqObj, tztZFNetPoolsCallBack callBack) {
        return getShareInstance().dobalance(reqObj, callBack);
    }

    /**
     * 退出
     */
    public static void Quit() {
        getShareInstance().onQuit();
    }

    private synchronized void onQuit() {
        tztZFLog.log(tztZFLogLevel.tztZFLogLevel_Debug, "Quite", "%s", "Quite");
        if (_poolNet != null) {
            _poolNet.QuitPools();
            _poolNet = null;
            tztZFLog.log(tztZFLogLevel.tztZFLogLevel_Debug, "Quite", "%s", "Quite end");
        }
    }
}
