package com.tztzf.network;

import com.tztzf.base.tztZFPublicFun;
import com.tztzf.baseos.tztZFTextUtils;
import com.tztzf.commbase.tztZFDataMapBase;
import com.tztzf.network.define.tztZFRecvType;
import com.tztzf.protocol.core.tztZFDataEncrypt;

import java.util.ArrayList;

/**
 * ****************************************************************
 * 文件名称 : tztZFNetWorkRespObj
 * 作   者 : Created by wlz
 * 创建时间 : 2019-12-18 15:53
 * 文件描述 : 应答
 * 注意事项 :
 * 版权声明 : Copyright (C) 2015-2025 杭州中焯信息技术股份有限公司
 * 修改历史 : 2019-12-18 1.00 初始版本
 * ****************************************************************
 */
public class tztZFNetWorkRespObj {

    //region 内部私有属性及函数

    //应答errorno
    private int mStatusCode = 0;
    //应答errormessage
    private String mMessage = "";
    //接收状态
    private tztZFRecvType mRecvType;
    //请求标志
    private int mReqObjTag;
    //请求数据
    private tztZFNetWorkReqObj mReqObj;
    //应答数据
    private tztZFDataMapBase<String, Object> mData = new tztZFDataMapBase<>();

    //endregion

    //region 具体实现

    //region 公布方法（允许其他包访问）

    /**
     * 获取应答数据
     *
     * @return 应答数据
     * @deprecated Use {getDataMap()}
     */
    @Deprecated
    public tztZFDataMapBase<String, Object> getResValues() {
        return getDataMap();
    }

    /**
     * 获取应答数据
     *
     * @return 应答数据
     */
    public tztZFDataMapBase<String, Object> getDataMap() {
        return mData;
    }

    /**
     * 获取字符串数据
     *
     * @param key key值
     * @return 字符串数据
     */
    public String getString(String key) {
        if (mData == null) {
            return "";
        }
        return mData.getStr(key);
    }


    /**
     * 获取字符串数据
     *
     * @param key key值
     * @return 字符串数据
     */
    public String getStr(String key) {
        if (mData == null) {
            return "";
        }
        return mData.getStr(key);
    }

    /**
     * 获取数据流数据
     *
     * @param key key值
     * @return 数据流数据
     */
    public byte[] getBytes(String key) {
        if (mData == null) {
            return null;
        }
        return mData.getBytes(key);
    }

    /**
     * 获取整型数据
     *
     * @param key key值
     * @return 整型数据
     */
    public int getInt(String key) {
        return getInt(key, 0);
    }

    /**
     * 获取整型数据
     *
     * @param key       key值
     * @param nDefValue 默认整型数据
     * @return 整型数据
     */
    public int getInt(String key, int nDefValue) {
        if (mData == null)
            return nDefValue;
        return mData.getInt(key, nDefValue);
    }

    /**
     * 获取base64解码数据
     *
     * @param key key值
     * @return base64解码数据
     */

    public byte[] getBase64(String key) {
        String value = getString(key);
        if (tztZFTextUtils.isEmpty(value))
            return null;
        return tztZFDataEncrypt.decryptBase64(value.getBytes());
    }

    /**
     * 获取ArrayList数据
     *
     * @param key key值
     * @return ArrayList数据
     */
    public ArrayList<ArrayList<String>> getArray(String key) {
        String value = getString(key);
        if (tztZFTextUtils.isEmpty(value))
            return null;

        ArrayList<ArrayList<String>> ayValues = new ArrayList<>();

        ArrayList<String> ayGrid = new ArrayList<>();
        tztZFPublicFun.split(value, "\r\n", ayGrid);
        for (String strValue : ayGrid) {
            if (strValue.length() < 1)
                continue;
            ArrayList<String> ayData = new ArrayList<>();
            tztZFPublicFun.split(strValue, "|", ayData);
            if (ayData.size() < 1)
                continue;
            ayValues.add(ayData);
        }
//        }
//        String[] ay = value.split("\\r\\n");
//        for (int i = 0; i < ay.length; i++){
//            String sData = ay[i];
//            if (tztZFTextUtils.isEmpty(sData))
//                continue;
//
//            ArrayList<String> ayGridData = new ArrayList<>();
//            boolean bRemoveLast = sData.endsWith("|");
//            String[] aySub = sData.split("\\|");
//            for (int j = 0; j < aySub.length; j++){
//                String item = aySub[j];
//                if (j == aySub.length - 1
//                    && item.length() < 1 && bRemoveLast)
//                    continue;
//
//                ayGridData.add(item);
//            }
//            ayValues.add(ayGridData);
//        }
        return ayValues;
    }

    /**
     * 获取tztZFDataMapBase数据
     *
     * @param key key值
     * @return tztZFDataMapBase数据
     */
    public tztZFDataMapBase<String, Object> getDataMap(String key) {
        if (mData != null)
            return mData.getDataMap(key);
        return null;
    }

    /**
     * 获取应答errorno
     *
     * @return 应答errorno
     */
    public int getStatusCode() {
        return mStatusCode;
    }

    /**
     * 设置应答errorno
     *
     * @param mStatusCode 应答errorno
     */
    public void setStatusCode(int mStatusCode) {
        this.mStatusCode = mStatusCode;
    }

    /**
     * 获取应答message
     *
     * @return 应答message
     */
    public String getMessage() {
        return mMessage;
    }

    /**
     * 设置应答message
     *
     * @param mMessage 应答message
     */
    public void setMessage(String mMessage) {
        this.mMessage = mMessage;
    }

    /**
     * 获取接收状态
     *
     * @return 接收状态
     */
    public tztZFRecvType getRecvType() {
        return mRecvType;
    }

    /**
     * 设置接收状态
     *
     * @param mRecvType 接收状态
     */
    public void setRecvType(tztZFRecvType mRecvType) {
        this.mRecvType = mRecvType;
    }

    /**
     * 获取请求标志
     *
     * @return 请求标志
     */
    public int getReqObjTag() {
        return mReqObjTag;
    }

    /**
     * 设置请求标志
     *
     * @param mReqObjTag 请求标志
     */
    public void setReqObjTag(int mReqObjTag) {
        this.mReqObjTag = mReqObjTag;
    }

    /**
     * 获取请求数据
     *
     * @return 请求数据
     */
    public tztZFNetWorkReqObj getReqObj() {
        return mReqObj;
    }

    /**
     * 设置请求数据
     *
     * @param mReqObj 请求数据
     */
    public void setReqObj(tztZFNetWorkReqObj mReqObj) {
        this.mReqObj = mReqObj;
    }

    //endregion

    //region 保护方法（允许子类访问、允许相同包中访问）
    //endregion

    //region 内部方法（只允许相同包中访问）

    /**
     * 设置应答数据
     *
     * @param dataMap 应答数据
     */
    public void setResponseData(tztZFDataMapBase<String, Object> dataMap) {
        mData = dataMap;
        if (mData != null) {
            this.mStatusCode = mData.getInt("errorno");
            this.mMessage = mData.getStr("errormessage");
        }
    }

    /**
     * 应答数据结构转String
     *
     * @return 应答数据String
     */
    @Override
    public String toString() {
        return mData.toString();
    }

    //endregion

    //region 私有方法（只允许自己调用）
    //endregion

    //endregion

    //region 开发新增处理代码

    //endregion

}