package com.tztzf.protocol.core;

import com.tztzf.base.tztZFPublicFun;
import com.tztzf.commbase.tztZFDataMapBase;
import com.tztzf.log.tztZFLog;
import com.tztzf.log.tztZFLogLevel;
import com.tztzf.log.tztZFLogPrintType;
import com.tztzf.protocol.callback.tztZFDataProtocolInterface;
import com.tztzf.protocol.define.tztZFHandshakeStatus;
import com.tztzf.protocol.define.tztZFProtocolError;
import com.tztzf.protocol.tztnative.tztNativePubFun;
import com.tztzf.protocol.tztnative.tztNativeZFProtocolObject;

import java.util.Locale;

import static com.tztzf.protocol.define.tztZFHandshakeStatus.tztZFHandshake_Error;
import static com.tztzf.protocol.define.tztZFHandshakeStatus.tztZFHandshake_Failed;
import static com.tztzf.protocol.define.tztZFHandshakeStatus.tztZFHandshake_Handing;
import static com.tztzf.protocol.define.tztZFHandshakeStatus.tztZFHandshake_Init;
import static com.tztzf.protocol.define.tztZFHandshakeStatus.tztZFHandshake_Non;
import static com.tztzf.protocol.define.tztZFHandshakeStatus.tztZFHandshake_Rehand;
import static com.tztzf.protocol.define.tztZFHandshakeStatus.tztZFHandshake_Success;
import static com.tztzf.protocol.define.tztZFPackageType.tztZFPackage_Handshake;
import static com.tztzf.protocol.define.tztZFPackageType.tztZFPackage_Socket;
import static com.tztzf.protocol.define.tztZFProtocolError.tztZFHandshakeError_Check;
import static com.tztzf.protocol.define.tztZFProtocolError.tztZFHandshakeError_DBMBegin;
import static com.tztzf.protocol.define.tztZFProtocolError.tztZFHandshakeError_DBMEnd;
import static com.tztzf.protocol.define.tztZFProtocolError.tztZFHandshakeError_Decode2016Error;
import static com.tztzf.protocol.define.tztZFProtocolError.tztZFHandshakeError_Error;
import static com.tztzf.protocol.define.tztZFProtocolError.tztZFHandshakeError_Failed;
import static com.tztzf.protocol.define.tztZFProtocolError.tztZFHandshakeError_HandBegin;
import static com.tztzf.protocol.define.tztZFProtocolError.tztZFHandshakeError_HandEnd;
import static com.tztzf.protocol.define.tztZFProtocolError.tztZFHandshakeError_NotAllowProtocol;
import static com.tztzf.protocol.define.tztZFProtocolError.tztZFHandshakeError_PackIDCheckError;
import static com.tztzf.protocol.define.tztZFProtocolError.tztZFHandshakeError_Protocol;
import static com.tztzf.protocol.define.tztZFProtocolError.tztZFHandshakeError_ServerDBM;
import static com.tztzf.protocol.define.tztZFProtocolError.tztZFHandshakeError_ServerHand;
import static com.tztzf.protocol.define.tztZFProtocolError.tztZFHandshakeError_ServerRecv;
import static com.tztzf.protocol.define.tztZFProtocolError.tztZFHandshakeError_UnKnowAction;
import static com.tztzf.protocol.define.tztZFProtocolError.tztZFProtocolError_Init;
import static com.tztzf.protocol.define.tztZFProtocolError.tztZFProtocolError_OK;
import static com.tztzf.protocol.define.tztZFProtocolType.tztZFProtocolType_1975;
import static com.tztzf.protocol.define.tztZFProtocolType.tztZFProtocolType_2013;
import static com.tztzf.protocol.define.tztZFProtocolType.tztZFProtocolType_2016;

/**
 * 作者: yangdl 2019.06
 * 用途: 通讯协议类，发送数据编码；接收数据检查、解码；协议握手处理等
 * 修改记录如下:
 *
 * @版本号: 1.00.001 2019.06.10
 * 更新说明: 第一版正式发布
 * 1.把数据编码、解码统一通过tztZFEncryptKit处理（NDK调用）
 * 2.支持RC(RC4)、SM(SM2)、RSA协议握手
 * 3.断网后，先初始化握手状态
 * 4.联网成功后，先判断握手状态，非握手成功状态，发起握手，握手成功，设置服务器可重连标记
 */
public class tztZFDataProtocolKit {

    private static int allPackageTag = 0;
    int linkType = 0;
    int sdkType = 0;
    int userID = 0;
    int cosigntype = 0;
    int bustype = 0;
    private Object context;
    private long dataObj = 0;
    @tztZFHandshakeStatus
    private int handStatus = tztZFHandshake_Non;
    private int handTag;
    private byte[] intactToServer = null;
    //心跳包
    private byte[] onlineData = null;
    /** 握手错误 */
    private @tztZFProtocolError int nErrorCode = tztZFProtocolError_Init;
    /**
     * 服务器返回错误码
     */
    private int nErrorNo = tztZFProtocolError_Init;
    private String errMsg = "";
    private int nRegisterSDKCert = tztZFProtocolError_Init;
    private int protocolType = 0;
    //协议头长
    private int protocolHeadLen = 6;
    //协议头 
    private short protocolHead = 2013;

    private tztZFDataProtocolInterface events = null;

    public tztZFDataProtocolKit(Object context, int linkType, tztZFDataProtocolInterface events) {
        init(context, linkType, events);
    }

    public static void setLogLevel(int level) {
        tztNativePubFun.tztZFCLog_setLevel(level);
    }

    //通讯协议对象初始化 inIndex 通讯类型序号
    private void init(Object context, int linkType, tztZFDataProtocolInterface events) {
        this.context = context;
        this.events = events;

        this.sdkType = 0;
        this.userID = 0;
        this.linkType = linkType;

        nErrorNo = tztZFProtocolError_Init;
        nErrorCode = nErrorNo;
        nRegisterSDKCert = tztZFProtocolError_OK;
        errMsg = "";
        handStatus = tztZFHandshake_Non;

        if (dataObj == 0) {
            dataObj = tztNativeZFProtocolObject.createObj();
        }
        tztlog("Init", "Init! linkType=%d", linkType);
    }

    protected void finalize() {
        free();
    }

    public void free() {
        if (dataObj != 0) {
            tztlog("free", "free:%d", dataObj);
            tztNativeZFProtocolObject.freeObj(dataObj);
            dataObj = 0;
        }
    }

    public boolean loadProtocolCert(int userID, int sdkType) {
        this.sdkType = sdkType;
        this.userID = userID;
        //获取SDK相关数据
        nRegisterSDKCert = tztNativeZFProtocolObject.loadProtocolCert(dataObj, this.userID, this.sdkType, linkType);
        //验证协议
        if (nRegisterSDKCert > 0 && checkProtocolType()) {
            nErrorNo = tztZFProtocolError_Init;
            nErrorCode = nErrorNo;
            cosigntype = 0;
            return true;
        }
        handStatus = tztZFHandshake_Failed;
        nErrorNo = nRegisterSDKCert;
        nErrorCode = nErrorNo;
        return false;
    }

    public boolean setProtocolCert(int userID, int sdkType, tztZFProtocolCert protocolCert) {
        this.sdkType = sdkType;
        this.userID = userID;
        if (protocolCert == null)
            return false;
        byte[] dataCert = protocolCert.toStream();
        if (dataCert == null || dataCert.length < 1)
            return false;
        nRegisterSDKCert = tztNativeZFProtocolObject.setProtocolCert(dataObj, userID, sdkType, linkType, dataCert);
        //验证协议
        if (nRegisterSDKCert > 0 && checkProtocolType()) {
            nErrorNo = tztZFProtocolError_Init;
            nErrorCode = nErrorNo;
            cosigntype = 1;
            return true;
        }
        handStatus = tztZFHandshake_Failed;
        nErrorNo = nRegisterSDKCert;
        nErrorCode = nErrorNo;
        return false;
    }

    public boolean setProtocolBus(int bus) {
        bustype = bus;
        if (bustype > 0) {

        }
        tztNativeZFProtocolObject.setProtocolBus(dataObj, bustype);
        return true;
    }

    public boolean switchToLoadCert() {
        cosigntype = 0;
        onlineData = null;
        if (bustype > 0) {
            return setProtocolBus(bustype);
        }
        return tztNativeZFProtocolObject.switchToLoadCert(dataObj) == 1;
    }

    /**
     * 检查协议类型
     *
     * @return 支持返回1，否则返回0
     */
    private boolean checkProtocolType() {
        return tztNativeZFProtocolObject.checkProtocolType(dataObj);
    }

    /**
     * 初始化协议
     *
     * @return 成功返回1，否则返回0
     */
    public int onProtocolInit() {
        if (nRegisterSDKCert > 0 && checkProtocolType()) {
            if (handStatus == tztZFHandshake_Init)
                return 1;
            return dataObjInitProtocol();
        } else {
            handStatus = tztZFHandshake_Failed;
            nErrorNo = tztZFHandshakeError_Protocol;
            nErrorCode = nErrorNo;
            return 0;
        }
    }

    /**
     * 处理协议包头
     */
    private void dealProtocolHead() {
        protocolType = tztNativeZFProtocolObject.getProtocolType(dataObj);
        if (protocolType == tztZFProtocolType_1975) {
            protocolHead = 1975;
        } else if (protocolType == tztZFProtocolType_2013) {
            protocolHead = 2013;
        } else if (protocolType == tztZFProtocolType_2016) {
            protocolHead = 2016;
        }
    }

    /**
     * 检索数据包头，若检索成功，返回对应的数据包包体长度，否则返回0
     *
     * @param cHeader 数据流
     * @return 成功返回包体长度，否则返回0
     */
    public int checkPackageHeader(byte[] cHeader) {
        if (cHeader == null || cHeader.length < protocolHeadLen)
            return -1;
        int head = tztZFPublicFun.getInt(cHeader, 0, 2);
        // 是否包头是否
        tztlog("Head", "PackageHeader:%d=%d", head, protocolHead);
        if (head != protocolHead) {
            dealProtocolHead();
            if (head != protocolHead)
                return -1;
        }
        return tztZFPublicFun.getInt(cHeader, 2, 4);
    }

    /**
     * 协议完整心跳发送包
     *
     * @return 成功返回完整心跳发送包 否则返回null
     */
    public byte[] transferOnlineData() {
        if (onlineData == null || onlineData.length <= 0)
            onlineData = tztNativeZFProtocolObject.transferOnlineData(dataObj);
        return onlineData;
    }

    /**
     * 将请求数据对象转换成协议完整发送数据流
     *
     * @param sendPackage 请求数据对象
     * @return 转换后的协议完整发送数据流
     */
    public byte[] transferSendData(tztZFSendPackage sendPackage) {
        if (sendPackage == null)
            return null;
        tztZFDataMapBase<String, Object> sendData = sendPackage.getSendData();
        if (sendData == null)
            return null;
        int nAction = sendPackage.getAction();

        tztlog("SendData", "[Action:%d]", nAction);
        if (sendPackage.getPackageType() == tztZFPackage_Handshake || nAction == handAction()) { //握手请求
            setHandshakeReqData(sendData);
            sendPackage.setPackageType(tztZFPackage_Handshake);
        } else if (sendPackage.getPackageType() == tztZFPackage_Socket) {
            sendData.setStr("Action", "46");
            sendData.setStr("SocketCheck", "1");
        } else if (nAction <= 0) {
            return null;
        }

        sendPackage.setIntactToServer(intactToServer);

        String strtfrom = sendData.getStr("tfrom");
        if (strtfrom == null || strtfrom.length() < 1) {
            sendData.setStr("tfrom", "Android.tztzf");
        }

        if (!getSendTag(sendPackage))
            return null;
        byte[] sendBytes = tztNativeZFProtocolObject.transferSendData(dataObj, sendPackage, sendPackage.getPackageIndex());
        if (sendBytes != null && sendBytes.length > 0 && sendPackage.getPackageType() == tztZFPackage_Handshake) {
            handTag = sendPackage.getTag();
            handStatus = tztZFHandshake_Handing;
        }
        if (sendBytes != null) {
            tztlog("SendData", "[Action:%d]%d", nAction, sendBytes.length);
        }
        return sendBytes;
    }

    /**
     * 协议应答数据流转换成应答对象
     *
     * @param recvPackage 协议应答对象
     * @param data        协议应答数据流
     * @return true 解析成功 false 解析失败
     */
    public boolean transferPackageData(tztZFRecvPackage recvPackage, byte[] data) {
        boolean bRet = tztNativeZFProtocolObject.transferPackage(dataObj, recvPackage, data);
        if (bRet && recvPackage != null) {
            tztZFDataMapBase<String, Object> mapData = recvPackage.getRecvData();
            byte[] intact = recvPackage.getIntacttoServer();
            if (intact != null && intact.length > 0) {
                intactToServer = intact;
            }
            int Action = mapData.getInt("Action");
            if (Action == 0)
                return false;
            //判断是否握手应答
            if (Action == handAction()) {
                if (checkHandshakeAns(recvPackage)) {
                    setHandshakeData(mapData);
                    recvPackage.setPackageType(tztZFPackage_Handshake);
                    if (events != null) {
                        events.didHandshake(handStatus);
                    }
                }
            }
            return true;
        }
        return false;
    }

    private int handAction() {
        return tztNativeZFProtocolObject.getHandShakeAction(dataObj);
    }

    //握手初始化
    private int dataObjInitProtocol() {
        int nHandKind = tztNativeZFProtocolObject.initProtocolType(dataObj);
        handTag = -1;
        if (nHandKind <= 0) { //协议不支持
            handStatus = tztZFHandshake_Failed;
            nErrorNo = tztZFHandshakeError_Protocol;
            nErrorCode = nErrorNo;
            return 0;
        }
        //不握手，不需要协商，直接置为协商成功
        if (nHandKind == 2) {
            handStatus = tztZFHandshake_Success;
        } else {
            handStatus = tztZFHandshake_Init;
        }
        dealProtocolHead();
        return 1;
    }

    /**
     * 协商状态 获取协商状态
     *
     * @return 返回当前协商状态
     */
    //当前握手状态
    public @tztZFHandshakeStatus int getCurrentStatus() {
        return handStatus;
    }

    /**
     * 协商失败 设置状态为协商失败（应答超时、应答服务器断开等）
     */
    //握手失败
    public void onHandshakeErr(int errorNo) {
        handStatus = tztZFHandshake_Error;
        nErrorNo = errorNo;
        nErrorCode = nErrorNo;
        errMsg = String.format(Locale.getDefault(), "[%d]协商请求失败", nErrorNo);
    }

    private boolean checkHandshakeAns(tztZFRecvPackage recvPackage) {
        if (recvPackage == null)
            return false;
        if (handTag <= 0)
            return false;
        if (handStatus != tztZFHandshake_Handing) {
            return false;
        }
        int reqTag = recvPackage.getTag();
        if (reqTag == handTag)
            return true;
        return false;
    }

    /**
     * 协商功能应答处理
     *
     * @param mapData 应答对象
     * @return true：握手应答数据成功处理；false：非对应握手应答数据
     */
    //握手数据应答
    private void setHandshakeData(tztZFDataMapBase<String, Object> mapData) {
        int nError = mapData.getInt("errorno");
        if (nError < 0) {
            nErrorNo = nError;
            errMsg = mapData.getStr("errormessage");
            //证书错误、验证不通过，直接返回握手失败，不允许使用
            if (nError == tztZFHandshakeError_Failed
                    || nError == tztZFHandshakeError_Check) {
                //秘钥已经作废或者不存在
                handStatus = tztZFHandshake_Failed;
            } else if (nError == tztZFHandshakeError_PackIDCheckError
                    || nError == tztZFHandshakeError_Decode2016Error) {
                //请求包校验或解码失败，重连服务器
                handStatus = tztZFHandshake_Rehand;
            } else if (nError == tztZFHandshakeError_NotAllowProtocol
                    || nError == tztZFHandshakeError_Error
                    || nError == tztZFHandshakeError_UnKnowAction) {
                //协议禁用，切换协议
                tztNativeZFProtocolObject.switchToFixProtocol(dataObj);
                //重新处理协议包头
                dealProtocolHead();
                handStatus = tztZFHandshake_Rehand;
            } else { //非无此功能号错误
                handStatus = tztZFHandshake_Error;
            }

            if (nError <= tztZFHandshakeError_HandBegin && nError >= tztZFHandshakeError_HandEnd) {
                nErrorCode = tztZFHandshakeError_ServerHand;
            } else if (nError <= tztZFHandshakeError_DBMBegin && nError >= tztZFHandshakeError_DBMEnd) {
                nErrorCode = tztZFHandshakeError_ServerDBM;
            } else {
                nErrorCode = tztZFHandshakeError_ServerRecv;
            }
            return;
        }
        nErrorNo = tztNativeZFProtocolObject.getHandShakeResponse(dataObj);
        nErrorCode = nErrorNo;
        if (nErrorNo != tztZFProtocolError_OK) {
            handStatus = tztZFHandshake_Failed;
            errMsg = "[" + nErrorNo + "]协商失败！";
            return;
        }
        handStatus = tztZFHandshake_Success;
    }

    //协商请求内容设置
    private boolean setHandshakeReqData(tztZFDataMapBase<String, Object> mapData) {
        errMsg = "";
        String tfrom = mapData.getStr("Tfrom");
        String cfrom = mapData.getStr("Cfrom");

        mapData.removeAllObjects();
        int nAction = handAction();
        mapData.setInt("Action", nAction);
        if (tfrom != null && tfrom.length() > 0) {
            mapData.setStr("Tfrom", tfrom);
        }
        if (cfrom != null && cfrom.length() > 0) {
            mapData.setStr("Cfrom", cfrom);
        }
        return true;
    }

    public String getHandAppID() {
        byte[] appID = tztNativeZFProtocolObject.getHandAppID(dataObj);
        if (appID == null)
            return "0";
        try {
            return new String(appID, "UTF-8");
        } catch (Exception e) {
            return new String(appID);
        }
    }

    public int getErrorNo() {
        return nErrorNo;
    }

    public int getErrorCode() {
        return nErrorCode;
    }

    public String getErrorInfo() {
        String strTip = "";
        if (nRegisterSDKCert <= 0) {
            return "";
        }
        if (nErrorNo == tztZFProtocolError_OK) {
            strTip = String.format(Locale.getDefault(), "[%d]服务器验证成功。", nErrorNo);
        } else if (nErrorNo == 0) {
            strTip = String.format(Locale.getDefault(), "[%d]服务器验证中...", nErrorNo);
        } else {
            strTip = String.format(Locale.getDefault(), "[%d]服务器验证失败。", nErrorNo);
        }
        return strTip;
    }

    private void tztlog(String tag, String format, Object... args) {
        if (tztZFLog.getLevel() <= tztZFLogLevel.tztZFLogLevel_Verbose.value()) {
            String cTag = "DataProtocol[" + this.hashCode() + "]";
            tztZFLog.log(tztZFLogLevel.tztZFLogLevel_Verbose, tztZFLogPrintType.tztZFLogPrintType_tztSocket.value(), cTag, "(" + tag + ")" + format, args);
        }
    }

    private boolean getSendTag(tztZFSendPackage sendPackage) {
        int nAction = sendPackage.getAction();
        if (nAction == 0)
            return false;
        synchronized (this) {
            allPackageTag++;
            if (allPackageTag <= 0) {
                allPackageTag = 1;
            }
            sendPackage.withProtocol(protocolType, allPackageTag);
        }
        return true;
    }
}
