package com.tztzf.protocol.core;

import static com.tztzf.protocol.define.tztZFAlgo_EncryptType.tztZFAlgo_EncryptType_RC;
import static com.tztzf.protocol.define.tztZFAlgo_HandshakeType.tztZFAlgo_HandshakeType_RC;
import static com.tztzf.protocol.define.tztZFProtocolType.tztZFProtocolType_1975;
import static com.tztzf.protocol.define.tztZFProtocolType.tztZFProtocolType_2013;
import static com.tztzf.protocol.define.tztZFProtocolType.tztZFProtocolType_2016;

import com.tztzf.network.cosign.tztZFInputStream;

///协议证书
public class tztZFProtocolCert {
    ///通讯协议
    public int protocolType = tztZFProtocolType_2013;
    ///握手协议
    public int handType = tztZFAlgo_HandshakeType_RC;
    ///握手协议参数 （RSA: 1024/2048; ECC:nid）
    public int handParameter = 0;
    public int encryptType = tztZFAlgo_EncryptType_RC;
    public int sPubKeyIndex = 0;
    public byte[] sPubKey = null;
    public int cPubKeyIndex = 0;
    public byte[] cPubKey = null;
    public byte[] iDB = null;
    public byte[] iDA = null;

    public tztZFProtocolCert() {

    }

    public boolean isInvalid() {
        return (sPubKey == null || sPubKey.length < 1
                || iDA == null || iDA.length < 1
                || iDB == null || iDB.length < 1);
    }

    public byte[] toStream() {
        switch (protocolType) {
            case 2013:
                protocolType = tztZFProtocolType_2013;
                break;
            case 2016:
                protocolType = tztZFProtocolType_2016;
                break;
            case 1975:
                protocolType = tztZFProtocolType_1975;
                break;
            case tztZFProtocolType_1975:
            case tztZFProtocolType_2013:
            case tztZFProtocolType_2016:
                break;
            default:
                protocolType = tztZFProtocolType_2013;
        }

        tztZFInputStream inputStream = new tztZFInputStream();
        inputStream.appendInt(protocolType);
        inputStream.appendInt(handType);
        inputStream.appendInt(encryptType);
        inputStream.appendInt(handParameter);
        inputStream.appendInt(cPubKeyIndex);
        inputStream.appendInt(sPubKeyIndex);
        inputStream.appendLenByte(iDA);
        inputStream.appendLenByte(iDB);
        inputStream.appendLenByte(cPubKey);
        inputStream.appendLenByte(sPubKey);
        return inputStream.getInputData();
    }
}
