package com.tztzf.protocol.core;

import android.app.Application;

import com.tztzf.base.tztZFPublicFun;
import com.tztzf.commbase.tztZFDataMapBase;
import com.tztzf.network.define.tztZFErrorMessageDefine;
import com.tztzf.protocol.callback.tztZFRegisterInterface;
import com.tztzf.protocol.tztnative.tztNativeZFProtocolObject;
import com.zfnetwork.BuildConfig;

import java.util.HashMap;
import java.util.Locale;

import static com.tztzf.protocol.define.tztZFProtocolError.tztZFRegisterError_SDKApp;
import static com.tztzf.protocol.define.tztZFProtocolError.tztZFRegisterError_SDKAppName;
import static com.tztzf.protocol.define.tztZFProtocolError.tztZFRegisterError_SDKBad;
import static com.tztzf.protocol.define.tztZFProtocolError.tztZFRegisterError_SDKCert;
import static com.tztzf.protocol.define.tztZFProtocolError.tztZFRegisterError_SDKErr;
import static com.tztzf.protocol.define.tztZFProtocolError.tztZFRegisterError_SDKFormat;
import static com.tztzf.protocol.define.tztZFProtocolError.tztZFRegisterError_SDKID;
import static com.tztzf.protocol.define.tztZFProtocolError.tztZFRegisterError_SDKKey;
import static com.tztzf.protocol.define.tztZFProtocolError.tztZFRegisterError_SDKSession;
import static com.tztzf.protocol.define.tztZFProtocolError.tztZFRegisterError_SDKUserID;
import static com.tztzf.protocol.define.tztZFProtocolError.tztZFRegisterError_SDKtype;
import static com.tztzf.protocol.define.tztZFProtocolError.tztZFRegisterError_SDKunCert;
import static com.tztzf.protocol.define.tztZFProtocolError.tztZFRegisterError_SDKunLinktype;

/**
 * ****************************************************************
 * 文件名称:tztZFRegister.java
 * 作    者:Created by yangdl
 * 创建时间:2019.10.16 11:26
 * 文件描述:中焯SDK注册
 * 注意事项:
 * 版权声明:Copyright (C) 2019-2029 杭州中焯信息技术股份有限公司
 * 修改历史:2019.10.16
 * ****************************************************************
 */

public class tztZFRegister {
    private static String regPackageName = "";
    private static HashMap<String, tztZFRegisterInterface> maphandler = new HashMap<>();

    public static boolean onRegisterSDKType(Object application, int sdkType, String sdkID, String sdkKey, byte[] sdkCert, String appPackage, String appName, tztZFRegisterInterface completion) {
        if (application == null || tztZFPublicFun.isEmpty(sdkKey) || tztZFPublicFun.isEmpty(sdkID)) {
            return false;
        }
        Application app = (Application) application;
        byte[] bytePackageName = null;
        String strAppPackage = app.getPackageName();
        regPackageName = strAppPackage.toLowerCase();
        bytePackageName = regPackageName.getBytes();

        byte[] byteAppName = null;
        String strAppName = "";
        if (app.getPackageManager() != null
                && app.getApplicationInfo() != null
                && app.getPackageManager().getApplicationLabel(app.getApplicationInfo()) != null) {
            strAppName = app.getPackageManager().getApplicationLabel(app.getApplicationInfo()).toString();
        }
        try {
            byteAppName = strAppName.getBytes("UTF8");
        } catch (Exception exception) {
            byteAppName = strAppName.getBytes();
        }

        if (!tztZFPublicFun.isEmpty(appPackage) && !tztZFPublicFun.isEmpty(appName) && BuildConfig.DEBUG) {
            try {
                bytePackageName = appPackage.getBytes("UTF8");
                byteAppName = appName.getBytes("UTF8");
            } catch (Exception exception) {
                bytePackageName = appPackage.getBytes();
                byteAppName = appName.getBytes();
            }
        }

        String strPackagePath = app.getPackageCodePath();
        byte[] bytePackagePath = strPackagePath.getBytes();
        byte[] byteSDKID = sdkID.getBytes();
        byte[] byteSDKKey = sdkKey.getBytes();

        maphandler.put(sdkID + "", completion);
        int[] result = tztNativeZFProtocolObject.registerKitNative(sdkType, byteSDKID, byteSDKKey, sdkCert, bytePackageName, byteAppName, bytePackagePath);
        if (result != null && completion != null) {
            int nRet = result[0];
            int nUserID = result[1];
            int nSdkType = result[2];
            int nDebug = result[3];
            tztZFDataMapBase<String, Object> mapBlock = new tztZFDataMapBase<>();
            mapBlock.setStr("ErrorNo", String.valueOf(nRet));
            mapBlock.setStr("ErrorMessage", registerErrMessage(nRet));
            mapBlock.setStr("SDKType", String.valueOf(nSdkType));
            mapBlock.setStr("UserID", String.valueOf(nUserID));
            mapBlock.setStr("Debug", String.valueOf(nDebug));
            mapBlock.setStr("LinkType", "-1");
            mapBlock.setStr("BlockType", "1");
            completion.onHandler(sdkID, mapBlock);
        }
        return true;
    }

    public static String getRegPackageName() {
        return regPackageName;
    }

    public static void onHandler(int sdkType, String sdkID, int nError) {
        tztZFRegisterInterface handler = maphandler.get(sdkID + "");
        if (handler != null) {
            String sErrMsg = "";
            tztZFDataMapBase<String, Object> blockData = new tztZFDataMapBase<>();
            blockData.setStr("ErrorNo", String.valueOf(nError));
            blockData.setStr("ErrorMessage", sErrMsg);
            blockData.setStr("SDKType", String.valueOf(sdkType));
            blockData.setStr("LinkType", "-1");
            blockData.setInt("BlockType", 1);
            handler.onHandler(String.valueOf(sdkType), blockData);
        }
    }

    private static String registerErrMessage(int errorno) {
        if (errorno > 0) {
            return "注册成功";
        }
        String certLable = tztZFErrorMessageDefine.getUserDefineError(tztZFErrorMessageDefine.tztZFNetWorkErrorKey_CertLable, tztZFErrorMessageDefine.tztZFCertLable);
        switch (errorno) {
            case tztZFRegisterError_SDKBad:
                return String.format(Locale.getDefault(), "参数错误[%d]", errorno);
            case tztZFRegisterError_SDKunCert:
                return String.format(Locale.getDefault(), "%s不存在[%d]", certLable, errorno);
            case tztZFRegisterError_SDKCert:
            case tztZFRegisterError_SDKFormat:
            case tztZFRegisterError_SDKSession:
                return String.format(Locale.getDefault(), "%s不合法[%d]", certLable, errorno);
            case tztZFRegisterError_SDKtype:
            case tztZFRegisterError_SDKID:
            case tztZFRegisterError_SDKKey:
            case tztZFRegisterError_SDKApp:
            case tztZFRegisterError_SDKAppName:
            case tztZFRegisterError_SDKunLinktype:
            case tztZFRegisterError_SDKUserID:
                return String.format(Locale.getDefault(), "%s不合法或不匹配[%d]", certLable, errorno);
            case tztZFRegisterError_SDKErr:
                return String.format(Locale.getDefault(), "%s注册失败[%d]", certLable, errorno);
            default:
                break;
        }
        return String.format(Locale.getDefault(), "其他错误[%d]", errorno);
    }
}
