package com.tztzf.protocol.tztnative;

import com.tztzf.base.tztZFPublicFun;
import com.tztzf.commbase.tztZFDataMapBase;
import com.tztzf.log.tztZFLog;
import com.tztzf.log.tztZFLogLevel;
import com.tztzf.log.tztZFLogPrintType;
import com.tztzf.protocol.core.tztZFRecvPackage;
import com.tztzf.protocol.core.tztZFSendPackage;
import com.tztzf.soloader.tztZFSoLoader;

/**
 * 作者: yangdl 2019.06
 * 用途: tztProtocol.so:NDK数据编解码处理
 * 修改记录如下:
 *
 * @版本号: 1.00.001 2019.06.10
 * 更新说明: 第一版正式发布
 */

public class tztNativeZFProtocolObject {

    private static tztNativeZFProtocolObject protocolObjKit = new tztNativeZFProtocolObject();

    private static byte[] byteseNull = null;

    static {
        tztZFSoLoader.loadLibrary("tztZFProtocol");
//        System.loadLibrary("tztZFProtocol");
    }

    private static tztNativeZFProtocolObject getShareInstance() {
        if (protocolObjKit == null)
            protocolObjKit = new tztNativeZFProtocolObject();
        return protocolObjKit;
    }

    private static int getlen(byte[] data) {
        if (data != null)
            return data.length;
        return 0;
    }

    //创建dataobj
    public static long createObj() {
        return getShareInstance().createObjNative();
    }

    //释放dataobj
    public static boolean freeObj(long dataObj) {
        return getShareInstance().freeObjNative(dataObj);
    }

    //加载注册证书信息:厂商userID、类别:sdkType、通讯链路:linkType
    //>0成功,其他失败
    public static int loadProtocolCert(long dataObj, int userID, int sdkType, int linkType) {
        return getShareInstance().loadProtocolCertNative(dataObj, userID, sdkType, linkType);
    }

    //设置协商证书信息
    public static int setProtocolCert(long dataObj, int userID, int sdkType, int linkType, byte[] cProtocolCert) {
        return getShareInstance().setProtocolCertNative(dataObj, userID, sdkType, linkType, cProtocolCert, getlen(cProtocolCert));
    }

    public static int setProtocolBus(long dataObj, int bus) {
        return getShareInstance().setProtocolBusNative(dataObj, bus);
    }

    //获取通讯协议类别
    public static int getProtocolType(long dataObj) {
        return getShareInstance().getProtocolTypeNative(dataObj);
    }

    //获取协商类型
    public static int getHandShakeType(long dataObj) {
        return getShareInstance().getHandShakeTypeNative(dataObj);
    }

    //获取协商功能号
    public static int getHandShakeAction(long dataObj) {
        return getShareInstance().getHandShakeActionNative(dataObj);
    }

    //验证协议
    public static boolean checkProtocolType(long dataObj) {
        return getShareInstance().checkProtocolTypeNative(dataObj);
    }

    //初始化协议所有相关数据
    public static int initProtocolType(long dataObj) {
        return getShareInstance().initProtocolTypeNative(dataObj);
    }

    //初始化请求(应答)数据(0:请求 1:应答)
    public static int initProtocolData(long dataObj, int kind) {
        return getShareInstance().initProtocolDataNative(dataObj, kind);
    }

    //设置协商请求
    public static int setHandShakeRequest(long dataObj) {
        return getShareInstance().setHandShakeRequestNative(dataObj);
    }

    //获取协商应答
    public static int getHandShakeResponse(long dataObj) {
        return getShareInstance().getHandShakeResponseNative(dataObj);
    }

    //切换2013协议
    public static int switchToFixProtocol(long dataObj) {
        return getShareInstance().switchToFixProtocolNative(dataObj);
    }

    //切换为导入证书
    public static int switchToLoadCert(long dataObj) {
        return getShareInstance().switchToLoadCertNative(dataObj);
    }

    //data:packagelen(4)+pos(4)+headlen(4)
    public static byte[] tztDataProtocolCheckPackage(long dataObj, byte[] cPackageHeader) {
        return getShareInstance().tztDataProtocolCheckPackageNative(dataObj, cPackageHeader, getlen(cPackageHeader));
    }

    //获取appid
    public static byte[] getHandAppID(long dataObj) {
        return getShareInstance().getHandAppIDNative(dataObj);
    }

    //协议心跳包
    public static byte[] transferOnlineData(long dataObj) {
        return getShareInstance().transferOnlineDataNative(dataObj);
    }

    //发送协议数据编码
    //协议数据包加密 data:keylen(1)+key+vallen(4)+val;
    public static byte[] transferSendData(long dataObj, tztZFSendPackage sendPackage, int packageIndex) {
        if (sendPackage == null || dataObj == 0)
            return null;
        tztZFDataMapBase<String, Object> mapData = sendPackage.getSendData();
        if (mapData == null)
            return null;
        int packagetype = sendPackage.getPackageType();
        //key字段长
        byte[] type = getShareInstance().tztGetAlgoInfoNative(dataObj, 0);
        int gbk = tztZFPublicFun.getInt(type, 0, 4);
        int kbit = tztZFPublicFun.getInt(type, 4, 4);
        String strTag = String.valueOf(sendPackage.getTag());
        byte[] reqno = strTag.getBytes();
        byte[] intact = mapData.getBytes("Intacttoserver");
        int action = sendPackage.getAction();
        if (action == 0)
            return null;
        mapData.removeByKey("Action");
        mapData.removeByKey("Reqno");
        mapData.removeByKey("Intacttoserver");
        byte[] sdata = mapData.getBuffer(gbk, kbit);
        tztZFLog.log(tztZFLogLevel.tztZFLogLevel_Verbose, tztZFLogPrintType.tztZFLogPrintType_tztSocket.value(), "SendData",
                "[Action:%d;Reqno:%s;%s];packageindex:%d;packagetype:%d transferSendDataKit",
                action, strTag, mapData.getStr("Reqtag"), packageIndex, packagetype);
        return getShareInstance().transferSendDataNative(dataObj, packagetype, packageIndex, action,
                reqno, getlen(reqno), intact, getlen(intact), sdata, getlen(sdata));
    }

    //应答协议数据解码
    public static boolean transferPackage(long dataObj, tztZFRecvPackage recvPackage, byte[] data) {
        if (data == null || data.length <= 0)
            return false;
        if (recvPackage == null)
            return false;
        tztZFDataMapBase<String, Object> mapData = recvPackage.getRecvData();
        if (mapData == null)
            return false;
        //协议应答数据解析 type[encoding(4)+keylen(4)+action(4)],data,reqno,intact;
        byte[][] result = getShareInstance().transferPackageNative(dataObj, data, getlen(data));
        if (result == null || result.length != 2 || result[0] == null) {
            return false;
        } else {
            //type[encoding(4)+keylen(4)],data;
            byte[] type = result[0];
            byte[] decode = result[1];
            int Encoding = tztZFPublicFun.getInt(type, 0, 4);
            int KeyLen = tztZFPublicFun.getInt(type, 4, 4);
            mapData.setBuffer(decode, Encoding, KeyLen);
            //设置应答包标示
            recvPackage.setRecvTag();
            int action = mapData.getAction();
            String reqno = mapData.getReqno();
            tztZFLog.log(tztZFLogLevel.tztZFLogLevel_Verbose, tztZFLogPrintType.tztZFLogPrintType_tztSocket.value(), "RecvData", "[Action:%d;Reqno:%s] transferPackage", action, reqno);
            return true;
        }
    }

    //证书注册
    public static int[] registerKitNative(int sdkType, byte[] sdkid, byte[] sdkkey, byte[] sdkcert, byte[] apppackage, byte[] appname,
                                          byte[] packagepath) {
        return getShareInstance().registerKitNative(sdkType, sdkid, getlen(sdkid),
                sdkkey, getlen(sdkkey),
                sdkcert, getlen(sdkcert),
                apppackage, getlen(apppackage),
                appname, getlen(appname),
                packagepath, getlen(packagepath));
    }
    //endregion

    //region NDK 函数
//------------------------------NDK 函数----------------------//
    //创建dataobj
    private native long createObjNative();

    //释放dataobj
    private native boolean freeObjNative(long dataObj);

    //加载注册证书信息:厂商userID、类别:sdkType、通讯链路:linkType
    //>0成功,其他失败
    private native int loadProtocolCertNative(long dataObj, int userID, int sdkType, int linkType);

    ///设置协商证书信息
    private native int setProtocolCertNative(long dataObj, int userID, int sdkType, int linkType, byte[] cProtocolCert, int nProtocolCert);

    private native int setProtocolBusNative(long dataObj, int bus);

    //获取通讯协议类别
    private native int getProtocolTypeNative(long dataObj);

    //获取协商类型
    private native int getHandShakeTypeNative(long dataObj);

    //获取协商功能号
    private native int getHandShakeActionNative(long dataObj);

    //验证协议
    private native boolean checkProtocolTypeNative(long dataObj);

    //初始化协议所有相关数据
    private native int initProtocolTypeNative(long dataObj);

    //初始化请求(应答)数据(0:请求 1:应答)
    private native int initProtocolDataNative(long dataObj, int kind);

    //设置协商请求
    private native int setHandShakeRequestNative(long dataObj);

    //获取协商应答
    private native int getHandShakeResponseNative(long dataObj);

    //切换2013协议
    private native int switchToFixProtocolNative(long dataObj);

    //切换为导入证书
    private native int switchToLoadCertNative(long dataObj);

    //协议信息
    private native byte[] tztGetAlgoInfoNative(long dataObj, int kind);

    //data:packagelen(4)+pos(4)+headlen(4)
    private native byte[] tztDataProtocolCheckPackageNative(long dataObj, byte[] cPackageHeader, int cPackageHeaderLength);

    //获取appid
    private native byte[] getHandAppIDNative(long dataObj);

    //协议心跳包
    private native byte[] transferOnlineDataNative(long dataObj);

    //协议数据包加密 data:keylen(1)+key+vallen(4)+val;
    private native byte[] transferSendDataNative(long dataObj, int packagetype,
                                                 int packageindex, int action,
                                                 byte[] reqno, int reqnolen,
                                                 byte[] intact, int intactlen,
                                                 byte[] data, int datalen);

    //协议应答数据解析 type[encoding(4)+keylen(4)+action(4)],data,reqno,intact;
    private native byte[][] transferPackageNative(long dataObj, byte[] data, int datalen);

    //证书注册
    private native int[] registerKitNative(int sdkType,
                                           byte[] sdkid, int sdkidlen,
                                           byte[] sdkkey, int sdkkeylen,
                                           byte[] sdkcert, int sdkcertlen,
                                           byte[] apppackage, int apppackagelen,
                                           byte[] appname, int appnamelen,
                                           byte[] packagepath, int packagepathlen);
    //endregion
}
